@@ -374,11 +374,6 @@ enum ha_base_keytype {
   update handler::auto_increment_value
 */
 #define HA_STATUS_AUTO          64
-/*
-  update the number of rows updated and deleted since last reset call.
-  handler::rows_updated, rows_deleted
-*/
-#define HA_STATUS_WRITTEN_ROWS  128
 /*
   Errorcodes given by handler functions
 
