@@ -245,11 +245,7 @@ typedef struct st_net {
 #if !defined(CHECK_EMBEDDED_DIFFERENCES) || !defined(EMBEDDED_LIBRARY)
   Vio *vio;
   unsigned char *buff,*buff_end,*write_pos,*read_pos;
-#ifdef __WIN__
-  SOCKET fd;
-#else
-  int fd;					/* For Perl DBI/dbd */
-#endif
+  my_socket fd;					/* For Perl DBI/dbd */
   /*
     The following variable is set if we are doing several queries in one
     command ( as in LOAD TABLE ... FROM MASTER ),
@@ -286,6 +282,16 @@ typedef struct st_net {
   /** Client library sqlstate buffer. Set along with the error message. */
   char sqlstate[SQLSTATE_LENGTH+1];
   void *extension;
+#if defined(MYSQL_SERVER) && !defined(EMBEDDED_LIBRARY)
+  /*
+    Controls whether a big packet should be skipped.
+
+    Initially set to FALSE by default. Unauthenticated sessions must have
+    this set to FALSE so that the server can't be tricked to read packets
+    indefinitely.
+  */
+  my_bool skip_big_packet;
+#endif
 } NET;
 
 
@@ -423,13 +429,8 @@ void my_net_set_read_timeout(NET *net, uint timeout);
   Currently it's used internally by manager.c
 */
 struct sockaddr;
-#ifdef __WIN__
-int my_connect(SOCKET s, const struct sockaddr *name, unsigned int namelen,
+int my_connect(my_socket s, const struct sockaddr *name, unsigned int namelen,
 	       unsigned int timeout);
-#else
-int my_connect(int s,    const struct sockaddr *name, unsigned int namelen,
-	       unsigned int timeout);
-#endif
 
 struct rand_struct {
   unsigned long seed1,seed2,max_value;
