@@ -33,6 +33,7 @@ sub mtr_options_from_test_file($$);
 
 my $do_test;
 my $skip_test;
+my %incompatible;
 
 sub init_pattern {
   my ($from, $what)= @_;
@@ -48,6 +49,15 @@ sub init_pattern {
 }
 
 
+sub collect_incomp_tests {
+  open (INCOMP, "lib/v1/incompatible.tests");
+  while (<INCOMP>)
+  {
+    next unless /^\w/;
+    s/\s.*\n//;		      # Ignore anything from first white space
+    $incompatible{$_}= 1;
+  }
+}
 
 ##############################################################################
 #
@@ -59,6 +69,8 @@ sub collect_test_cases ($) {
   $do_test= init_pattern($::opt_do_test, "--do-test");
   $skip_test= init_pattern($::opt_skip_test, "--skip-test");
 
+  collect_incomp_tests();
+
   my $suites= shift; # Semicolon separated list of test suites
   my $cases = [];    # Array of hash
 
@@ -529,6 +541,17 @@ sub collect_one_test_case($$$$$$$$$) {
   $tinfo->{'component_id'} = $component_id;
   push(@$cases, $tinfo);
 
+  # Remove "combinations" part of test name
+  my $test_base_name= $tinfo->{'name'};
+  $test_base_name=~ s/\s.*\n//;
+
+  if (exists ($incompatible{$test_base_name}))
+  {
+    $tinfo->{'skip'}= 1;
+    $tinfo->{'comment'}= "Test cannot run in mtr v1";
+    return;
+  }
+  
   # ----------------------------------------------------------------------
   # Skip some tests but include in list, just mark them to skip
   # ----------------------------------------------------------------------
@@ -842,7 +865,7 @@ sub collect_one_test_case($$$$$$$$$) {
     if ( $tinfo->{'innodb_test'} )
     {
       # This is a test that need innodb
-      if ( $::mysqld_variables{'innodb'} ne "TRUE" )
+      if ( $::mysqld_variables{'innodb'} eq "OFF" )
       {
 	# innodb is not supported, skip it
 	$tinfo->{'skip'}= 1;
