@@ -265,8 +265,9 @@ sub spawn_parent_impl {
           if ( $timer_name eq "suite" )
           {
             # We give up here
-            # FIXME we should only give up the suite, not all of the run?
             print STDERR "\n";
+            kill(9, $pid); # Kill mysqltest
+            mtr_kill_leftovers(); # Kill servers the hard way
             mtr_error("Test suite timeout");
           }
           elsif ( $timer_name eq "testcase" )
