@@ -15,17 +15,18 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
-int ndbcluster_connect(int (*connect_callback)(void));
-int ndbcluster_disconnect();
+int ndbcluster_connect(int (*connect_callback)(void),
+                       ulong wait_connected,
+                       uint connection_pool_size,
+                       bool optimized_node_select,
+                       const char* connect_string, uint force_nodeid);
+void ndbcluster_disconnect(void);
 
 Ndb_cluster_connection *ndb_get_cluster_connection();
 ulonglong ndb_get_latest_trans_gci();
 void ndb_set_latest_trans_gci(ulonglong val);
 int ndb_has_node_id(uint id);
 
-/* options from from mysqld.cc */
-extern ulong opt_ndb_cluster_connection_pool;
-
 /* perform random sleep in the range milli_sleep to 2*milli_sleep */
 inline void do_retry_sleep(unsigned milli_sleep)
 {
