@@ -19,11 +19,7 @@
 /**
   @file
 
-  This file is the net layer API for the MySQL client/server protocol,
-  which is a tightly coupled, proprietary protocol owned by MySQL AB.
-  @note
-  Any re-implementations of this protocol must also be under GPL
-  unless one has got an license from MySQL AB stating otherwise.
+  This file is the net layer API for the MySQL client/server protocol.
 
   Write and read of logical packets to/from socket.
 
@@ -48,7 +44,6 @@
 #include <my_sys.h>
 #include <m_string.h>
 #include <my_net.h>
-#include <my_socket.h>
 #include <violite.h>
 #include <signal.h>
 #include <errno.h>
@@ -140,11 +135,13 @@ my_bool my_net_init(NET *net, Vio* vio)
 #else
   net->query_cache_query= 0;
 #endif
+#if defined(MYSQL_SERVER) && !defined(EMBEDDED_LIBRARY)
+  net->skip_big_packet= FALSE;
+#endif
 
   if (vio != 0)					/* If real connection */
   {
-    /* Return native socket type in net->fd as is part of ABI */
-    net->fd  = MY_SOCKET_FORMAT_VALUE(vio_fd(vio));	/* For perl DBI/DBD */
+    net->fd  = vio_fd(vio);	/* For perl DBI/DBD */
 #if defined(MYSQL_SERVER) && !defined(__WIN__)
     if (!(test_flags & TEST_BLOCKING))
     {
@@ -232,7 +229,7 @@ static int net_data_is_ready(my_socket sd)
   struct pollfd ufds;
   int res;
 
-  ufds.fd= sd.fd;
+  ufds.fd= sd;
   ufds.events= POLLIN | POLLPRI;
   if (!(res= poll(&ufds, 1, 0)))
     return 0;
@@ -252,14 +249,14 @@ static int net_data_is_ready(my_socket sd)
 #endif
 
   FD_ZERO(&sfds);
-  my_FD_SET(sd, &sfds);
+  FD_SET(sd, &sfds);
 
   tv.tv_sec= tv.tv_usec= 0;
 
-  if ((res= select(my_socket_nfds(sd,0)+1, &sfds, NULL, NULL, &tv)) < 0)
+  if ((res= select((int) (sd + 1), &sfds, NULL, NULL, &tv)) < 0)
     return 0;
   else
-    return test(res ? my_FD_ISSET(sd, &sfds) : 0);
+    return test(res ? FD_ISSET(sd, &sfds) : 0);
 #endif /* HAVE_POLL */
 }
 
@@ -908,7 +905,13 @@ my_real_read(NET *net, size_t *complen)
 		       ("Packets out of order (Found: %d, expected %u)",
 			(int) net->buff[net->where_b + 3],
 			net->pkt_nr));
-#ifdef EXTRA_DEBUG
+            /* 
+              We don't make noise server side, since the client is expected
+              to break the protocol for e.g. --send LOAD DATA .. LOCAL where
+              the server expects the client to send a file, but the client
+              may reply with a new command instead.
+            */
+#if defined (EXTRA_DEBUG) && !defined (MYSQL_SERVER)
             fflush(stdout);
 	    fprintf(stderr,"Error: Packets out of order (Found: %d, expected %d)\n",
 		    (int) net->buff[net->where_b + 3],
@@ -954,6 +957,7 @@ my_real_read(NET *net, size_t *complen)
 	  {
 #if defined(MYSQL_SERVER) && !defined(NO_ALARM)
 	    if (!net->compress &&
+                net->skip_big_packet &&
 		!my_net_skip_rest(net, (uint32) len, &alarmed, &alarm_buff))
 	      net->error= 3;		/* Successfully skiped packet */
 #endif
