@@ -29,21 +29,37 @@
 int init_intvar_from_file(int* var, IO_CACHE* f, int default_val);
 int init_strvar_from_file(char *var, int max_size, IO_CACHE *f,
 			  const char *default_val);
+#ifndef MCP_WL342
 int init_floatvar_from_file(float* var, IO_CACHE* f, float default_val);
+#endif
+#ifndef MCP_BUG47037
+int init_dynarray_intvar_from_file(DYNAMIC_ARRAY* arr, IO_CACHE* f);
+#endif
 
 Master_info::Master_info()
   :Slave_reporting_capability("I/O"),
-   ssl(0), ssl_verify_server_cert(0), fd(-1), io_thd(0),
-   heartbeat_period(0), received_heartbeats(0),
-   inited(0), abort_slave(0),slave_running(0), slave_run_id(0)
+   ssl(0), ssl_verify_server_cert(0), fd(-1), io_thd(0), inited(0),
+   abort_slave(0),slave_running(0),
+   slave_run_id(0)
 {
-  host[0] = 0; user[0] = 0; password[0] = 0; bind_addr[0] = 0;
+  host[0] = 0; user[0] = 0; password[0] = 0;
   ssl_ca[0]= 0; ssl_capath[0]= 0; ssl_cert[0]= 0;
   ssl_cipher[0]= 0; ssl_key[0]= 0;
 
-  master_server_id= (uint32)ULONG_MAX;
+#ifndef MCP_WL3127
+  bind_addr[0] = 0;
+#endif
+#ifndef MCP_WL4080
   master_epoch= 0;
-
+#endif
+#ifndef MCP_BUG47037
+  master_id = 0;
+  my_init_dynamic_array(&ignore_server_ids, sizeof(::server_id), 16, 16);
+#endif
+#ifndef MCP_WL342
+  heartbeat_period= 0;
+  received_heartbeats= 0;
+#endif
   bzero((char*) &file, sizeof(file));
   pthread_mutex_init(&run_lock, MY_MUTEX_INIT_FAST);
   pthread_mutex_init(&data_lock, MY_MUTEX_INIT_FAST);
@@ -54,6 +70,9 @@ Master_info::Master_info()
 
 Master_info::~Master_info()
 {
+#ifndef MCP_BUG47037
+  delete_dynamic(&ignore_server_ids);
+#endif
   pthread_mutex_destroy(&run_lock);
   pthread_mutex_destroy(&data_lock);
   pthread_cond_destroy(&data_cond);
@@ -61,6 +80,45 @@ Master_info::~Master_info()
   pthread_cond_destroy(&stop_cond);
 }
 
+#ifndef MCP_BUG47037
+/**
+   A comparison function to be supplied as argument to @c sort_dynamic()
+   and @c bsearch()
+
+   @return -1 if first argument is less, 0 if it equal to, 1 if it is greater
+   than the second
+*/
+int change_master_server_id_cmp(ulong *id1, ulong *id2)
+{
+  return *id1 < *id2? -1 : (*id1 > *id2? 1 : 0);
+}
+
+
+/**
+   Reports if the s_id server has been configured to ignore events 
+   it generates with
+
+      CHANGE MASTER IGNORE_SERVER_IDS= ( list of server ids )
+
+   Method is called from the io thread event receiver filtering.
+
+   @param      s_id    the master server identifier
+
+   @retval   TRUE    if s_id is in the list of ignored master  servers,
+   @retval   FALSE   otherwise.
+ */
+bool Master_info::shall_ignore_server_id(ulong s_id)
+{
+  if (likely(ignore_server_ids.elements == 1))
+    return (* (ulong*) dynamic_array_ptr(&ignore_server_ids, 0)) == s_id;
+  else      
+    return bsearch((const ulong *) &s_id,
+                   ignore_server_ids.buffer,
+                   ignore_server_ids.elements, sizeof(ulong),
+                   (int (*) (const void*, const void*)) change_master_server_id_cmp)
+      != NULL;
+}
+#endif
 
 void init_master_info_with_options(Master_info* mi)
 {
@@ -69,9 +127,10 @@ void init_master_info_with_options(Master_info* mi)
   mi->master_log_name[0] = 0;
   mi->master_log_pos = BIN_LOG_HEADER_SIZE;             // skip magic number
 
+#ifndef MCP_WL3127
   /* Reset Master Bind to empty string (bind all IPv4 + IPv6) */
   strmake(mi->bind_addr, "", sizeof(mi->bind_addr));
-
+#endif
   if (master_host)
     strmake(mi->host, master_host, sizeof(mi->host) - 1);
   if (master_user)
@@ -80,6 +139,8 @@ void init_master_info_with_options(Master_info* mi)
     strmake(mi->password, master_password, MAX_PASSWORD_LENGTH);
   mi->port = master_port;
   mi->connect_retry = master_connect_retry;
+
+#ifndef MCP_WL342
   /* 
     always request heartbeat unless master_heartbeat_period is set
     explicitly zero.  Here is the default value for heartbeat period
@@ -90,6 +151,7 @@ void init_master_info_with_options(Master_info* mi)
                                     (slave_net_timeout/2.0));
   DBUG_ASSERT(mi->heartbeat_period > (float) 0.001
               || mi->heartbeat_period == 0);
+#endif
   mi->ssl= master_ssl;
   if (master_ssl_ca)
     strmake(mi->ssl_ca, master_ssl_ca, sizeof(mi->ssl_ca)-1);
@@ -113,13 +175,26 @@ enum {
   /* 5.1.16 added value of master_ssl_verify_server_cert */
   LINE_FOR_MASTER_SSL_VERIFY_SERVER_CERT= 15,
 
+#ifndef MCP_WL342
   /* 5.1.23 added value of master_heartbeat_period */
   LINE_FOR_MASTER_HEARTBEAT_PERIOD= 16,
+#endif
 
+#ifndef MCP_WL3127
   LINES_IN_MASTER_INFO_WITH_SSL_AND_BIND_ADDR= 17,
+#endif
+
+#ifndef MCP_BUG47037
+  /* 6.0 added value of master_ignore_server_id */
+  LINE_FOR_REPLICATE_IGNORE_SERVER_IDS= 18,
+#endif
 
   /* Number of lines currently used when saving master info file */
-  LINES_IN_MASTER_INFO= LINES_IN_MASTER_INFO_WITH_SSL_AND_BIND_ADDR
+#ifndef MCP_BUG47037
+  LINES_IN_MASTER_INFO= LINE_FOR_REPLICATE_IGNORE_SERVER_IDS
+#else
+  LINES_IN_MASTER_INFO= LINE_FOR_MASTER_SSL_VERIFY_SERVER_CERT
+#endif
 };
 
 int init_master_info(Master_info* mi, const char* master_info_fname,
@@ -221,7 +296,9 @@ file '%s')", fname);
     mi->fd = fd;
     int port, connect_retry, master_log_pos, lines;
     int ssl= 0, ssl_verify_server_cert= 0;
+#ifndef MCP_WL342
     float master_heartbeat_period= 0.0;
+#endif
     char *first_non_digit;
 
     /*
@@ -307,18 +384,31 @@ file '%s')", fname);
           init_intvar_from_file(&ssl_verify_server_cert, &mi->file, 0))
         goto errwithmsg;
 
-      /*
-        Starting from 5.1.23 master_heartbeat_period might be
-        in the file
-      */
+#ifndef MCP_WL342
       if (lines >= LINE_FOR_MASTER_HEARTBEAT_PERIOD &&
           init_floatvar_from_file(&master_heartbeat_period, &mi->file, 0.0))
         goto errwithmsg;
+#endif
 
+#ifndef MCP_WL3127
       if (lines >= LINES_IN_MASTER_INFO_WITH_SSL_AND_BIND_ADDR &&
           init_strvar_from_file(mi->bind_addr, sizeof(mi->bind_addr),
                                 &mi->file, ""))
         goto errwithmsg;
+#endif
+
+#ifndef MCP_BUG47037
+      /*
+        Starting from 6.0 list of server_id of ignorable servers might be
+        in the file
+      */
+      if (lines >= LINE_FOR_REPLICATE_IGNORE_SERVER_IDS &&
+          init_dynarray_intvar_from_file(&mi->ignore_server_ids, &mi->file))
+      {
+        sql_print_error("Failed to initialize master info ignore_server_ids");
+        goto errwithmsg;
+      }
+#endif
     }
 
 #ifndef HAVE_OPENSSL
@@ -327,6 +417,7 @@ file '%s')", fname);
                       "('%s') are ignored because this MySQL slave was compiled "
                       "without SSL support.", fname);
 #endif /* HAVE_OPENSSL */
+
     /*
       This has to be handled here as init_intvar_from_file can't handle
       my_off_t types
@@ -336,7 +427,9 @@ file '%s')", fname);
     mi->connect_retry= (uint) connect_retry;
     mi->ssl= (my_bool) ssl;
     mi->ssl_verify_server_cert= ssl_verify_server_cert;
+#ifndef MCP_WL342
     mi->heartbeat_period= master_heartbeat_period;
+#endif
   }
   DBUG_PRINT("master_info",("log_file_name: %s  position: %ld",
                             mi->master_log_name,
@@ -349,7 +442,7 @@ file '%s')", fname);
   mi->inited = 1;
   // now change cache READ -> WRITE - must do this before flush_master_info
   reinit_io_cache(&mi->file, WRITE_CACHE, 0L, 0, 1);
-  if ((error=test(flush_master_info(mi, 1))))
+  if ((error=test(flush_master_info(mi, TRUE, TRUE))))
     sql_print_error("Failed to flush master info file");
   pthread_mutex_unlock(&mi->data_lock);
   DBUG_RETURN(error);
@@ -375,10 +468,13 @@ err:
      1 - flush master info failed
      0 - all ok
 */
-int flush_master_info(Master_info* mi, bool flush_relay_log_cache)
+int flush_master_info(Master_info* mi, 
+                      bool flush_relay_log_cache, 
+                      bool need_lock_relay_log)
 {
   IO_CACHE* file = &mi->file;
   char lbuf[22];
+  int err= 0;
 
   DBUG_ENTER("flush_master_info");
   DBUG_PRINT("enter",("master_pos: %ld", (long) mi->master_log_pos));
@@ -395,9 +491,46 @@ int flush_master_info(Master_info* mi, bool flush_relay_log_cache)
     When we come to this place in code, relay log may or not be initialized;
     the caller is responsible for setting 'flush_relay_log_cache' accordingly.
   */
-  if (flush_relay_log_cache &&
-      flush_io_cache(mi->rli.relay_log.get_log_file()))
-    DBUG_RETURN(2);
+
+#ifndef MCP_BUG47037
+  /*
+    produce a line listing the total number and all the ignored server_id:s
+  */
+  char* ignore_server_ids_buf;
+  {
+    ignore_server_ids_buf=
+      (char *) my_malloc((sizeof(::server_id) * 3 + 1) *
+                         (1 + mi->ignore_server_ids.elements), MYF(MY_WME));
+    if (!ignore_server_ids_buf)
+      DBUG_RETURN(1);
+    for (ulong i= 0, cur_len= sprintf(ignore_server_ids_buf,
+                                      "%u", mi->ignore_server_ids.elements);
+         i < mi->ignore_server_ids.elements; i++)
+    {
+      ulong s_id;
+      get_dynamic(&mi->ignore_server_ids, (uchar*) &s_id, i);
+      cur_len +=sprintf(ignore_server_ids_buf + cur_len, " %lu", s_id);
+    }
+  }
+#endif
+
+  if (flush_relay_log_cache)
+  {
+    pthread_mutex_t *log_lock= mi->rli.relay_log.get_log_lock();
+    IO_CACHE *log_file= mi->rli.relay_log.get_log_file();
+
+    if (need_lock_relay_log)
+      pthread_mutex_lock(log_lock);
+
+    safe_mutex_assert_owner(log_lock);
+    err= flush_io_cache(log_file);
+
+    if (need_lock_relay_log)
+      pthread_mutex_unlock(log_lock);
+
+    if (err)
+      DBUG_RETURN(2);
+  }
 
   /*
     We flushed the relay log BEFORE the master.info file, because if we crash
@@ -415,18 +548,30 @@ int flush_master_info(Master_info* mi, bool flush_relay_log_cache)
      contents of file). But because of number of lines in the first line
      of file we don't care about this garbage.
   */
+
+#ifndef MCP_WL342
   char heartbeat_buf[sizeof(mi->heartbeat_period) * 4]; // buffer to suffice always
-  my_sprintf(heartbeat_buf, (heartbeat_buf, "%.3f", mi->heartbeat_period));
+  sprintf(heartbeat_buf, "%.3f", mi->heartbeat_period);
+#endif
   my_b_seek(file, 0L);
   my_b_printf(file,
-              "%u\n%s\n%s\n%s\n%s\n%s\n%d\n%d\n%d\n%s\n%s\n%s\n%s\n%s\n%d\n%s\n",
+              "%u\n%s\n%s\n%s\n%s\n%s\n%d\n%d\n%d\n%s\n%s\n%s\n%s\n%s\n%d\n%s\n%s\n%s\n",
               LINES_IN_MASTER_INFO,
               mi->master_log_name, llstr(mi->master_log_pos, lbuf),
               mi->host, mi->user,
               mi->password, mi->port, mi->connect_retry,
               (int)(mi->ssl), mi->ssl_ca, mi->ssl_capath, mi->ssl_cert,
               mi->ssl_cipher, mi->ssl_key, mi->ssl_verify_server_cert,
-              heartbeat_buf);
+#ifndef MCP_WL342
+              heartbeat_buf,
+#endif
+#ifndef MCP_WL3127
+              mi->bind_addr,
+#endif
+#ifndef MCP_BUG47037
+              ignore_server_ids_buf);
+  my_free(ignore_server_ids_buf, MYF(0));
+#endif
   DBUG_RETURN(-flush_io_cache(file));
 }
 
