@@ -27,8 +27,23 @@
 #define HA_FT_WLEN   4
 #define FT_SEGS      2
 
+/**
+  Accessor methods for the weight and the number of subkeys in a buffer.
+
+  The weight is of float type and subkeys number is of integer type. Both
+  are stored in the same position of the buffer and the stored object is
+  identified by the sign (bit): the weight value is positive whilst the
+  number of subkeys is negative.
+
+  In light of C's strict-aliasing rules, which roughly state that an object
+  must not be accessed through incompatible types, these methods are used to
+  avoid any problems arising from the type duality inside the buffer. The
+  values are retrieved using a character type which can access any object.
+*/
 #define ft_sintXkorr(A)    mi_sint4korr(A)
 #define ft_intXstore(T,A)  mi_int4store(T,A)
+#define ft_floatXget(V,M)  mi_float4get(V,M)
+
 
 extern const HA_KEYSEG ft_keysegs[FT_SEGS];
 
