@@ -1,6 +1,4 @@
-/*
-   Copyright (C) 2000-2006 MySQL AB
-    All rights reserved. Use is subject to license terms.
+/* Copyright (c) 2000, 2010 Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -217,36 +215,14 @@ const char *ha_myisammrg::index_type(uint key_number)
 static int myisammrg_parent_open_callback(void *callback_param,
                                           const char *filename)
 {
-  ha_myisammrg  *ha_myrg;
-  TABLE         *parent;
+  ha_myisammrg  *ha_myrg= (ha_myisammrg*) callback_param;
+  TABLE         *parent= ha_myrg->table_ptr();
   TABLE_LIST    *child_l;
-  const char    *db;
-  const char    *table_name;
   size_t        dirlen;
   char          dir_path[FN_REFLEN];
+  char          name_buf[NAME_LEN];
   DBUG_ENTER("myisammrg_parent_open_callback");
 
-  /* Extract child table name and database name from filename. */
-  dirlen= dirname_length(filename);
-  if (dirlen >= FN_REFLEN)
-  {
-    /* purecov: begin inspected */
-    DBUG_PRINT("error", ("name too long: '%.64s'", filename));
-    my_errno= ENAMETOOLONG;
-    DBUG_RETURN(1);
-    /* purecov: end */
-  }
-  table_name= filename + dirlen;
-  dirlen--; /* Strip off trailing '/'. */
-  memcpy(dir_path, filename, dirlen);
-  dir_path[dirlen]= '\0';
-  db= base_name(dir_path);
-  dirlen-= db - dir_path; /* This is now the length of 'db'. */
-  DBUG_PRINT("myrg", ("open: '%s'.'%s'", db, table_name));
-
-  ha_myrg= (ha_myisammrg*) callback_param;
-  parent= ha_myrg->table_ptr();
-
   /* Get a TABLE_LIST object. */
   if (!(child_l= (TABLE_LIST*) alloc_root(&parent->mem_root,
                                           sizeof(TABLE_LIST))))
@@ -258,13 +234,70 @@ static int myisammrg_parent_open_callback(void *callback_param,
   }
   bzero((char*) child_l, sizeof(TABLE_LIST));
 
-  /* Set database (schema) name. */
-  child_l->db_length= dirlen;
-  child_l->db= strmake_root(&parent->mem_root, db, dirlen);
-  /* Set table name. */
-  child_l->table_name_length= strlen(table_name);
-  child_l->table_name= strmake_root(&parent->mem_root, table_name,
-                                    child_l->table_name_length);
+  /*
+    Depending on MySQL version, filename may be encoded by table name to
+    file name encoding or not. Always encoded if parent table is created
+    by 5.1.46+. Encoded if parent is created by 5.1.6+ and child table is
+    in different database.
+  */
+  if (!has_path(filename))
+  {
+    /* Child is in the same database as parent. */
+    child_l->db_length= parent->s->db.length;
+    child_l->db= strmake_root(&parent->mem_root, parent->s->db.str,
+                              child_l->db_length);
+    /* Child table name is encoded in parent dot-MRG starting with 5.1.46. */
+    if (parent->s->mysql_version >= 50146)
+    {
+      child_l->table_name_length= filename_to_tablename(filename, name_buf,
+                                                        sizeof(name_buf));
+      child_l->table_name= strmake_root(&parent->mem_root, name_buf,
+                                        child_l->table_name_length);
+    }
+    else
+    {
+      child_l->table_name_length= strlen(filename);
+      child_l->table_name= strmake_root(&parent->mem_root, filename,
+                                        child_l->table_name_length);
+    }
+  }
+  else
+  {
+    DBUG_ASSERT(strlen(filename) < sizeof(dir_path));
+    fn_format(dir_path, filename, "", "", 0);
+    /* Extract child table name and database name from filename. */
+    dirlen= dirname_length(dir_path);
+    /* Child db/table name is encoded in parent dot-MRG starting with 5.1.6. */
+    if (parent->s->mysql_version >= 50106)
+    {
+      child_l->table_name_length= filename_to_tablename(dir_path + dirlen,
+                                                        name_buf,
+                                                        sizeof(name_buf));
+      child_l->table_name= strmake_root(&parent->mem_root, name_buf,
+                                        child_l->table_name_length);
+      dir_path[dirlen - 1]= 0;
+      dirlen= dirname_length(dir_path);
+      child_l->db_length= filename_to_tablename(dir_path + dirlen, name_buf,
+                                                sizeof(name_buf));
+      child_l->db= strmake_root(&parent->mem_root, name_buf, child_l->db_length);
+    }
+    else
+    {
+      child_l->table_name_length= strlen(dir_path + dirlen);
+      child_l->table_name= strmake_root(&parent->mem_root, dir_path + dirlen,
+                                        child_l->table_name_length);
+      dir_path[dirlen - 1]= 0;
+      dirlen= dirname_length(dir_path);
+      child_l->db_length= strlen(dir_path + dirlen);
+      child_l->db= strmake_root(&parent->mem_root, dir_path + dirlen,
+                                child_l->db_length);
+    }
+  }
+
+  DBUG_PRINT("myrg", ("open: '%.*s'.'%.*s'", (int) child_l->db_length,
+                      child_l->db, (int) child_l->table_name_length,
+                      child_l->table_name));
+
   /* Convert to lowercase if required. */
   if (lower_case_table_names && child_l->table_name_length)
     child_l->table_name_length= my_casedn_str(files_charset_info,
@@ -310,7 +343,7 @@ static MI_INFO *myisammrg_attach_children_callback(void *callback_param)
   TABLE         *parent;
   TABLE         *child;
   TABLE_LIST    *child_l;
-  MI_INFO       *myisam;
+  MI_INFO       *UNINIT_VAR(myisam);
   DBUG_ENTER("myisammrg_attach_children_callback");
 
   my_errno= 0;
@@ -383,7 +416,7 @@ static MI_INFO *myisammrg_attach_children_callback(void *callback_param)
     my_errno= HA_ERR_WRONG_MRG_TABLE_DEF;
   }
   DBUG_PRINT("myrg", ("MyISAM handle: 0x%lx  my_errno: %d",
-                      (long) myisam, my_errno));
+                      my_errno ? NULL : (long) myisam, my_errno));
 
  err:
   DBUG_RETURN(my_errno ? NULL : myisam);
@@ -396,24 +429,24 @@ static MI_INFO *myisammrg_attach_children_callback(void *callback_param)
   @detail This function initializes the MERGE storage engine structures
     and adds a child list of TABLE_LIST to the parent TABLE.
 
-  @param[in]    name            MERGE table path name
-  @param[in]    mode            read/write mode, unused
-  @param[in]    test_if_locked  open flags
+  @param[in]    name                MERGE table path name
+  @param[in]    mode                read/write mode, unused
+  @param[in]    test_if_locked_arg  open flags
 
   @return       status
-    @retval     0               OK
-    @retval     -1              Error, my_errno gives reason
+    @retval     0                   OK
+    @retval     -1                  Error, my_errno gives reason
 */
 
 int ha_myisammrg::open(const char *name, int mode __attribute__((unused)),
-                       uint test_if_locked)
+                       uint test_if_locked_arg)
 {
   DBUG_ENTER("ha_myisammrg::open");
   DBUG_PRINT("myrg", ("name: '%s'  table: 0x%lx", name, (long) table));
-  DBUG_PRINT("myrg", ("test_if_locked: %u", test_if_locked));
+  DBUG_PRINT("myrg", ("test_if_locked_arg: %u", test_if_locked_arg));
 
   /* Save for later use. */
-  this->test_if_locked= test_if_locked;
+  test_if_locked= test_if_locked_arg;
 
   /* retrieve children table list. */
   my_errno= 0;
@@ -1099,8 +1132,8 @@ void ha_myisammrg::update_create_info(HA_CREATE_INFO *create_info)
 	goto err;
 
       create_info->merge_list.elements++;
-      (*create_info->merge_list.next) = (uchar*) ptr;
-      create_info->merge_list.next= (uchar**) &ptr->next_local;
+      (*create_info->merge_list.next) = ptr;
+      create_info->merge_list.next= &ptr->next_local;
     }
     *create_info->merge_list.next=0;
   }
@@ -1122,7 +1155,7 @@ int ha_myisammrg::create(const char *name, register TABLE *form,
 {
   char buff[FN_REFLEN];
   const char **table_names, **pos;
-  TABLE_LIST *tables= (TABLE_LIST*) create_info->merge_list.first;
+  TABLE_LIST *tables= create_info->merge_list.first;
   THD *thd= current_thd;
   size_t dirlgt= dirname_length(name);
   DBUG_ENTER("ha_myisammrg::create");
@@ -1135,7 +1168,7 @@ int ha_myisammrg::create(const char *name, register TABLE *form,
   /* Create child path names. */
   for (pos= table_names; tables; tables= tables->next_local)
   {
-    const char *table_name;
+    const char *table_name= buff;
 
     /*
       Construct the path to the MyISAM table. Try to meet two conditions:
@@ -1161,10 +1194,12 @@ int ha_myisammrg::create(const char *name, register TABLE *form,
       as the MyISAM tables are from the same database as the MERGE table.
     */
     if ((dirname_length(buff) == dirlgt) && ! memcmp(buff, name, dirlgt))
-      table_name= tables->table_name;
-    else
-      if (! (table_name= thd->strmake(buff, length)))
-        DBUG_RETURN(HA_ERR_OUT_OF_MEM); /* purecov: inspected */
+    {
+      table_name+= dirlgt;
+      length-= dirlgt;
+    }
+    if (!(table_name= thd->strmake(table_name, length)))
+      DBUG_RETURN(HA_ERR_OUT_OF_MEM); /* purecov: inspected */
 
     *pos++= table_name;
   }
@@ -1185,7 +1220,7 @@ void ha_myisammrg::append_create_info(String *packet)
   const char *current_db;
   size_t db_length;
   THD *thd= current_thd;
-  MYRG_TABLE *open_table, *first;
+  TABLE_LIST *open_table, *first;
 
   if (file->merge_insert_method != MERGE_INSERT_DISABLED)
   {
@@ -1203,14 +1238,11 @@ void ha_myisammrg::append_create_info(String *packet)
   current_db= table->s->db.str;
   db_length=  table->s->db.length;
 
-  for (first=open_table=file->open_tables ;
-       open_table != file->end_table ;
-       open_table++)
+  for (first= open_table= table->child_l;;
+       open_table= open_table->next_global)
   {
-    LEX_STRING db, name;
-    LINT_INIT(db.str);
+    LEX_STRING db= { open_table->db, open_table->db_length };
 
-    split_file_name(open_table->table->filename, &db, &name);
     if (open_table != first)
       packet->append(',');
     /* Report database for mapped table if it isn't in current database */
@@ -1221,7 +1253,10 @@ void ha_myisammrg::append_create_info(String *packet)
       append_identifier(thd, packet, db.str, db.length);
       packet->append('.');
     }
-    append_identifier(thd, packet, name.str, name.length);
+    append_identifier(thd, packet, open_table->table_name,
+                      open_table->table_name_length);
+    if (&open_table->next_global == table->child_last_l)
+      break;
   }
   packet->append(')');
 }
