@@ -230,9 +230,7 @@ MYRG_INFO *myrg_parent_open(const char *parent_name,
   int       save_errno;
   int       insert_method;
   uint      length;
-  uint      dir_length;
   uint      child_count;
-  size_t    name_buff_length;
   File      fd;
   IO_CACHE  file_cache;
   char      parent_name_buff[FN_REFLEN * 2];
@@ -302,7 +300,6 @@ MYRG_INFO *myrg_parent_open(const char *parent_name,
   }
 
   /* Call callback for each child. */
-  dir_length= dirname_part(parent_name_buff, parent_name, &name_buff_length);
   my_b_seek(&file_cache, 0);
   while ((length= my_b_gets(&file_cache, child_name_buff, FN_REFLEN - 1)))
   {
@@ -314,14 +311,6 @@ MYRG_INFO *myrg_parent_open(const char *parent_name,
     if (!child_name_buff[0] || (child_name_buff[0] == '#'))
       continue;
 
-    if (!has_path(child_name_buff))
-    {
-      VOID(strmake(parent_name_buff + dir_length, child_name_buff,
-                   sizeof(parent_name_buff) - 1 - dir_length));
-      VOID(cleanup_dirname(child_name_buff, parent_name_buff));
-    }
-    else
-      fn_format(child_name_buff, child_name_buff, "", "", 0);
     DBUG_PRINT("info", ("child: '%s'", child_name_buff));
 
     /* Callback registers child with handler table. */
@@ -390,12 +379,11 @@ int myrg_attach_children(MYRG_INFO *m_info, int handle_locking,
 {
   ulonglong  file_offset;
   MI_INFO    *myisam;
-  int        rc;
   int        errpos;
   int        save_errno;
   uint       idx;
   uint       child_nr;
-  uint       key_parts;
+  uint       UNINIT_VAR(key_parts);
   uint       min_keys;
   my_bool    bad_children= FALSE;
   DBUG_ENTER("myrg_attach_children");
@@ -409,10 +397,8 @@ int myrg_attach_children(MYRG_INFO *m_info, int handle_locking,
     here and in ha_myisammrg::store_lock() forces consistent data.
   */
   pthread_mutex_lock(&m_info->mutex);
-  rc= 1;
   errpos= 0;
   file_offset= 0;
-  LINT_INIT(key_parts);
   min_keys= 0;
   child_nr= 0;
   while ((myisam= (*callback)(callback_param)))
