@@ -27,6 +27,7 @@
  */
 
 typedef Bitmask<MAXNROFATTRIBUTESINWORDS> AttributeMask;
+typedef BitmaskPOD<MAXNROFATTRIBUTESINWORDS> AttributeMaskPOD;
 
 template <Uint32 SZ>
 struct Id_array
@@ -35,6 +36,6 @@ struct Id_array
   Uint32 id[SZ];
 };
 
-typedef Id_array<MAX_ATTRIBUTES_IN_INDEX> AttributeList;
+typedef Id_array<MAX_ATTRIBUTES_IN_INDEX> IndexAttributeList;
 
 #endif
