@@ -191,8 +191,8 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 /* 120 not unused */
 #define GSN_ROUTE_ORD                   121
 #define GSN_NODE_VERSION_REP            122
-/* 123 unused */
-/* 124 unused */
+/* 123 not unused */
+#define GSN_FSSUSPENDORD                124 /* local */
 #define GSN_CHECK_LCP_STOP              125
 #define GSN_CLOSE_COMCONF               126 /* local */
 #define GSN_CLOSE_COMREQ                127 /* local */
@@ -295,9 +295,9 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 #define GSN_DROP_TAB_REF                203
 #define GSN_DROP_TAB_CONF               204
 
-#define GSN_208
-#define GSN_209
-#define GSN_216
+#define GSN_DIH_GET_TABINFO_REQ         208 /* distr. */
+#define GSN_DIH_GET_TABINFO_REF         209 /* distr. */
+#define GSN_DIH_GET_TABINFO_CONF        232 /* distr. */
 
 /*****************************************/
 
@@ -334,7 +334,7 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 /* 230 not unused */
 /* 231 not unused */
 
-/* 232 unused */
+/* 232 not unused */
 /* 233 unused */
 /* 234 unused */
 #define GSN_DISCONNECT_REP              235
@@ -673,6 +673,7 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 #define GSN_TCINDXNEXTCONF              525
 #define GSN_TCINDXNEXREF                526
 #define GSN_FIRE_TRIG_ORD               527
+#define GSN_FIRE_TRIG_ORD_L             123 /* local from TUP to SUMA */
 
 /**
  * These are used only by kernel
@@ -819,12 +820,13 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 
 #define GSN_SYNC_THREAD_REQ             608
 #define GSN_SYNC_THREAD_CONF            609
-#define GSN_610
-#define GSN_611
 
-#define GSN_612
-#define GSN_613
-#define GSN_614
+#define GSN_SYNC_REQ                    610
+#define GSN_SYNC_REF                    611
+#define GSN_SYNC_CONF                   612
+
+#define GSN_SYNC_PATH_REQ               613
+#define GSN_SYNC_PATH_CONF              614
 #define GSN_615
 #define GSN_616
 #define GSN_617
