@@ -31,10 +31,10 @@
 // sql/my_decimal.h requires many more sql/*.h new to ndb
 // for now, copy the bit we need  TODO proper fix
 
-#define DECIMAL_MAX_LENGTH ((8 * 9) - 8)
+#define DECIMAL_MAX_FIELD_SIZE ((9 * 9) - (2 * 8))
 
-#ifndef NOT_FIXED_DEC
-#define NOT_FIXED_DEC                   31
+#ifndef DECIMAL_NOT_SPECIFIED
+#define DECIMAL_NOT_SPECIFIED 31
 #endif
 
 C_MODE_START
@@ -166,8 +166,14 @@ public:
     AttributeExtScale      = 1015, //Default 0
     AttributeExtLength     = 1016, //Default 0
     AttributeAutoIncrement = 1017, //Default false
-    AttributeDefaultValue  = 1018, //Default value (printable string),
     AttributeArrayType     = 1019, //Default NDB_ARRAYTYPE_FIXED
+    AttributeDefaultValueLen = 1020, //Actual Length saved in AttributeDefaultValue
+    /* Default value (Binary type, not printable as string),
+     * For backward compatibility, the new keyValue
+     * (not use the old keyValue 1018) is added
+       when restoring data from low backup version data.
+    */
+    AttributeDefaultValue = 1021,
     AttributeEnd           = 1999  //
   };
   // ----------------------------------------------------------------------
@@ -434,7 +440,8 @@ public:
     Uint32 AttributeAutoIncrement;
     Uint32 AttributeStorageType;
     Uint32 AttributeDynamic;
-    char   AttributeDefaultValue[MAX_ATTR_DEFAULT_VALUE_SIZE];
+    Uint32 AttributeDefaultValueLen;  //byte sizes
+    Uint8  AttributeDefaultValue[MAX_ATTR_DEFAULT_VALUE_SIZE];
     
     Attribute() {}
     void init();
@@ -502,8 +509,9 @@ public:
           // copy from Field_new_decimal ctor
           uint precision = AttributeExtPrecision;
           uint scale = AttributeExtScale;
-          if (precision > DECIMAL_MAX_LENGTH || scale >= NOT_FIXED_DEC)
-            precision = DECIMAL_MAX_LENGTH;
+          if (precision > DECIMAL_MAX_FIELD_SIZE ||
+              scale >= DECIMAL_NOT_SPECIFIED)
+            return false;
           uint bin_size = my_decimal_get_binary_size(precision, scale);
           AttributeSize = DictTabInfo::an8Bit;
           AttributeArraySize = bin_size * AttributeExtLength;
@@ -588,8 +596,11 @@ public:
       fprintf(out, "AttributeExtPrecision = %d\n", AttributeExtPrecision);
       fprintf(out, "AttributeExtScale = %d\n", AttributeExtScale);
       fprintf(out, "AttributeExtLength = %d\n", AttributeExtLength);
-      fprintf(out, "AttributeDefaultValue = \"%s\"\n",
-        AttributeDefaultValue ? AttributeDefaultValue : "");
+      fprintf(out, "AttributeDefaultValueLen = %d\n",
+              AttributeDefaultValueLen);
+      fprintf(out, "AttributeDefaultValue: \n");
+      for (unsigned int i = 0; i < AttributeDefaultValueLen; i++)
+        fprintf(out, "0x%x", AttributeDefaultValue[i]);
     }
   };
   
@@ -617,7 +628,7 @@ private:
   Uint32 tabInfoData[DataLength];
 
 public:
-  enum Depricated
+  enum Deprecated
   {
     AttributeDGroup    = 1009, //Default NotDGroup
     AttributeStoredInd = 1011, //Default NotStored
