@@ -490,7 +490,7 @@ extern "C" {
   const char *ndb_mgm_get_connectstring(NdbMgmHandle handle, char *buf, int buf_sz);
 
   /**
-   * DEPRICATED: use ndb_mgm_set_timeout instead.
+   * DEPRECATED: use ndb_mgm_set_timeout instead.
    *
    * @param handle  NdbMgmHandle
    * @param seconds number of seconds
@@ -583,6 +583,25 @@ extern "C" {
    */
   const char *ndb_mgm_get_connected_bind_address(NdbMgmHandle handle);
 
+  /**
+   * Get the version of the mgm server we're talking to.
+   *
+   * @param   handle         Management handle
+   * @param   major          Returns the major version number for NDB
+   * @param   minor          Returns the minor version number for NDB
+   * @param   build          Returns the build version number for NDB
+   * @param   len            Specifies the max size of the buffer
+   *                         available to return version string in
+   * @param   str            Pointer to buffer where to return the
+   *                         version string which is in the
+   *                         form "mysql-X.X.X ndb-Y.Y.Y-status"
+   *
+   * @return  0 for error and 1 for success
+   */
+  int ndb_mgm_get_version(NdbMgmHandle handle,
+                          int *major, int *minor, int* build,
+                          int len, char* str);
+
 #ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
   /** @} *********************************************************************/
   /**
@@ -752,6 +771,29 @@ extern "C" {
   int ndb_mgm_stop3(NdbMgmHandle handle, int no_of_nodes,
 		    const int * node_list, int abort, int *disconnect);
 
+  /**
+   * Stops cluster nodes
+   *
+    * @param   handle        Management handle.
+   * @param   no_of_nodes   Number of database nodes to stop<br>
+   *                         -1: All database and management nodes<br>
+   *                          0: All database nodes in cluster<br>
+   *                          n: Stop the <var>n</var> node(s) specified in
+   *                            the array node_list
+   * @param   node_list     List of node IDs of database nodes to be stopped
+   * @param   abort         Don't perform graceful stop,
+   *                        but rather stop immediately
+   * @param   force         Force stop of nodes even if it means the
+   *                        whole cluster will be shutdown
+   * @param   disconnect    Returns true if you need to disconnect to apply
+   *                        the stop command (e.g. stopping the mgm server
+   *                        that handle is connected to)
+   *
+   * @return                Number of nodes stopped (-1 on error).
+   */
+   int ndb_mgm_stop4(NdbMgmHandle handle, int no_of_nodes,
+		    const int * node_list, int abort, int force,
+                    int *disconnect);
 
   /**
    * Restart database nodes
@@ -817,6 +859,33 @@ extern "C" {
 		       const int * node_list, int initial,
 		       int nostart, int abort, int *disconnect);
 
+  /**
+   * Restart nodes
+   *
+   * @param   handle        Management handle.
+   * @param   no_of_nodes   Number of database nodes to be restarted:<br>
+   *                          0: Restart all database nodes in the cluster<br>
+   *                          n: Restart the <var>n</var> node(s) specified
+   *                             in the array node_list
+   * @param   node_list     List of node IDs of database nodes to be restarted
+   * @param   initial       Remove filesystem from restarting node(s)
+   * @param   nostart       Don't actually start node(s) but leave them
+   *                        waiting for start command
+   * @param   abort         Don't perform graceful restart,
+   *                        but rather restart immediately
+   * @param   force         Force restart of nodes even if it means the
+   *                        whole cluster will be restarted
+   * @param   disconnect    Returns true if mgmapi client must disconnect from
+   *                        server to apply the requested operation. (e.g.
+   *                        restart the management server)
+   *
+   *
+   * @return                Number of nodes stopped (-1 on error).
+   */
+  int ndb_mgm_restart4(NdbMgmHandle handle, int no_of_nodes,
+		       const int * node_list, int initial,
+		       int nostart, int abort, int force, int *disconnect);
+
   /**
    * Start database nodes
    *
@@ -1216,38 +1285,6 @@ extern "C" {
    */
   Uint32 ndb_mgm_get_mgmd_nodeid(NdbMgmHandle handle);
 
-  /**
-   * Get the version of the mgm server we're talking to.
-   * Designed to allow switching of protocol depending on version
-   * so that new clients can speak to old servers in a compat mode
-   */
-  int ndb_mgm_get_version(NdbMgmHandle handle,
-                          int *major, int *minor, int* build,
-                          int len, char* str);
-
-  /**
-   * Query NDB$INFO.
-   * On success, returns number of rows.
-   * ndb_mgm_ndbinfo_getrow() *MUST* be called that many times.
-   */
-  int ndb_mgm_ndbinfo(NdbMgmHandle handle, const char *query, int *rows);
-
-  /**
-   * Gets the column names for NDBINFO query
-   *
-   * Must be called after successful ndb_mgm_ndbinfo() but BEFORE
-   * ndb_mgm_getrows.
-   */
-  int ndb_mgm_ndbinfo_colcount(NdbMgmHandle h);
-  int ndb_mgm_ndbinfo_getcolums(NdbMgmHandle h, int n, int l, char** c);
-  char* ndb_mgm_ndbinfo_nextcolumn(char* row, int *len);
-
-  /**
-   * returns zero on success.
-   * Must be called after ndb_mgm_ndbinfo and after ndb_mgm_ndbinfo_getcolumns
-   */
-  int ndb_mgm_ndbinfo_getrow(NdbMgmHandle h, char* row, int len);
-
   /**
    * Config iterator
    */
@@ -1326,6 +1363,45 @@ extern "C" {
 
 #endif
 
+  /**
+   *   Struct containing array of ndb_logevents
+   *   of the requested type, describing for example
+   *   memoryusage or baclupstatus for the whole cluster,
+   *   returned from ndb_mgm_dump_events()
+   */
+  struct ndb_mgm_events {
+    /** Number of entries in the logevents array */
+    int no_of_events;
+    /** Array of ndb_logevents  */
+    struct ndb_logevent events [
+#ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
+                    1
+#endif
+    ];
+  };
+
+  /**
+   * Get an array of ndb_logevent of a specified type, describing
+   * for example memoryusage or backupstatus for the whole cluster
+   *
+   * @note The caller must free the pointer returned by this function.
+   *
+   * @param   handle        Management handle.
+   * @param   type          Which ndb_logevent to request
+   * @param   no_of_nodes   Number of database nodes to request info from<br>
+   *                          0: All database nodes in cluster<br>
+   *                          n: Only the <var>n</var> node(s) specified in<br>
+   *                             the array node_list
+   * @param   node_list     List of node IDs of database nodes to request<br>
+   *                        info from
+   *
+   * @return                struct with array of ndb_logevent's
+   *                        (or <var>NULL</var> on error).
+   */
+  struct ndb_mgm_events*
+  ndb_mgm_dump_events(NdbMgmHandle handle, enum Ndb_logevent_type type,
+                      int no_of_nodes, const int * node_list);
+
 #ifdef __cplusplus
 }
 #endif
@@ -1333,3 +1409,4 @@ extern "C" {
 /** @} */
 
 #endif
+
