@@ -20,7 +20,7 @@
    @mainpage                            NDB API Programmers' Guide
 
    This guide assumes a basic familiarity with MySQL Cluster concepts found
-   on http://dev.mysql.com/doc/mysql/en/mysql-cluster.html.
+   in the MySQL Cluster documentation.
    Some of the fundamental ones are also described in section @ref secConcepts.
 
    The NDB API is a MySQL Cluster application interface 
@@ -997,6 +997,7 @@ class TransporterFacade;
 class PollGuard;
 class Ndb_local_table_info;
 template <class T> struct Ndb_free_list_t;
+class NdbLockHandle;
 
 typedef void (* NdbEventCallback)(NdbEventOperation*, Ndb*, void*);
 
@@ -1071,6 +1072,7 @@ class Ndb
   friend class Ndb_cluster_connection_impl;
   friend class Ndb_internal;
   friend class NdbScanFilterImpl;
+  friend class PollGuard;
 #endif
 
 public:
@@ -1624,6 +1626,18 @@ public:
    */
   const NdbError & getNdbError(int errorCode);
 
+  /**
+   * Get a string containing any extra error details in the supplied
+   * buffer
+   * Where there is extra detail available a ptr to the start of
+   * the supplied buffer will be returned.
+   * If the extra detail string is longer than the passed buffer 
+   * then it will be truncated to fit.
+   * Where there is no extra detail, NULL will be returned.
+   */
+  const char* getNdbErrorDetail(const NdbError& err, 
+                                char* buff, 
+                                Uint32 buffLen) const;
 
   /** @} *********************************************************************/
 
@@ -1650,23 +1664,6 @@ public:
                               ///< (may crash system)
   };
 
-  /**
-   * For testing purposes it is possible to tamper with the NDB Cluster
-   * (i.e. send a special signal to DBDIH, the NDB distribution handler).
-   * <b>This feature should only used for debugging purposes.</b>
-   * In a release versions of NDB Cluster,
-   * this call always return -1 and does nothing.
-   * 
-   * @param aAction Action to be taken according to TamperType above
-   *
-   * @param aNode  Which node the action will be taken
-   *              -1:   Master DIH.
-   *            0-16:   Nodnumber.
-   * @return -1 indicates error, other values have meaning dependent 
-   *          on type of tampering.
-   */
-  int NdbTamper(TamperType aAction, int aNode);  
-
   /**
    * Return a unique tuple id for a table.  The id sequence is
    * ascending but may contain gaps.  Methods which have no
@@ -1755,17 +1752,17 @@ public:
   };
 
   Free_list_usage * get_free_list_usage(Free_list_usage*);
+
+  /* Get minimum known DB node version */
+  Uint32 getMinDbNodeVersion() const;
 #endif
 
 private:
 /*****************************************************************************
  *     These are service routines used by the other classes in the NDBAPI.
  ****************************************************************************/
-  Uint32 get_cond_wait_index() { return cond_wait_index; }
-  void set_cond_wait_index(Uint32 index) { cond_wait_index = index; }
-  Uint32 cond_wait_index;
-  Ndb *cond_signal_ndb;
-  void cond_signal();
+  Uint32 _unused;
+  void *_unused2;
 
   Ndb(const Ndb&); // Not impl.
   Ndb&operator=(const Ndb&);
@@ -1798,6 +1795,8 @@ private:
 
   NdbBlob*              getNdbBlob();// Get a blob handle etc
 
+  NdbLockHandle*        getLockHandle(); // Get a lock handle.
+
   void			releaseSignal(NdbApiSignal* anApiSignal);
   void                  releaseSignals(Uint32, NdbApiSignal*, NdbApiSignal*);
   void                  releaseSignalsInList(NdbApiSignal** pList);
@@ -1810,6 +1809,7 @@ private:
   void		 	releaseOperation(NdbOperation* anOperation);	
   void		 	releaseScanOperation(NdbIndexScanOperation*);
   void                  releaseNdbBlob(NdbBlob* aBlob);
+  void                  releaseLockHandle(NdbLockHandle* lh);
 
   void                  check_send_timeout();
   void                  remove_sent_list(Uint32);
@@ -1818,7 +1818,8 @@ private:
 
   // Handle a received signal. Used by both
   // synchronous and asynchronous interface
-  void handleReceivedSignal(NdbApiSignal* anApiSignal, struct LinearSectionPtr ptr[3]);
+  void handleReceivedSignal(const NdbApiSignal* anApiSignal,
+			    const struct LinearSectionPtr ptr[3]);
   
   int			sendRecSignal(Uint16 aNodeId,
 				      Uint32 aWaitState,
@@ -1826,9 +1827,6 @@ private:
                                       Uint32 nodeSequence,
                                       Uint32 *ret_conn_seq= 0);
   
-  // Sets Restart GCI in Ndb object
-  void			RestartGCI(int aRestartGCI);
-
   // Get block number of this NDBAPI object
   int			getBlockNumber();
   
@@ -1969,6 +1967,9 @@ private:
 
   NdbApiSignal* theCommitAckSignal;
 
+  /* Cached minimum connected Db node version */
+  Uint32 theCachedMinDbNodeVersion;
+
 
 #ifdef POORMANSPURIFY
   int cfreeSignals;
@@ -1977,9 +1978,6 @@ private:
   int creleaseSignals;
 #endif
 
-  static void executeMessage(void*, NdbApiSignal *, 
-			     struct LinearSectionPtr ptr[3]);
-  static void statusMessage(void*, Uint32, bool, bool);
 #ifdef VM_TRACE
 #include <my_attribute.h>
   void printState(const char* fmt, ...)
