@@ -365,6 +365,10 @@ public:
      */
     CHARSET_INFO* getCharset() const;
 
+    /**
+     * Returns mysql's internal number for the column's character set.
+     */
+    int getCharsetNumber() const;
 
     /**
      * For blob, get "inline size" i.e. number of initial bytes
@@ -561,14 +565,32 @@ public:
 
     /** @} *******************************************************************/
 
+#ifndef DOXYGEN_SHOULD_SKIP_DEPRECATED
+    int setDefaultValue(const char*);
+#endif
+    /* setDefaultValue
+     * Set buf to NULL for no default value, or null default value for
+     * NULLABLE column, otherwise set buf to pointer to default value.
+     * The len parameter is the number of significant bytes of default
+     * value supplied, which is the type size for fixed size types.
+     * For variable length types, the leading 1 or 2 bytes pointed to 
+     * by buf also contain length information as normal for the type.
+     */
+    int setDefaultValue(const void* buf, unsigned int len);
+
+    /* getDefaultValue
+     * Get the default value data for this column.
+     * Optional int len* will be updated with the significant length 
+     * of the default value, or set to 0 for NULL or no default.
+     */
+    const void* getDefaultValue(unsigned int* len = 0) const;
+
 #ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
     const Table * getBlobTable() const;
 
     void setAutoIncrement(bool);
     bool getAutoIncrement() const;
     void setAutoIncrementInitialValue(Uint64 val);
-    int setDefaultValue(const char*);   
-    const char* getDefaultValue() const;
 
     static const Column * FRAGMENT;
     static const Column * FRAGMENT_FIXED_MEMORY;
@@ -583,6 +605,8 @@ public:
     static const Column * ROW_GCI;
     static const Column * ANY_VALUE;
     static const Column * COPY_ROWID;
+    static const Column * LOCK_REF;
+    static const Column * OP_ID;
     static const Column * OPTIMIZE;
     static const Column * FRAGMENT_EXTENT_SPACE;
     static const Column * FRAGMENT_FREE_EXTENT_SPACE;
@@ -740,6 +764,11 @@ public:
      */
     int getNoOfColumns() const;
     
+    /**
+     * Get number of auto_increment columns in the table
+     */
+    int getNoOfAutoIncrementColumns() const;
+    
     /**
      * Get number of primary keys in the table
      */
@@ -933,6 +962,17 @@ public:
     const Int32 *getRangeListData() const;
     Uint32 getRangeListDataLen() const;
 
+    /**
+     * Get list of nodes storing given fragment, primary
+     * is normally entry 0
+     * Returns : 0 for error, > 0 for fragment count
+     * If fragment count is > arraySize param, only arraySize
+     * entries are written.
+     */
+    Uint32 getFragmentNodes(Uint32 fragmentId, 
+                            Uint32* nodeIdArrayPtr,
+                            Uint32 arraySize) const;
+
     /**
      * Set table object type
      */
@@ -1041,6 +1081,12 @@ public:
      */
     Uint32 getPartitionId(Uint32 hashvalue) const ;
 
+    /*
+     * Return TRUE if any of the columns in the table have a 
+     * non NULL default value defined
+     */ 
+    bool hasDefaultValues() const;
+
   private:
 #ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
     friend class Ndb;
@@ -1082,7 +1128,7 @@ public:
 #ifndef DOXYGEN_SHOULD_SKIP_DEPRECATED
     /**
      * Get the number of columns in the index
-     * Depricated, use getNoOfColumns instead.
+     * Deprecated, use getNoOfColumns instead.
      */
     int getNoOfIndexColumns() const;
 #endif
@@ -1095,7 +1141,7 @@ public:
 #ifndef DOXYGEN_SHOULD_SKIP_DEPRECATED
     /**
      * Get a specific column name in the index
-     * Depricated, use getColumn instead.
+     * Deprecated, use getColumn instead.
      */
     const char * getIndexColumn(int no) const ;
 #endif
@@ -1195,7 +1241,7 @@ public:
      * Add a column name to the index definition
      * Note that the order of indexes will be in
      * the order they are added (only matters for ordered indexes).
-     * Depricated, use addColumnName instead.
+     * Deprecated, use addColumnName instead.
      */
     int addIndexColumn(const char * name);
 #endif
@@ -1212,7 +1258,7 @@ public:
      * Add several column names to the index definition
      * Note that the order of indexes will be in
      * the order they are added (only matters for ordered indexes).
-     * Depricated, use addColumnNames instead.
+     * Deprecated, use addColumnNames instead.
      */
     int addIndexColumns(int noOfNames, const char ** names);
 #endif
@@ -1400,7 +1446,8 @@ public:
     enum EventReport {
       ER_UPDATED = 0,
       ER_ALL = 1, // except not-updated blob inlines
-      ER_SUBSCRIBE = 2
+      ER_SUBSCRIBE = 2,
+      ER_DDL = 4
     };
 
     /**
@@ -2033,6 +2080,9 @@ public:
 #endif
     int listObjects(List & list,
 		    Object::Type type = Object::TypeUndefined) const;
+    int listObjects(List & list,
+                    Object::Type type,
+                    bool fullyQualified) const;
 
     /**
      * Get the latest error
@@ -2041,6 +2091,13 @@ public:
      */			     
     const struct NdbError & getNdbError() const;
 
+    /**
+     * Get warning flags.  The value is valid only if the operation did
+     * not return an error and can return warnings.  The flags are
+     * specific to the operation.
+     */
+    int getWarningFlags() const;
+
     /** @} *******************************************************************/
 
     /** 
@@ -2238,8 +2295,8 @@ public:
      * @param index Index to create
      * @return 0 if successful otherwise -1.
      */
-    int createIndex(const Index &index);
-    int createIndex(const Index &index, const Table &table);
+    int createIndex(const Index &index, bool offline = false);
+    int createIndex(const Index &index, const Table &table, bool offline = false);
 
     /**
      * Drop index with given name
@@ -2264,6 +2321,12 @@ public:
      * Force gcp and wait for gcp complete
      */
     int forceGCPWait();
+    int forceGCPWait(int type);
+
+    /**
+     * Get restart gci
+     */
+    int getRestartGCI(Uint32 * gci);
 #endif
 
     /** @} *******************************************************************/
@@ -2274,6 +2337,18 @@ public:
      * @{
      */
     
+    /*
+     * The four "create" operations can return warning flags defined
+     * below.  See getWarningFlags().
+     */
+    enum {
+      WarnUndobufferRoundUp = 0x1,  // rounded up to kernel page size
+      WarnUndofileRoundDown = 0x2,  // rounded down to kernel page size
+      WarnExtentRoundUp = 0x4,      // rounded up to kernel page size
+      WarnDatafileRoundDown = 0x8,  // rounded down to kernel page size
+      WarnDatafileRoundUp = 0x10    // rounded up to extent size
+    };
+
     int createLogfileGroup(const LogfileGroup &, ObjectId* = 0);
     int dropLogfileGroup(const LogfileGroup&);
     LogfileGroup getLogfileGroup(const char * name);
@@ -2438,6 +2513,7 @@ public:
     int dropIndexGlobal(const Index &index);
     int removeIndexGlobal(const Index &ndbidx, int invalidate) const;
     int removeTableGlobal(const Table &ndbtab, int invalidate) const;
+    void invalidateDbGlobal(const char * dbname);
 #endif
 
     /*
@@ -2474,8 +2550,32 @@ public:
       createRecord
     */
     void releaseRecord(NdbRecord *rec);
+  }; // class Dictionary
+
+  class NdbDataPrintFormat
+  {
+  public:
+    NdbDataPrintFormat();
+    virtual ~NdbDataPrintFormat();
+    const char *lines_terminated_by;
+    const char *fields_terminated_by;
+    const char *start_array_enclosure;
+    const char *end_array_enclosure;
+    const char *fields_enclosed_by;
+    const char *fields_optionally_enclosed_by;
+    const char *hex_prefix;
+    const char *null_string;
+    int hex_format;
   };
-};
+
+  static 
+  class NdbOut& printFormattedValue(class NdbOut& out, 
+                                    const NdbDataPrintFormat& format,
+                                    const NdbDictionary::Column* c,
+                                    const void* val);
+  
+
+}; // class NdbDictionary
 
 class NdbOut& operator <<(class NdbOut& out, const NdbDictionary::Column& col);
 
