@@ -36,6 +36,8 @@ class NdbBlob;
 class TcKeyReq;
 class NdbRecord;
 class NdbInterpretedCode;
+struct GenericSectionPtr;
+class NdbLockHandle;
 
 /**
  * @class NdbOperation
@@ -228,7 +230,7 @@ public:
    * dirtyRead is a deprecated name for committedRead
    *
    * @return 0 if successful otherwise -1.
-   * @depricated
+   * @deprecated
    */
   virtual int			dirtyRead();
 
@@ -898,7 +900,7 @@ public:
   /**
    * Get the type of access for this operation
    */
-  const Type getType() const;
+  Type getType() const;
 
   /** @} *********************************************************************/
 
@@ -913,6 +915,7 @@ public:
     DeleteRequest = 3,            ///< Delete Operation
     WriteRequest = 4,             ///< Write Operation
     ReadExclusive = 5,            ///< Read exclusive
+    UnlockRequest = 7,            ///< Unlock operation
     OpenScanRequest,              ///< Scan Operation
     OpenRangeScanRequest,         ///< Range scan operation
     NotDefined2,                  ///< Internal for debugging
@@ -1035,7 +1038,8 @@ public:
                  OO_PARTITION_ID = 0x08, 
                  OO_INTERPRETED  = 0x10,
                  OO_ANYVALUE     = 0x20,
-                 OO_CUSTOMDATA   = 0x40 };
+                 OO_CUSTOMDATA   = 0x40,
+                 OO_LOCKHANDLE   = 0x80 };
 
     /* An operation-specific abort option.
      * Only necessary if the default abortoption behaviour
@@ -1066,6 +1070,16 @@ public:
     void * customData;
   };
 
+  /* getLockHandle
+   * Returns a pointer to this operation's LockHandle.
+   * For NdbRecord, the lock handle must first be requested using
+   * the OO_LOCKHANDLE operation option.
+   * For non-NdbRecord operations, this call can be used alone.
+   * The returned LockHandle cannot be used until the operation
+   * has been executed.
+   */
+  const NdbLockHandle* getLockHandle() const;
+  const NdbLockHandle* getLockHandle();
 
 protected:
 /******************************************************************************
@@ -1208,8 +1222,8 @@ protected:
  * was sent, then the connection object is told about this situation.
  *****************************************************************************/
 
+  int    doSendKeyReq(int processorId, GenericSectionPtr* secs, Uint32 numSecs);
   int    doSend(int ProcessorId, Uint32 lastFlag);
-  int    doSendNdbRecord(int aNodeId);
   virtual int	 prepareSend(Uint32  TC_ConnectPtr,
                              Uint64  TransactionId,
 			     AbortOption);
@@ -1239,7 +1253,8 @@ protected:
                                      Uint32 &wordsWritten);
 
   // Method which prepares signals at operation definition time.
-  int    buildSignalsNdbRecord(Uint32 aTC_ConnectPtr, Uint64 aTransId);
+  int    buildSignalsNdbRecord(Uint32 aTC_ConnectPtr, Uint64 aTransId,
+                               const Uint32 * read_mask);
 
   // Method which does final preparations at execute time.
   int    prepareSendNdbRecord(AbortOption ao);
@@ -1257,11 +1272,11 @@ protected:
   int    insertATTRINFOData_NdbRecord(const char *value,
                                       Uint32 size);
 
-  int	 receiveTCKEYREF(NdbApiSignal*); 
+  int	 receiveTCKEYREF(const NdbApiSignal*);
 
   int	 checkMagicNumber(bool b = true); // Verify correct object
 
-  int    checkState_TransId(NdbApiSignal* aSignal);
+  int    checkState_TransId(const NdbApiSignal* aSignal);
 
 /******************************************************************************
  *	These are support methods only used locally in this class.
@@ -1285,8 +1300,9 @@ protected:
   NdbBlob *linkInBlobHandle(NdbTransaction *aCon,
                             const NdbColumnImpl *column,
                             NdbBlob * & lastPtr);
-  int getBlobHandlesNdbRecord(NdbTransaction* aCon);
-  int getBlobHandlesNdbRecordDelete(NdbTransaction* aCon, bool checkReadSet);
+  int getBlobHandlesNdbRecord(NdbTransaction* aCon, const Uint32 * mask);
+  int getBlobHandlesNdbRecordDelete(NdbTransaction* aCon, 
+                                    bool checkReadSet, const Uint32 * mask);
 
   // Handle ATTRINFO signals   
   int insertATTRINFO(Uint32 aData);
@@ -1315,6 +1331,10 @@ protected:
   // get table or index key from prepared signals
   int getKeyFromTCREQ(Uint32* data, Uint32 & size);
 
+  int getLockHandleImpl();
+  int prepareGetLockHandle();
+  int prepareGetLockHandleNdbRecord();
+
   virtual void setReadLockMode(LockMode lockMode);
 
 /******************************************************************************
@@ -1449,7 +1469,7 @@ protected:
     Do not use clas Bitmask/BitmaskPOD here, to avoid having to
     #include <Bitmask.hpp> in application code.
   */
-  Uint32 m_read_mask[(NDB_MAX_ATTRIBUTES_IN_TABLE+31)>>5];
+  Uint32 m_unused_read_mask[(128+31)>>5];
   /* Interpreted program for NdbRecord operations. */
   const NdbInterpretedCode *m_interpreted_code;
 
@@ -1485,6 +1505,12 @@ protected:
 
   Uint32 repack_read(Uint32 len);
 
+  NdbLockHandle* theLockHandle;
+
+  bool m_blob_lock_upgraded; /* Did Blob code upgrade LM_CommittedRead
+                              * to LM_Read?
+                              */
+
 private:
   NdbOperation(const NdbOperation&); // Not impl.
   NdbOperation&operator=(const NdbOperation&);
@@ -1582,7 +1608,7 @@ Return Value    Return the Type.
 Remark:         Gets type of access.
 ******************************************************************************/
 inline
-const NdbOperation::Type
+NdbOperation::Type
 NdbOperation::getType() const
 {
   return m_type;
