@@ -21,12 +21,15 @@
 
 #include <ndb_global.h>
 #include <ndb_net.h>
-#include <my_socket.h>
+#include <ndb_socket.h>
+#include <portlib/ndb_socket_poller.h>
 
-#define NDB_SOCKET_TYPE my_socket
-#define _NDB_CLOSE_SOCKET(x) my_socket_close(x)
+#define NDB_SOCKET_TYPE ndb_socket_t
 
-#define NDB_SOCKLEN_T SOCKET_SIZE_TYPE
+static inline
+void NDB_CLOSE_SOCKET(ndb_socket_t s) {
+  my_socket_close(s);
+}
 
 #ifdef	__cplusplus
 extern "C" {
@@ -45,21 +48,8 @@ extern "C" {
  */
 int Ndb_getInAddr(struct in_addr * dst, const char *address);
 
-#ifdef DBUG_OFF
-#define NDB_CLOSE_SOCKET(fd) _NDB_CLOSE_SOCKET(fd)
-#else
-int NDB_CLOSE_SOCKET(my_socket fd);
-#endif
-
 int Ndb_check_socket_hup(NDB_SOCKET_TYPE sock);
 
-#ifdef NDB_WIN
-#define NONBLOCKERR(E) (E!=SOCKET_EAGAIN && E!=SOCKET_EWOULDBLOCK)
-#else
-#define NONBLOCKERR(E) (E!=EINPROGRESS)
-#endif
-
-
 #ifdef	__cplusplus
 }
 #endif
