@@ -18,7 +18,7 @@
 
 
 /*
-  ndbapi_blob.cpp:
+  ndbapi_blob_ndbrecord
 
   Illustrates the manipulation of BLOB (actually TEXT in this example).
   This example uses the NdbRecord style way of accessing tuples.
@@ -133,7 +133,7 @@ static void setup_records(Ndb *myNdb)
   NdbDictionary::RecordSpecification spec[2];
 
   NdbDictionary::Dictionary *myDict= myNdb->getDictionary();
-  const NdbDictionary::Table *myTable= myDict->getTable("my_text");
+  const NdbDictionary::Table *myTable= myDict->getTable("api_blob_ndbrecord");
   if (myTable == NULL)
     APIERROR(myDict->getNdbError());
   const NdbDictionary::Column *col1= myTable->getColumn("my_id");
@@ -168,7 +168,7 @@ static void setup_records(Ndb *myNdb)
 */
 void drop_table(MYSQL &mysql)
 {
-  if (mysql_query(&mysql, "DROP TABLE my_text"))
+  if (mysql_query(&mysql, "DROP TABLE api_blob_ndbrecord"))
     MYSQLERROR(mysql);
 }
 
@@ -180,7 +180,7 @@ int try_create_table(MYSQL &mysql)
 {
   return mysql_query(&mysql,
                      "CREATE TABLE"
-                     "  my_text"
+                     "  api_blob_ndbrecord"
                      "    (my_id INT UNSIGNED NOT NULL,"
                      "     my_text TEXT NOT NULL,"
                      "     PRIMARY KEY USING HASH (my_id))"
@@ -193,7 +193,7 @@ void create_table(MYSQL &mysql)
   {
     if (mysql_errno(&mysql) != ER_TABLE_EXISTS_ERROR)
       MYSQLERROR(mysql);
-    std::cout << "MySQL Cluster already has example table: my_text. "
+    std::cout << "MySQL Cluster already has example table: api_blob_ndbrecord. "
               << "Dropping it..." << std::endl;
     /******************
      * Recreate table *
@@ -540,8 +540,8 @@ int main(int argc, char**argv)
                              0, mysqld_sock, 0) )
       MYSQLERROR(mysql);
 
-    mysql_query(&mysql, "CREATE DATABASE TEST_DB");
-    if (mysql_query(&mysql, "USE TEST_DB") != 0)
+    mysql_query(&mysql, "CREATE DATABASE ndb_examples");
+    if (mysql_query(&mysql, "USE ndb_examples") != 0)
       MYSQLERROR(mysql);
 
     create_table(mysql);
@@ -562,7 +562,7 @@ int main(int argc, char**argv)
     exit(-1);
   }
 
-  Ndb myNdb(&cluster_connection,"TEST_DB");
+  Ndb myNdb(&cluster_connection,"ndb_examples");
   if (myNdb.init(1024) == -1) {      // Set max 1024 parallel transactions
     APIERROR(myNdb.getNdbError());
     exit(-1);
@@ -588,8 +588,5 @@ int main(int argc, char**argv)
   if(delete_key(&myNdb) > 0)
     std::cout << "delete_key: Success!" << std::endl;
 
-  /* Drop table. */
-  drop_table(mysql);
-
   return 0;
 }
