@@ -54,6 +54,10 @@ const GsnName SignalNames [] = {
   ,{ GSN_TCSEIZECONF,            "TCSEIZECONF" }
   ,{ GSN_TCSEIZEREF,             "TCSEIZEREF" }
   ,{ GSN_TCSEIZEREQ,             "TCSEIZEREQ" }
+  ,{ GSN_DBINFO_SCANREQ,         "DBINFO_SCANREQ" }
+  ,{ GSN_DBINFO_SCANCONF,        "DBINFO_SCANCONF" }
+  ,{ GSN_DBINFO_SCANREF,         "DBINFO_SCANREF" }
+  ,{ GSN_DBINFO_TRANSID_AI,      "DBINFO_TRANSID_AI" }
   ,{ GSN_TRANSID_AI,             "TRANSID_AI" }
   ,{ GSN_TRANSID_AI_R,           "TRANSID_AI_R" }
   ,{ GSN_ABORT,                  "ABORT" }
@@ -89,6 +93,7 @@ const GsnName SignalNames [] = {
   ,{ GSN_ADD_FRAGCONF,           "ADD_FRAGCONF" }
   ,{ GSN_ADD_FRAGREF,            "ADD_FRAGREF" }
   ,{ GSN_ADD_FRAGREQ,            "ADD_FRAGREQ" }
+  ,{ GSN_API_START_REP,          "API_START_REP" }
   ,{ GSN_API_FAILCONF,           "API_FAILCONF" }
   ,{ GSN_API_FAILREQ,            "API_FAILREQ" }
   ,{ GSN_CHECK_LCP_STOP,         "CHECK_LCP_STOP" }
@@ -745,5 +750,16 @@ const GsnName SignalNames [] = {
   ,{ GSN_ALLOC_MEM_REQ, "ALLOC_MEM_REQ" }
   ,{ GSN_ALLOC_MEM_REF, "ALLOC_MEM_REF" }
   ,{ GSN_ALLOC_MEM_CONF, "ALLOC_MEM_CONF" }
+
+  ,{ GSN_DIH_GET_TABINFO_REQ, "DIH_GET_TABINFO_REQ" }
+  ,{ GSN_DIH_GET_TABINFO_REF, "DIH_GET_TABINFO_REF" }
+  ,{ GSN_DIH_GET_TABINFO_CONF, "DIH_GET_TABINFO_CONF" }
+
+  ,{ GSN_SYNC_REQ, "SYNC_REQ" }
+  ,{ GSN_SYNC_REF, "SYNC_REF" }
+  ,{ GSN_SYNC_CONF, "SYNC_CONF" }
+
+  ,{ GSN_SYNC_PATH_REQ, "SYNC_PATH_REQ" }
+  ,{ GSN_SYNC_PATH_CONF, "SYNC_PATH_CONF" }
 };
 const unsigned short NO_OF_SIGNAL_NAMES = sizeof(SignalNames)/sizeof(GsnName);
