@@ -17,7 +17,6 @@
 */
 
 #include <ndb_global.h>
-#include <my_pthread.h>
 
 #include <TransporterRegistry.hpp>
 #include "TransporterInternalDefinitions.hpp"
@@ -111,7 +110,6 @@ TransporterRegistry::TransporterRegistry(TransporterCallback *callback,
  {
    /* Failure to allocate data or get epoll socket, abort */
    perror("Failed to alloc epoll-array or calling epoll_create... falling back to select!");
-   ndbout_c("Falling back to select");
    if (m_epoll_fd != -1)
    {
      close(m_epoll_fd);
@@ -294,6 +292,10 @@ TransporterRegistry::init(NodeId nodeId) {
 
   DEBUG("TransporterRegistry started node: " << localNodeId);
 
+  if (!m_socket_poller.set_max_count(maxTransporters +
+				     1 /* wakeup socket */))
+    DBUG_RETURN(false);
+
   DBUG_RETURN(true);
 }
 
@@ -899,18 +901,30 @@ TransporterRegistry::wakeup()
 }
 
 Uint32
-TransporterRegistry::pollReceive(Uint32 timeOutMillis){
+TransporterRegistry::pollReceive(Uint32 timeOutMillis,
+                                 NodeBitmask& mask)
+{
   Uint32 retVal = 0;
 
-  if((nSCITransporters) > 0)
+  /**
+   * If any transporters have left-over data that was not fully executed in
+   * last loop, don't wait and return 'data available' even if nothing new
+   */
+  if (!mask.isclear())
+  {
+    timeOutMillis = 0;
+    retVal = 1;
+  }
+
+  if (nSCITransporters > 0)
   {
     timeOutMillis=0;
   }
 
 #ifdef NDB_SHM_TRANSPORTER
-  if(nSHMTransporters > 0)
+  if (nSHMTransporters > 0)
   {
-    Uint32 res = poll_SHM(0);
+    Uint32 res = poll_SHM(0, mask);
     if(res)
     {
       retVal |= res;
@@ -923,17 +937,7 @@ TransporterRegistry::pollReceive(Uint32 timeOutMillis){
 #if defined(HAVE_EPOLL_CREATE)
   if (likely(m_epoll_fd != -1))
   {
-    Uint32 num_trps = nTCPTransporters;
-    /**
-     * If any transporters have left-over data that was not fully executed in
-     * last loop, don't wait and return 'data available' even if nothing new
-     * from epoll.
-     */
-    if (!m_has_data_transporters.isclear())
-    {
-      timeOutMillis = 0;
-      retVal = 1;
-    }
+    Uint32 num_trps = nTCPTransporters + (m_has_extra_wakeup_socket ? 1 : 0);
     
     if (num_trps)
     {
@@ -941,26 +945,39 @@ TransporterRegistry::pollReceive(Uint32 timeOutMillis){
                                       num_trps, timeOutMillis);
       retVal |= tcpReadSelectReply;
     }
+
+    int num_socket_events = tcpReadSelectReply;
+    if (num_socket_events > 0)
+    {
+      for (int i = 0; i < num_socket_events; i++)
+      {
+        mask.set(m_epoll_events[i].data.u32);
+      }
+    }
+    else if (num_socket_events < 0)
+    {
+      assert(errno == EINTR);
+    }
   }
   else
 #endif
   {
-    if(nTCPTransporters > 0 || m_has_extra_wakeup_socket || retVal == 0)
+    if (nTCPTransporters > 0 || m_has_extra_wakeup_socket)
     {
-      retVal |= poll_TCP(timeOutMillis);
+      retVal |= poll_TCP(timeOutMillis, mask);
     }
     else
       tcpReadSelectReply = 0;
   }
 #endif
 #ifdef NDB_SCI_TRANSPORTER
-  if(nSCITransporters > 0)
-    retVal |= poll_SCI(timeOutMillis);
+  if (nSCITransporters > 0)
+    retVal |= poll_SCI(timeOutMillis, mask);
 #endif
 #ifdef NDB_SHM_TRANSPORTER
-  if(nSHMTransporters > 0 && retVal == 0)
+  if (nSHMTransporters > 0)
   {
-    int res = poll_SHM(0);
+    int res = poll_SHM(0, mask);
     retVal |= res;
   }
 #endif
@@ -970,17 +987,23 @@ TransporterRegistry::pollReceive(Uint32 timeOutMillis){
 
 #ifdef NDB_SCI_TRANSPORTER
 Uint32
-TransporterRegistry::poll_SCI(Uint32 timeOutMillis)
+TransporterRegistry::poll_SCI(Uint32 timeOutMillis, NodeBitmask& mask)
 {
-  for (int i=0; i<nSCITransporters; i++) {
+  Uint32 retVal = 0;
+  for (int i = 0; i < nSCITransporters; i++)
+  {
     SCI_Transporter * t = theSCITransporters[i];
-    Uint32 node_id= t->getRemoteNodeId();
-    if (t->isConnected() && is_connected(node_id)) {
-      if(t->hasDataToRead())
-	return 1;
+    Uint32 node_id = t->getRemoteNodeId();
+    if (t->isConnected() && is_connected(node_id))
+    {
+      if (t->hasDataToRead())
+      {
+        mask.set(node_id);
+	retVal = 1;
+      }
     }
   }
-  return 0;
+  return retVal;
 }
 #endif
 
@@ -988,21 +1011,27 @@ TransporterRegistry::poll_SCI(Uint32 timeOutMillis)
 #ifdef NDB_SHM_TRANSPORTER
 static int g_shm_counter = 0;
 Uint32
-TransporterRegistry::poll_SHM(Uint32 timeOutMillis)
+TransporterRegistry::poll_SHM(Uint32 timeOutMillis, NodeBitmask& mask)
 {  
-  for(int j=0; j < 100; j++)
+  Uint32 retVal = 0;
+  for (int j = 0; j < 100; j++)
   {
-    for (int i=0; i<nSHMTransporters; i++) {
+    for (int i = 0; i<nSHMTransporters; i++)
+    {
       SHM_Transporter * t = theSHMTransporters[i];
-      Uint32 node_id= t->getRemoteNodeId();
-      if (t->isConnected() && is_connected(node_id)) {
-	if(t->hasDataToRead()) {
-	  return 1;
+      Uint32 node_id = t->getRemoteNodeId();
+      if (t->isConnected() && is_connected(node_id))
+      {
+	if (t->hasDataToRead())
+        {
+          j = 100;
+          mask.set(node_id);
+          retVal = 1;
 	}
       }
     }
   }
-  return 0;
+  return retVal;
 }
 #endif
 
@@ -1016,69 +1045,58 @@ TransporterRegistry::poll_SHM(Uint32 timeOutMillis)
  * protected by transporter locks on upper layer).
  */
 Uint32 
-TransporterRegistry::poll_TCP(Uint32 timeOutMillis)
+TransporterRegistry::poll_TCP(Uint32 timeOutMillis, NodeBitmask& mask)
 {
-  bool hasdata = false;
-  if (false && nTCPTransporters == 0)
-  {
-    tcpReadSelectReply = 0;
-    return 0;
-  }
+  m_socket_poller.clear();
 
-  int maxSocketValue = 0;
-
-  // Needed for TCP/IP connections
-  // The read- and writeset are used by select
-
-  FD_ZERO(&tcpReadset);
-
-  // Prepare for sending and receiving
-  for (int i = 0; i < nTCPTransporters; i++) {
-    TCP_Transporter * t = theTCPTransporters[i];
-    Uint32 node_id= t->getRemoteNodeId();
-    
-    // If the transporter is connected
-    if (is_connected(node_id) && t->isConnected()) {
-      
-      const NDB_SOCKET_TYPE socket = t->getSocket();
-      if (!my_socket_valid(socket))
-        continue;
-
-      maxSocketValue = my_socket_nfds(socket, maxSocketValue);
-
-      // Put the connected transporters in the socket read-set 
-      my_FD_SET(socket, &tcpReadset);
-    }
-    hasdata |= t->hasReceiveData();
-  }
-  
   if (m_has_extra_wakeup_socket)
   {
     const NDB_SOCKET_TYPE socket = m_extra_wakeup_sockets[0];
-    maxSocketValue = my_socket_nfds(socket, maxSocketValue);
 
-    // Put the wakup-socket in the read-set
-    my_FD_SET(socket, &tcpReadset);
+    // Poll the wakup-socket for read
+    m_socket_poller.add(socket, true, false, false);
   }
 
-  timeOutMillis = hasdata ? 0 : timeOutMillis;
-  
-  struct timeval timeout;
-  timeout.tv_sec  = timeOutMillis / 1000;
-  timeout.tv_usec = (timeOutMillis % 1000) * 1000;
-
-  tcpReadSelectReply = select(maxSocketValue+1, &tcpReadset, 0, 0, &timeout);
-  if(false && tcpReadSelectReply == -1 && errno == EINTR)
-    g_eventLogger->info("woke-up by signal");
-
-#ifdef NDB_WIN32
-  if(tcpReadSelectReply == SOCKET_ERROR)
+  Uint16 idx[MAX_NODES];
+  for (int i = 0; i < nTCPTransporters; i++)
   {
-    NdbSleep_MilliSleep(timeOutMillis);
+    TCP_Transporter * t = theTCPTransporters[i];
+    const NDB_SOCKET_TYPE socket = t->getSocket();
+    Uint32 node_id = t->getRemoteNodeId();
+
+    if (is_connected(node_id) && t->isConnected() && my_socket_valid(socket))
+    {
+      idx[i] = m_socket_poller.add(socket, true, false, false);
+    }
+    else
+    {
+      idx[i] = MAX_NODES + 1;
+    }
   }
-#endif
-  
-  return tcpReadSelectReply || hasdata;
+
+  tcpReadSelectReply = m_socket_poller.poll_unsafe(timeOutMillis);
+
+  if (tcpReadSelectReply > 0)
+  {
+    if (m_extra_wakeup_sockets)
+    {
+      if (m_socket_poller.has_read(0))
+        mask.set((Uint32)0);
+    }
+
+    for (int i = 0; i < nTCPTransporters; i++)
+    {
+      TCP_Transporter * t = theTCPTransporters[i];
+      if (idx[i] != MAX_NODES + 1)
+      {
+        Uint32 node_id = t->getRemoteNodeId();
+        if (m_socket_poller.has_read(idx[i]))
+          mask.set(node_id);
+      }
+    }
+  }
+
+  return tcpReadSelectReply;
 }
 #endif
 
@@ -1088,7 +1106,7 @@ TransporterRegistry::change_epoll(TCP_Transporter *t, bool add)
 {
   struct epoll_event event_poll;
   bzero(&event_poll, sizeof(event_poll));
-  my_socket sock_fd = t->getSocket();
+  NDB_SOCKET_TYPE sock_fd = t->getSocket();
   int node_id = t->getRemoteNodeId();
   int op = add ? EPOLL_CTL_ADD : EPOLL_CTL_DEL;
   int ret_val, error;
@@ -1136,104 +1154,46 @@ ok:
   return FALSE;
 }
 
+#endif
+
 /**
  * In multi-threaded cases, this must be protected by a global receive lock.
  */
-void
-TransporterRegistry::get_tcp_data(TCP_Transporter *t)
-{
-  const NodeId node_id = t->getRemoteNodeId();
-  bool hasdata = false;
-  if (is_connected(node_id) && t->isConnected())
-  {
-    callbackObj->checkJobBuffer();
-    t->doReceive();
-    
-    Uint32 *ptr;
-    Uint32 sz = t->getReceiveData(&ptr);
-    callbackObj->transporter_recv_from(node_id);
-    Uint32 szUsed = unpack(ptr, sz, node_id, ioStates[node_id]);
-    t->updateReceiveDataPtr(szUsed);
-    hasdata = t->hasReceiveData();
-  }
-  m_has_data_transporters.set(node_id, hasdata);
-}
-
-#endif
-
 void
 TransporterRegistry::performReceive()
 {
   bool hasReceived = false;
-#ifdef NDB_TCP_TRANSPORTER
-#if defined(HAVE_EPOLL_CREATE)
-  if (likely(m_epoll_fd != -1))
-  {
-    int num_socket_events = tcpReadSelectReply;
-    int i;
-    
-    if (num_socket_events > 0)
-    {
-      for (i = 0; i < num_socket_events; i++)
-      {
-        m_has_data_transporters.set(m_epoll_events[i].data.u32);
-      }
-    }
-    else if (num_socket_events < 0)
-    {
-      assert(errno == EINTR);
-    }
 
-    if (m_has_data_transporters.get(0))
-    {
-      m_has_data_transporters.clear(Uint32(0));
-      consume_extra_sockets();
-    }
-    
-    Uint32 id = 0;
-    while ((id = m_has_data_transporters.find(id + 1)) != BitmaskImpl::NotFound)
-    {
-      get_tcp_data((TCP_Transporter*)theTransporters[id]);
-    }
+  if (m_has_data_transporters.get(0))
+  {
+    m_has_data_transporters.clear(Uint32(0));
+    consume_extra_sockets();
   }
-  else
-#endif
-  {
-    if (m_has_extra_wakeup_socket)
-    {
-      if (my_FD_ISSET(m_extra_wakeup_sockets[0], &tcpReadset))
-      {
-        consume_extra_sockets();
-      }
-    }
 
-    for (int i=0; i<nTCPTransporters; i++) 
+#ifdef NDB_TCP_TRANSPORTER
+  Uint32 id = 0;
+  while ((id = m_has_data_transporters.find(id + 1)) != BitmaskImpl::NotFound)
+  {
+    bool hasdata = false;
+    TCP_Transporter * t = (TCP_Transporter*)theTransporters[id];
+    if (is_connected(id))
     {
-      TCP_Transporter *t = theTCPTransporters[i];
-      const NodeId nodeId = t->getRemoteNodeId();
-      const NDB_SOCKET_TYPE socket    = t->getSocket();
-      if(is_connected(nodeId)){
-        if(t->isConnected())
-        {
-          if (my_FD_ISSET(socket, &tcpReadset))
-          {
-            t->doReceive();
-          }
-          
-          if (t->hasReceiveData())
-          {
-            if (hasReceived)
-              callbackObj->checkJobBuffer();
-            hasReceived = true;
-            Uint32 * ptr;
-            Uint32 sz = t->getReceiveData(&ptr);
-            callbackObj->transporter_recv_from(nodeId);
-            Uint32 szUsed = unpack(ptr, sz, nodeId, ioStates[nodeId]);
-            t->updateReceiveDataPtr(szUsed);
-          }
-        } 
+      if (t->isConnected())
+      {
+        t->doReceive();
+        if (hasReceived)
+          callbackObj->checkJobBuffer();
+        hasReceived = true;
+        Uint32 * ptr;
+        Uint32 sz = t->getReceiveData(&ptr);
+        callbackObj->transporter_recv_from(id);
+        Uint32 szUsed = unpack(ptr, sz, id, ioStates[id]);
+        t->updateReceiveDataPtr(szUsed);
+        hasdata = t->hasReceiveData();
       }
     }
+    // If transporter still have data, make sure that it's remember to next time
+    m_has_data_transporters.set(id, hasdata);
   }
 #endif
   
@@ -1416,7 +1376,7 @@ TransporterRegistry::setIOState(NodeId nodeId, IOState state) {
   ioStates[nodeId] = state;
 }
 
-static void * 
+extern "C" void *
 run_start_clients_C(void * me)
 {
   ((TransporterRegistry*) me)->start_clients_thread();
@@ -1520,9 +1480,7 @@ TransporterRegistry::report_disconnect(NodeId node_id, int errnum)
   DBUG_ENTER("TransporterRegistry::report_disconnect");
   DBUG_PRINT("info",("performStates[%d]=DISCONNECTED",node_id));
   performStates[node_id] = DISCONNECTED;
-#ifdef HAVE_EPOLL_CREATE
   m_has_data_transporters.clear(node_id);
-#endif
   callbackObj->reportDisconnect(node_id, errnum);
   DBUG_VOID_RETURN;
 }
@@ -1844,7 +1802,7 @@ TransporterRegistry::start_service(SocketServer& socket_server)
 }
 
 #ifdef NDB_SHM_TRANSPORTER
-static
+extern "C"
 RETSIGTYPE 
 shm_sig_handler(int signo)
 {
@@ -1955,7 +1913,7 @@ bool TransporterRegistry::connect_client(NdbMgmHandle *h)
 NDB_SOCKET_TYPE TransporterRegistry::connect_ndb_mgmd(NdbMgmHandle *h)
 {
   struct ndb_mgm_reply mgm_reply;
-  my_socket sockfd;
+  NDB_SOCKET_TYPE sockfd;
   my_socket_invalidate(&sockfd);
 
   DBUG_ENTER("TransporterRegistry::connect_ndb_mgmd(NdbMgmHandle)");
@@ -2013,7 +1971,7 @@ NDB_SOCKET_TYPE TransporterRegistry::connect_ndb_mgmd(NdbMgmHandle *h)
 NDB_SOCKET_TYPE TransporterRegistry::connect_ndb_mgmd(SocketClient *sc)
 {
   NdbMgmHandle h= ndb_mgm_create_handle();
-  my_socket s;
+  NDB_SOCKET_TYPE s;
   my_socket_invalidate(&s);
 
   DBUG_ENTER("TransporterRegistry::connect_ndb_mgmd(SocketClient)");
@@ -2055,12 +2013,11 @@ TransporterRegistry::getWritePtr(TransporterSendBufferHandle *handle,
                                           t->get_max_send_buffer());
 
   if (insertPtr == 0) {
-    struct timeval timeout = {0, 10000};
     //-------------------------------------------------
     // Buffer was completely full. We have severe problems.
     // We will attempt to wait for a small time
     //-------------------------------------------------
-    if(t->send_is_possible(&timeout)) {
+    if(t->send_is_possible(10)) {
       //-------------------------------------------------
       // Send is possible after the small timeout.
       //-------------------------------------------------
@@ -2099,8 +2056,7 @@ TransporterRegistry::updateWritePtr(TransporterSendBufferHandle *handle,
     // we will not worry since we will soon be back for
     // a renewed trial.
     //-------------------------------------------------
-    struct timeval no_timeout = {0,0};
-    if(t->send_is_possible(&no_timeout)) {
+    if(t->send_is_possible(0)) {
       //-------------------------------------------------
       // Send was possible, attempt at a send.
       //-------------------------------------------------
