@@ -16,11 +16,7 @@
  *
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-
-#include <NdbMem.h>
+#include <ndb_global.h>
 
 /**
  * This is a casual hack to do static memory allocation
@@ -32,7 +28,9 @@
 #include "../../../../zlib/inflate.h"
 #include "../../../../zlib/deflate.h"
 
-#include "azlib.h"
+#include <util/azlib.h>
+
+#include <my_sys.h>
 
 #ifdef HAVE_VALGRIND
 #include <valgrind/memcheck.h>
@@ -57,7 +55,7 @@ static int const az_magic[3] = {0xfe, 0x03, 0x01}; /* az magic header */
 int az_open(azio_stream *s, const char *path, int Flags, File  fd);
 int do_flush(azio_stream *file, int flush);
 int    get_byte(azio_stream *s);
-char* get_block(azio_stream *s, int blksz);
+unsigned char* get_block(azio_stream *s, int blksz);
 int   check_header(azio_stream *s);
 int write_header(azio_stream *s);
 int    destroy(azio_stream *s);
@@ -109,6 +107,34 @@ void az_free(voidpf opaque, voidpf address)
     VALGRIND_MAKE_MEM_NOACCESS(r->mem,r->size);
 }
 
+#ifdef _WIN32
+/* Windows doesn't define ENOTSUP, define it same as Solaris */
+#define ENOTSUP 48
+#endif
+
+#ifndef HAVE_POSIX_MEMALIGN
+static inline int posix_memalign(void **memptr, size_t alignment, size_t size)
+{
+#ifdef HAVE_MEMALIGN
+  /* Solaris 10 has memalign but not posix_memalign */
+  *memptr = memalign(alignment,size);
+  if(!*memptr)
+    return ENOMEM;
+  return 0;
+#else
+  /* But Darwin 7.9.0 doesn't have posix_memalign OR memalign */
+  (void)memptr;
+  (void)alignment;
+  (void)size;
+  return ENOTSUP; /* POSIX says we can return EINVAL or ENOMEM...
+                     but we cheat here and do ENOTSUP so that code
+                     elsewhere can work out if it can fall back to
+                     plain malloc() or not as we cannot reasonably do aligned
+                     memory allocation and free without leaking memory
+                  */
+#endif
+}
+#endif
 
 /* ===========================================================================
   Opens a gzip (.gz) file for reading or writing. The mode parameter
@@ -361,9 +387,9 @@ int get_byte(s)
  * *MUST* be < buffer size
  * *MUST* be aligned to IO size (i.e. not be only partially in buffer)
  */
-char* get_block(azio_stream *s,int blksz)
+unsigned char* get_block(azio_stream *s,int blksz)
 {
-  char *r= s->stream.next_in;
+  unsigned char *r= s->stream.next_in;
   if (s->stream.avail_in == 0)
     if(read_buffer(s))
       return NULL;
@@ -436,7 +462,7 @@ int check_header(azio_stream *s)
   else if (    s->stream.next_in[0] == az_magic[0]
             && s->stream.next_in[1] == az_magic[1])
   {
-    char *header_block;
+    unsigned char *header_block;
     if(s->stream.avail_in < AZHEADER_SIZE + AZMETA_BUFFER_SIZE)
     {
       s->z_err = Z_DATA_ERROR;
