@@ -16,8 +16,19 @@
 #ifndef ANGEL_HPP
 #define ANGEL_HPP
 
-int angel_run(const char* connect_str,
-              const char* bind_address,
-              bool initialstart,
-              bool daemon);
+#include <util/BaseString.hpp>
+
+void
+angel_run(const char* progname,
+          const Vector<BaseString>& original_args,
+          const char* connect_str,
+          int force_nodeid,
+          const char* bind_address,
+          bool initial,
+          bool no_start,
+          bool daemon);
+
+void
+angel_stop(void);
+
 #endif
