@@ -83,6 +83,18 @@ LocalProxy::LocalProxy(BlockNumber blockNumber, Block_context& ctx) :
   addRecSignal(GSN_DROP_TRIG_IMPL_REQ, &LocalProxy::execDROP_TRIG_IMPL_REQ);
   addRecSignal(GSN_DROP_TRIG_IMPL_CONF, &LocalProxy::execDROP_TRIG_IMPL_CONF);
   addRecSignal(GSN_DROP_TRIG_IMPL_REF, &LocalProxy::execDROP_TRIG_IMPL_REF);
+
+  // GSN_DBINFO_SCANREQ
+  addRecSignal(GSN_DBINFO_SCANREQ, &LocalProxy::execDBINFO_SCANREQ);
+  addRecSignal(GSN_DBINFO_SCANCONF, &LocalProxy::execDBINFO_SCANCONF);
+
+  // GSN_SYNC_REQ
+  addRecSignal(GSN_SYNC_REQ, &LocalProxy::execSYNC_REQ, true);
+  addRecSignal(GSN_SYNC_REF, &LocalProxy::execSYNC_REF);
+  addRecSignal(GSN_SYNC_CONF, &LocalProxy::execSYNC_CONF);
+
+  // GSN_SYNC_PATH_REQ
+  addRecSignal(GSN_SYNC_PATH_REQ, &LocalProxy::execSYNC_PATH_REQ, true);
 }
 
 LocalProxy::~LocalProxy()
@@ -97,7 +109,10 @@ LocalProxy::sendREQ(Signal* signal, SsSequential& ss)
 {
   ss.m_worker = 0;
   ndbrequire(ss.m_sendREQ != 0);
-  (this->*ss.m_sendREQ)(signal, ss.m_ssId);
+  SectionHandle handle(this);
+  restoreHandle(handle, ss);
+  (this->*ss.m_sendREQ)(signal, ss.m_ssId, &handle);
+  saveSections(ss, handle);
 }
 
 void
@@ -110,7 +125,8 @@ LocalProxy::recvCONF(Signal* signal, SsSequential& ss)
   if (ss.m_worker < c_workers) {
     jam();
     ndbrequire(ss.m_sendREQ != 0);
-    (this->*ss.m_sendREQ)(signal, ss.m_ssId);
+    SectionHandle handle(this);
+    (this->*ss.m_sendREQ)(signal, ss.m_ssId, &handle);
     return;
   }
 }
@@ -134,6 +150,26 @@ LocalProxy::skipConf(SsSequential& ss)
 {
 }
 
+void
+LocalProxy::saveSections(SsCommon& ss, SectionHandle & handle)
+{
+  ss.m_sec_cnt = handle.m_cnt;
+  for (Uint32 i = 0; i<ss.m_sec_cnt; i++)
+    ss.m_sec_ptr[i] = handle.m_ptr[i].i;
+  handle.clear();
+}
+
+void
+LocalProxy::restoreHandle(SectionHandle & handle, SsCommon& ss)
+{
+  handle.m_cnt = ss.m_sec_cnt;
+  for (Uint32 i = 0; i<ss.m_sec_cnt; i++)
+    handle.m_ptr[i].i = ss.m_sec_ptr[i];
+
+  getSections(handle.m_cnt, handle.m_ptr);
+  ss.m_sec_cnt = 0;
+}
+
 bool
 LocalProxy::firstReply(const SsSequential& ss)
 {
@@ -154,12 +190,15 @@ LocalProxy::sendREQ(Signal* signal, SsParallel& ss)
   ss.m_workerMask.clear();
   ss.m_worker = 0;
   const Uint32 count = ss.m_extraLast ? c_lqhWorkers : c_workers;
+  SectionHandle handle(this);
+  restoreHandle(handle, ss);
   while (ss.m_worker < count) {
     jam();
     ss.m_workerMask.set(ss.m_worker);
-    (this->*ss.m_sendREQ)(signal, ss.m_ssId);
+    (this->*ss.m_sendREQ)(signal, ss.m_ssId, &handle);
     ss.m_worker++;
   }
+  releaseSections(handle);
 }
 
 void
@@ -226,11 +265,12 @@ LocalProxy::lastReply(const SsParallel& ss)
 bool
 LocalProxy::lastExtra(Signal* signal, SsParallel& ss)
 {
+  SectionHandle handle(this);
   if (c_lqhWorkers + ss.m_extraSent < c_workers) {
     jam();
     ss.m_worker = c_lqhWorkers + ss.m_extraSent;
     ss.m_workerMask.set(ss.m_worker);
-    (this->*ss.m_sendREQ)(signal, ss.m_ssId);
+    (this->*ss.m_sendREQ)(signal, ss.m_ssId, &handle);
     ss.m_extraSent++;
     return false;
   }
@@ -307,7 +347,8 @@ LocalProxy::backREAD_CONFIG_REQ(Signal* signal)
 }
 
 void
-LocalProxy::sendREAD_CONFIG_REQ(Signal* signal, Uint32 ssId)
+LocalProxy::sendREAD_CONFIG_REQ(Signal* signal, Uint32 ssId,
+                                SectionHandle* handle)
 {
   Ss_READ_CONFIG_REQ& ss = ssFind<Ss_READ_CONFIG_REQ>(ssId);
 
@@ -315,8 +356,8 @@ LocalProxy::sendREAD_CONFIG_REQ(Signal* signal, Uint32 ssId)
   req->senderRef = reference();
   req->senderData = ssId;
   req->noOfParameters = 0;
-  sendSignal(workerRef(ss.m_worker), GSN_READ_CONFIG_REQ,
-             signal, ReadConfigReq::SignalLength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_READ_CONFIG_REQ,
+                      signal, ReadConfigReq::SignalLength, JBB, handle);
 }
 
 void
@@ -336,6 +377,10 @@ LocalProxy::sendREAD_CONFIG_CONF(Signal* signal, Uint32 ssId)
   if (!lastReply(ss))
     return;
 
+  SectionHandle handle(this);
+  restoreHandle(handle, ss);
+  releaseSections(handle);
+
   ReadConfigConf* conf = (ReadConfigConf*)signal->getDataPtrSend();
   conf->senderRef = reference();
   conf->senderData = ss.m_req.senderData;
@@ -380,13 +425,13 @@ LocalProxy::backSTTOR(Signal* signal)
 }
 
 void
-LocalProxy::sendSTTOR(Signal* signal, Uint32 ssId)
+LocalProxy::sendSTTOR(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
   Ss_STTOR& ss = ssFind<Ss_STTOR>(ssId);
 
   memcpy(signal->getDataPtrSend(), ss.m_reqdata, ss.m_reqlength << 2);
-  sendSignal(workerRef(ss.m_worker), GSN_STTOR,
-             signal, ss.m_reqlength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_STTOR,
+                      signal, ss.m_reqlength, JBB, handle);
 }
 
 void
@@ -450,15 +495,15 @@ LocalProxy::backNDB_STTOR(Signal* signal)
 }
 
 void
-LocalProxy::sendNDB_STTOR(Signal* signal, Uint32 ssId)
+LocalProxy::sendNDB_STTOR(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
   Ss_NDB_STTOR& ss = ssFind<Ss_NDB_STTOR>(ssId);
 
   NdbSttor* req = (NdbSttor*)signal->getDataPtrSend();
   *req = ss.m_req;
   req->senderRef = reference();
-  sendSignal(workerRef(ss.m_worker), GSN_NDB_STTOR,
-             signal, ss.m_reqlength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_NDB_STTOR,
+                      signal, ss.m_reqlength, JBB, handle);
 }
 
 void
@@ -559,14 +604,14 @@ LocalProxy::execNODE_FAILREP(Signal* signal)
 }
 
 void
-LocalProxy::sendNODE_FAILREP(Signal* signal, Uint32 ssId)
+LocalProxy::sendNODE_FAILREP(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
   Ss_NODE_FAILREP& ss = ssFind<Ss_NODE_FAILREP>(ssId);
 
   NodeFailRep* req = (NodeFailRep*)signal->getDataPtrSend();
   *req = ss.m_req;
-  sendSignal(workerRef(ss.m_worker), GSN_NODE_FAILREP,
-             signal, NodeFailRep::SignalLength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_NODE_FAILREP,
+                      signal, NodeFailRep::SignalLength, JBB, handle);
 }
 
 void
@@ -634,7 +679,7 @@ LocalProxy::execINCL_NODEREQ(Signal* signal)
 }
 
 void
-LocalProxy::sendINCL_NODEREQ(Signal* signal, Uint32 ssId)
+LocalProxy::sendINCL_NODEREQ(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
   Ss_INCL_NODEREQ& ss = ssFind<Ss_INCL_NODEREQ>(ssId);
 
@@ -643,8 +688,8 @@ LocalProxy::sendINCL_NODEREQ(Signal* signal, Uint32 ssId)
 
   memcpy(req, &ss.m_req, ss.m_reqlength << 2);
   req->senderRef = reference();
-  sendSignal(workerRef(ss.m_worker), GSN_INCL_NODEREQ,
-             signal, ss.m_reqlength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_INCL_NODEREQ,
+                      signal, ss.m_reqlength, JBB, handle);
 }
 
 void
@@ -685,12 +730,13 @@ LocalProxy::execNODE_STATE_REP(Signal* signal)
 }
 
 void
-LocalProxy::sendNODE_STATE_REP(Signal* signal, Uint32 ssId)
+LocalProxy::sendNODE_STATE_REP(Signal* signal, Uint32 ssId,
+                               SectionHandle* handle)
 {
   Ss_NODE_STATE_REP& ss = ssFind<Ss_NODE_STATE_REP>(ssId);
 
-  sendSignal(workerRef(ss.m_worker), GSN_NODE_STATE_REP,
-             signal,NodeStateRep::SignalLength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_NODE_STATE_REP,
+                      signal,NodeStateRep::SignalLength, JBB, handle);
 }
 
 // GSN_CHANGE_NODE_STATE_REQ
@@ -707,15 +753,16 @@ LocalProxy::execCHANGE_NODE_STATE_REQ(Signal* signal)
 }
 
 void
-LocalProxy::sendCHANGE_NODE_STATE_REQ(Signal* signal, Uint32 ssId)
+LocalProxy::sendCHANGE_NODE_STATE_REQ(Signal* signal, Uint32 ssId,
+                                      SectionHandle* handle)
 {
   Ss_CHANGE_NODE_STATE_REQ& ss = ssFind<Ss_CHANGE_NODE_STATE_REQ>(ssId);
 
   ChangeNodeStateReq * req = (ChangeNodeStateReq*)signal->getDataPtrSend();
   req->senderRef = reference();
 
-  sendSignal(workerRef(ss.m_worker), GSN_CHANGE_NODE_STATE_REQ,
-             signal, ChangeNodeStateReq::SignalLength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_CHANGE_NODE_STATE_REQ,
+                      signal, ChangeNodeStateReq::SignalLength, JBB, handle);
 }
 
 void
@@ -759,13 +806,14 @@ LocalProxy::execDUMP_STATE_ORD(Signal* signal)
 }
 
 void
-LocalProxy::sendDUMP_STATE_ORD(Signal* signal, Uint32 ssId)
+LocalProxy::sendDUMP_STATE_ORD(Signal* signal, Uint32 ssId,
+                               SectionHandle* handle)
 {
   Ss_DUMP_STATE_ORD& ss = ssFind<Ss_DUMP_STATE_ORD>(ssId);
 
   memcpy(signal->getDataPtrSend(), ss.m_reqdata, ss.m_reqlength << 2);
-  sendSignal(workerRef(ss.m_worker), GSN_DUMP_STATE_ORD,
-             signal, ss.m_reqlength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_DUMP_STATE_ORD,
+                      signal, ss.m_reqlength, JBB, handle);
 }
 
 // GSN_NDB_TAMPER
@@ -784,13 +832,13 @@ LocalProxy::execNDB_TAMPER(Signal* signal)
 }
 
 void
-LocalProxy::sendNDB_TAMPER(Signal* signal, Uint32 ssId)
+LocalProxy::sendNDB_TAMPER(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
   Ss_NDB_TAMPER& ss = ssFind<Ss_NDB_TAMPER>(ssId);
 
   signal->theData[0] = ss.m_errorInsert;
-  sendSignal(workerRef(ss.m_worker), GSN_NDB_TAMPER,
-             signal, 1, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_NDB_TAMPER,
+                      signal, 1, JBB, handle);
 }
 
 // GSN_TIME_SIGNAL
@@ -807,12 +855,12 @@ LocalProxy::execTIME_SIGNAL(Signal* signal)
 }
 
 void
-LocalProxy::sendTIME_SIGNAL(Signal* signal, Uint32 ssId)
+LocalProxy::sendTIME_SIGNAL(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
   Ss_TIME_SIGNAL& ss = ssFind<Ss_TIME_SIGNAL>(ssId);
   signal->theData[0] = 0;
-  sendSignal(workerRef(ss.m_worker), GSN_TIME_SIGNAL,
-             signal, 1, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_TIME_SIGNAL,
+                      signal, 1, JBB, handle);
 }
 
 // GSN_CREATE_TRIG_IMPL_REQ
@@ -820,17 +868,25 @@ LocalProxy::sendTIME_SIGNAL(Signal* signal, Uint32 ssId)
 void
 LocalProxy::execCREATE_TRIG_IMPL_REQ(Signal* signal)
 {
+  if (!assembleFragments(signal))
+    return;
+
   if (ssQueue<Ss_CREATE_TRIG_IMPL_REQ>(signal))
     return;
   const CreateTrigImplReq* req = (const CreateTrigImplReq*)signal->getDataPtr();
   Ss_CREATE_TRIG_IMPL_REQ& ss = ssSeize<Ss_CREATE_TRIG_IMPL_REQ>();
   ss.m_req = *req;
-  ndbrequire(signal->getLength() <= CreateTrigImplReq::SignalLengthLocal);
+  ndbrequire(signal->getLength() <= CreateTrigImplReq::SignalLength);
+
+  SectionHandle handle(this, signal);
+  saveSections(ss, handle);
+
   sendREQ(signal, ss);
 }
 
 void
-LocalProxy::sendCREATE_TRIG_IMPL_REQ(Signal* signal, Uint32 ssId)
+LocalProxy::sendCREATE_TRIG_IMPL_REQ(Signal* signal, Uint32 ssId,
+                                     SectionHandle * handle)
 {
   Ss_CREATE_TRIG_IMPL_REQ& ss = ssFind<Ss_CREATE_TRIG_IMPL_REQ>(ssId);
 
@@ -838,8 +894,9 @@ LocalProxy::sendCREATE_TRIG_IMPL_REQ(Signal* signal, Uint32 ssId)
   *req = ss.m_req;
   req->senderRef = reference();
   req->senderData = ssId;
-  sendSignal(workerRef(ss.m_worker), GSN_CREATE_TRIG_IMPL_REQ,
-             signal, CreateTrigImplReq::SignalLengthLocal, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_CREATE_TRIG_IMPL_REQ,
+                      signal, CreateTrigImplReq::SignalLength, JBB,
+                      handle);
 }
 
 void
@@ -909,7 +966,8 @@ LocalProxy::execDROP_TRIG_IMPL_REQ(Signal* signal)
 }
 
 void
-LocalProxy::sendDROP_TRIG_IMPL_REQ(Signal* signal, Uint32 ssId)
+LocalProxy::sendDROP_TRIG_IMPL_REQ(Signal* signal, Uint32 ssId,
+                                   SectionHandle * handle)
 {
   Ss_DROP_TRIG_IMPL_REQ& ss = ssFind<Ss_DROP_TRIG_IMPL_REQ>(ssId);
 
@@ -917,8 +975,8 @@ LocalProxy::sendDROP_TRIG_IMPL_REQ(Signal* signal, Uint32 ssId)
   *req = ss.m_req;
   req->senderRef = reference();
   req->senderData = ssId;
-  sendSignal(workerRef(ss.m_worker), GSN_DROP_TRIG_IMPL_REQ,
-             signal, DropTrigImplReq::SignalLength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_DROP_TRIG_IMPL_REQ,
+                      signal, DropTrigImplReq::SignalLength, JBB, handle);
 }
 
 void
@@ -971,4 +1029,300 @@ LocalProxy::sendDROP_TRIG_IMPL_CONF(Signal* signal, Uint32 ssId)
   ssRelease<Ss_DROP_TRIG_IMPL_REQ>(ssId);
 }
 
+// GSN_DBINFO_SCANREQ
+
+//#define DBINFO_SCAN_TRACE
+#ifdef DBINFO_SCAN_TRACE
+#include <debugger/DebuggerNames.hpp>
+#endif
+
+static Uint32
+switchRef(Uint32 block, Uint32 instance, Uint32 node)
+{
+  const Uint32 ref = numberToRef(block, instance,  node);
+#ifdef DBINFO_SCAN_TRACE
+  ndbout_c("Dbinfo::LocalProxy: switching to %s(%d) in node %d, ref: 0x%.8x",
+           getBlockName(block, "<unknown>"), instance, node, ref);
+#endif
+  return ref;
+}
+
+
+bool
+LocalProxy::find_next(Ndbinfo::ScanCursor* cursor) const
+{
+  const Uint32 node = refToNode(cursor->currRef);
+  const Uint32 block = refToMain(cursor->currRef);
+  Uint32 instance = refToInstance(cursor->currRef);
+
+  ndbrequire(node == getOwnNodeId());
+  ndbrequire(block == number());
+
+
+  Uint32 worker = (instance > 0) ? workerIndex(instance) + 1 : 0;
+
+  if (worker < c_workers)
+  {
+    jam();
+    cursor->currRef = switchRef(block, workerInstance(worker), node);
+    return true;
+  }
+
+  cursor->currRef = numberToRef(block, node);
+  return false;
+}
+
+
+
+void
+LocalProxy::execDBINFO_SCANREQ(Signal* signal)
+{
+  jamEntry();
+  const DbinfoScanReq* req = (const DbinfoScanReq*) signal->getDataPtr();
+  Uint32 signal_length = signal->getLength();
+  ndbrequire(signal_length == DbinfoScanReq::SignalLength+req->cursor_sz);
+
+  Ndbinfo::ScanCursor* cursor =
+    (Ndbinfo::ScanCursor*)DbinfoScan::getCursorPtr(req);
+
+  if (Ndbinfo::ScanCursor::getHasMoreData(cursor->flags) &&
+      cursor->saveCurrRef)
+  {
+    jam();
+    /* Continue in the saved block ref */
+    cursor->currRef = cursor->saveCurrRef;
+    cursor->saveCurrRef = 0;
+
+    // Set this block as sender and remember original sender
+    cursor->saveSenderRef = cursor->senderRef;
+    cursor->senderRef = reference();
+
+    sendSignal(cursor->currRef, GSN_DBINFO_SCANREQ,
+               signal, signal_length, JBB);
+    return;
+  }
+
+  Ndbinfo::ScanCursor::setHasMoreData(cursor->flags, false);
+
+  if (find_next(cursor))
+  {
+    jam();
+    ndbrequire(cursor->currRef);
+    ndbrequire(cursor->saveCurrRef == 0);
+
+    // Set this block as sender and remember original sender
+    cursor->saveSenderRef = cursor->senderRef;
+    cursor->senderRef = reference();
+
+    sendSignal(cursor->currRef, GSN_DBINFO_SCANREQ,
+               signal, signal_length, JBB);
+    return;
+  }
+
+  /* Scan is done, send SCANCONF back to caller  */
+  ndbrequire(cursor->saveSenderRef == 0);
+
+  ndbrequire(cursor->currRef);
+  ndbrequire(cursor->saveCurrRef == 0);
+
+  ndbrequire(refToInstance(cursor->currRef) == 0);
+  sendSignal(cursor->senderRef, GSN_DBINFO_SCANCONF, signal, signal_length, JBB);
+  return;
+}
+
+void
+LocalProxy::execDBINFO_SCANCONF(Signal* signal)
+{
+  jamEntry();
+  const DbinfoScanConf* conf = (const DbinfoScanConf*)signal->getDataPtr();
+  Uint32 signal_length = signal->getLength();
+  ndbrequire(signal_length == DbinfoScanConf::SignalLength+conf->cursor_sz);
+
+  Ndbinfo::ScanCursor* cursor =
+    (Ndbinfo::ScanCursor*)DbinfoScan::getCursorPtr(conf);
+
+  if (Ndbinfo::ScanCursor::getHasMoreData(cursor->flags))
+  {
+    /* The instance has more data and want to continue */
+    jam();
+
+    /* Swap back saved senderRef */
+    const Uint32 senderRef = cursor->senderRef = cursor->saveSenderRef;
+    cursor->saveSenderRef = 0;
+
+    /* Save currRef to continue with same instance again */
+    cursor->saveCurrRef = cursor->currRef;
+    cursor->currRef = reference();
+
+    sendSignal(senderRef, GSN_DBINFO_SCANCONF, signal, signal_length, JBB);
+    return;
+  }
+
+  if (conf->returnedRows)
+  {
+    jam();
+    /*
+       The instance has no more data, but it has sent rows
+       to the API which need to be CONFed
+    */
+
+    /* Swap back saved senderRef */
+    const Uint32 senderRef = cursor->senderRef = cursor->saveSenderRef;
+    cursor->saveSenderRef = 0;
+
+    if (find_next(cursor))
+    {
+      /*
+        There is another instance to continue in - signal 'more data'
+        and setup saveCurrRef to continue in that instance
+      */
+      jam();
+      Ndbinfo::ScanCursor::setHasMoreData(cursor->flags, true);
+
+      cursor->saveCurrRef = cursor->currRef;
+      cursor->currRef = reference();
+    }
+    else
+    {
+      /* There was no more instances to continue in */
+      ndbrequire(Ndbinfo::ScanCursor::getHasMoreData(cursor->flags) == false);
+
+      ndbrequire(cursor->currRef == reference());
+      cursor->saveCurrRef = 0;
+    }
+
+    sendSignal(senderRef, GSN_DBINFO_SCANCONF, signal, signal_length, JBB);
+    return;
+  }
+
+
+  /* The underlying block reported completed, find next if any */
+  if (find_next(cursor))
+  {
+    jam();
+
+    ndbrequire(cursor->senderRef == reference());
+    ndbrequire(cursor->saveSenderRef); // Should already be set
+
+    ndbrequire(cursor->saveCurrRef == 0);
+
+    sendSignal(cursor->currRef, GSN_DBINFO_SCANREQ,
+               signal, signal_length, JBB);
+    return;
+  }
+
+  /* Scan in this block and its instances are completed */
+
+  /* Swap back saved senderRef */
+  const Uint32 senderRef = cursor->senderRef = cursor->saveSenderRef;
+  cursor->saveSenderRef = 0;
+
+  ndbrequire(cursor->currRef);
+  ndbrequire(cursor->saveCurrRef == 0);
+
+  sendSignal(senderRef, GSN_DBINFO_SCANCONF, signal, signal_length, JBB);
+  return;
+}
+
+// GSN_SYNC_REQ
+
+void
+LocalProxy::execSYNC_REQ(Signal* signal)
+{
+  Ss_SYNC_REQ& ss = ssSeize<Ss_SYNC_REQ>();
+
+  ss.m_req = * CAST_CONSTPTR(SyncReq, signal->getDataPtr());
+
+  sendREQ(signal, ss);
+}
+
+void
+LocalProxy::sendSYNC_REQ(Signal* signal, Uint32 ssId,
+                         SectionHandle* handle)
+{
+  Ss_SYNC_REQ& ss = ssFind<Ss_SYNC_REQ>(ssId);
+
+  SyncReq * req = CAST_PTR(SyncReq, signal->getDataPtrSend());
+  req->senderRef = reference();
+  req->senderData = ssId;
+  req->prio = ss.m_req.prio;
+
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_SYNC_REQ,
+                      signal, SyncReq::SignalLength,
+                      JobBufferLevel(ss.m_req.prio), handle);
+}
+
+void
+LocalProxy::execSYNC_REF(Signal* signal)
+{
+  SyncRef ref = * CAST_CONSTPTR(SyncRef, signal->getDataPtr());
+  Ss_SYNC_REQ& ss = ssFind<Ss_SYNC_REQ>(ref.senderData);
+
+  recvREF(signal, ss, ref.errorCode);
+}
+
+void
+LocalProxy::execSYNC_CONF(Signal* signal)
+{
+  SyncConf conf = * CAST_CONSTPTR(SyncConf, signal->getDataPtr());
+  Ss_SYNC_REQ& ss = ssFind<Ss_SYNC_REQ>(conf.senderData);
+
+  recvCONF(signal, ss);
+}
+
+void
+LocalProxy::sendSYNC_CONF(Signal* signal, Uint32 ssId)
+{
+  Ss_SYNC_REQ& ss = ssFind<Ss_SYNC_REQ>(ssId);
+
+  if (!lastReply(ss))
+    return;
+
+  /**
+   * SimulatedBlock::execSYNC_REQ will reply
+   */
+  if (ss.m_error == 0)
+  {
+    jam();
+    SyncConf * conf = CAST_PTR(SyncConf, signal->getDataPtrSend());
+    conf->senderRef = reference();
+    conf->senderData = ss.m_req.senderData;
+
+    Uint32 prio = ss.m_req.prio;
+    sendSignal(ss.m_req.senderRef, GSN_SYNC_CONF, signal,
+               SyncConf::SignalLength,
+               JobBufferLevel(prio));
+  }
+  else
+  {
+    jam();
+    SyncRef * ref = CAST_PTR(SyncRef, signal->getDataPtrSend());
+    ref->senderRef = reference();
+    ref->senderData = ss.m_req.senderData;
+    ref->errorCode = ss.m_error;
+
+    Uint32 prio = ss.m_req.prio;
+    sendSignal(ss.m_req.senderRef, GSN_SYNC_REF, signal,
+               SyncRef::SignalLength,
+               JobBufferLevel(prio));
+  }
+  ssRelease<Ss_SYNC_REQ>(ssId);
+}
+
+void
+LocalProxy::execSYNC_PATH_REQ(Signal* signal)
+{
+  SyncPathReq* req = CAST_PTR(SyncPathReq, signal->getDataPtrSend());
+  req->count *= c_workers;
+
+  for (Uint32 i = 0; i < c_workers; i++)
+  {
+    jam();
+    Uint32 ref = numberToRef(number(), workerInstance(i), getOwnNodeId());
+    sendSignal(ref, GSN_SYNC_PATH_REQ, signal,
+               signal->getLength(),
+               JobBufferLevel(req->prio));
+  }
+}
+
 BLOCK_FUNCTIONS(LocalProxy)
