@@ -28,6 +28,8 @@
 #include <signaldata/NFCompleteRep.hpp>
 #include <signaldata/CreateTrigImpl.hpp>
 #include <signaldata/DropTrigImpl.hpp>
+#include <signaldata/DbinfoScan.hpp>
+#include <signaldata/Sync.hpp>
 
 /*
  * Proxy blocks for MT LQH.
@@ -93,7 +95,7 @@ protected:
 
   // convert between worker index and worker instance
 
-  Uint32 workerInstance(Uint32 i) {
+  Uint32 workerInstance(Uint32 i) const {
     ndbrequire(i < c_workers);
     Uint32 ino;
     if (i < c_lqhWorkers)
@@ -103,7 +105,7 @@ protected:
     return ino;
   }
 
-  Uint32 workerIndex(Uint32 ino) {
+  Uint32 workerIndex(Uint32 ino) const {
     ndbrequire(ino != 0);
     Uint32 i;
     if (ino != 1 + MaxLqhWorkers)
@@ -116,14 +118,17 @@ protected:
 
   // support routines and classes ("Ss" = signal state)
 
-  typedef void (LocalProxy::*SsFUNC)(Signal*, Uint32 ssId);
+  typedef void (LocalProxy::*SsFUNCREQ)(Signal*, Uint32 ssId, SectionHandle*);
+  typedef void (LocalProxy::*SsFUNCREP)(Signal*, Uint32 ssId);
 
   struct SsCommon {
     Uint32 m_ssId;      // unique id in SsPool (below)
-    SsFUNC m_sendREQ;   // from proxy to worker
-    SsFUNC m_sendCONF;  // from proxy to caller
+    SsFUNCREQ m_sendREQ;   // from proxy to worker
+    SsFUNCREP m_sendCONF;  // from proxy to caller
     Uint32 m_worker;    // current worker
     Uint32 m_error;
+    Uint32 m_sec_cnt;
+    Uint32 m_sec_ptr[3];
     static const char* name() { return "UNDEF"; }
     SsCommon() {
       m_ssId = 0;
@@ -131,6 +136,7 @@ protected:
       m_sendCONF = 0;
       m_worker = 0;
       m_error = 0;
+      m_sec_cnt = 0;
     }
   };
 
@@ -148,6 +154,9 @@ protected:
   bool firstReply(const SsSequential& ss);
   bool lastReply(const SsSequential& ss);
 
+  void saveSections(SsCommon&ss, SectionHandle&);
+  void restoreHandle(SectionHandle&, SsCommon&);
+
   // run workers in parallel
   struct SsParallel : SsCommon {
     WorkerMask m_workerMask;
@@ -293,10 +302,11 @@ protected:
     SsPool<Ss>& sp = Ss::pool(this);
     if (sp.m_usage < Ss::poolSize)
       return false;
-    ndbrequire(signal->getNoOfSections() == 0);
+
+    SectionHandle handle(this, signal);
     GlobalSignalNumber gsn = signal->header.theVerId_signalNumber & 0xFFFF;
     sendSignalWithDelay(reference(), gsn,
-                        signal, 10, signal->length());
+                        signal, 10, signal->length(), &handle);
     return true;
   }
 
@@ -321,7 +331,7 @@ protected:
   void execREAD_CONFIG_REQ(Signal*);
   virtual void callREAD_CONFIG_REQ(Signal*);
   void backREAD_CONFIG_REQ(Signal*);
-  void sendREAD_CONFIG_REQ(Signal*, Uint32 ssId);
+  void sendREAD_CONFIG_REQ(Signal*, Uint32 ssId, SectionHandle*);
   void execREAD_CONFIG_CONF(Signal*);
   void sendREAD_CONFIG_CONF(Signal*, Uint32 ssId);
 
@@ -344,7 +354,7 @@ protected:
   void execSTTOR(Signal*);
   virtual void callSTTOR(Signal*);
   void backSTTOR(Signal*);
-  void sendSTTOR(Signal*, Uint32 ssId);
+  void sendSTTOR(Signal*, Uint32 ssId, SectionHandle*);
   void execSTTORRY(Signal*);
   void sendSTTORRY(Signal*, Uint32 ssId);
 
@@ -365,7 +375,7 @@ protected:
   void execNDB_STTOR(Signal*);
   virtual void callNDB_STTOR(Signal*);
   void backNDB_STTOR(Signal*);
-  void sendNDB_STTOR(Signal*, Uint32 ssId);
+  void sendNDB_STTOR(Signal*, Uint32 ssId, SectionHandle*);
   void execNDB_STTORRY(Signal*);
   void sendNDB_STTORRY(Signal*, Uint32 ssId);
 
@@ -402,7 +412,7 @@ protected:
   };
   SsPool<Ss_NODE_FAILREP> c_ss_NODE_FAILREP;
   void execNODE_FAILREP(Signal*);
-  void sendNODE_FAILREP(Signal*, Uint32 ssId);
+  void sendNODE_FAILREP(Signal*, Uint32 ssId, SectionHandle*);
   void execNF_COMPLETEREP(Signal*);
   void sendNF_COMPLETEREP(Signal*, Uint32 ssId);
 
@@ -431,7 +441,7 @@ protected:
   };
   SsPool<Ss_INCL_NODEREQ> c_ss_INCL_NODEREQ;
   void execINCL_NODEREQ(Signal*);
-  void sendINCL_NODEREQ(Signal*, Uint32 ssId);
+  void sendINCL_NODEREQ(Signal*, Uint32 ssId, SectionHandle*);
   void execINCL_NODECONF(Signal*);
   void sendINCL_NODECONF(Signal*, Uint32 ssId);
 
@@ -448,7 +458,7 @@ protected:
   };
   SsPool<Ss_NODE_STATE_REP> c_ss_NODE_STATE_REP;
   void execNODE_STATE_REP(Signal*);
-  void sendNODE_STATE_REP(Signal*, Uint32 ssId);
+  void sendNODE_STATE_REP(Signal*, Uint32 ssId, SectionHandle*);
 
   // GSN_CHANGE_NODE_STATE_REQ
   struct Ss_CHANGE_NODE_STATE_REQ : SsParallel {
@@ -464,7 +474,7 @@ protected:
   };
   SsPool<Ss_CHANGE_NODE_STATE_REQ> c_ss_CHANGE_NODE_STATE_REQ;
   void execCHANGE_NODE_STATE_REQ(Signal*);
-  void sendCHANGE_NODE_STATE_REQ(Signal*, Uint32 ssId);
+  void sendCHANGE_NODE_STATE_REQ(Signal*, Uint32 ssId, SectionHandle*);
   void execCHANGE_NODE_STATE_CONF(Signal*);
   void sendCHANGE_NODE_STATE_CONF(Signal*, Uint32 ssId);
 
@@ -483,7 +493,7 @@ protected:
   };
   SsPool<Ss_DUMP_STATE_ORD> c_ss_DUMP_STATE_ORD;
   void execDUMP_STATE_ORD(Signal*);
-  void sendDUMP_STATE_ORD(Signal*, Uint32 ssId);
+  void sendDUMP_STATE_ORD(Signal*, Uint32 ssId, SectionHandle*);
 
   // GSN_NDB_TAMPER
   struct Ss_NDB_TAMPER : SsParallel {
@@ -499,7 +509,7 @@ protected:
   };
   SsPool<Ss_NDB_TAMPER> c_ss_NDB_TAMPER;
   void execNDB_TAMPER(Signal*);
-  void sendNDB_TAMPER(Signal*, Uint32 ssId);
+  void sendNDB_TAMPER(Signal*, Uint32 ssId, SectionHandle*);
 
   // GSN_TIME_SIGNAL
   struct Ss_TIME_SIGNAL : SsParallel {
@@ -514,7 +524,7 @@ protected:
   };
   SsPool<Ss_TIME_SIGNAL> c_ss_TIME_SIGNAL;
   void execTIME_SIGNAL(Signal*);
-  void sendTIME_SIGNAL(Signal*, Uint32 ssId);
+  void sendTIME_SIGNAL(Signal*, Uint32 ssId, SectionHandle*);
 
   // GSN_CREATE_TRIG_IMPL_REQ
   struct Ss_CREATE_TRIG_IMPL_REQ : SsParallel {
@@ -530,7 +540,7 @@ protected:
   };
   SsPool<Ss_CREATE_TRIG_IMPL_REQ> c_ss_CREATE_TRIG_IMPL_REQ;
   void execCREATE_TRIG_IMPL_REQ(Signal*);
-  void sendCREATE_TRIG_IMPL_REQ(Signal*, Uint32 ssId);
+  void sendCREATE_TRIG_IMPL_REQ(Signal*, Uint32 ssId, SectionHandle*);
   void execCREATE_TRIG_IMPL_CONF(Signal*);
   void execCREATE_TRIG_IMPL_REF(Signal*);
   void sendCREATE_TRIG_IMPL_CONF(Signal*, Uint32 ssId);
@@ -549,10 +559,36 @@ protected:
   };
   SsPool<Ss_DROP_TRIG_IMPL_REQ> c_ss_DROP_TRIG_IMPL_REQ;
   void execDROP_TRIG_IMPL_REQ(Signal*);
-  void sendDROP_TRIG_IMPL_REQ(Signal*, Uint32 ssId);
+  void sendDROP_TRIG_IMPL_REQ(Signal*, Uint32 ssId, SectionHandle*);
   void execDROP_TRIG_IMPL_CONF(Signal*);
   void execDROP_TRIG_IMPL_REF(Signal*);
   void sendDROP_TRIG_IMPL_CONF(Signal*, Uint32 ssId);
+
+  // GSN_DBINFO_SCANREQ
+  bool find_next(Ndbinfo::ScanCursor* cursor) const;
+  void execDBINFO_SCANREQ(Signal*);
+  void execDBINFO_SCANCONF(Signal*);
+
+  // GSN_SYNC_REQ
+  void execSYNC_REQ(Signal*);
+  void execSYNC_REF(Signal*);
+  void execSYNC_CONF(Signal*);
+  void sendSYNC_REQ(Signal*, Uint32 ssId, SectionHandle*);
+  void sendSYNC_CONF(Signal*, Uint32 ssId);
+  struct Ss_SYNC_REQ : SsParallel {
+    SyncReq m_req;
+    Ss_SYNC_REQ() {
+      m_sendREQ = &LocalProxy::sendSYNC_REQ;
+      m_sendCONF = &LocalProxy::sendSYNC_CONF;
+    }
+    enum { poolSize = 4 };
+    static SsPool<Ss_SYNC_REQ>& pool(LocalProxy* proxy) {
+      return proxy->c_ss_SYNC_REQ;
+    }
+  };
+  SsPool<Ss_SYNC_REQ> c_ss_SYNC_REQ;
+
+  void execSYNC_PATH_REQ(Signal*);
 };
 
 #endif
