@@ -23,6 +23,7 @@
 #include <TransporterRegistry.hpp>
 #include <NdbOut.hpp>
 #include <NdbMem.h>
+#include <NdbTick.h>
 
 #include <SignalLoggerManager.hpp>
 #include <FastScheduler.hpp>
@@ -40,6 +41,8 @@
 #include <signaldata/DisconnectRep.hpp>
 #include <signaldata/EnableCom.hpp>
 #include <signaldata/RouteOrd.hpp>
+#include <signaldata/DbinfoScan.hpp>
+#include <signaldata/Sync.hpp>
 
 #include <EventLogger.hpp>
 #include <TimeQueue.hpp>
@@ -53,6 +56,9 @@
 extern EventLogger * g_eventLogger;
 extern int simulate_error_during_shutdown;
 
+// Index pages used by ACC instances
+Uint32 g_acc_pages_used[1 + MAX_NDBMT_LQH_WORKERS];
+
 extern void mt_set_section_chunk_size();
 
 Cmvmi::Cmvmi(Block_context& ctx) :
@@ -108,9 +114,15 @@ Cmvmi::Cmvmi(Block_context& ctx) :
 
   addRecSignal(GSN_CONTINUEB, &Cmvmi::execCONTINUEB);
   addRecSignal(GSN_ROUTE_ORD, &Cmvmi::execROUTE_ORD);
-  
+  addRecSignal(GSN_DBINFO_SCANREQ, &Cmvmi::execDBINFO_SCANREQ);
+
+  addRecSignal(GSN_SYNC_REQ, &Cmvmi::execSYNC_REQ, true);
+  addRecSignal(GSN_SYNC_REF, &Cmvmi::execSYNC_REF);
+  addRecSignal(GSN_SYNC_CONF, &Cmvmi::execSYNC_CONF);
+
   subscriberPool.setSize(5);
-  
+  c_syncReqPool.setSize(5);
+
   const ndb_mgm_configuration_iterator * db = m_ctx.m_config.getOwnConfigIterator();
   for(unsigned j = 0; j<LogLevel::LOGLEVEL_CATEGORIES; j++){
     Uint32 logLevel;
@@ -147,6 +159,11 @@ Cmvmi::Cmvmi(Block_context& ctx) :
   setNodeInfo(getOwnNodeId()).m_mysql_version = NDB_MYSQL_VERSION_D;
 
   c_memusage_report_frequency = 0;
+
+  m_start_time = NdbTick_CurrentMillisecond() / 1000; // seconds
+
+  bzero(g_acc_pages_used, sizeof(g_acc_pages_used));
+
 }
 
 Cmvmi::~Cmvmi()
@@ -204,6 +221,136 @@ void Cmvmi::execNDB_TAMPER(Signal* signal)
 #endif
 }//execNDB_TAMPER()
 
+static Uint32 blocks[] =
+{
+  QMGR_REF,
+  NDBCNTR_REF,
+  DBTC_REF,
+  DBDIH_REF,
+  DBDICT_REF,
+  DBLQH_REF,
+  DBTUP_REF,
+  DBACC_REF,
+  NDBFS_REF,
+  BACKUP_REF,
+  DBUTIL_REF,
+  SUMA_REF,
+  TRIX_REF,
+  DBTUX_REF,
+  LGMAN_REF,
+  TSMAN_REF,
+  PGMAN_REF,
+  DBINFO_REF,
+  0
+};
+
+void
+Cmvmi::execSYNC_REQ(Signal* signal)
+{
+  jamEntry();
+  SyncReq req = * CAST_CONSTPTR(SyncReq, signal->getDataPtr());
+  Ptr<SyncRecord> ptr;
+  if (!c_syncReqPool.seize(ptr))
+  {
+    jam();
+    SyncRecord tmp;
+    ptr.p = &tmp;
+    tmp.m_senderRef = req.senderRef;
+    tmp.m_senderData = req.senderData;
+    tmp.m_prio = req.prio;
+    tmp.m_error = SyncRef::SR_OUT_OF_MEMORY;
+    sendSYNC_REP(signal, ptr);
+    return;
+  }
+
+  ptr.p->m_senderRef = req.senderRef;
+  ptr.p->m_senderData = req.senderData;
+  ptr.p->m_prio = req.prio;
+  ptr.p->m_error = 0;
+
+  SyncReq* out = CAST_PTR(SyncReq, signal->getDataPtrSend());
+  out->senderRef = reference();
+  out->senderData = ptr.i;
+  out->prio = ptr.p->m_prio;
+  Uint32 i = 0;
+  for (i = 0; blocks[i] != 0; i++)
+  {
+    sendSignal(blocks[i], GSN_SYNC_REQ, signal, SyncReq::SignalLength,
+               JobBufferLevel(ptr.p->m_prio));
+  }
+  ptr.p->m_cnt = i;
+}
+
+void
+Cmvmi::execSYNC_CONF(Signal* signal)
+{
+  jamEntry();
+  SyncConf conf = * CAST_CONSTPTR(SyncConf, signal->getDataPtr());
+
+  Ptr<SyncRecord> ptr;
+  c_syncReqPool.getPtr(ptr, conf.senderData);
+  ndbrequire(ptr.p->m_cnt > 0);
+  ptr.p->m_cnt--;
+  if (ptr.p->m_cnt == 0)
+  {
+    jam();
+
+    sendSYNC_REP(signal, ptr);
+    c_syncReqPool.release(ptr);
+  }
+}
+
+void
+Cmvmi::execSYNC_REF(Signal* signal)
+{
+  jamEntry();
+  SyncRef ref = * CAST_CONSTPTR(SyncRef, signal->getDataPtr());
+
+  Ptr<SyncRecord> ptr;
+  c_syncReqPool.getPtr(ptr, ref.senderData);
+  ndbrequire(ptr.p->m_cnt > 0);
+  ptr.p->m_cnt--;
+
+  if (ptr.p->m_error == 0)
+  {
+    jam();
+    ptr.p->m_error = ref.errorCode;
+  }
+
+  if (ptr.p->m_cnt == 0)
+  {
+    jam();
+
+    sendSYNC_REP(signal, ptr);
+    c_syncReqPool.release(ptr);
+  }
+}
+
+void
+Cmvmi::sendSYNC_REP(Signal * signal, Ptr<SyncRecord> ptr)
+{
+  if (ptr.p->m_error == 0)
+  {
+    jam();
+    SyncConf* conf = CAST_PTR(SyncConf, signal->getDataPtrSend());
+    conf->senderRef = reference();
+    conf->senderData = ptr.p->m_senderData;
+    sendSignal(ptr.p->m_senderRef, GSN_SYNC_CONF, signal,
+               SyncConf::SignalLength,
+               JobBufferLevel(ptr.p->m_prio));
+  }
+  else
+  {
+    jam();
+    SyncRef* ref = CAST_PTR(SyncRef, signal->getDataPtrSend());
+    ref->senderRef = reference();
+    ref->senderData = ptr.p->m_senderData;
+    ref->errorCode = ptr.p->m_error;
+    sendSignal(ptr.p->m_senderRef, GSN_SYNC_REF, signal, SyncRef::SignalLength,
+               JobBufferLevel(ptr.p->m_prio));
+  }
+}
+
 void Cmvmi::execSET_LOGLEVELORD(Signal* signal) 
 {
   SetLogLevelOrd * const llOrd = (SetLogLevelOrd *)&signal->theData[0];
@@ -219,6 +366,148 @@ void Cmvmi::execSET_LOGLEVELORD(Signal* signal)
   }
 }//execSET_LOGLEVELORD()
 
+struct SavedEvent
+{
+  Uint32 m_len;
+  Uint32 m_seq;
+  Uint32 m_time;
+  Uint32 m_data[25];
+
+  STATIC_CONST( HeaderLength = 3 );
+};
+
+static
+struct SavedEventBuffer
+{
+  SavedEventBuffer() {
+    m_read_pos = m_write_pos = 0;
+    m_sequence = 0;
+    m_buffer_len = 0;
+    m_data = 0;
+  }
+
+  void init(Uint32 bytes) {
+    if (bytes < 128)
+    {
+      return; // min size...unless set to 0
+    }
+    Uint32 words = bytes / 4;
+    m_data = new Uint32[words];
+    if (m_data)
+    {
+      m_buffer_len = words;
+    }
+  }
+
+  Uint32 m_sequence;
+  Uint16 m_write_pos;
+  Uint16 m_read_pos;
+  Uint32 m_buffer_len;
+  Uint32 * m_data;
+
+  void alloc(Uint32 len);
+  void purge();
+  void save(const Uint32 * theData, Uint32 len);
+
+  Uint32 free() const;
+
+  Uint32 m_scan_pos;
+  void startScan();
+  int scan(SavedEvent * dst, Uint32 filter[]);
+} m_saved_event_buffer;
+
+void
+SavedEventBuffer::alloc(Uint32 len)
+{
+  assert(m_buffer_len > 0);
+
+  while (free() <= len)
+    purge();
+}
+
+Uint32
+SavedEventBuffer::free() const
+{
+  if (m_write_pos == m_read_pos)
+    return m_buffer_len;
+  else if (m_write_pos > m_read_pos)
+    return (m_buffer_len - m_write_pos) + m_read_pos;
+  else
+    return m_read_pos - m_write_pos;
+}
+
+void
+SavedEventBuffer::purge()
+{
+  const Uint32 * ptr = m_data + m_read_pos;
+  const SavedEvent * header = (SavedEvent*)ptr;
+  Uint32 len = SavedEvent::HeaderLength + header->m_len;
+  m_read_pos = (m_read_pos + len) % m_buffer_len;
+}
+
+void
+SavedEventBuffer::save(const Uint32 * theData, Uint32 len)
+{
+  if (m_buffer_len == 0)
+    return;
+
+  Uint32 total = len + SavedEvent::HeaderLength;
+  alloc(total);
+
+  SavedEvent s;
+  s.m_len = len; // size of SavedEvent
+  s.m_seq = m_sequence++;
+  s.m_time = (Uint32)time(0);
+  const Uint32 * src = (const Uint32*)&s;
+  Uint32 * dst = m_data + m_write_pos;
+
+  Uint32 remain = m_buffer_len - m_write_pos;
+  if (remain >= total)
+  {
+    memcpy(dst, src, 4 * SavedEvent::HeaderLength);
+    memcpy(dst+SavedEvent::HeaderLength, theData, 4*len);
+  }
+  else
+  {
+    memcpy(s.m_data, theData, 4 * len);
+    memcpy(dst, src, 4 * remain);
+    memcpy(m_data, src + remain, 4 * (total - remain));
+  }
+  m_write_pos = (m_write_pos + total) % m_buffer_len;
+}
+
+void
+SavedEventBuffer::startScan()
+{
+  m_scan_pos = m_read_pos;
+}
+
+int
+SavedEventBuffer::scan(SavedEvent* _dst, Uint32 filter[])
+{
+  Uint32 * dst = (Uint32*)_dst;
+  while (m_scan_pos != m_write_pos)
+  {
+    const Uint32 * ptr = m_data + m_scan_pos;
+    SavedEvent * s = (SavedEvent*)ptr;
+    assert(s->m_len <= 25);
+    Uint32 total = s->m_len + SavedEvent::HeaderLength;
+    if (m_scan_pos + total <= m_buffer_len)
+    {
+      memcpy(dst, s, 4 * total);
+    }
+    else
+    {
+      Uint32 remain = m_buffer_len - m_scan_pos;
+      memcpy(dst, s, 4 * remain);
+      memcpy(dst + remain, m_data, 4 * (total - remain));
+    }
+    m_scan_pos = (m_scan_pos + total) % m_buffer_len;
+    return 1;
+  }
+  return 0;
+}
+
 void Cmvmi::execEVENT_REP(Signal* signal) 
 {
   //-----------------------------------------------------------------------
@@ -266,7 +555,9 @@ void Cmvmi::execEVENT_REP(Signal* signal)
     
     sendSignal(ptr.p->blockRef, GSN_EVENT_REP, signal, signal->length(), JBB);
   }
-  
+
+  m_saved_event_buffer.save(signal->theData, signal->getLength());
+
   if(clogLevel.getLogLevel(eventCategory) < threshold){
     return;
   }
@@ -359,6 +650,9 @@ void Cmvmi::sendSTTORRY(Signal* signal)
 }//Cmvmi::sendSTTORRY
 
 
+static Uint32 f_accpages = 0;
+extern Uint32 compute_acc_32kpages(const ndb_mgm_configuration_iterator * p);
+
 void 
 Cmvmi::execREAD_CONFIG_REQ(Signal* signal)
 {
@@ -389,6 +683,12 @@ Cmvmi::execREAD_CONFIG_REQ(Signal* signal)
     m_shared_page_pool.set((GlobalPage*)ptr, ~0);
   }
 
+  f_accpages = compute_acc_32kpages(p);
+
+  Uint32 eventlog = 4096;
+  ndb_mgm_get_int_parameter(p, CFG_DB_EVENTLOG_BUFFER_SIZE, &eventlog);
+  m_saved_event_buffer.init(eventlog);
+
   ReadConfigConf * conf = (ReadConfigConf*)signal->getDataPtrSend();
   conf->senderRef = reference();
   conf->senderData = senderData;
@@ -398,11 +698,6 @@ Cmvmi::execREAD_CONFIG_REQ(Signal* signal)
   c_memusage_report_frequency = 0;
   ndb_mgm_get_int_parameter(p, CFG_DB_MEMREPORT_FREQUENCY, 
                             &c_memusage_report_frequency);
-  
-  signal->theData[0] = ZREPORT_MEMORY_USAGE;
-  signal->theData[1] = 0;
-  signal->theData[2] = 3;
-  execCONTINUEB(signal);
 }
 
 void Cmvmi::execSTTOR(Signal* signal)
@@ -422,9 +717,14 @@ void Cmvmi::execSTTOR(Signal* signal)
        */
       refresh_watch_dog(9);
       int res = NdbMem_MemLockAll(0);
-      if(res != 0){
-        g_eventLogger->warning("Failed to memlock pages");
-	warningEvent("Failed to memlock pages");
+      if (res != 0)
+      {
+        char buf[100];
+        BaseString::snprintf(buf, sizeof(buf), 
+                             "Failed to memlock pages, error: %d (%s)",
+                             errno, strerror(errno));
+        g_eventLogger->warning("%s", buf);
+        warningEvent("%s", buf);
       }
     }
     
@@ -440,6 +740,15 @@ void Cmvmi::execSTTOR(Signal* signal)
       ndbrequire(db_watchdog_interval);
       update_watch_dog_timer(db_watchdog_interval);
     }
+
+    /**
+     * Start auto-mem reporting
+     */
+    signal->theData[0] = ZREPORT_MEMORY_USAGE;
+    signal->theData[1] = 0;
+    signal->theData[2] = 0;
+    signal->theData[3] = 0;
+    execCONTINUEB(signal);
     
     sendSTTORRY(signal);
     return;
@@ -574,7 +883,8 @@ void Cmvmi::execOPEN_COMREQ(Signal* signal)
     }
   }
   
-  if (userRef != 0) {
+  if (userRef != 0)
+  {
     jam(); 
     signal->theData[0] = tStartingNode;
     signal->theData[1] = tData2;
@@ -629,7 +939,6 @@ void Cmvmi::execDISCONNECT_REP(Signal *signal)
 {
   const DisconnectRep * const rep = (DisconnectRep *)&signal->theData[0];
   const Uint32 hostId = rep->nodeId;
-
   jamEntry();
 
   setNodeInfo(hostId).m_connected = false;
@@ -814,7 +1123,6 @@ void Cmvmi::execSTOP_ORD(Signal* signal)
 
 void
 Cmvmi::execSTART_ORD(Signal* signal) {
-
   StartOrd * const startOrd = (StartOrd *)&signal->theData[0];
   jamEntry();
   
@@ -833,7 +1141,7 @@ Cmvmi::execSTART_ORD(Signal* signal) {
       type = NRT_DoStart_InitialStart;
     if(!StopReq::getNoStart(tmp)&&!StopReq::getInitialStart(tmp))
       type = NRT_DoStart_Restart;
-    NdbShutdown(NST_Restart, type);
+    NdbShutdown(0, NST_Restart, type);
   }
 
   if(globalData.theRestartFlag == system_started){
@@ -881,8 +1189,12 @@ Cmvmi::execSTART_ORD(Signal* signal) {
       int res = NdbMem_MemLockAll(1);
       if(res != 0)
       {
-        g_eventLogger->warning("Failed to memlock pages");
-	warningEvent("Failed to memlock pages");
+        char buf[100];
+        BaseString::snprintf(buf, sizeof(buf),
+                             "Failed to memlock pages, error: %d (%s)",
+                             errno, strerror(errno));
+        g_eventLogger->warning("%s", buf);
+        warningEvent("%s", buf);
       }
       else
       {
@@ -927,12 +1239,188 @@ void Cmvmi::execTAMPER_ORD(Signal* signal)
   // to be able to indicate if we really introduced an error.
 #ifdef ERROR_INSERT
   TamperOrd* const tamperOrd = (TamperOrd*)&signal->theData[0];
-  signal->theData[2] = 0;
-  signal->theData[1] = tamperOrd->errorNo;
-  signal->theData[0] = 5;
-  sendSignal(DBDIH_REF, GSN_DIHNDBTAMPER, signal, 3,JBB);
-#endif
+  Uint32 errNo = tamperOrd->errorNo;
 
+  if (errNo == 0)
+  {
+    jam();
+    signal->theData[0] = 0;
+    sendSignal(QMGR_REF, GSN_NDB_TAMPER, signal, 1, JBB);
+    sendSignal(NDBCNTR_REF, GSN_NDB_TAMPER, signal, 1, JBB);
+    sendSignal(NDBFS_REF, GSN_NDB_TAMPER, signal, 1, JBB);
+    sendSignal(DBACC_REF, GSN_NDB_TAMPER, signal, 1, JBB);
+    sendSignal(DBTUP_REF, GSN_NDB_TAMPER, signal, 1, JBB);
+    sendSignal(DBLQH_REF, GSN_NDB_TAMPER, signal, 1, JBB);
+    sendSignal(DBDICT_REF, GSN_NDB_TAMPER, signal, 1, JBB);
+    sendSignal(DBDIH_REF, GSN_NDB_TAMPER, signal, 1, JBB);
+    sendSignal(DBTC_REF, GSN_NDB_TAMPER, signal, 1, JBB);
+    sendSignal(CMVMI_REF, GSN_NDB_TAMPER, signal, 1, JBB);
+    return;
+  }
+
+  Uint32 tuserblockref = 0;
+  if (errNo < 1000)
+  {
+    /*--------------------------------------------------------------------*/
+    // Insert errors into QMGR.
+    /*--------------------------------------------------------------------*/
+    jam();
+    tuserblockref = QMGR_REF;
+  }
+  else if (errNo < 2000)
+  {
+    /*--------------------------------------------------------------------*/
+    // Insert errors into NDBCNTR.
+    /*--------------------------------------------------------------------*/
+    jam();
+    tuserblockref = NDBCNTR_REF;
+  }
+  else if (errNo < 3000)
+  {
+    /*--------------------------------------------------------------------*/
+    // Insert errors into NDBFS.
+    /*--------------------------------------------------------------------*/
+    jam();
+    tuserblockref = NDBFS_REF;
+  }
+  else if (errNo < 4000)
+  {
+    /*--------------------------------------------------------------------*/
+    // Insert errors into DBACC.
+    /*--------------------------------------------------------------------*/
+    jam();
+    tuserblockref = DBACC_REF;
+  }
+  else if (errNo < 5000)
+  {
+    /*--------------------------------------------------------------------*/
+    // Insert errors into DBTUP.
+    /*--------------------------------------------------------------------*/
+    jam();
+    tuserblockref = DBTUP_REF;
+  }
+  else if (errNo < 6000)
+  {
+    /*---------------------------------------------------------------------*/
+    // Insert errors into DBLQH.
+    /*---------------------------------------------------------------------*/
+    jam();
+    tuserblockref = DBLQH_REF;
+  }
+  else if (errNo < 7000)
+  {
+    /*---------------------------------------------------------------------*/
+    // Insert errors into DBDICT.
+    /*---------------------------------------------------------------------*/
+    jam();
+    tuserblockref = DBDICT_REF;
+  }
+  else if (errNo < 8000)
+  {
+    /*---------------------------------------------------------------------*/
+    // Insert errors into DBDIH.
+    /*--------------------------------------------------------------------*/
+    jam();
+    tuserblockref = DBDIH_REF;
+  }
+  else if (errNo < 9000)
+  {
+    /*--------------------------------------------------------------------*/
+    // Insert errors into DBTC.
+    /*--------------------------------------------------------------------*/
+    jam();
+    tuserblockref = DBTC_REF;
+  }
+  else if (errNo < 10000)
+  {
+    /*--------------------------------------------------------------------*/
+    // Insert errors into CMVMI.
+    /*--------------------------------------------------------------------*/
+    jam();
+    tuserblockref = CMVMI_REF;
+  }
+  else if (errNo < 11000)
+  {
+    jam();
+    tuserblockref = BACKUP_REF;
+  }
+  else if (errNo < 12000)
+  {
+    // DBUTIL_REF ?
+    jam();
+  }
+  else if (errNo < 13000)
+  {
+    jam();
+    tuserblockref = DBTUX_REF;
+  }
+  else if (errNo < 14000)
+  {
+    jam();
+    tuserblockref = SUMA_REF;
+  }
+  else if (errNo < 15000)
+  {
+    jam();
+    tuserblockref = DBDICT_REF;
+  }
+  else if (errNo < 16000)
+  {
+    jam();
+    tuserblockref = LGMAN_REF;
+  }
+  else if (errNo < 17000)
+  {
+    jam();
+    tuserblockref = TSMAN_REF;
+  }
+  else if (errNo < 30000)
+  {
+    /*--------------------------------------------------------------------*/
+    // Ignore errors in the 20000-range.
+    /*--------------------------------------------------------------------*/
+    jam();
+    return;
+  }
+  else if (errNo < 40000)
+  {
+    jam();
+    /*--------------------------------------------------------------------*/
+    // Redirect errors to master DIH in the 30000-range.
+    /*--------------------------------------------------------------------*/
+
+    /**
+     * since CMVMI doesnt keep track of master,
+     * send to local DIH
+     */
+    signal->theData[0] = 5;
+    signal->theData[1] = errNo;
+    signal->theData[2] = 0;
+    sendSignal(DBDIH_REF, GSN_DIHNDBTAMPER, signal, 3, JBB);
+    return;
+  }
+  else if (errNo < 50000)
+  {
+    jam();
+
+    /**
+     * since CMVMI doesnt keep track of master,
+     * send to local DIH
+     */
+    signal->theData[0] = 5;
+    signal->theData[1] = errNo;
+    signal->theData[2] = 0;
+    sendSignal(DBDIH_REF, GSN_DIHNDBTAMPER, signal, 3, JBB);
+    return;
+  }
+
+  ndbassert(tuserblockref != 0); // mapping missing ??
+  if (tuserblockref != 0)
+  {
+    signal->theData[0] = errNo;
+    sendSignal(tuserblockref, GSN_NDB_TAMPER, signal, 1, JBB);
+  }
+#endif
 }//execTAMPER_ORD()
 
 #ifdef VM_TRACE
@@ -972,6 +1460,7 @@ recurse(char * buf, int loops, int arg){
 void
 Cmvmi::execDUMP_STATE_ORD(Signal* signal)
 {
+  jamEntry();
   Uint32 val = signal->theData[0];
   if (val >= DumpStateOrd::OneBlockOnly)
   {
@@ -989,24 +1478,10 @@ Cmvmi::execDUMP_STATE_ORD(Signal* signal)
     return;
   }
 
-  sendSignal(QMGR_REF, GSN_DUMP_STATE_ORD,    signal, signal->length(), JBB);
-  sendSignal(NDBCNTR_REF, GSN_DUMP_STATE_ORD, signal, signal->length(), JBB);
-  sendSignal(DBTC_REF, GSN_DUMP_STATE_ORD,    signal, signal->length(), JBB);
-  sendSignal(DBDIH_REF, GSN_DUMP_STATE_ORD,   signal, signal->length(), JBB);
-  sendSignal(DBDICT_REF, GSN_DUMP_STATE_ORD,  signal, signal->length(), JBB);
-  sendSignal(DBLQH_REF, GSN_DUMP_STATE_ORD,   signal, signal->length(), JBB);
-  sendSignal(DBTUP_REF, GSN_DUMP_STATE_ORD,   signal, signal->length(), JBB);
-  sendSignal(DBACC_REF, GSN_DUMP_STATE_ORD,   signal, signal->length(), JBB);
-  sendSignal(NDBFS_REF, GSN_DUMP_STATE_ORD,   signal, signal->length(), JBB);
-  sendSignal(BACKUP_REF, GSN_DUMP_STATE_ORD,  signal, signal->length(), JBB);
-  sendSignal(DBUTIL_REF, GSN_DUMP_STATE_ORD,  signal, signal->length(), JBB);
-  sendSignal(SUMA_REF, GSN_DUMP_STATE_ORD,    signal, signal->length(), JBB);
-  sendSignal(TRIX_REF, GSN_DUMP_STATE_ORD,    signal, signal->length(), JBB);
-  sendSignal(DBTUX_REF, GSN_DUMP_STATE_ORD,   signal, signal->length(), JBB);
-  sendSignal(LGMAN_REF, GSN_DUMP_STATE_ORD,   signal, signal->length(), JBB);
-  sendSignal(TSMAN_REF, GSN_DUMP_STATE_ORD,   signal, signal->length(), JBB);
-  sendSignal(PGMAN_REF, GSN_DUMP_STATE_ORD,   signal, signal->length(), JBB);
-  sendSignal(DBINFO_REF,GSN_DUMP_STATE_ORD,   signal, signal->length(), JBB);
+  for (Uint32 i = 0; blocks[i] != 0; i++)
+  {
+    sendSignal(blocks[i], GSN_DUMP_STATE_ORD, signal, signal->length(), JBB);
+  }
 
   /**
    *
@@ -1083,31 +1558,41 @@ Cmvmi::execDUMP_STATE_ORD(Signal* signal)
 	      g_sectionSegmentPool.getNoOfFree());
   }
 
-  if (dumpState->args[0] == 1000)
+  if (dumpState->args[0] == DumpStateOrd::DumpPageMemory)
   {
-    Uint32 len = signal->getLength();
-    if (signal->getLength() == 1)
+    const Uint32 len = signal->getLength();
+    if (len == 1)
     {
+      // Start dumping resource limits
       signal->theData[1] = 0;
       signal->theData[2] = ~0;
       sendSignal(reference(), GSN_DUMP_STATE_ORD, signal, 3, JBB);
 
+      // Dump data and index memory
       reportDMUsage(signal, 0);
+      reportIMUsage(signal, 0);
       return;
     }
-    Uint32 id = signal->theData[1];
-    Resource_limit rl;
-    if (!m_ctx.m_mm.get_resource_limit(id, rl))
-      len = 2;
-    else
+
+    if (len == 2)
     {
-      if (rl.m_min || rl.m_curr || rl.m_max)
-	infoEvent("Resource %d min: %d max: %d curr: %d",
-		  id, rl.m_min, rl.m_max, rl.m_curr);
+      // Dump data and index memory to specific ref
+      Uint32 result_ref = signal->theData[1];
+      reportDMUsage(signal, 0, result_ref);
+      reportIMUsage(signal, 0, result_ref);
+      return;
     }
 
-    if (len == 3)
+    Uint32 id = signal->theData[1];
+    Resource_limit rl;
+    if (m_ctx.m_mm.get_resource_limit(id, rl))
     {
+      if (rl.m_min || rl.m_curr || rl.m_max)
+      {
+        infoEvent("Resource %d min: %d max: %d curr: %d",
+                  id, rl.m_min, rl.m_max, rl.m_curr);
+      }
+
       signal->theData[0] = 1000;
       signal->theData[1] = id+1;
       signal->theData[2] = ~0;
@@ -1190,6 +1675,42 @@ Cmvmi::execDUMP_STATE_ORD(Signal* signal)
     sendSignal(reference(), GSN_TESTSIG, signal, 8, JBB, ptr, 2);
   }
 
+  if (arg == DumpStateOrd::DumpEventLog)
+  {
+    Uint32 result_ref = signal->theData[1];
+    m_saved_event_buffer.startScan();
+    SavedEvent s;
+    Uint32 cnt = 0;
+    EventReport * rep = CAST_PTR(EventReport, signal->getDataPtrSend());
+    rep->setEventType(NDB_LE_SavedEvent);
+    rep->setNodeId(getOwnNodeId());
+    while (m_saved_event_buffer.scan(&s, 0))
+    {
+      jam();
+      cnt++;
+      signal->theData[1] = s.m_len;
+      signal->theData[2] = s.m_seq;
+      signal->theData[3] = s.m_time;
+      if (s.m_len <= 21)
+      {
+        jam();
+        memcpy(signal->theData+4, s.m_data, 4*s.m_len);
+        sendSignal(result_ref, GSN_EVENT_REP, signal, 4 + s.m_len, JBB);
+      }
+      else
+      {
+        jam();
+        LinearSectionPtr ptr[3];
+        ptr[0].p = s.m_data;
+        ptr[0].sz = s.m_len;
+        sendSignal(result_ref, GSN_EVENT_REP, signal, 4, JBB, ptr, 1);
+      }
+    }
+    signal->theData[1] = 0; // end of stream
+    sendSignal(result_ref, GSN_EVENT_REP, signal, 2, JBB);
+    return;
+  }
+
   if (arg == DumpStateOrd::CmvmiTestLongSig)
   {
     /* Forward as GSN_TESTSIG to self */
@@ -1299,6 +1820,162 @@ Cmvmi::execDUMP_STATE_ORD(Signal* signal)
   }
 }//Cmvmi::execDUMP_STATE_ORD()
 
+
+void Cmvmi::execDBINFO_SCANREQ(Signal *signal)
+{
+  DbinfoScanReq req= *(DbinfoScanReq*)signal->theData;
+  const Ndbinfo::ScanCursor* cursor = 
+    CAST_CONSTPTR(Ndbinfo::ScanCursor, DbinfoScan::getCursorPtr(&req));
+  Ndbinfo::Ratelimit rl;
+
+  jamEntry();
+
+  switch(req.tableId){
+  case Ndbinfo::TRANSPORTERS_TABLEID:
+  {
+    jam();
+    Uint32 rnode = cursor->data[0];
+    if (rnode == 0)
+      rnode++; // Skip node 0
+
+    while(rnode < MAX_NODES)
+    {
+      switch(getNodeInfo(rnode).m_type)
+      {
+      default:
+      {
+        jam();
+        Ndbinfo::Row row(signal, req);
+        row.write_uint32(getOwnNodeId()); // Node id
+        row.write_uint32(rnode); // Remote node id
+        row.write_uint32(globalTransporterRegistry.getPerformState(rnode)); // State
+        ndbinfo_send_row(signal, req, row, rl);
+       break;
+      }
+
+      case NodeInfo::INVALID:
+        jam();
+       break;
+      }
+
+      rnode++;
+      if (rl.need_break(req))
+      {
+        jam();
+        ndbinfo_send_scan_break(signal, req, rl, rnode);
+        return;
+      }
+    }
+    break;
+  }
+
+  case Ndbinfo::RESOURCES_TABLEID:
+  {
+    jam();
+    Uint32 resource_id = cursor->data[0];
+    Resource_limit resource_limit;
+
+    while(m_ctx.m_mm.get_resource_limit(resource_id, resource_limit))
+    {
+      jam();
+      Ndbinfo::Row row(signal, req);
+      row.write_uint32(getOwnNodeId()); // Node id
+      row.write_uint32(resource_id);
+
+      row.write_uint32(resource_limit.m_min);
+      row.write_uint32(resource_limit.m_curr);
+      row.write_uint32(resource_limit.m_max);
+      row.write_uint32(0); //TODO
+      ndbinfo_send_row(signal, req, row, rl);
+      resource_id++;
+
+      if (rl.need_break(req))
+      {
+        jam();
+        ndbinfo_send_scan_break(signal, req, rl, resource_id);
+        return;
+      }
+    }
+    break;
+  }
+
+  case Ndbinfo::NODES_TABLEID:
+  {
+    jam();
+    const NodeState& nodeState = getNodeState();
+    const Uint32 start_level = nodeState.startLevel;
+    const NDB_TICKS uptime = (NdbTick_CurrentMillisecond()/1000) - m_start_time;
+
+    Ndbinfo::Row row(signal, req);
+    row.write_uint32(getOwnNodeId()); // Node id
+
+    row.write_uint64(uptime);         // seconds
+    row.write_uint32(start_level);
+    row.write_uint32(start_level == NodeState::SL_STARTING ?
+                     nodeState.starting.startPhase : 0);
+    ndbinfo_send_row(signal, req, row, rl);
+    break;
+  }
+
+  case Ndbinfo::POOLS_TABLEID:
+  {
+    jam();
+
+    Resource_limit res_limit;
+    m_ctx.m_mm.get_resource_limit(RG_DATAMEM, res_limit);
+
+    const Uint32 tup_pages_used = res_limit.m_curr - f_accpages;
+    const Uint32 tup_pages_total = res_limit.m_min - f_accpages;
+
+    Ndbinfo::pool_entry pools[] =
+    {
+      { "Data memory",
+        tup_pages_used,
+        tup_pages_total,
+        sizeof(GlobalPage),
+        0,
+        { CFG_DB_DATA_MEM,0,0,0 }},
+      { NULL, 0,0,0,0,{ 0,0,0,0 }}
+    };
+
+    static const size_t num_config_params =
+      sizeof(pools[0].config_params)/sizeof(pools[0].config_params[0]);
+    Uint32 pool = cursor->data[0];
+    BlockNumber bn = blockToMain(number());
+    while(pools[pool].poolname)
+    {
+      jam();
+      Ndbinfo::Row row(signal, req);
+      row.write_uint32(getOwnNodeId());
+      row.write_uint32(bn);           // block number
+      row.write_uint32(instance());   // block instance
+      row.write_string(pools[pool].poolname);
+
+      row.write_uint64(pools[pool].used);
+      row.write_uint64(pools[pool].total);
+      row.write_uint64(pools[pool].used_hi);
+      row.write_uint64(pools[pool].entry_size);
+      for (size_t i = 0; i < num_config_params; i++)
+        row.write_uint32(pools[pool].config_params[i]);
+      ndbinfo_send_row(signal, req, row, rl);
+      pool++;
+      if (rl.need_break(req))
+      {
+        jam();
+        ndbinfo_send_scan_break(signal, req, rl, pool);
+        return;
+      }
+    }
+  }
+
+  default:
+    break;
+  }
+
+  ndbinfo_send_scan_conf(signal, req, rl);
+}
+
+
 void
 Cmvmi::execNODE_START_REP(Signal* signal)
 {
@@ -2054,6 +2731,58 @@ Cmvmi::sendFragmentedComplete(Signal* signal, Uint32 data, Uint32 returnCode){
   }
 }
 
+
+static Uint32
+calc_percent(Uint32 used, Uint32 total)
+{
+  return (total ? (used * 100)/total : 0);
+}
+
+
+static Uint32
+sum_array(const Uint32 array[], unsigned sz)
+{
+  Uint32 sum = 0;
+  for (unsigned i = 0; i < sz; i++)
+    sum += array[i];
+  return sum;
+}
+
+
+/*
+  Check if any of the given thresholds has been
+  passed since last
+
+  Returns:
+   -1       no threshold passed
+   0 - 100  threshold passed
+*/
+
+static int
+check_threshold(Uint32 last, Uint32 now)
+{
+  assert(last <= 100 && now <= 100);
+
+  static const Uint32 thresholds[] = { 100, 99, 90, 80, 0 };
+
+  Uint32 passed;
+  for (size_t i = 0; i < NDB_ARRAY_SIZE(thresholds); i++)
+  {
+    if (now >= thresholds[i])
+    {
+      passed = thresholds[i];
+      break;
+    }
+  }
+  assert(passed <= 100);
+
+  if (passed == last)
+    return -1; // Already reported this level
+
+  return passed;
+}
+
+
 void
 Cmvmi::execCONTINUEB(Signal* signal)
 {
@@ -2062,36 +2791,54 @@ Cmvmi::execCONTINUEB(Signal* signal)
   {
     jam();
     Uint32 cnt = signal->theData[1];
-    Uint32 usedLast = signal->theData[2];
+    Uint32 tup_percent_last = signal->theData[2];
+    Uint32 acc_percent_last = signal->theData[3];
 
-    Resource_limit rl;
-    m_ctx.m_mm.get_resource_limit(RG_DATAMEM, rl);
-
-    Uint32 tmp = rl.m_min;
-    Uint32 now = tmp ? (rl.m_curr * 100)/tmp : 0;
-    static const Uint32 thresholds[] = { 100, 90, 80, 0 };
-    
-    Uint32 i = 0;
-    const Uint32 sz = sizeof(thresholds)/sizeof(thresholds[0]);
-    for(i = 0; i<sz; i++)
     {
-      if(now >= thresholds[i])
+      // Data memory threshold
+      Resource_limit rl;
+      m_ctx.m_mm.get_resource_limit(RG_DATAMEM, rl);
+
+      const Uint32 tup_pages_used = rl.m_curr - f_accpages;
+      const Uint32 tup_pages_total = rl.m_min - f_accpages;
+      const Uint32 tup_percent_now = calc_percent(tup_pages_used,
+                                                  tup_pages_total);
+
+      int passed;
+      if ((passed = check_threshold(tup_percent_last, tup_percent_now)) != -1)
       {
-	now = thresholds[i];
-	break;
+        jam();
+        reportDMUsage(signal, tup_percent_now >= tup_percent_last ? 1 : -1);
+        tup_percent_last = passed;
       }
     }
-    
-    if(now != usedLast || 
-       (c_memusage_report_frequency && cnt + 1 == c_memusage_report_frequency))
+
+    {
+      // Index memory threshold
+      const Uint32 acc_pages_used =
+        sum_array(g_acc_pages_used, NDB_ARRAY_SIZE(g_acc_pages_used));
+      const Uint32 acc_pages_total = f_accpages * 4;
+      const Uint32 acc_percent_now = calc_percent(acc_pages_used,
+                                                  acc_pages_total);
+
+      int passed;
+      if ((passed = check_threshold(acc_percent_last, acc_percent_now)) != -1)
+      {
+        jam();
+        reportIMUsage(signal, acc_percent_now >= acc_percent_last ? 1 : -1);
+        acc_percent_last = passed;
+      }
+    }
+
+    // Index and data memory report frequency
+    if(c_memusage_report_frequency &&
+       cnt + 1 == c_memusage_report_frequency)
     {
       jam();
-      reportDMUsage(signal, 
-                    now > usedLast ? 1 : 
-                    now < usedLast ? -1 : 0);
+      reportDMUsage(signal, 0);
+      reportIMUsage(signal, 0);
       cnt = 0;
-      usedLast = 0;
-    } 
+    }
     else
     {
       jam();
@@ -2099,26 +2846,46 @@ Cmvmi::execCONTINUEB(Signal* signal)
     }
     signal->theData[0] = ZREPORT_MEMORY_USAGE;
     signal->theData[1] = cnt; // seconds since last report
-    signal->theData[2] = usedLast; // last threshold
-    sendSignalWithDelay(reference(), GSN_CONTINUEB, signal, 1000, 3);
+    signal->theData[2] = tup_percent_last; // last reported threshold for TUP
+    signal->theData[3] = acc_percent_last; // last reported threshold for ACC
+    sendSignalWithDelay(reference(), GSN_CONTINUEB, signal, 1000, 4);
     return;
   }
   }
 }
 
 void
-Cmvmi::reportDMUsage(Signal* signal, int incDec)
+Cmvmi::reportDMUsage(Signal* signal, int incDec, BlockReference ref)
 {
   Resource_limit rl;
   m_ctx.m_mm.get_resource_limit(RG_DATAMEM, rl);
-  
+
+  const Uint32 tup_pages_used = rl.m_curr - f_accpages;
+  const Uint32 tup_pages_total = rl.m_min - f_accpages;
+
   signal->theData[0] = NDB_LE_MemoryUsage;
   signal->theData[1] = incDec;
   signal->theData[2] = sizeof(GlobalPage);
-  signal->theData[3] = rl.m_curr;
-  signal->theData[4] = rl.m_min;
+  signal->theData[3] = tup_pages_used;
+  signal->theData[4] = tup_pages_total;
   signal->theData[5] = DBTUP;
-  sendSignal(CMVMI_REF, GSN_EVENT_REP, signal, 6, JBB);
+  sendSignal(ref, GSN_EVENT_REP, signal, 6, JBB);
+}
+
+
+void
+Cmvmi::reportIMUsage(Signal* signal, int incDec, BlockReference ref)
+{
+  const Uint32 acc_pages_used =
+    sum_array(g_acc_pages_used, NDB_ARRAY_SIZE(g_acc_pages_used));
+
+  signal->theData[0] = NDB_LE_MemoryUsage;
+  signal->theData[1] = incDec;
+  signal->theData[2] = 8192;
+  signal->theData[3] = acc_pages_used;
+  signal->theData[4] = f_accpages * 4;
+  signal->theData[5] = DBACC;
+  sendSignal(ref, GSN_EVENT_REP, signal, 6, JBB);
 }
 
 
