@@ -1,3 +1,4 @@
+/** **/
 /*
    Copyright (C) 2003 MySQL AB
     All rights reserved. Use is subject to license terms.
@@ -154,7 +155,6 @@ ndbout << "Ptr: " << ptr.p->word32 << " \tIndex: " << tmp_string << " \tValue: "
 #define ZREL_ROOT_FRAG 5
 #define ZREL_FRAG 6
 #define ZREL_DIR 7
-#define ZREPORT_MEMORY_USAGE 8
 
 /* ------------------------------------------------------------------------- */
 /* ERROR CODES                                                               */
@@ -180,6 +180,7 @@ ndbout << "Ptr: " << ptr.p->word32 << " \tIndex: " << tmp_string << " \tValue: "
 #define ZWRITE_ERROR 630
 #define ZTO_OP_STATE_ERROR 631
 #define ZTOO_EARLY_ACCESS_ERROR 632
+#define ZDIR_RANGE_FULL_ERROR 633 // on fragment
 #endif
 
 class ElementHeader {
@@ -474,6 +475,11 @@ struct Fragmentrec {
 // flag to avoid accessing table record if no char attributes
 //-----------------------------------------------------------------------------
   Uint8 hasCharAttr;
+
+//-----------------------------------------------------------------------------
+// flag to mark that execEXPANDCHECK2 has failed due to DirRange full
+//-----------------------------------------------------------------------------
+  Uint8 dirRangeFull;
 };
 
   typedef Ptr<Fragmentrec> FragmentrecPtr;
@@ -846,7 +852,6 @@ private:
 
   void zpagesize_error(const char* where);
 
-  void reportMemoryUsage(Signal* signal, int gth);
   void reenable_expand_after_redo_log_exection_complete(Signal*);
 
   // charsets
@@ -856,6 +861,12 @@ private:
   void initData();
   void initRecords();
 
+#ifdef VM_TRACE
+  void debug_lh_vars(const char* where);
+#else
+  void debug_lh_vars(const char* where) {}
+#endif
+
   // Variables
 /* --------------------------------------------------------------------------------- */
 /* DIRECTORY RANGE                                                                   */
@@ -957,9 +968,10 @@ private:
   Page8Ptr slPageptr;
   Page8Ptr spPageptr;
   Uint32 cfirstfreepage;
-  Uint32 cfreepage;
   Uint32 cpagesize;
+  Uint32 cpageCount;
   Uint32 cnoOfAllocatedPages;
+  Uint32 cnoOfAllocatedPagesMax;
 /* --------------------------------------------------------------------------------- */
 /* ROOTFRAGMENTREC                                                                   */
 /*          DURING EXPAND FRAGMENT PROCESS, EACH FRAGMEND WILL BE EXPAND INTO TWO    */
