@@ -69,6 +69,7 @@
 // Error Codes for Add Table
 // --------------------------------------
 #define ZREPLERROR1 306
+#define ZREPLERROR2 307
 
 // --------------------------------------
 // Crash Codes
@@ -141,7 +142,8 @@ public:
       STARTED = 2,
       ALTER_TABLE = 3,
       ALTER_TABLE_ABORT = 4, // "local" abort
-      ALTER_TABLE_REVERT = 5
+      ALTER_TABLE_REVERT = 5,
+      GET_TABINFO = 6
     };
     union {
       Uint32 nodes[MAX_REPLICAS];
@@ -154,6 +156,9 @@ public:
       struct {
         Uint32 m_map_ptr_i;
       } m_create;
+      struct {
+        Uint32 m_requestInfo;
+      } m_get_tabinfo;
     };
     ConnectState connectState;
     Uint32 nextPool;
@@ -455,6 +460,7 @@ public:
       CS_INVALIDATE_NODE_LCP,
       CS_ALTER_TABLE,
       CS_COPY_TO_SAVE
+      ,CS_GET_TABINFO
     };
     /**
      * State for copying pages to disk
@@ -565,7 +571,10 @@ public:
       ,TO_CREATE_FRAG_COMMIT = 12  // Waiting for all (CREATE_FRAGREQ commit)
       ,TO_UPDATE_AFTER_COMMIT = 13 // Waiting for master (UPDATE_TOREQ)
 
-      ,TO_END_TO = 14               // Waiting for master (EBND_TOREQ)
+      ,TO_START_LOGGING = 14        // Enabling logging on all fragments
+      ,TO_SL_COPY_ACTIVE = 15       // Start logging: Copy active (local)
+      ,TO_SL_CREATE_FRAG = 16       // Start logging: Create Frag (dist)
+      ,TO_END_TO = 17               // Waiting for master (EBND_TOREQ)
     };
 
     /**
@@ -692,6 +701,7 @@ private:
   void execGCP_NODEFINISH(Signal *);
   void execGCP_COMMIT(Signal *);
   void execSUB_GCP_COMPLETE_REP(Signal *);
+  void execSUB_GCP_COMPLETE_ACK(Signal *);
   void execDIHNDBTAMPER(Signal *);
   void execCONTINUEB(Signal *);
   void execCOPY_GCIREQ(Signal *);
@@ -701,6 +711,8 @@ private:
   void execTCGETOPSIZECONF(Signal *);
   void execTC_CLOPSIZECONF(Signal *);
 
+  void execDIH_GET_TABINFO_REQ(Signal*);
+
   int handle_invalid_lcp_no(const struct LcpFragRep*, ReplicaRecordPtr);
   void execLCP_FRAG_REP(Signal *);
   void execLCP_COMPLETE_REP(Signal *);
@@ -815,6 +827,7 @@ private:
   void sendGCP_COMMIT(Signal *, Uint32 nodeId, Uint32);
   void sendGCP_PREPARE(Signal *, Uint32 nodeId, Uint32);
   void sendGCP_SAVEREQ(Signal *, Uint32 nodeId, Uint32);
+  void sendSUB_GCP_COMPLETE_REP(Signal*, Uint32 nodeId, Uint32);
   void sendINCL_NODEREQ(Signal *, Uint32 nodeId, Uint32);
   void sendMASTER_GCPREQ(Signal *, Uint32 nodeId, Uint32);
   void sendMASTER_LCPREQ(Signal *, Uint32 nodeId, Uint32);
@@ -857,6 +870,7 @@ private:
   void addTable_closeConf(Signal* signal, Uint32 tabPtrI);
   void resetReplicaSr(TabRecordPtr tabPtr);
   void resetReplicaLcp(ReplicaRecord * replicaP, Uint32 stopGci);
+  void resetReplica(Ptr<ReplicaRecord>);
 
 //------------------------------------
 // Methods for LCP functionality
@@ -963,7 +977,6 @@ private:
   void copyTabReq_complete(Signal* signal, TabRecordPtr tabPtr);
 
   void gcpcommitreqLab(Signal *);
-  void gcpsavereqLab(Signal *);
   void copyGciLab(Signal *, CopyGCIReq::CopyReason reason);
   void storeNewLcpIdLab(Signal *);
   void startLcpRoundLoopLab(Signal *, Uint32 startTableId, Uint32 startFragId);
@@ -978,6 +991,8 @@ private:
   void checkLocalNodefailComplete(Signal*, Uint32 failedNodeId,
 				  NodefailHandlingStep step);
   
+  Callback m_sendSTTORRY;
+  void sendSTTORRY(Signal*, Uint32 senderData = 0, Uint32 retVal = 0);
   void ndbsttorry10Lab(Signal *, Uint32 _line);
   void createMutexes(Signal* signal, Uint32 no);
   void createMutex_done(Signal* signal, Uint32 no, Uint32 retVal);
@@ -1087,11 +1102,12 @@ private:
                     Uint32 tfstStopGci,
                     Uint32& tfstStartGci,
                     Uint32& tfstLcp);
-  void newCrashedReplica(Uint32 nodeId, ReplicaRecordPtr ncrReplicaPtr);
+  void newCrashedReplica(ReplicaRecordPtr ncrReplicaPtr);
   void packCrashedReplicas(ReplicaRecordPtr pcrReplicaPtr);
   void releaseReplicas(Uint32 * replicaPtr);
-  void removeOldCrashedReplicas(ReplicaRecordPtr rocReplicaPtr);
-  void removeTooNewCrashedReplicas(ReplicaRecordPtr rtnReplicaPtr);
+  void removeOldCrashedReplicas(Uint32, Uint32, ReplicaRecordPtr rocReplicaPtr);
+  void removeTooNewCrashedReplicas(ReplicaRecordPtr rtnReplicaPtr, Uint32 lastCompletedGCI);
+  void mergeCrashedReplicas(ReplicaRecordPtr pcrReplicaPtr);
   void seizeReplicaRec(ReplicaRecordPtr& replicaPtr);
 
 //------------------------------------
@@ -1227,7 +1243,14 @@ private:
   void nr_start_fragments(Signal*, TakeOverRecordPtr);
   void nr_start_fragment(Signal*, TakeOverRecordPtr, ReplicaRecordPtr);
   void nr_run_redo(Signal*, TakeOverRecordPtr);
-  
+  void nr_start_logging(Signal*, TakeOverRecordPtr);
+
+  void getTabInfo(Signal*);
+  void getTabInfo_send(Signal*, TabRecordPtr);
+  void getTabInfo_sendComplete(Signal*, Uint32, Uint32);
+  int getTabInfo_copyTableToSection(SegmentedSectionPtr & ptr, CopyTableNode);
+  int getTabInfo_copySectionToPages(TabRecordPtr, SegmentedSectionPtr);
+
   // Initialisation
   void initData();
   void initRecords();
@@ -1332,7 +1355,8 @@ private:
       M_GCP_IDLE      = 0,
       M_GCP_PREPARE   = 1,
       M_GCP_COMMIT    = 2,
-      M_GCP_COMMITTED = 3
+      M_GCP_COMMITTED = 3,
+      M_GCP_COMPLETE  = 4
     } m_state;
 
     struct {
@@ -1523,18 +1547,6 @@ private:
   Uint32 c_set_initial_start_flag;
   Uint64 c_current_time; // Updated approx. every 10ms
 
-  enum GcpMasterTakeOverState {
-    GMTOS_IDLE = 0,
-    GMTOS_INITIAL = 1,
-    ALL_READY = 2,
-    ALL_PREPARED = 3,
-    COMMIT_STARTED_NOT_COMPLETED = 4,
-    COMMIT_COMPLETED = 5,
-    PREPARE_STARTED_NOT_COMMITTED = 6,
-    SAVE_STARTED_NOT_COMPLETED = 7
-  };
-  GcpMasterTakeOverState cgcpMasterTakeOverState;
-
 public:
   enum LcpMasterTakeOverState {
     LMTOS_IDLE = 0,
@@ -1611,6 +1623,7 @@ private:
   SignalCounter c_GCP_COMMIT_Counter;
   SignalCounter c_GCP_PREPARE_Counter;
   SignalCounter c_GCP_SAVEREQ_Counter;
+  SignalCounter c_SUB_GCP_COMPLETE_REP_Counter;
   SignalCounter c_INCL_NODEREQ_Counter;
   SignalCounter c_MASTER_GCPREQ_Counter;
   SignalCounter c_MASTER_LCPREQ_Counter;
