@@ -1,4 +1,5 @@
-/* Copyright (C) 2007 MySQL AB
+/* Copyright (C) 2007-2008 MySQL AB, 2009-2010 Sun Microsystems Inc.
+   All rights reserved. Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,10 +19,6 @@
 
 #include <SimulatedBlock.hpp>
 
-#include <NodeBitmask.hpp>
-#include <SLList.hpp>
-
-
 class Dbinfo : public SimulatedBlock
 {
 public:
@@ -30,31 +27,16 @@ public:
   BLOCK_DEFINES(Dbinfo);
 
 protected:
-  struct Node {
-    Uint32 nodeId;
-    Uint32 alive;
-    Uint32 nextList;
-    union { Uint32 prevList; Uint32 nextPool; };
-  };
-  typedef Ptr<Node> NodePtr;
-
-  NodeId c_masterNodeId;
-  ArrayPool<Node> c_nodePool;
-  SLList<Node> c_nodes;
-  NdbNodeBitmask c_aliveNodes;
-
-
   void execSTTOR(Signal* signal);
   void sendSTTORRY(Signal*);
   void execREAD_CONFIG_REQ(Signal*);
   void execDUMP_STATE_ORD(Signal* signal);
 
+  Uint32 find_next_block(Uint32 block) const;
+  bool find_next(Ndbinfo::ScanCursor* cursor) const;
   void execDBINFO_SCANREQ(Signal *signal);
-  void execDBINFO_TRANSID_AI(Signal* signal);
   void execDBINFO_SCANCONF(Signal *signal);
 
-  /* for maintaining c_aliveNodes */
-  void execREAD_NODESCONF(Signal* signal);
   void execINCL_NODEREQ(Signal* signal);
   void execNODE_FAILREP(Signal* signal);
 };
