@@ -18,6 +18,7 @@
 #include "DblqhCommon.hpp"
 
 #include <signaldata/StartFragReq.hpp>
+#include <signaldata/ExecFragReq.hpp>
 
 DblqhProxy::DblqhProxy(Block_context& ctx) :
   LocalProxy(DBLQH, ctx),
@@ -88,6 +89,10 @@ DblqhProxy::DblqhProxy(Block_context& ctx) :
   addRecSignal(GSN_EXEC_SRREQ, &DblqhProxy::execEXEC_SRREQ);
   addRecSignal(GSN_EXEC_SRCONF, &DblqhProxy::execEXEC_SRCONF);
 
+  // GSN_EXEC_FRAG
+  addRecSignal(GSN_EXEC_FRAGREQ, &DblqhProxy::execEXEC_FRAGREQ);
+  addRecSignal(GSN_EXEC_FRAGCONF, &DblqhProxy::execEXEC_FRAGCONF);
+
   // GSN_DROP_FRAG_REQ
   addRecSignal(GSN_DROP_FRAG_REQ, &DblqhProxy::execDROP_FRAG_REQ);
   addRecSignal(GSN_DROP_FRAG_CONF, &DblqhProxy::execDROP_FRAG_CONF);
@@ -162,7 +167,8 @@ DblqhProxy::execCREATE_TAB_REQ(Signal* signal)
 }
 
 void
-DblqhProxy::sendCREATE_TAB_REQ(Signal* signal, Uint32 ssId)
+DblqhProxy::sendCREATE_TAB_REQ(Signal* signal, Uint32 ssId,
+                               SectionHandle* handle)
 {
   Ss_CREATE_TAB_REQ& ss = ssFind<Ss_CREATE_TAB_REQ>(ssId);
 
@@ -170,8 +176,8 @@ DblqhProxy::sendCREATE_TAB_REQ(Signal* signal, Uint32 ssId)
   *req = ss.m_req;
   req->senderRef = reference();
   req->senderData = ssId;
-  sendSignal(workerRef(ss.m_worker), GSN_CREATE_TAB_REQ,
-             signal, CreateTabReq::SignalLengthLDM, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_CREATE_TAB_REQ,
+                      signal, CreateTabReq::SignalLengthLDM, JBB, handle);
 }
 
 void
@@ -263,11 +269,17 @@ DblqhProxy::execLQHADDATTREQ(Signal* signal)
   ndbrequire(ss_main.m_req.noOfAttributes >= req->noOfAttributes);
   ss_main.m_req.noOfAttributes -= req->noOfAttributes;
 
+  /* Save long section(s) in ss for forwarding to 
+   * workers
+   */
+  SectionHandle handle(this, signal);
+  saveSections(ss, handle);
+  
   sendREQ(signal, ss);
 }
 
 void
-DblqhProxy::sendLQHADDATTREQ(Signal* signal, Uint32 ssId)
+DblqhProxy::sendLQHADDATTREQ(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
   Ss_LQHADDATTREQ& ss = ssFind<Ss_LQHADDATTREQ>(ssId);
   Ss_CREATE_TAB_REQ& ss_main = ssFind<Ss_CREATE_TAB_REQ>(ssId);
@@ -279,8 +291,8 @@ DblqhProxy::sendLQHADDATTREQ(Signal* signal, Uint32 ssId)
   req->noOfAttributes = ss.m_req.noOfAttributes;
   req->senderData = ssId;
   req->senderAttrPtr = ss.m_req.senderAttrPtr;
-  sendSignal(workerRef(ss.m_worker), GSN_LQHADDATTREQ,
-             signal, reqlength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_LQHADDATTREQ,
+                      signal, reqlength, JBB, handle);
 }
 
 void
@@ -370,7 +382,8 @@ DblqhProxy::execTAB_COMMITREQ(Signal* signal)
 }
 
 void
-DblqhProxy::sendTAB_COMMITREQ(Signal* signal, Uint32 ssId)
+DblqhProxy::sendTAB_COMMITREQ(Signal* signal, Uint32 ssId,
+                              SectionHandle* handle)
 {
   Ss_TAB_COMMITREQ& ss = ssFind<Ss_TAB_COMMITREQ>(ssId);
 
@@ -378,8 +391,8 @@ DblqhProxy::sendTAB_COMMITREQ(Signal* signal, Uint32 ssId)
   req->senderRef = reference();
   req->senderData = ssId;
   req->tableId = ss.m_req.tableId;
-  sendSignal(workerRef(ss.m_worker), GSN_TAB_COMMITREQ,
-             signal, TabCommitReq::SignalLength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_TAB_COMMITREQ,
+                      signal, TabCommitReq::SignalLength, JBB, handle);
 }
 
 void
@@ -817,7 +830,7 @@ DblqhProxy::execGCP_SAVEREQ(Signal* signal)
 }
 
 void
-DblqhProxy::sendGCP_SAVEREQ(Signal* signal, Uint32 ssId)
+DblqhProxy::sendGCP_SAVEREQ(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
   Ss_GCP_SAVEREQ& ss = ssFind<Ss_GCP_SAVEREQ>(ssId);
 
@@ -826,8 +839,8 @@ DblqhProxy::sendGCP_SAVEREQ(Signal* signal, Uint32 ssId)
 
   req->dihBlockRef = reference();
   req->dihPtr = ss.m_worker;
-  sendSignal(workerRef(ss.m_worker), GSN_GCP_SAVEREQ,
-             signal, GCPSaveReq::SignalLength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_GCP_SAVEREQ,
+                      signal, GCPSaveReq::SignalLength, JBB, handle);
 }
 
 void
@@ -909,7 +922,8 @@ DblqhProxy::execPREP_DROP_TAB_REQ(Signal* signal)
 }
 
 void
-DblqhProxy::sendPREP_DROP_TAB_REQ(Signal* signal, Uint32 ssId)
+DblqhProxy::sendPREP_DROP_TAB_REQ(Signal* signal, Uint32 ssId,
+                                  SectionHandle * handle)
 {
   Ss_PREP_DROP_TAB_REQ& ss = ssFind<Ss_PREP_DROP_TAB_REQ>(ssId);
 
@@ -917,8 +931,8 @@ DblqhProxy::sendPREP_DROP_TAB_REQ(Signal* signal, Uint32 ssId)
   *req = ss.m_req;
   req->senderRef = reference();
   req->senderData = ssId; // redundant since tableId is used
-  sendSignal(workerRef(ss.m_worker), GSN_PREP_DROP_TAB_REQ,
-             signal, PrepDropTabReq::SignalLength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_PREP_DROP_TAB_REQ,
+                      signal, PrepDropTabReq::SignalLength, JBB, handle);
 }
 
 void
@@ -988,7 +1002,7 @@ DblqhProxy::execDROP_TAB_REQ(Signal* signal)
 }
 
 void
-DblqhProxy::sendDROP_TAB_REQ(Signal* signal, Uint32 ssId)
+DblqhProxy::sendDROP_TAB_REQ(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
   Ss_DROP_TAB_REQ& ss = ssFind<Ss_DROP_TAB_REQ>(ssId);
 
@@ -996,8 +1010,8 @@ DblqhProxy::sendDROP_TAB_REQ(Signal* signal, Uint32 ssId)
   *req = ss.m_req;
   req->senderRef = reference();
   req->senderData = ssId; // redundant since tableId is used
-  sendSignal(workerRef(ss.m_worker), GSN_DROP_TAB_REQ,
-             signal, DropTabReq::SignalLength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_DROP_TAB_REQ,
+                      signal, DropTabReq::SignalLength, JBB, handle);
 }
 
 void
@@ -1060,29 +1074,27 @@ DblqhProxy::sendDROP_TAB_CONF(Signal* signal, Uint32 ssId)
 void
 DblqhProxy::execALTER_TAB_REQ(Signal* signal)
 {
+  jamEntry();
+  if (!assembleFragments(signal))
+  {
+    jam();
+    return;
+  }
   const AlterTabReq* req = (const AlterTabReq*)signal->getDataPtr();
   Uint32 ssId = getSsId(req);
   Ss_ALTER_TAB_REQ& ss = ssSeize<Ss_ALTER_TAB_REQ>(ssId);
   ss.m_req = *req;
   ndbrequire(signal->getLength() == AlterTabReq::SignalLength);
 
-  {
-    SectionHandle handle(this, signal);
-    ss.m_sections = handle.m_cnt;
-    ndbrequire(ss.m_sections <= 1);
-    if (ss.m_sections >= 1) {
-      ss.m_sz0 = handle.m_ptr[0].p->m_sz;
-      ndbrequire(ss.m_sz0 <= ss.MaxSection0);
-      ::copy(ss.m_section0, handle.m_ptr[0]);
-    }
-    releaseSections(handle);
-  }
+  SectionHandle handle(this, signal);
+  saveSections(ss, handle);
 
   sendREQ(signal, ss);
 }
 
 void
-DblqhProxy::sendALTER_TAB_REQ(Signal* signal, Uint32 ssId)
+DblqhProxy::sendALTER_TAB_REQ(Signal* signal, Uint32 ssId,
+                              SectionHandle* handle)
 {
   Ss_ALTER_TAB_REQ& ss = ssFind<Ss_ALTER_TAB_REQ>(ssId);
 
@@ -1090,18 +1102,8 @@ DblqhProxy::sendALTER_TAB_REQ(Signal* signal, Uint32 ssId)
   *req = ss.m_req;
   req->senderRef = reference();
   req->senderData = ssId;
-  if (ss.m_sections == 0) {
-    jam();
-    sendSignal(workerRef(ss.m_worker), GSN_ALTER_TAB_REQ,
-               signal, AlterTabReq::SignalLength, JBB);
-  } else {
-    jam();
-    LinearSectionPtr ptr[3];
-    ptr[0].sz = ss.m_sz0;
-    ptr[0].p = ss.m_section0;
-    sendSignal(workerRef(ss.m_worker), GSN_ALTER_TAB_REQ,
-               signal, AlterTabReq::SignalLength, JBB, ptr, 1);
-  }
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_ALTER_TAB_REQ,
+                      signal, AlterTabReq::SignalLength, JBB, handle);
 }
 
 void
@@ -1197,7 +1199,7 @@ DblqhProxy::execSTART_RECREQ(Signal* signal)
 }
 
 void
-DblqhProxy::sendSTART_RECREQ(Signal* signal, Uint32 ssId)
+DblqhProxy::sendSTART_RECREQ(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
   Ss_START_RECREQ& ss = ssFind<Ss_START_RECREQ>(ssId);
 
@@ -1206,8 +1208,8 @@ DblqhProxy::sendSTART_RECREQ(Signal* signal, Uint32 ssId)
 
   req->senderRef = reference();
   req->senderData = ssId;
-  sendSignal(workerRef(ss.m_worker), GSN_START_RECREQ,
-             signal, StartRecReq::SignalLength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_START_RECREQ,
+                      signal, StartRecReq::SignalLength, JBB, handle);
 }
 
 void
@@ -1283,7 +1285,18 @@ DblqhProxy::sendSTART_RECREQ_2(Signal* signal, Uint32 ssId)
   if (firstReply(ss)) {
     ss.m_req = *req;
   } else {
-    // ndbrequire(ss.m_req.lcpId == req->lcpId); wl4391_todo
+    jam();
+    /*
+     * Fragments can be started from different lcpId's.  LGMAN must run
+     * UNDO until lowest lcpId.  Each DBLQH instance computes the lowest
+     * lcpId in START_FRAGREQ.  In MT case the proxy further computes
+     * the lowest of the lcpId's from worker instances.
+     */
+    if (req->lcpId < ss.m_req.lcpId)
+    {
+      jam();
+      ss.m_req.lcpId = req->lcpId;
+    }
     ndbrequire(ss.m_req.proxyBlockNo == req->proxyBlockNo);
   }
 
@@ -1295,8 +1308,8 @@ DblqhProxy::sendSTART_RECREQ_2(Signal* signal, Uint32 ssId)
       (Ss_START_RECREQ_2::Req*)signal->getDataPtrSend();
     *req = ss.m_req;
     BlockReference ref = numberToRef(req->proxyBlockNo, getOwnNodeId());
-    sendSignal(ref, GSN_START_RECREQ,
-               signal, Ss_START_RECREQ_2::Req::SignalLength, JBB);
+    sendSignal(ref, GSN_START_RECREQ, signal,
+               Ss_START_RECREQ_2::Req::SignalLength, JBB);
   }
 }
 
@@ -1316,15 +1329,16 @@ DblqhProxy::execSTART_RECCONF_2(Signal* signal)
 }
 
 void
-DblqhProxy::sendSTART_RECCONF_2(Signal* signal, Uint32 ssId)
+DblqhProxy::sendSTART_RECCONF_2(Signal* signal, Uint32 ssId,
+                                SectionHandle* handle)
 {
   Ss_START_RECREQ_2& ss = ssFind<Ss_START_RECREQ_2>(ssId);
 
   Ss_START_RECREQ_2::Conf* conf =
     (Ss_START_RECREQ_2::Conf*)signal->getDataPtrSend();
   *conf = ss.m_conf;
-  sendSignal(workerRef(ss.m_worker), GSN_START_RECCONF,
-             signal, Ss_START_RECREQ_2::Conf::SignalLength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_START_RECCONF, signal,
+                      Ss_START_RECREQ_2::Conf::SignalLength, JBB, handle);
 }
 
 // GSN_LQH_TRANSREQ
@@ -1363,7 +1377,7 @@ DblqhProxy::execLQH_TRANSREQ(Signal* signal)
 }
 
 void
-DblqhProxy::sendLQH_TRANSREQ(Signal* signal, Uint32 ssId)
+DblqhProxy::sendLQH_TRANSREQ(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
   Ss_LQH_TRANSREQ& ss = ssFind<Ss_LQH_TRANSREQ>(ssId);
 
@@ -1372,8 +1386,8 @@ DblqhProxy::sendLQH_TRANSREQ(Signal* signal, Uint32 ssId)
 
   req->senderData = ssId;
   req->senderRef = reference();
-  sendSignal(workerRef(ss.m_worker), GSN_LQH_TRANSREQ,
-             signal, LqhTransReq::SignalLength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_LQH_TRANSREQ,
+                      signal, LqhTransReq::SignalLength, JBB, handle);
 }
 
 void
@@ -1529,11 +1543,11 @@ DblqhProxy::execEXEC_SR_1(Signal* signal, GlobalSignalNumber gsn)
 }
 
 void
-DblqhProxy::sendEXEC_SR_1(Signal* signal, Uint32 ssId)
+DblqhProxy::sendEXEC_SR_1(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
   Ss_EXEC_SR_1& ss = ssFind<Ss_EXEC_SR_1>(ssId);
   signal->theData[0] = ss.m_sig.nodeId;
-  sendSignal(workerRef(ss.m_worker), ss.m_gsn, signal, 1, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), ss.m_gsn, signal, 1, JBB, handle);
 }
 
 // GSN_EXEC_SRREQ_2 [ fictional gsn ]
@@ -1590,6 +1604,55 @@ DblqhProxy::sendEXEC_SR_2(Signal* signal, Uint32 ssId)
   ssRelease<Ss_EXEC_SR_2>(ssId);
 }
 
+// GSN_EXEC_FRAGREQ
+void
+DblqhProxy::execEXEC_FRAGREQ(Signal* signal)
+{
+  Uint32 ref = ((ExecFragReq*)signal->getDataPtr())->dst;
+
+  if (refToNode(ref) == getOwnNodeId())
+  {
+    jam();
+    sendSignal(ref, GSN_EXEC_FRAGREQ, signal, signal->getLength(), JBB);
+  }
+  else if (ndb_route_exec_frag(getNodeInfo(refToNode(ref)).m_version))
+  {
+    jam();
+    sendSignal(numberToRef(DBLQH, refToNode(ref)), GSN_EXEC_FRAGREQ, signal,
+               signal->getLength(), JBB);
+  }
+  else
+  {
+    jam();
+    sendSignal(ref, GSN_EXEC_FRAGREQ, signal,
+               signal->getLength(), JBB);
+  }
+}
+
+// GSN_EXEC_FRAGCONF
+void
+DblqhProxy::execEXEC_FRAGCONF(Signal* signal)
+{
+  Uint32 ref = signal->theData[1];
+
+  if (refToNode(ref) == getOwnNodeId())
+  {
+    jam();
+    sendSignal(ref, GSN_EXEC_FRAGCONF, signal, 1, JBB);
+  }
+  else if (ndb_route_exec_frag(getNodeInfo(refToNode(ref)).m_version))
+  {
+    jam();
+    sendSignal(numberToRef(DBLQH, refToNode(ref)), GSN_EXEC_FRAGCONF,
+               signal, 2, JBB);
+  }
+  else
+  {
+    jam();
+    sendSignal(ref, GSN_EXEC_FRAGCONF, signal, 2, JBB);
+  }
+}
+
 // GSN_DROP_FRAG_REQ
 
 void
@@ -1604,7 +1667,8 @@ DblqhProxy::execDROP_FRAG_REQ(Signal* signal)
 }
 
 void
-DblqhProxy::sendDROP_FRAG_REQ(Signal* signal, Uint32 ssId)
+DblqhProxy::sendDROP_FRAG_REQ(Signal* signal, Uint32 ssId,
+                              SectionHandle* handle)
 {
   Ss_DROP_FRAG_REQ& ss = ssFind<Ss_DROP_FRAG_REQ>(ssId);
 
@@ -1612,8 +1676,8 @@ DblqhProxy::sendDROP_FRAG_REQ(Signal* signal, Uint32 ssId)
   *req = ss.m_req;
   req->senderRef = reference();
   req->senderData = ssId;
-  sendSignal(workerRef(ss.m_worker), GSN_DROP_FRAG_REQ,
-             signal, DropFragReq::SignalLength, JBB);
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_DROP_FRAG_REQ,
+                      signal, DropFragReq::SignalLength, JBB, handle);
 }
 
 void
