@@ -138,6 +138,9 @@
 #define ZNOTUNIQUE 893
 
 #define ZINVALID_KEY 290
+#define ZUNLOCKED_IVAL_TOO_HIGH 294
+#define ZUNLOCKED_OP_HAS_BAD_STATE 295 
+#define ZBAD_DIST_KEY 298
 #endif
 
 class Dbtc: public SimulatedBlock {
@@ -515,7 +518,7 @@ public:
     /**
      * Index attribute list.  Only the length is used in v21x.
      */
-    AttributeList attributeList;
+    IndexAttributeList attributeList;
 
     /**
      * Primary table id, the primary table to be indexed
@@ -763,6 +766,7 @@ public:
     UintR clientData;
     Uint32 errorData;
     UintR attrInfoLen;
+    Uint32 immediateTriggerId;  // Id of trigger op being fired NOW
     
     UintR accumulatingIndexOp;
     UintR executingIndexOp;
@@ -910,6 +914,7 @@ public:
       UintR  hashValue;    /* THE HASH VALUE USED TO LOCATE FRAGMENT       */
     
       Uint8  m_special_hash; // collation or distribution key
+      Uint8  m_no_hash;      // Hash not required for LQH (special variant)
       Uint8  m_no_disk_flag; 
       Uint8  lenAiInTckeyreq;  /* LENGTH OF ATTRIBUTE INFORMATION IN TCKEYREQ */
     
@@ -929,6 +934,7 @@ public:
       Uint32 scanInfo;
     
       Uint32 scanTakeOverInd;
+      Uint32 unlockNodeId;     /* NodeId for unlock operation */
     /* End of TCKEYREQ/TCINDXREQ only fields */
 
   };
@@ -1393,10 +1399,10 @@ private:
                             HostRecord * ahostptr,
                             UintR hostId);
   void sendPackedSignalLqh(Signal* signal, HostRecord * ahostptr);
-  void sendCommitLqh(Signal* signal,
-                     TcConnectRecord * const regTcPtr);
-  void sendCompleteLqh(Signal* signal,
+  Uint32 sendCommitLqh(Signal* signal,
                        TcConnectRecord * const regTcPtr);
+  Uint32 sendCompleteLqh(Signal* signal,
+                         TcConnectRecord * const regTcPtr);
   void sendTCKEY_FAILREF(Signal* signal, ApiConnectRecord *);
   void sendTCKEY_FAILCONF(Signal* signal, ApiConnectRecord *);
   void routeTCKEY_FAILREFCONF(Signal* signal, const ApiConnectRecord *, 
@@ -1549,6 +1555,11 @@ private:
                            TcFiredTriggerData* firedTriggerData,
                            ApiConnectRecordPtr* transPtr,
                            TcConnectRecordPtr* opPtr);
+  bool appendAttrDataToSection(Uint32& sectionIVal,
+                               DataBuffer<11>& values,
+                               bool withHeaders,
+                               Uint32& attrId,
+                               bool& hasNull);
   void insertIntoIndexTable(Signal* signal, 
                             TcFiredTriggerData* firedTriggerData, 
                             ApiConnectRecordPtr* transPtr,
@@ -1569,6 +1580,8 @@ private:
                            TcConnectRecordPtr* opPtr);
 
   void releaseFiredTriggerData(DLFifoList<TcFiredTriggerData>* triggers);
+  void abortTransFromTrigger(Signal* signal, const ApiConnectRecordPtr& transPtr, 
+                             Uint32 error);
   // Generated statement blocks
   void warningHandlerLab(Signal* signal, int line);
   void systemErrorLab(Signal* signal, int line);
@@ -1693,40 +1706,88 @@ private:
 
   Uint64 tcheckGcpId;
 
-  struct TransCounters {
-    TransCounters() {}
-    enum { Off, Timer, Started } c_trans_status;
-    UintR cattrinfoCount;
-    UintR ctransCount;
-    UintR ccommitCount;
-    UintR creadCount;
-    UintR csimpleReadCount;
-    UintR cwriteCount;
-    UintR cabortCount;
-    UintR cconcurrentOp;
-    Uint32 c_scan_count;
-    Uint32 c_range_scan_count;
-    void reset () { 
-      cattrinfoCount = ctransCount = ccommitCount = creadCount =
-	csimpleReadCount = cwriteCount = cabortCount =
-	c_scan_count = c_range_scan_count = 0; 
-    }
-    Uint32 report(Signal* signal){
+  // Montonically increasing counters
+  struct MonotonicCounters {
+    Uint64 cattrinfoCount;
+    Uint64 ctransCount;
+    Uint64 ccommitCount;
+    Uint64 creadCount;
+    Uint64 csimpleReadCount;
+    Uint64 cwriteCount;
+    Uint64 cabortCount;
+    Uint64 c_scan_count;
+    Uint64 c_range_scan_count;
+
+    // Resource usage counter(not monotonic)
+    Uint32 cconcurrentOp;
+
+    MonotonicCounters() :
+     cattrinfoCount(0),
+     ctransCount(0),
+     ccommitCount(0),
+     creadCount(0),
+     csimpleReadCount(0),
+     cwriteCount(0),
+     cabortCount(0),
+     c_scan_count(0),
+     c_range_scan_count(0),
+     cconcurrentOp(0) {}
+
+    Uint32 build_event_rep(Signal* signal)
+    {
+      /*
+        Read saved value from CONTINUEB, subtract from
+        counter and write to EVENT_REP
+      */
+      const Uint32 attrinfoCount =    diff(signal, 1, cattrinfoCount);
+      const Uint32 transCount =       diff(signal, 3, ctransCount);
+      const Uint32 commitCount =      diff(signal, 5, ccommitCount);
+      const Uint32 readCount =        diff(signal, 7, creadCount);
+      const Uint32 simpleReadCount =  diff(signal, 9, csimpleReadCount);
+      const Uint32 writeCount =       diff(signal, 11, cwriteCount);
+      const Uint32 abortCount =       diff(signal, 13, cabortCount);
+      const Uint32 scan_count =       diff(signal, 15, c_scan_count);
+      const Uint32 range_scan_count = diff(signal, 17, c_range_scan_count);
+
       signal->theData[0] = NDB_LE_TransReportCounters;
-      signal->theData[1] = ctransCount;
-      signal->theData[2] = ccommitCount;
-      signal->theData[3] = creadCount;
-      signal->theData[4] = csimpleReadCount;
-      signal->theData[5] = cwriteCount;
-      signal->theData[6] = cattrinfoCount;
-      signal->theData[7] = cconcurrentOp;
-      signal->theData[8] = cabortCount;
-      signal->theData[9] = c_scan_count;
-      signal->theData[10] = c_range_scan_count;
+      signal->theData[1] = transCount;
+      signal->theData[2] = commitCount;
+      signal->theData[3] = readCount;
+      signal->theData[4] = simpleReadCount;
+      signal->theData[5] = writeCount;
+      signal->theData[6] = attrinfoCount;
+      signal->theData[7] = cconcurrentOp; // Exception that confirms the rule!
+      signal->theData[8] = abortCount;
+      signal->theData[9] = scan_count;
+      signal->theData[10] = range_scan_count;
       return 11;
     }
+
+    Uint32 build_continueB(Signal* signal) const
+    {
+      /* Save current value of counters to CONTINUEB */
+      const Uint64* vars[] = {
+        &cattrinfoCount, &ctransCount, &ccommitCount,
+        &creadCount, &csimpleReadCount, &cwriteCount,
+        &cabortCount, &c_scan_count, &c_range_scan_count };
+      const size_t num = sizeof(vars)/sizeof(vars[0]);
+
+      for (size_t i = 0; i < num; i++)
+      {
+        signal->theData[1+i*2] = Uint32(*vars[i] >> 32);
+        signal->theData[1+i*2+1] = Uint32(*vars[i]);
+      }
+      return 1 + num * 2;
+    }
+  private:
+    Uint32 diff(Signal* signal, size_t pos, Uint64 curr) const
+    {
+      const Uint64 old =
+        (signal->theData[pos+1] | (Uint64(signal->theData[pos]) << 32));
+      return (Uint32)(curr - old);
+    }
   } c_counters;
-  
+
   Uint16 cownNodeid;
   Uint16 terrorCode;
 
