@@ -204,6 +204,7 @@ Dbtup::dealloc_tuple(Signal* signal,
     * ptr->get_mm_gci(regTabPtr) = gci;
   }
 }
+
 #if 0
 static void dump_buf_hex(unsigned char *p, Uint32 bytes)
 {
@@ -240,8 +241,7 @@ Dbtup::commit_operation(Signal* signal,
   Uint32 bits= tuple_ptr->m_header_bits;
 
   Tuple_header *disk_ptr= 0;
-  Tuple_header *copy=
-    get_copy_tuple(regTabPtr, &regOperPtr->m_copy_tuple_location);
+  Tuple_header *copy= get_copy_tuple(&regOperPtr->m_copy_tuple_location);
   
   Uint32 copy_bits= copy->m_header_bits;
 
@@ -331,7 +331,7 @@ Dbtup::commit_operation(Signal* signal,
     memcpy(&key, copy->get_disk_ref_ptr(regTabPtr), sizeof(Local_key));
     Uint32 logfile_group_id= regFragPtr->m_logfile_group_id;
 
-    PagePtr diskPagePtr = *(PagePtr*)&m_pgman_ptr;
+    PagePtr diskPagePtr = { (Tup_page*)m_pgman_ptr.p, m_pgman_ptr.i };
     ndbassert(diskPagePtr.p->m_page_no == key.m_page_no);
     ndbassert(diskPagePtr.p->m_file_no == key.m_file_no);
     Uint32 sz, *dst;
@@ -501,7 +501,7 @@ void Dbtup::execTUP_COMMITREQ(Signal* signal)
   FragrecordPtr regFragPtr;
   OperationrecPtr regOperPtr;
   TablerecPtr regTabPtr;
-  KeyReqStruct req_struct;
+  KeyReqStruct req_struct(this);
   TransState trans_state;
   Uint32 no_of_fragrec, no_of_tablerec;
 
@@ -605,8 +605,7 @@ void Dbtup::execTUP_COMMITREQ(Signal* signal)
     if(!regOperPtr.p->m_copy_tuple_location.isNull())
     {
       jam();
-      Tuple_header* tmp=
-        get_copy_tuple(regTabPtr.p, &regOperPtr.p->m_copy_tuple_location);
+      Tuple_header* tmp= get_copy_tuple(&regOperPtr.p->m_copy_tuple_location);
       
       memcpy(&req.m_page, 
 	     tmp->get_disk_ref_ptr(regTabPtr.p), sizeof(Local_key));
@@ -752,7 +751,9 @@ skip_disk:
     {
       jam();
       if (get_page)
+      {
 	ndbassert(tuple_ptr->m_header_bits & Tuple_header::DISK_PART);
+      }
       dealloc_tuple(signal, gci_hi, page.p, tuple_ptr,
 		    regOperPtr.p, regFragPtr.p, regTabPtr.p); 
     }
@@ -792,9 +793,18 @@ Dbtup::set_commit_change_mask_info(const Tablerec* regTabPtr,
   else
   {
     Uint32 * dst = req_struct->changeMask.rep.data;
-    Tuple_header* ptr = get_copy_tuple(regTabPtr,
-                                       &regOperPtr->m_copy_tuple_location);
-    const Uint32 * maskptr = get_change_mask_ptr(regTabPtr, ptr);
-    memcpy(dst, maskptr, 4*masklen);
+    Uint32 * maskptr = get_copy_tuple_raw(&regOperPtr->m_copy_tuple_location);
+    Uint32 cols = * maskptr;
+    if (cols == regTabPtr->m_no_of_attributes)
+    {
+      memcpy(dst, maskptr + 1, 4*masklen);
+    }
+    else
+    {
+      ndbassert(regTabPtr->m_no_of_attributes > cols); // no drop column
+      memcpy(dst, maskptr + 1, 4*((cols + 31) >> 5));
+      req_struct->changeMask.setRange(cols,
+                                      regTabPtr->m_no_of_attributes - cols);
+    }
   }
 }
