@@ -92,6 +92,7 @@ check_free(const Dbtup::Extent_info* extP)
 void 
 Dbtup::dump_disk_alloc(Dbtup::Disk_alloc_info & alloc)
 {
+  const Uint32 limit = 512;
   ndbout_c("dirty pages");
   for(Uint32 i = 0; i<MAX_FREE_LIST; i++)
   {
@@ -99,10 +100,15 @@ Dbtup::dump_disk_alloc(Dbtup::Disk_alloc_info & alloc)
     PagePtr ptr;
     ArrayPool<Page> *pool= (ArrayPool<Page>*)&m_global_page_pool;
     LocalDLList<Page> list(*pool, alloc.m_dirty_pages[i]);
-    for(list.first(ptr); !ptr.isNull(); list.next(ptr))
+    Uint32 c = 0;
+    for (list.first(ptr); c < limit && !ptr.isNull(); c++, list.next(ptr))
     {
       ndbout << ptr << " ";
     }
+    if (c == limit)
+    {
+      ndbout << "MAXLIMIT ";
+    }
     ndbout_c(" ");
   }
   ndbout_c("page requests");
@@ -112,10 +118,15 @@ Dbtup::dump_disk_alloc(Dbtup::Disk_alloc_info & alloc)
     Ptr<Page_request> ptr;
     Local_page_request_list list(c_page_request_pool, 
 				 alloc.m_page_requests[i]);
-    for(list.first(ptr); !ptr.isNull(); list.next(ptr))
+    Uint32 c = 0;
+    for (list.first(ptr); c < limit && !ptr.isNull(); c++, list.next(ptr))
     {
       ndbout << ptr << " ";
     }
+    if (c == limit)
+    {
+      ndbout << "MAXLIMIT ";
+    }
     ndbout_c(" ");
   }
 
@@ -125,10 +136,15 @@ Dbtup::dump_disk_alloc(Dbtup::Disk_alloc_info & alloc)
     printf("  %d : ", i);
     Ptr<Extent_info> ptr;
     Local_extent_info_list list(c_extent_pool, alloc.m_free_extents[i]);
-    for(list.first(ptr); !ptr.isNull(); list.next(ptr))
+    Uint32 c = 0;
+    for (list.first(ptr); c < limit && !ptr.isNull(); c++, list.next(ptr))
     {
       ndbout << ptr << " ";
     }
+    if (c == limit)
+    {
+      ndbout << "MAXLIMIT ";
+    }
     ndbout_c(" ");
   }
 
@@ -601,6 +617,7 @@ Dbtup::disk_page_prealloc(Signal* signal,
       bzero(ext.p->m_free_page_count, sizeof(ext.p->m_free_page_count));
       ext.p->m_free_space= alloc.m_page_free_bits_map[0] * pages; 
       ext.p->m_free_page_count[0]= pages; // All pages are "free"-est
+      ext.p->m_empty_page_no = 0;
       c_extent_hash.add(ext);
 
       Local_fragment_extent_list list1(c_extent_pool, alloc.m_extent_list);
@@ -664,8 +681,15 @@ Dbtup::disk_page_prealloc(Signal* signal,
   if (pageBits == 0)
   {
     jam();
-    //XXX empty page -> fast to map
-    flags |= Page_cache_client::EMPTY_PAGE;
+
+    if (ext.p->m_first_page_no + ext.p->m_empty_page_no == key->m_page_no)
+    {
+      jam();
+      flags |= Page_cache_client::EMPTY_PAGE;
+      //ndbout << "EMPTY_PAGE " << ext.p->m_empty_page_no << " " << *key << endl;
+      ext.p->m_empty_page_no++;
+    }
+
     preq.m_callback.m_callbackFunction = 
       safe_cast(&Dbtup::disk_page_prealloc_initial_callback);
   }
@@ -1631,7 +1655,7 @@ Dbtup::disk_restart_undo_lcp(Uint32 tableId, Uint32 fragId, Uint32 flag,
   tabPtr.i= tableId;
   ptrCheckGuard(tabPtr, cnoOfTablerec, tablerec);
   
-  if (tabPtr.p->tableStatus == DEFINED)
+  if (tabPtr.p->tableStatus == DEFINED && tabPtr.p->m_no_of_disk_attributes)
   {
     jam();
     FragrecordPtr fragPtr;
@@ -1713,7 +1737,8 @@ Dbtup::disk_restart_undo_callback(Signal* signal,
   undo->m_table_ptr.i = tableId;
   ptrCheckGuard(undo->m_table_ptr, cnoOfTablerec, tablerec);
   
-  if (undo->m_table_ptr.p->tableStatus != DEFINED)
+  if (! (undo->m_table_ptr.p->tableStatus == DEFINED && 
+         undo->m_table_ptr.p->m_no_of_disk_attributes))
   {
     jam();
     if (DBG_UNDO)
@@ -1895,7 +1920,7 @@ Dbtup::disk_restart_alloc_extent(Uint32 tableId, Uint32 fragId,
   FragrecordPtr fragPtr;
   tabPtr.i = tableId;
   ptrCheckGuard(tabPtr, cnoOfTablerec, tablerec);
-  if (tabPtr.p->tableStatus == DEFINED)
+  if (tabPtr.p->tableStatus == DEFINED && tabPtr.p->m_no_of_disk_attributes)
   {
     getFragmentrec(fragPtr, fragId, tabPtr.p);
 
@@ -1920,6 +1945,7 @@ Dbtup::disk_restart_alloc_extent(Uint32 tableId, Uint32 fragId,
       ext.p->m_key = *key;
       ext.p->m_first_page_no = ext.p->m_key.m_page_no;
       ext.p->m_free_space= 0;
+      ext.p->m_empty_page_no = (1 << 16); // We don't know, so assume none
       bzero(ext.p->m_free_page_count, sizeof(ext.p->m_free_page_count));
       
       if (alloc.m_curr_extent_info_ptr_i != RNIL)
@@ -1956,17 +1982,21 @@ Dbtup::disk_restart_page_bits(Uint32 tableId, Uint32 fragId,
   FragrecordPtr fragPtr;
   tabPtr.i = tableId;
   ptrCheckGuard(tabPtr, cnoOfTablerec, tablerec);
-  getFragmentrec(fragPtr, fragId, tabPtr.p);
-  Disk_alloc_info& alloc= fragPtr.p->m_disk_alloc_info;
-  
-  Ptr<Extent_info> ext;
-  c_extent_pool.getPtr(ext, alloc.m_curr_extent_info_ptr_i);
-  
-  Uint32 size= alloc.calc_page_free_space(bits);  
-  
-  ext.p->m_free_page_count[bits]++;
-  update_extent_pos(alloc, ext, size); // actually only to update free_space
-  ndbassert(ext.p->m_free_matrix_pos == RNIL);
+  if (tabPtr.p->tableStatus == DEFINED && tabPtr.p->m_no_of_disk_attributes)
+  {
+    jam();
+    getFragmentrec(fragPtr, fragId, tabPtr.p);
+    Disk_alloc_info& alloc= fragPtr.p->m_disk_alloc_info;
+    
+    Ptr<Extent_info> ext;
+    c_extent_pool.getPtr(ext, alloc.m_curr_extent_info_ptr_i);
+    
+    Uint32 size= alloc.calc_page_free_space(bits);  
+    
+    ext.p->m_free_page_count[bits]++;
+    update_extent_pos(alloc, ext, size); // actually only to update free_space
+    ndbassert(ext.p->m_free_matrix_pos == RNIL);
+  }
 }
 
 void
