@@ -68,6 +68,8 @@ Dbtup::Dbtup(Block_context& ctx, Uint32 instanceNumber)
     c_storedProcPool(),
     c_buildIndexList(c_buildIndexPool),
     c_undo_buffer(&ctx.m_mm),
+    m_pages_allocated(0),
+    m_pages_allocated_max(0),
     f_undo_done(true)
 {
   BLOCK_CONSTRUCTOR(Dbtup);
@@ -107,6 +109,10 @@ Dbtup::Dbtup(Block_context& ctx, Uint32 instanceNumber)
 
   // Ordered index related
   addRecSignal(GSN_BUILD_INDX_IMPL_REQ, &Dbtup::execBUILD_INDX_IMPL_REQ);
+  addRecSignal(GSN_BUILD_INDX_IMPL_REF, &Dbtup::execBUILD_INDX_IMPL_REF);
+  addRecSignal(GSN_BUILD_INDX_IMPL_CONF, &Dbtup::execBUILD_INDX_IMPL_CONF);
+  addRecSignal(GSN_ALTER_TAB_CONF, &Dbtup::execALTER_TAB_CONF);
+  m_max_parallel_index_build = 0;
 
   // Tup scan
   addRecSignal(GSN_ACC_SCANREQ, &Dbtup::execACC_SCANREQ);
@@ -121,6 +127,7 @@ Dbtup::Dbtup(Block_context& ctx, Uint32 instanceNumber)
   addRecSignal(GSN_FSREMOVECONF, &Dbtup::execFSREMOVECONF, true);
 
   addRecSignal(GSN_DROP_FRAG_REQ, &Dbtup::execDROP_FRAG_REQ);
+  addRecSignal(GSN_SUB_GCP_COMPLETE_REP, &Dbtup::execSUB_GCP_COMPLETE_REP);
 
   fragoperrec = 0;
   fragrecord = 0;
@@ -128,8 +135,7 @@ Dbtup::Dbtup(Block_context& ctx, Uint32 instanceNumber)
   hostBuffer = 0;
   tablerec = 0;
   tableDescriptor = 0;
-  totNoOfPagesAllocated = 0;
-  
+
   initData();
   CLEAR_ERROR_INSERT_VALUE;
 
@@ -439,7 +445,8 @@ void Dbtup::execREAD_CONFIG_REQ(Signal* signal)
   pc.m_block = this;
   c_page_request_pool.wo_pool_init(RT_DBTUP_PAGE_REQUEST, pc);
   c_extent_pool.init(RT_DBTUP_EXTENT_INFO, pc);
-  c_page_map_pool.init(RT_DBTUP_PAGE_MAP, pc);
+  NdbMutex_Init(&c_page_map_pool_mutex);
+  c_page_map_pool.init(&c_page_map_pool_mutex, RT_DBTUP_PAGE_MAP, pc);
   
   Uint32 nScanOp;       // use TUX config for now
   ndbrequire(!ndb_mgm_get_int_parameter(p, CFG_TUX_SCAN_OP, &nScanOp));
@@ -472,6 +479,19 @@ void Dbtup::execREAD_CONFIG_REQ(Signal* signal)
   clastBitMask = 1;
   clastBitMask = clastBitMask << 31;
 
+  ndb_mgm_get_int_parameter(p, CFG_DB_MT_BUILD_INDEX,
+                            &m_max_parallel_index_build);
+
+  if (isNdbMtLqh() && globalData.ndbMtLqhThreads > 1)
+  {
+    /**
+     * Divide by LQH threads
+     */
+    Uint32 val = m_max_parallel_index_build;
+    val = (val + instance() - 1) / globalData.ndbMtLqhThreads;
+    m_max_parallel_index_build = val;
+  }
+  
   initialiseRecordsLab(signal, 0, ref, senderData);
 }//Dbtup::execSIZEALT_REP()
 
@@ -611,6 +631,7 @@ void Dbtup::execNDB_STTOR(Signal* signal)
     jam();
     cownNodeId = ownNodeId;
     cownref = calcInstanceBlockRef(DBTUP);
+    initializeDefaultValuesFrag();
     break;
   case ZSTARTPHASE2:
     jam();
@@ -638,6 +659,30 @@ void Dbtup::startphase3Lab(Signal* signal, Uint32 config1, Uint32 config2)
 {
 }//Dbtup::startphase3Lab()
 
+void Dbtup::initializeDefaultValuesFrag()
+{
+  /* Grab and initialize a fragment record for storing default
+   * values for the table fragments held by this TUP instance
+   */
+  seizeFragrecord(DefaultValuesFragment);
+  DefaultValuesFragment.p->fragStatus = Fragrecord::FS_ONLINE;
+  DefaultValuesFragment.p->m_undo_complete= false;
+  DefaultValuesFragment.p->m_lcp_scan_op = RNIL;
+  DefaultValuesFragment.p->m_lcp_keep_list = RNIL;
+  DefaultValuesFragment.p->noOfPages = 0;
+  DefaultValuesFragment.p->noOfVarPages = 0;
+  DefaultValuesFragment.p->m_max_page_no = 0;
+  DefaultValuesFragment.p->m_free_page_id_list = FREE_PAGE_RNIL;
+  ndbrequire(DefaultValuesFragment.p->m_page_map.isEmpty());
+  DefaultValuesFragment.p->m_restore_lcp_id = RNIL;
+  for (Uint32 i = 0; i<MAX_FREE_LIST+1; i++)
+    ndbrequire(DefaultValuesFragment.p->free_var_page_array[i].isEmpty());
+
+  DefaultValuesFragment.p->m_logfile_group_id = RNIL;
+
+  return;
+}
+
 void Dbtup::initializeFragoperrec() 
 {
   FragoperrecPtr fragoperPtr;
@@ -727,6 +772,10 @@ Dbtup::initTab(Tablerec* const regTabPtr)
   regTabPtr->tabDescriptor = RNIL;
   regTabPtr->readKeyArray = RNIL;
   regTabPtr->dynTabDescriptor = RNIL;
+  regTabPtr->dynFixSizeMask[MM] = NULL;
+  regTabPtr->dynVarSizeMask[MM] = NULL;
+  regTabPtr->dynFixSizeMask[DD] = NULL;
+  regTabPtr->dynVarSizeMask[DD] = NULL;
 
   regTabPtr->m_bits = 0;
 
@@ -736,6 +785,7 @@ Dbtup::initTab(Tablerec* const regTabPtr)
   regTabPtr->m_dropTable.tabUserPtr = RNIL;
   regTabPtr->m_dropTable.tabUserRef = 0;
   regTabPtr->tableStatus = NOT_DEFINED;
+  regTabPtr->m_default_value_location.setNull();
 
   // Clear trigger data
   if (!regTabPtr->afterInsertTriggers.isEmpty())
