@@ -38,6 +38,7 @@
 #include "AttributeOffset.hpp"
 #include <my_sys.h>
 #include <signaldata/LqhFrag.hpp>
+#include <signaldata/AttrInfo.hpp>
 
 #include <EventLogger.hpp>
 extern EventLogger * g_eventLogger;
@@ -78,6 +79,7 @@ Dbtup::execCREATE_TAB_REQ(Signal* signal)
   fragOperPtr.p->lqhBlockrefFrag = req->senderRef;
 
   regTabPtr.p->m_createTable.m_fragOpPtrI = fragOperPtr.i;
+  regTabPtr.p->m_createTable.defValSectionI = RNIL;
   regTabPtr.p->tableStatus= DEFINING;
   regTabPtr.p->m_bits = 0;
   regTabPtr.p->m_bits |= (req->checksumIndicator ? Tablerec::TR_Checksum : 0);
@@ -213,7 +215,12 @@ void Dbtup::execTUP_ADD_ATTRREQ(Signal* signal)
       jam();
       regTabPtr.p->m_attributes[ind].m_no_of_varsize++;
     }
-    
+    if (null_pos > AO_NULL_FLAG_POS_MASK)
+    {
+      jam();
+      terrorCode = ZTOO_MANY_BITS_ERROR;
+      goto error;
+    }      
     AttributeOffset::setNullFlagPos(attrDes2, null_pos);
   } else {
     /* A dynamic attribute. */
@@ -283,8 +290,20 @@ void Dbtup::execTUP_ADD_ATTRREQ(Signal* signal)
                    fragOperPtr.p->charsetIndex, attrDes2);
   setTabDescrWord(firstTabDesIndex + 1, attrDes2);
 
-  if (ERROR_INSERTED(4009) && attrId == 0 ||
-      ERROR_INSERTED(4010) && lastAttr)
+  if ((ERROR_INSERTED(4009) && attrId == 0) ||
+      (ERROR_INSERTED(4010) && lastAttr))
+  {
+    jam();
+    CLEAR_ERROR_INSERT_VALUE;
+    terrorCode = 1;
+    goto error;
+  }
+
+  if (!receive_defvalue(signal, regTabPtr))
+    goto error;
+
+  if ((ERROR_INSERTED(4032) && (attrId == 0)) ||
+      (ERROR_INSERTED(4033) && lastAttr))
   {
     jam();
     CLEAR_ERROR_INSERT_VALUE;
@@ -377,6 +396,12 @@ void Dbtup::execTUP_ADD_ATTRREQ(Signal* signal)
   }
 #endif
 
+  if (store_default_record(regTabPtr) < 0)
+  {
+    jam();
+    goto error;
+  }
+
   ndbrequire(regTabPtr.p->tableStatus == DEFINING);
   regTabPtr.p->tableStatus= DEFINED;
 
@@ -388,6 +413,17 @@ void Dbtup::execTUP_ADD_ATTRREQ(Signal* signal)
   releaseFragoperrec(fragOperPtr);
   return;
 error:
+  {
+    /* Release any unprocessed sections */
+    SectionHandle handle(this, signal);
+    releaseSections(handle);
+  }
+  /* Release segmented section used to receive Attr default value */
+  releaseSection(regTabPtr.p->m_createTable.defValSectionI);
+  regTabPtr.p->m_createTable.defValSectionI = RNIL;
+  free_var_part(DefaultValuesFragment.p, regTabPtr.p, &regTabPtr.p->m_default_value_location);
+  regTabPtr.p->m_default_value_location.setNull();
+
   signal->theData[0]= fragOperPtr.p->lqhPtrFrag;
   signal->theData[1]= terrorCode;
   sendSignal(fragOperPtr.p->lqhBlockrefFrag,
@@ -396,6 +432,156 @@ error:
   return;
 }
 
+bool Dbtup::receive_defvalue(Signal* signal, const TablerecPtr& regTabPtr)
+{
+  jam();
+  Uint32 defValueBytes = 0;
+  Uint32 defValueWords = 0;
+  Uint32 attrId = signal->theData[2];
+  Uint32 attrDescriptor = signal->theData[3];
+
+  Uint32 attrLen = AttributeDescriptor::getSize(attrDescriptor);
+  Uint32 arrayType = AttributeDescriptor::getArrayType(attrDescriptor);
+  Uint32 arraySize = AttributeDescriptor::getArraySize(attrDescriptor);
+
+  const Uint32 numSections = signal->getNoOfSections();
+
+  if (numSections == 0)
+    return true;
+
+  jam();
+  SectionHandle handle(this, signal);
+  SegmentedSectionPtr ptr;
+  handle.getSection(ptr, TupAddAttrReq::DEFAULT_VALUE_SECTION_NUM);
+
+  SimplePropertiesSectionReader r(ptr, getSectionSegmentPool());
+  r.reset();
+
+  Uint32 ahIn;
+  ndbrequire(r.getWord(&ahIn));
+
+  defValueBytes = AttributeHeader::getByteSize(ahIn);
+  defValueWords = (defValueBytes + 3) / 4;
+
+  Uint32 *dst = NULL;
+  AttributeHeader ah(attrId, defValueBytes);
+
+  if (defValueBytes == 0)
+  {
+    jam();
+    releaseSections(handle);
+    return true;
+  }
+
+  /* We have a default value, double check to be sure this is not
+   * a primary key
+   */
+  if (AttributeDescriptor::getPrimaryKey(attrDescriptor))
+  {
+    jam();
+    releaseSections(handle);
+    /* Default value for primary key column not supported */
+    terrorCode = 792;
+    return false;
+  }
+
+  Uint32 bytes;
+  if (attrLen)
+    bytes= AttributeDescriptor::getSizeInBytes(attrDescriptor);
+  else
+    bytes= ((arraySize + AD_SIZE_IN_WORDS_OFFSET)
+                              >> AD_SIZE_IN_WORDS_SHIFT) * 4;
+
+  terrorCode= 0;
+
+  if (attrLen)
+  {
+    if (arrayType == NDB_ARRAYTYPE_FIXED)
+    {
+      if (defValueBytes != bytes)
+      {
+        terrorCode = ZBAD_DEFAULT_VALUE_LEN;
+      }
+    }
+    else
+    {
+      if (defValueBytes > bytes)
+      {
+        terrorCode = ZBAD_DEFAULT_VALUE_LEN;
+      }
+    }
+  }
+  else
+  {
+    /*
+     * The condition is for BIT type.
+     * Even though it is fixed, the compare operator should be > rather than ==,
+     * for the 4-byte alignemnt, the space for BIT type occupied 4 bytes at least.
+     * yet the bytes of default value can be 1, 2, 3, 4, 5, 6, 7, 8 bytes.
+     */
+    if (defValueBytes > bytes)
+    {
+      terrorCode = ZBAD_DEFAULT_VALUE_LEN;
+    }
+  }
+  
+  jam();
+
+  if (likely( !terrorCode ))
+  {
+    dst = cinBuffer;
+    
+    ndbrequire(r.getWords(dst, defValueWords));
+    
+    /* Check that VAR types have valid inline length */
+    if ((attrLen) &&
+        (arrayType != NDB_ARRAYTYPE_FIXED))
+    {
+      jam();
+      const uchar* valPtr = (const uchar*) dst;
+      Uint32 internalVarSize = 0;
+      
+      if (arrayType == NDB_ARRAYTYPE_SHORT_VAR)
+      {
+        internalVarSize = 1 + valPtr[0];
+      }
+      else if (arrayType == NDB_ARRAYTYPE_MEDIUM_VAR)
+      {
+        internalVarSize = 2 + valPtr[0] + (256 * Uint32(valPtr[1]));
+      }
+      else
+      {
+        ndbrequire(false);
+      }
+      
+      if (unlikely(internalVarSize != defValueBytes))
+      {
+        jam();
+        terrorCode = ZBAD_DEFAULT_VALUE_LEN;
+        releaseSections(handle);
+        return false;
+      }
+    }
+
+    if (likely( appendToSection(regTabPtr.p->m_createTable.defValSectionI, 
+                                (const Uint32 *)&ah, 1) ))
+    {  
+      if (likely( appendToSection(regTabPtr.p->m_createTable.defValSectionI, 
+                                  (const Uint32*)dst, 
+                                  defValueWords)))
+      {
+        jam();
+        releaseSections(handle);
+        return true;
+      }
+    }
+    terrorCode = ZMEM_NOMEM_ERROR;
+  }
+
+  releaseSections(handle);
+  return false;
+}
+
 void Dbtup::execTUPFRAGREQ(Signal* signal)
 {
   jamEntry();
@@ -490,8 +676,8 @@ void Dbtup::execTUPFRAGREQ(Signal* signal)
     goto sendref;
   }
 
-  if (ERROR_INSERTED(4007) && regTabPtr.p->fragid[0] == fragId ||
-      ERROR_INSERTED(4008) && regTabPtr.p->fragid[1] == fragId ||
+  if ((ERROR_INSERTED(4007) && regTabPtr.p->fragid[0] == fragId) ||
+      (ERROR_INSERTED(4008) && regTabPtr.p->fragid[1] == fragId) ||
       ERROR_INSERTED(4050))
   {
     jam();
@@ -548,12 +734,75 @@ void Dbtup::execTUPFRAGREQ(Signal* signal)
   return;
 
 sendref:
-
   signal->theData[0]= userptr;
   signal->theData[1]= terrorCode;
   sendSignal(userRef, GSN_TUPFRAGREF, signal, 2, JBB);
 }
 
+/*
+  Store the default values for a table, as the ATTRINFO "program"
+  (i.e AttributeHeader|Data AttributeHeader|Data...)
+  in varsize memory associated with the dummy fragment(DefaultValuesFragment).
+  There is a DBTUP global set of defaults records in DefaultValuesFragment.
+  One record per table stored on varsize pages.
+  
+  Each Table_record has a Local_key pointing to start of its default values
+  in TUP's default values fragment.
+*/
+int Dbtup::store_default_record(const TablerecPtr& regTabPtr)
+{
+  Uint32 RdefValSectionI = regTabPtr.p->m_createTable.defValSectionI;
+  jam();
+
+  if (RdefValSectionI == RNIL) //No default values are stored for the table
+  {
+    jam();
+    if (ERROR_INSERTED(4034))
+    {
+      jam();
+      CLEAR_ERROR_INSERT_VALUE;
+      terrorCode = 1;
+      return -1;
+    }
+
+    return 0;
+  }
+
+  SegmentedSectionPtr defValSection;
+  getSection(defValSection, RdefValSectionI);
+  Uint32 sizes = defValSection.p->m_sz;
+  /**
+   * Alloc var-length memory for storing defaults
+   */
+  Uint32* var_data_ptr= alloc_var_part(&terrorCode,
+                                       DefaultValuesFragment.p,
+                                       regTabPtr.p,
+                                       sizes,
+                                       &regTabPtr.p->m_default_value_location);
+  if (unlikely( var_data_ptr == 0 ))
+  {
+    jam();
+    /* Caller releases the default values section */
+    return -1;
+  }
+
+  if (ERROR_INSERTED(4034))
+  {
+    jam();
+    CLEAR_ERROR_INSERT_VALUE;
+    terrorCode = 1;
+    return -1;
+  }
+      
+  
+  copy(var_data_ptr, RdefValSectionI);
+  releaseSection(RdefValSectionI);
+  regTabPtr.p->m_createTable.defValSectionI= RNIL;
+
+  return 0;
+}
+
+
 bool Dbtup::addfragtotab(Tablerec* const regTabPtr,
                          Uint32 fragId,
                          Uint32 fragIndex)
@@ -717,6 +966,11 @@ Dbtup::execALTER_TAB_REQ(Signal *signal)
     signal->theData[0] = ~Uint32(0);
     return;
   }
+  case AlterTabReq::AlterTableReadOnly:
+  case AlterTabReq::AlterTableReadWrite:
+    signal->theData[0] = 0;
+    signal->theData[1] = RNIL;
+    return;
   default:
     break;
   }
@@ -1160,8 +1414,8 @@ Dbtup::computeTableMetaData(Tablerec *regTabPtr)
   Uint32 dynamic_count= 0;
   regTabPtr->blobAttributeMask.clear();
   regTabPtr->notNullAttributeMask.clear();
-  bzero(regTabPtr->dynVarSizeMask, dyn_null_words<<2);
-  bzero(regTabPtr->dynFixSizeMask, dyn_null_words<<2);
+  bzero(regTabPtr->dynVarSizeMask[MM], dyn_null_words<<2);
+  bzero(regTabPtr->dynFixSizeMask[MM], dyn_null_words<<2);
 
   for(Uint32 i= 0; i<regTabPtr->m_no_of_attributes; i++)
   {
@@ -1223,7 +1477,7 @@ Dbtup::computeTableMetaData(Tablerec *regTabPtr)
           while(size_in_words-- > 0)
 	  {
 	    BitmaskImpl::set(dyn_null_words, 
-			     regTabPtr->dynFixSizeMask, null_pos++);
+			     regTabPtr->dynFixSizeMask[ind], null_pos++);
 	  }
         }
         else
@@ -1234,7 +1488,7 @@ Dbtup::computeTableMetaData(Tablerec *regTabPtr)
       treat_as_varsize:
         jam();
         off= dynvar_count++;
-	BitmaskImpl::set(dyn_null_words, regTabPtr->dynVarSizeMask, null_pos);
+	BitmaskImpl::set(dyn_null_words, regTabPtr->dynVarSizeMask[ind], null_pos);
       }
     }
     AttributeOffset::setOffset(attrDes2, off);
@@ -1300,7 +1554,7 @@ Dbtup::computeTableMetaData(Tablerec *regTabPtr)
     total_rec_size+= Tuple_header::HeaderSize;
 
   /* Room for changemask */
-  total_rec_size += (regTabPtr->m_no_of_attributes + 31) >> 5;
+  total_rec_size += 1 + ((regTabPtr->m_no_of_attributes + 31) >> 5);
 
   regTabPtr->total_rec_size= total_rec_size;
 
@@ -1397,8 +1651,8 @@ void Dbtup::setupDynDescriptorReferences(Uint32 dynDescr,
 {
   regTabPtr->dynTabDescriptor= dynDescr;
   Uint32* desc= &tableDescriptor[dynDescr].tabDescr;
-  regTabPtr->dynVarSizeMask= desc+offset[0];
-  regTabPtr->dynFixSizeMask= desc+offset[1];
+  regTabPtr->dynVarSizeMask[MM] = desc+offset[0];
+  regTabPtr->dynFixSizeMask[MM] = desc+offset[1];
 }
 
 Uint32
@@ -1584,8 +1838,8 @@ void Dbtup::releaseTabDescr(Tablerec* const regTabPtr)
   {
     jam();
     regTabPtr->dynTabDescriptor= RNIL;
-    regTabPtr->dynVarSizeMask= NULL;
-    regTabPtr->dynFixSizeMask= NULL;
+    regTabPtr->dynVarSizeMask[MM]= NULL;
+    regTabPtr->dynFixSizeMask[MM]= NULL;
     releaseTabDescr(descriptor);
   }
 }
@@ -1879,6 +2133,8 @@ Dbtup::drop_table_logsync_callback(Signal* signal,
              signal, DropTabConf::SignalLength, JBB);
   
   releaseTabDescr(tabPtr.p);
+  free_var_part(DefaultValuesFragment.p, tabPtr.p, &tabPtr.p->m_default_value_location);
+  tabPtr.p->m_default_value_location.setNull();
   initTab(tabPtr.p);
 }
 
@@ -2182,8 +2438,16 @@ Dbtup::start_restore_lcp(Uint32 tableId, Uint32 fragId)
   tabPtr.i= tableId;
   ptrCheckGuard(tabPtr, cnoOfTablerec, tablerec);
   
-  tabPtr.p->m_dropTable.tabUserPtr= tabPtr.p->m_attributes[DD].m_no_of_fixsize;
-  tabPtr.p->m_dropTable.tabUserRef= tabPtr.p->m_attributes[DD].m_no_of_varsize;
+  ndbassert(tabPtr.p->m_attributes[DD].m_no_of_fixsize < (1 << 16));
+  ndbassert(tabPtr.p->m_attributes[DD].m_no_of_varsize < (1 << 16));
+  
+  Uint32 saveAttrCounts = 
+    (tabPtr.p->m_attributes[DD].m_no_of_fixsize << 16) |
+    (tabPtr.p->m_attributes[DD].m_no_of_varsize << 0);
+  
+  tabPtr.p->m_dropTable.tabUserPtr= saveAttrCounts;
+  tabPtr.p->m_dropTable.tabUserRef= (tabPtr.p->m_bits & Tablerec::TR_RowGCI)? 1 : 0;
+  tabPtr.p->m_createTable.defValLocation = tabPtr.p->m_default_value_location;
   
   Uint32 *tabDesc = (Uint32*)(tableDescriptor+tabPtr.p->tabDescriptor);
   for(Uint32 i= 0; i<tabPtr.p->m_no_of_attributes; i++)
@@ -2200,6 +2464,9 @@ Dbtup::start_restore_lcp(Uint32 tableId, Uint32 fragId)
   tabPtr.p->m_no_of_disk_attributes = 0;
   tabPtr.p->m_attributes[DD].m_no_of_fixsize = 0;
   tabPtr.p->m_attributes[DD].m_no_of_varsize = 0;
+  /* Avoid LQH trampling GCI restored in raw format */
+  tabPtr.p->m_bits &= ~((Uint16) Tablerec::TR_RowGCI);
+  tabPtr.p->m_default_value_location.setNull();
 }
 void
 Dbtup::complete_restore_lcp(Signal* signal, 
@@ -2210,12 +2477,16 @@ Dbtup::complete_restore_lcp(Signal* signal,
   tabPtr.i= tableId;
   ptrCheckGuard(tabPtr, cnoOfTablerec, tablerec);
   
-  tabPtr.p->m_attributes[DD].m_no_of_fixsize= tabPtr.p->m_dropTable.tabUserPtr;
-  tabPtr.p->m_attributes[DD].m_no_of_varsize= tabPtr.p->m_dropTable.tabUserRef;
-  
+  Uint32 restoreAttrCounts = tabPtr.p->m_dropTable.tabUserPtr;
+
+  tabPtr.p->m_attributes[DD].m_no_of_fixsize= restoreAttrCounts >> 16;
+  tabPtr.p->m_attributes[DD].m_no_of_varsize= restoreAttrCounts & 0xffff;
+  tabPtr.p->m_bits |= ((tabPtr.p->m_dropTable.tabUserRef & 1) ? Tablerec::TR_RowGCI : 0);
+
   tabPtr.p->m_no_of_disk_attributes = 
     tabPtr.p->m_attributes[DD].m_no_of_fixsize + 
     tabPtr.p->m_attributes[DD].m_no_of_varsize;
+  tabPtr.p->m_default_value_location = tabPtr.p->m_createTable.defValLocation;
   
   Uint32 *tabDesc = (Uint32*)(tableDescriptor+tabPtr.p->tabDescriptor);
   for(Uint32 i= 0; i<tabPtr.p->m_no_of_attributes; i++)
