@@ -124,6 +124,12 @@ void
 Dbtup::execCREATE_TRIG_IMPL_REQ(Signal* signal)
 {
   jamEntry();
+  if (!assembleFragments(signal))
+  {
+    jam();
+    return;
+  }
+
   const CreateTrigImplReq* req = (const CreateTrigImplReq*)signal->getDataPtr();
   const Uint32 senderRef = req->senderRef;
   const Uint32 senderData = req->senderData;
@@ -133,38 +139,64 @@ Dbtup::execCREATE_TRIG_IMPL_REQ(Signal* signal)
 
   CreateTrigRef::ErrorCode error = CreateTrigRef::NoError;
 
-  // Find table
-  TablerecPtr tabPtr;
-  tabPtr.i = req->tableId;
-  ptrCheckGuard(tabPtr, cnoOfTablerec, tablerec);
-
-  if (tabPtr.p->tableStatus != DEFINED )
+  AttributeMask mask;
+  SectionHandle handle(this, signal);
+  if (handle.m_cnt <= CreateTrigImplReq::ATTRIBUTE_MASK_SECTION)
   {
     jam();
-    error = CreateTrigRef::InvalidTable;
-  }
-  // Create trigger and associate it with the table
-  else if (createTrigger(tabPtr.p, req))
-  {
-    jam();
-    // Send conf
-    CreateTrigImplConf* conf = (CreateTrigImplConf*)signal->getDataPtrSend();
-    conf->senderRef = reference();
-    conf->senderData = senderData;
-    conf->tableId = tableId;
-    conf->triggerId = triggerId;
-    conf->triggerInfo = triggerInfo;
-
-    sendSignal(senderRef, GSN_CREATE_TRIG_IMPL_CONF, 
-               signal, CreateTrigImplConf::SignalLength, JBB);
-    return;
+    ndbassert(false);
+    error = CreateTrigRef::BadRequestType;
   }
   else
   {
-    jam();
-    error = CreateTrigRef::TooManyTriggers;
+    SegmentedSectionPtr ptr;
+    handle.getSection(ptr, CreateTrigImplReq::ATTRIBUTE_MASK_SECTION);
+    ndbrequire(ptr.sz == mask.getSizeInWords());
+    ::copy(mask.rep.data, ptr);
   }
 
+  releaseSections(handle);
+
+  if (error != CreateTrigRef::NoError)
+  {
+    goto err;
+  }
+
+  {
+    // Find table
+    TablerecPtr tabPtr;
+    tabPtr.i = req->tableId;
+    ptrCheckGuard(tabPtr, cnoOfTablerec, tablerec);
+
+    if (tabPtr.p->tableStatus != DEFINED )
+    {
+      jam();
+      error = CreateTrigRef::InvalidTable;
+    }
+    // Create trigger and associate it with the table
+    else if (createTrigger(tabPtr.p, req, mask))
+    {
+      jam();
+      // Send conf
+      CreateTrigImplConf* conf = (CreateTrigImplConf*)signal->getDataPtrSend();
+      conf->senderRef = reference();
+      conf->senderData = senderData;
+      conf->tableId = tableId;
+      conf->triggerId = triggerId;
+      conf->triggerInfo = triggerInfo;
+
+      sendSignal(senderRef, GSN_CREATE_TRIG_IMPL_CONF,
+                 signal, CreateTrigImplConf::SignalLength, JBB);
+      return;
+    }
+    else
+    {
+      jam();
+      error = CreateTrigRef::TooManyTriggers;
+    }
+  }
+
+err:
   ndbassert(error != CreateTrigRef::NoError);
   // Send ref
   CreateTrigImplRef* ref = (CreateTrigImplRef*)signal->getDataPtrSend();
@@ -197,7 +229,16 @@ Dbtup::execDROP_TRIG_IMPL_REQ(Signal* signal)
 
   // Drop trigger
   Uint32 r = dropTrigger(tabPtr.p, req, refToBlock(receiverRef));
-  if (r == 0){
+  if (r == 0)
+  {
+    /**
+     * make sure that any trigger data is sent before DROP_TRIG_CONF
+     *   NOTE: This is only needed for SUMA triggers
+     *         (which are the only buffered ones) but it shouldn't
+     *         be too bad to do it for all triggers...
+     */
+    flush_ndbmtd_suma_buffer(signal);
+
     // Send conf
     DropTrigImplConf* conf = (DropTrigImplConf*)signal->getDataPtrSend();
     conf->senderRef = reference();
@@ -235,7 +276,8 @@ Dbtup::execDROP_TRIG_IMPL_REQ(Signal* signal)
 /* ---------------------------------------------------------------- */
 bool
 Dbtup::createTrigger(Tablerec* table,
-                     const CreateTrigImplReq* req)
+                     const CreateTrigImplReq* req,
+                     const AttributeMask& mask)
 {
   if (ERROR_INSERTED(4003)) {
     CLEAR_ERROR_INSERT_VALUE;
@@ -343,7 +385,7 @@ Dbtup::createTrigger(Tablerec* table,
     {
       jam();
       // Set attribute mask
-      tptr.p->attributeMask = req->attributeMask;
+      tptr.p->attributeMask = mask;
     }
   }
   return true;
@@ -577,8 +619,7 @@ void Dbtup::checkDetachedTriggers(KeyReqStruct *req_struct,
   switch (save_type) {
   case ZUPDATE:
   case ZINSERT:
-    req_struct->m_tuple_ptr =
-      get_copy_tuple(regTablePtr, &regOperPtr->m_copy_tuple_location);
+    req_struct->m_tuple_ptr =get_copy_tuple(&regOperPtr->m_copy_tuple_location);
     break;
   }
 
@@ -939,6 +980,7 @@ out:
     jam();
     return;
   }
+
 //--------------------------------------------------------------------
 // Now all data for this trigger has been read. It is now time to send
 // the trigger information consisting of two or three sets of TRIG_
@@ -1090,7 +1132,7 @@ out:
     if (executeDirect)
     {
       jam();
-      EXECUTE_DIRECT(refToMain(trigPtr->m_receiverRef),
+      EXECUTE_DIRECT(refToMain(ref),
                      GSN_FIRE_TRIG_ORD,
                      signal,
                      FireTrigOrd::SignalLengthSuma);
@@ -1106,8 +1148,17 @@ out:
       ptr[1].sz = noBeforeWords;
       ptr[2].p = afterBuffer;
       ptr[2].sz = noAfterWords;
-      sendSignal(trigPtr->m_receiverRef, GSN_FIRE_TRIG_ORD,
-                 signal, FireTrigOrd::SignalLengthSuma, JBB, ptr, 3);
+      if (refToMain(ref) == SUMA && (refToInstance(ref) != instance()))
+      {
+        jam();
+        ndbmtd_buffer_suma_trigger(signal, FireTrigOrd::SignalLengthSuma, ptr);
+      }
+      else
+      {
+        jam();
+        sendSignal(ref, GSN_FIRE_TRIG_ORD,
+                   signal, FireTrigOrd::SignalLengthSuma, JBB, ptr, 3);
+      }
     }
     break;
   case (TriggerType::SUBSCRIPTION):
@@ -1119,7 +1170,7 @@ out:
     if (executeDirect)
     {
       jam();
-      EXECUTE_DIRECT(refToMain(trigPtr->m_receiverRef),
+      EXECUTE_DIRECT(refToMain(ref),
                      GSN_FIRE_TRIG_ORD,
                      signal,
                      FireTrigOrd::SignalWithGCILength);
@@ -1137,7 +1188,7 @@ out:
       ptr[1].sz = noBeforeWords;
       ptr[2].p = afterBuffer;
       ptr[2].sz = noAfterWords;
-      sendSignal(trigPtr->m_receiverRef, GSN_FIRE_TRIG_ORD,
+      sendSignal(ref, GSN_FIRE_TRIG_ORD,
                  signal, FireTrigOrd::SignalWithGCILength, JBB, ptr, 3);
     }
     break;
@@ -1181,6 +1232,8 @@ bool Dbtup::readTriggerInfo(TupTriggerData* const trigPtr,
 //---------------------------------------------------------------------------
 // Set-up variables needed by readAttributes operPtr.p, tabptr.p
 //---------------------------------------------------------------------------
+  Ptr<Tablerec> tabptr;
+  Ptr<Operationrec> operPtr;
   operPtr.p = regOperPtr;
   tabptr.i = regFragPtr->fragTableId;
   ptrCheckGuard(tabptr, cnoOfTablerec, tablerec);
@@ -1190,6 +1243,8 @@ bool Dbtup::readTriggerInfo(TupTriggerData* const trigPtr,
   Uint32 descr_start= regTabPtr->tabDescriptor;
   ndbrequire(descr_start + (num_attr << ZAD_LOG_SIZE) <= cnoOfTabDescrRec);
 
+  req_struct->tablePtrP = regTabPtr;
+  req_struct->operPtrP = regOperPtr;
   req_struct->check_offset[MM]= regTabPtr->get_check_offset(MM);
   req_struct->check_offset[DD]= regTabPtr->get_check_offset(DD);
   req_struct->attr_descr= &tableDescriptor[descr_start];
@@ -1203,8 +1258,7 @@ bool Dbtup::readTriggerInfo(TupTriggerData* const trigPtr,
   {
     jam();
     req_struct->m_tuple_ptr=
-      get_copy_tuple(regTabPtr,
-                     &req_struct->prevOpPtr.p->m_copy_tuple_location);
+      get_copy_tuple(&req_struct->prevOpPtr.p->m_copy_tuple_location);
   }
 
   if (regTabPtr->need_expand(disk)) 
@@ -1216,7 +1270,7 @@ bool Dbtup::readTriggerInfo(TupTriggerData* const trigPtr,
 			   keyBuffer,
 			   ZATTR_BUFFER_SIZE,
 			   false);
-  ndbrequire(ret != -1);
+  ndbrequire(ret >= 0);
   noPrimKey= ret;
   
   req_struct->m_tuple_ptr = save0;
@@ -1258,7 +1312,7 @@ bool Dbtup::readTriggerInfo(TupTriggerData* const trigPtr,
     numAttrsToRead = setAttrIds(attributeMask, regTabPtr->m_no_of_attributes,
                                 &readBuffer[0]);
   }
-  ndbrequire(numAttrsToRead < MAX_ATTRIBUTES_IN_TABLE);
+  ndbrequire(numAttrsToRead <= MAX_ATTRIBUTES_IN_TABLE);
 //--------------------------------------------------------------------
 // Read Main tuple values
 //--------------------------------------------------------------------
@@ -1271,7 +1325,7 @@ bool Dbtup::readTriggerInfo(TupTriggerData* const trigPtr,
 			     afterBuffer,
 			     ZATTR_BUFFER_SIZE,
 			     false);
-    ndbrequire(ret != -1);
+    ndbrequire(ret >= 0);
     noAfterWords= ret;
   } else {
     jam();
@@ -1298,8 +1352,7 @@ bool Dbtup::readTriggerInfo(TupTriggerData* const trigPtr,
     else
     {
       req_struct->m_tuple_ptr =
-        get_copy_tuple(regTabPtr,
-                       &req_struct->prevOpPtr.p->m_copy_tuple_location);
+        get_copy_tuple(&req_struct->prevOpPtr.p->m_copy_tuple_location);
     }
 
     if (regTabPtr->need_expand(disk)) 
@@ -1312,7 +1365,7 @@ bool Dbtup::readTriggerInfo(TupTriggerData* const trigPtr,
 			     ZATTR_BUFFER_SIZE,
 			     false);
     req_struct->m_tuple_ptr= save;
-    ndbrequire(ret != -1);
+    ndbrequire(ret >= 0);
     noBeforeWords = ret;
     if (refToMain(trigPtr->m_receiverRef) != SUMA &&
         (noAfterWords == noBeforeWords) &&
@@ -1366,17 +1419,6 @@ void Dbtup::sendTrigAttrInfo(Signal* signal,
   } while (dataLen != dataIndex);
 }
 
-void Dbtup::sendFireTrigOrd(Signal* signal,
-                            KeyReqStruct *req_struct,
-                            Operationrec * const regOperPtr, 
-                            TupTriggerData* const trigPtr, 
-			    Uint32 fragmentId,
-                            Uint32 noPrimKeyWords, 
-                            Uint32 noBeforeValueWords, 
-                            Uint32 noAfterValueWords)
-{
-}
-
 /*
  * Ordered index triggers.
  *
@@ -1579,3 +1621,119 @@ Dbtup::removeTuxEntries(Signal* signal,
     triggerList.next(triggerPtr);
   }
 }
+
+void
+Dbtup::ndbmtd_buffer_suma_trigger(Signal * signal,
+                                  Uint32 len,
+                                  LinearSectionPtr sec[3])
+{
+  jam();
+  Uint32 tot = len + 5;
+  for (Uint32 i = 0; i<3; i++)
+    tot += sec[i].sz;
+
+  Uint32 * ptr = 0;
+  Uint32 free = m_suma_trigger_buffer.m_freeWords;
+  Uint32 pageId = m_suma_trigger_buffer.m_pageId;
+  Uint32 oom = m_suma_trigger_buffer.m_out_of_memory;
+  if (free < tot)
+  {
+    jam();
+    if (pageId != RNIL)
+    {
+      flush_ndbmtd_suma_buffer(signal);
+    }
+    if (oom == 0)
+    {
+      jam();
+      ndbassert(m_suma_trigger_buffer.m_pageId == RNIL);
+      void * vptr = m_ctx.m_mm.alloc_page(RT_DBTUP_PAGE,
+                                          &m_suma_trigger_buffer.m_pageId,
+                                          Ndbd_mem_manager::NDB_ZONE_ANY);
+      ptr = reinterpret_cast<Uint32*>(vptr);
+      free = GLOBAL_PAGE_SIZE_WORDS - tot;
+    }
+  }
+  else
+  {
+    jam();
+    ptr = reinterpret_cast<Uint32*>(c_page_pool.getPtr(pageId));
+    ptr += (GLOBAL_PAGE_SIZE_WORDS - free);
+    free -= tot;
+  }
+
+  if (likely(ptr != 0))
+  {
+    jam();
+    * ptr++ = tot;
+    * ptr++ = len;
+    * ptr++ = sec[0].sz;
+    * ptr++ = sec[1].sz;
+    * ptr++ = sec[2].sz;
+    memcpy(ptr, signal->getDataPtrSend(), 4 * len);
+    ptr += len;
+    for (Uint32 i = 0; i<3; i++)
+    {
+      memcpy(ptr, sec[i].p, 4 * sec[i].sz);
+      ptr += sec[i].sz;
+    }
+
+    m_suma_trigger_buffer.m_freeWords = free;
+    if (free < (len + 5))
+    {
+      flush_ndbmtd_suma_buffer(signal);
+    }
+  }
+  else
+  {
+    jam();
+    m_suma_trigger_buffer.m_out_of_memory = 1;
+  }
+}
+
+void
+Dbtup::flush_ndbmtd_suma_buffer(Signal* signal)
+{
+  jam();
+
+  Uint32 pageId = m_suma_trigger_buffer.m_pageId;
+  Uint32 free = m_suma_trigger_buffer.m_freeWords;
+  Uint32 oom = m_suma_trigger_buffer.m_out_of_memory;
+
+  if (pageId != RNIL)
+  {
+    jam();
+    Uint32 save[2];
+    save[0] = signal->theData[0];
+    save[1] = signal->theData[1];
+    signal->theData[0] = pageId;
+    signal->theData[1] =  GLOBAL_PAGE_SIZE_WORDS - free;
+    sendSignal(SUMA_REF, GSN_FIRE_TRIG_ORD_L, signal, 2, JBB);
+
+    signal->theData[0] = save[0];
+    signal->theData[1] = save[1];
+  }
+  else if (oom)
+  {
+    jam();
+    Uint32 save[2];
+    save[0] = signal->theData[0];
+    save[1] = signal->theData[1];
+    signal->theData[0] = RNIL;
+    signal->theData[1] =  0;
+    sendSignal(SUMA_REF, GSN_FIRE_TRIG_ORD_L, signal, 2, JBB);
+
+    signal->theData[0] = save[0];
+    signal->theData[1] = save[1];
+  }
+
+  m_suma_trigger_buffer.m_pageId = RNIL;
+  m_suma_trigger_buffer.m_freeWords = 0;
+  m_suma_trigger_buffer.m_out_of_memory = 0;
+}
+
+void
+Dbtup::execSUB_GCP_COMPLETE_REP(Signal* signal)
+{
+  flush_ndbmtd_suma_buffer(signal);
+}
