@@ -45,8 +45,6 @@
 
 #include <signaldata/DbinfoScan.hpp>
 #include <signaldata/TransIdAI.hpp>
-#include <ndbinfo.h>
-#include <dbinfo/ndbinfo_tableids.h>
 
 /**************************************************************************
  * ------------------------------------------------------------------------
@@ -761,65 +759,98 @@ DbUtil::execDUMP_STATE_ORD(Signal* signal){
 void DbUtil::execDBINFO_SCANREQ(Signal *signal)
 {
   DbinfoScanReq req= *(DbinfoScanReq*)signal->theData;
-  char buf[512];
-  struct dbinfo_row r;
-  struct dbinfo_ratelimit rl;
-
-  dbinfo_ratelimit_init(&rl, &req);
+  const Ndbinfo::ScanCursor* cursor =
+    CAST_CONSTPTR(Ndbinfo::ScanCursor, DbinfoScan::getCursorPtr(&req));
+  Ndbinfo::Ratelimit rl;
 
   jamEntry();
 
-  if(req.tableId == NDBINFO_POOLS_TABLEID)
+  switch(req.tableId){
+  case Ndbinfo::POOLS_TABLEID:
   {
-    struct {
-      const char* poolname;
-      Uint32 free;
-      Uint32 size;
-    } pools[] =
-        {
-          {"Page",
-           c_pagePool.getNoOfFree(),
-           c_pagePool.getSize() },
-          {"Prepare",
-           c_preparePool.getNoOfFree(),
-           c_preparePool.getSize() },
-          {"Prepared Operation",
-           c_preparedOperationPool.getNoOfFree(),
-           c_preparedOperationPool.getSize() },
-          {"Operation",
-           c_operationPool.getNoOfFree(),
-           c_operationPool.getSize() },
-          {"Transaction",
-           c_transactionPool.getNoOfFree(),
-           c_transactionPool.getSize() },
-          {"Attribute Mapping",
-           c_attrMappingPool.getNoOfFree(),
-           c_attrMappingPool.getSize() },
-          {"Data Buffer",
-           c_dataBufPool.getNoOfFree(),
-           c_dataBufPool.getSize() },
-          { NULL, 0, 0}
-        };
-
-    for(int i=0; pools[i].poolname; i++)
+    Ndbinfo::pool_entry pools[] =
     {
-      dbinfo_write_row_init(&r, buf, sizeof(buf));
-      dbinfo_write_row_column_uint32(&r, getOwnNodeId());
-      const char *blockname= "DBUTIL";
-      dbinfo_write_row_column(&r, blockname, strlen(blockname));
-      dbinfo_write_row_column(&r, pools[i].poolname, strlen(pools[i].poolname));
-      dbinfo_write_row_column_uint32(&r, pools[i].free);
-      dbinfo_write_row_column_uint32(&r, pools[i].size);
-      dbinfo_send_row(signal, r, rl, req.apiTxnId, req.senderRef);
+      { "Page",
+        c_pagePool.getUsed(),
+        c_pagePool.getSize(),
+        c_pagePool.getEntrySize(),
+        c_pagePool.getUsedHi(),
+        { 0,0,0,0 }},
+      { "Prepare",
+        c_preparePool.getUsed(),
+        c_preparePool.getSize(),
+        c_preparePool.getEntrySize(),
+        c_preparePool.getUsedHi(),
+        { 0,0,0,0 }},
+      { "Prepared Operation",
+        c_preparedOperationPool.getUsed(),
+        c_preparedOperationPool.getSize(),
+        c_preparedOperationPool.getEntrySize(),
+        c_preparedOperationPool.getUsedHi(),
+        { 0,0,0,0 }},
+      { "Operation",
+        c_operationPool.getUsed(),
+        c_operationPool.getSize(),
+        c_operationPool.getEntrySize(),
+        c_operationPool.getUsedHi(),
+        { 0,0,0,0 }},
+      { "Transaction",
+        c_transactionPool.getUsed(),
+        c_transactionPool.getSize(),
+        c_transactionPool.getEntrySize(),
+        c_transactionPool.getUsedHi(),
+        { 0,0,0,0 }},
+      { "Attribute Mapping",
+        c_attrMappingPool.getUsed(),
+        c_attrMappingPool.getSize(),
+        c_attrMappingPool.getEntrySize(),
+        c_attrMappingPool.getUsedHi(),
+        { 0,0,0,0 }},
+      { "Data Buffer",
+        c_dataBufPool.getUsed(),
+        c_dataBufPool.getSize(),
+        c_dataBufPool.getEntrySize(),
+        c_dataBufPool.getUsedHi(),
+        { 0,0,0,0 }},
+      { NULL, 0,0,0,0, { 0,0,0,0 }}
+    };
+
+    const size_t num_config_params =
+      sizeof(pools[0].config_params) / sizeof(pools[0].config_params[0]);
+    Uint32 pool = cursor->data[0];
+    BlockNumber bn = blockToMain(number());
+    while(pools[pool].poolname)
+    {
+      jam();
+      Ndbinfo::Row row(signal, req);
+      row.write_uint32(getOwnNodeId());
+      row.write_uint32(bn);           // block number
+      row.write_uint32(instance());   // block instance
+      row.write_string(pools[pool].poolname);
+      row.write_uint64(pools[pool].used);
+      row.write_uint64(pools[pool].total);
+      row.write_uint64(pools[pool].used_hi);
+      row.write_uint64(pools[pool].entry_size);
+      for (size_t i = 0; i < num_config_params; i++)
+        row.write_uint32(pools[pool].config_params[i]);
+      ndbinfo_send_row(signal, req, row, rl);
+      pool++;
+      if (rl.need_break(req))
+      {
+        jam();
+        ndbinfo_send_scan_break(signal, req, rl, pool);
+        return;
+      }
     }
+    break;
+  }
+  default:
+    break;
   }
 
-  DbinfoScanConf *conf= (DbinfoScanConf*)signal->getDataPtrSend();
-  memcpy(conf,&req, DbinfoScanReq::SignalLengthWithCursor * sizeof(Uint32));
-  conf->requestInfo &= ~(DbinfoScanConf::MoreData);
-  sendSignal(DBINFO_REF, GSN_DBINFO_SCANCONF,
-             signal, DbinfoScanConf::SignalLengthWithCursor, JBB);
+  ndbinfo_send_scan_conf(signal, req, rl);
 }
+
 void
 DbUtil::mutex_created(Signal* signal, Uint32 ptrI, Uint32 retVal){
   MutexManager::ActiveMutexPtr ptr; ptr.i = ptrI;
@@ -916,10 +947,12 @@ DbUtil::execUTIL_RELEASE_REF(Signal* signal) {
 
 void
 DbUtil::sendUtilPrepareRef(Signal* signal, UtilPrepareRef::ErrorCode error, 
-			   Uint32 recipient, Uint32 senderData){
+			   Uint32 recipient, Uint32 senderData,
+                           Uint32 errCode2){
   UtilPrepareRef * ref = (UtilPrepareRef *)signal->getDataPtrSend();
   ref->errorCode = error;
   ref->senderData = senderData;
+  ref->dictErrCode = errCode2;
 
   sendSignal(recipient, GSN_UTIL_PREPARE_REF, signal, 
 	     UtilPrepareRef::SignalLength, JBB);
@@ -1156,7 +1189,8 @@ DbUtil::execGET_TABINFOREF(Signal* signal){
   c_runningPrepares.getPtr(prepPtr, prepI);
 
   sendUtilPrepareRef(signal, UtilPrepareRef::DICT_TAB_INFO_ERROR,
-		     prepPtr.p->clientRef, prepPtr.p->clientData);
+		     prepPtr.p->clientRef, prepPtr.p->clientData,
+                     ref->errorCode);
 
   releasePrepare(prepPtr);
 }
@@ -1774,7 +1808,6 @@ DbUtil::execUTIL_SEQUENCE_REQ(Signal* signal){
   OperationPtr opPtr;
   ndbrequire(transPtr.p->operations.seize(opPtr));
   
-  ndbrequire(opPtr.p->rs.seize(prepOp->rsLen));
   ndbrequire(opPtr.p->keyInfo.seize(1));
 
   transPtr.p->gci_hi = 0;
@@ -1848,7 +1881,7 @@ DbUtil::getResultSet(Signal* signal, const Transaction * transP,
   // extract headers
   for(rs.first(it); it.curr.i != RNIL; ) {
     *tmpBuf++ = it.data[0];
-    rs.next(it, ((AttributeHeader*)&it.data[0])->getDataSize() + 1);
+    rs.next(it, AttributeHeader::getDataSize(it.data[0]) + 1);
     noAttr++;
   }
 
@@ -1858,10 +1891,13 @@ DbUtil::getResultSet(Signal* signal, const Transaction * transP,
   const Uint32* dataBuffer = tmpBuf;
 
   // extract data
-  for(rs.first(it); it.curr.i != RNIL; ) {
-    int sz = ((AttributeHeader*)&it.data[0])->getDataSize();
+  for(rs.first(it); it.curr.i != RNIL; )
+  {
+    jam();
+    int sz = AttributeHeader::getDataSize(it.data[0]);
     rs.next(it,1);
-    for (int i = 0; i < sz; i++) {
+    for (int i = 0; i < sz; i++)
+    {
       *tmpBuf++ = *it.data;
       rs.next(it,1);
       dataSz++;
@@ -2048,11 +2084,6 @@ DbUtil::execUTIL_EXECUTE_REQ(Signal* signal)
   opPtr.p->prepOp_i = prepOpPtr.i;
   opPtr.p->m_scanTakeOver = scanTakeOver;
 
-#if 0 //def EVENT_DEBUG
-  printf("opPtr.p->rs.seize( %u )\n", prepOpPtr.p->rsLen);
-#endif
-  ndbrequire(opPtr.p->rs.seize(prepOpPtr.p->rsLen));
-  
  /***********************************************************
    * Store signal data on linear memory in Transaction record 
    ***********************************************************/
@@ -2194,8 +2225,6 @@ DbUtil::runOperation(Signal* signal, TransactionPtr & transPtr,
   /**
    * Init operation w.r.t result set
    */
-  initResultSet(op->rs, pop->rsInfo);
-  op->rs.first(op->rsIterator);
   op->rsRecv = 0;
 #if 0 //def EVENT_DEBUG
   printf("pop->rsLen %u\n", pop->rsLen);
@@ -2327,27 +2356,6 @@ DbUtil::sendAttrInfo(Signal* signal,
   }
 }
 
-void
-DbUtil::initResultSet(ResultSetBuffer & rs, 
-		      const ResultSetInfoBuffer & rsi){
-  
-  ResultSetBuffer::DataBufferIterator rsit;
-  rs.first(rsit);
-  
-  ResultSetInfoBuffer::ConstDataBufferIterator rsiit;
-  for(rsi.first(rsiit); rsiit.curr.i != RNIL; rsi.next(rsiit)){
-    ndbrequire(rsit.curr.i != RNIL);
-    
-    rsit.data[0] = rsiit.data[0];
-#if 0 //def EVENT_DEBUG
-    printf("Init resultset %u, sz %d\n",
-	   rsit.curr.i,
-	   ((AttributeHeader*)&rsit.data[0])->getDataSize() + 1);
-#endif
-    rs.next(rsit, ((AttributeHeader*)&rsit.data[0])->getDataSize() + 1);
-  }
-}
-
 void
 DbUtil::getTransId(Transaction * transP){
 
@@ -2415,30 +2423,24 @@ DbUtil::execTRANSID_AI(Signal* signal){
   /**
    * Save result
    */
-  ResultSetBuffer::DataBufferIterator rs = opP->rsIterator;
-
   if (longSignal)
   {
     SectionSegment * ptrP = dataPtr.p;
     while (dataLen > NDB_SECTION_SEGMENT_SZ)
     {
-      ndbrequire(opP->rs.import(rs, &ptrP->theData[0], NDB_SECTION_SEGMENT_SZ));
-      opP->rs.next(rs, NDB_SECTION_SEGMENT_SZ);
+      ndbrequire(opP->rs.append(ptrP->theData, NDB_SECTION_SEGMENT_SZ));
       dataLen -= NDB_SECTION_SEGMENT_SZ;
       ptrP = g_sectionSegmentPool.getPtr(ptrP->m_nextSegment);
     }
-    ndbrequire(opP->rs.import(rs, &ptrP->theData[0], dataLen));
-    opP->rs.next(rs, dataLen);
+    ndbrequire(opP->rs.append(ptrP->theData, dataLen));
 
     releaseSections(handle);
   }
   else
   {
     const Uint32 *src = &signal->theData[3];
-    ndbrequire(opP->rs.import(rs,src,dataLen));
-    opP->rs.next(rs, dataLen);
+    ndbrequire(opP->rs.append(src, dataLen));
   }
-  opP->rsIterator = rs;
 
   if(!opP->complete()){
    jam();
