@@ -30,6 +30,7 @@
 #include <signaldata/LgmanContinueB.hpp>
 #include <signaldata/GetTabInfo.hpp>
 #include <signaldata/NodeFailRep.hpp>
+#include <signaldata/DbinfoScan.hpp>
 #include "dbtup/Dbtup.hpp"
 
 #include <EventLogger.hpp>
@@ -69,6 +70,7 @@ Lgman::Lgman(Block_context & ctx) :
   addRecSignal(GSN_STTOR, &Lgman::execSTTOR);
   addRecSignal(GSN_READ_CONFIG_REQ, &Lgman::execREAD_CONFIG_REQ);
   addRecSignal(GSN_DUMP_STATE_ORD, &Lgman::execDUMP_STATE_ORD);
+  addRecSignal(GSN_DBINFO_SCANREQ, &Lgman::execDBINFO_SCANREQ);
   addRecSignal(GSN_CONTINUEB, &Lgman::execCONTINUEB);
   addRecSignal(GSN_NODE_FAILREP, &Lgman::execNODE_FAILREP);
 
@@ -420,6 +422,134 @@ Lgman::execDUMP_STATE_ORD(Signal* signal){
   }
 }
 
+void
+Lgman::execDBINFO_SCANREQ(Signal *signal)
+{
+  DbinfoScanReq req= *(DbinfoScanReq*)signal->theData;
+  const Ndbinfo::ScanCursor* cursor = 
+    CAST_CONSTPTR(Ndbinfo::ScanCursor, DbinfoScan::getCursorPtr(&req));
+  Ndbinfo::Ratelimit rl;
+
+  jamEntry();
+
+  switch(req.tableId) {
+  case Ndbinfo::LOGSPACES_TABLEID:
+  {
+    jam();
+    Uint32 startBucket = cursor->data[0];
+    Logfile_group_hash_iterator iter;
+    m_logfile_group_hash.next(startBucket, iter);
+
+    while (!iter.curr.isNull())
+    {
+      jam();
+
+      Uint32 currentBucket = iter.bucket;
+      Ptr<Logfile_group> ptr = iter.curr;
+
+      Uint64 free = ptr.p->m_free_file_words*4;
+
+      Uint64 total = 0;
+      Local_undofile_list list(m_file_pool, ptr.p->m_files);
+      Ptr<Undofile> filePtr;
+      for (list.first(filePtr); !filePtr.isNull(); list.next(filePtr))
+      {
+        jam();
+        total += (Uint64)filePtr.p->m_file_size *
+          (Uint64)File_formats::NDB_PAGE_SIZE;
+      }
+
+      Uint64 high = 0; // TODO
+
+      Ndbinfo::Row row(signal, req);
+      row.write_uint32(getOwnNodeId());
+      row.write_uint32(1); // log type, 1 = DD-UNDO
+      row.write_uint32(ptr.p->m_logfile_group_id); // log id
+      row.write_uint32(0); // log part
+
+      row.write_uint64(total);          // total allocated
+      row.write_uint64((total-free));   // currently in use
+      row.write_uint64(high);           // in use high water mark
+      ndbinfo_send_row(signal, req, row, rl);
+
+      // move to next
+      if (m_logfile_group_hash.next(iter) == false)
+      {
+        jam(); // no more...
+        break;
+      }
+      else if (iter.bucket == currentBucket)
+      {
+        jam();
+        continue; // we need to iterate an entire bucket
+      }
+      else if (rl.need_break(req))
+      {
+        jam();
+        ndbinfo_send_scan_break(signal, req, rl, iter.bucket);
+        return;
+      }
+    }
+    break;
+  }
+
+  case Ndbinfo::LOGBUFFERS_TABLEID:
+  {
+    jam();
+    Uint32 startBucket = cursor->data[0];
+    Logfile_group_hash_iterator iter;
+    m_logfile_group_hash.next(startBucket, iter);
+
+    while (!iter.curr.isNull())
+    {
+      jam();
+
+      Uint32 currentBucket = iter.bucket;
+      Ptr<Logfile_group> ptr = iter.curr;
+
+      Uint64 free = ptr.p->m_free_buffer_words*4;
+      Uint64 total = ptr.p->m_total_buffer_words*4;
+      Uint64 high = 0; // TODO
+
+      Ndbinfo::Row row(signal, req);
+      row.write_uint32(getOwnNodeId());
+      row.write_uint32(1); // log type, 1 = DD-UNDO
+      row.write_uint32(ptr.p->m_logfile_group_id); // log id
+      row.write_uint32(0); // log part
+
+      row.write_uint64(total);          // total allocated
+      row.write_uint64((total-free));   // currently in use
+      row.write_uint64(high);           // in use high water mark
+      ndbinfo_send_row(signal, req, row, rl);
+
+      // move to next
+      if (m_logfile_group_hash.next(iter) == false)
+      {
+        jam(); // no more...
+        break;
+      }
+      else if (iter.bucket == currentBucket)
+      {
+        jam();
+        continue; // we need to iterate an entire bucket
+      }
+      else if (rl.need_break(req))
+      {
+        jam();
+        ndbinfo_send_scan_break(signal, req, rl, iter.bucket);
+        return;
+      }
+    }
+    break;
+  }
+
+  default:
+    break;
+  }
+
+  ndbinfo_send_scan_conf(signal, req, rl);
+}
+
 void
 Lgman::execCREATE_FILEGROUP_IMPL_REQ(Signal* signal){
   jamEntry();
@@ -1021,6 +1151,7 @@ Lgman::Logfile_group::Logfile_group(const CreateFilegroupImplReq* req)
   m_file_pos[0].m_ptr_i= m_file_pos[1].m_ptr_i = RNIL;
 
   m_free_file_words = 0;
+  m_total_buffer_words = 0;
   m_free_buffer_words = 0;
   m_callback_buffer_words = 0;
 
@@ -1087,6 +1218,9 @@ Lgman::alloc_logbuffer_memory(Ptr<Logfile_group> ptr, Uint32 bytes)
 		 " , requested %d pages", 
 		 (requested-pages), ptr.p->m_logfile_group_id, requested);
   }
+#if defined VM_TRACE || defined ERROR_INSERT
+  ndbout << "DD lgman: fg id:" << ptr.p->m_logfile_group_id << " undo buffer pages/bytes:" << (requested-pages) << "/" << (requested-pages)*File_formats::NDB_PAGE_SIZE << endl;
+#endif
   
   init_logbuffer_pointers(ptr);
   return true;
@@ -1126,7 +1260,8 @@ Lgman::init_logbuffer_pointers(Ptr<Logfile_group> ptr)
     pages += range.m_idx;
   }
   
-  ptr.p->m_free_buffer_words = pages * File_formats::UNDO_PAGE_WORDS;
+  ptr.p->m_total_buffer_words =
+    ptr.p->m_free_buffer_words = pages * File_formats::UNDO_PAGE_WORDS;
 }
 
 Uint32
@@ -1192,6 +1327,9 @@ Lgman::Undofile::Undofile(const struct CreateFileImplReq* req, Uint32 ptrI)
   pages = (pages << 32) | req->file_size_lo;
   pages /= GLOBAL_PAGE_SIZE;
   m_file_size = Uint32(pages);
+#if defined VM_TRACE || defined ERROR_INSERT
+  ndbout << "DD lgman: file id:" << m_file_id << " undofile pages/bytes:" << m_file_size << "/" << m_file_size*GLOBAL_PAGE_SIZE << endl;
+#endif
 
   m_create.m_senderRef = req->senderRef; // During META
   m_create.m_senderData = req->senderData; // During META
@@ -1388,7 +1526,7 @@ Lgman::get_log_buffer(Ptr<Logfile_group> ptr, Uint32 sz)
   {
 next:
     // fits this page wo/ problem
-    ndbrequire(total_free > sz);
+    ndbrequire(total_free >= sz);
     ptr.p->m_free_buffer_words = total_free - sz;
     ptr.p->m_pos[PRODUCER].m_current_pos.m_idx = pos + sz;
     return ((File_formats::Undofile::Undo_page*)page)->m_data + pos;
@@ -3112,7 +3250,9 @@ Lgman::get_next_undo_record(Uint64 * this_lsn)
     prev = pageP->m_data + pos - 1;
     
     if(((* prev) >> 16) & File_formats::Undofile::UNDO_NEXT_LSN)
+    {
       ndbrequire(lsn + 1 == ptr.p->m_last_read_lsn);
+    }
 
     ptr.p->m_pos[CONSUMER] = consumer;
     ptr.p->m_free_buffer_words += File_formats::UNDO_PAGE_WORDS;
