@@ -27,8 +27,6 @@
 #include "PosixAsyncFile.hpp"
 #endif
 
-#include "Filename.hpp"
-
 #include <signaldata/FsOpenReq.hpp>
 #include <signaldata/FsCloseReq.hpp>
 #include <signaldata/FsReadWriteReq.hpp>
@@ -41,7 +39,7 @@
 #include <signaldata/AllocMem.hpp>
 
 #include <RefConvert.hpp>
-#include <NdbSleep.h>
+#include <portlib/NdbDir.hpp>
 #include <NdbOut.hpp>
 #include <Configuration.hpp>
 
@@ -80,7 +78,9 @@ Ndbfs::Ndbfs(Block_context& ctx) :
   addRecSignal(GSN_FSREMOVEREQ, &Ndbfs::execFSREMOVEREQ);
   addRecSignal(GSN_ALLOC_MEM_REQ, &Ndbfs::execALLOC_MEM_REQ);
   addRecSignal(GSN_SEND_PACKED, &Ndbfs::execSEND_PACKED, true);
+  addRecSignal(GSN_BUILD_INDX_IMPL_REQ, &Ndbfs::execBUILD_INDX_IMPL_REQ);
    // Set send signals
+  addRecSignal(GSN_FSSUSPENDORD, &Ndbfs::execFSSUSPENDORD);
 
   theRequestPool = new Pool<Request>;
 }
@@ -136,11 +136,9 @@ static
 bool
 do_mkdir(const char * path)
 {
-#ifdef NDB_WIN32
-  return CreateDirectory(path, 0);
-#else
-  return ::mkdir(path, S_IRUSR | S_IWUSR | S_IXUSR | S_IXGRP | S_IRGRP) == 0;
-#endif
+  return NdbDir::create(path,
+                        NdbDir::u_rwx() | NdbDir::g_r() | NdbDir::g_x(),
+                        true /* ignore_existing */);
 }
 
 static
@@ -398,7 +396,7 @@ Ndbfs::execFSOPENREQ(Signal* signal)
       return;
     }
     m_shared_page_pool.getPtr(page_ptr);
-    file->set_buffer(page_ptr, cnt);
+    file->set_buffer(RT_DBTUP_PAGE, page_ptr, cnt);
   } 
   else if (fsOpenReq->fileFlags & FsOpenReq::OM_WRITE_BUFFER)
   {
@@ -406,8 +404,9 @@ Ndbfs::execFSOPENREQ(Signal* signal)
     Uint32 cnt = NDB_FILE_BUFFER_SIZE / GLOBAL_PAGE_SIZE; // 256k
     Ptr<GlobalPage> page_ptr;
     m_ctx.m_mm.alloc_pages(RT_FILE_BUFFER, &page_ptr.i, &cnt, 1);
-    if(cnt == 0)
+    if (cnt == 0)
     {
+      jam();
       file->m_page_ptr.setNull();
       file->m_page_cnt = 0;
 
@@ -419,7 +418,7 @@ Ndbfs::execFSOPENREQ(Signal* signal)
       return;
     }
     m_shared_page_pool.getPtr(page_ptr);
-    file->set_buffer(page_ptr, cnt);
+    file->set_buffer(RT_FILE_BUFFER, page_ptr, cnt);
   }
   else
   {
@@ -614,7 +613,7 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
 	}//if
 	request->par.readWrite.pages[i].buf = &tWA[varIndex * tClusterSize];
 	request->par.readWrite.pages[i].size = tPageSize;
-	request->par.readWrite.pages[i].offset = fileOffset * tPageSize;
+	request->par.readWrite.pages[i].offset = (off_t)(fileOffset*tPageSize);
       }//for
       request->par.readWrite.numberOfPages = fsRWReq->numberOfPages;
       break;
@@ -630,7 +629,7 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
       const UintPtr varIndex = fsRWReq->data.arrayOfPages.varIndex;
       const UintPtr fileOffset = fsRWReq->data.arrayOfPages.fileOffset;
       
-      request->par.readWrite.pages[0].offset = fileOffset * tPageSize;
+      request->par.readWrite.pages[0].offset = (off_t)(fileOffset * tPageSize);
       request->par.readWrite.pages[0].size = tPageSize * fsRWReq->numberOfPages;
       request->par.readWrite.numberOfPages = 1;
       request->par.readWrite.pages[0].buf = &tWA[varIndex * tPageSize];
@@ -654,7 +653,8 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
 	}//if
 	request->par.readWrite.pages[i].buf = &tWA[varIndex * tClusterSize];
 	request->par.readWrite.pages[i].size = tPageSize;
-	request->par.readWrite.pages[i].offset = tPageOffset + (i*tPageSize);
+	request->par.readWrite.pages[i].offset = (off_t)
+          (tPageOffset + (i*tPageSize));
       }//for
       request->par.readWrite.numberOfPages = fsRWReq->numberOfPages;
       break;
@@ -789,6 +789,34 @@ Ndbfs::execFSSYNCREQ(Signal * signal)
   ndbrequire(forward(openFile,request));
 }
 
+/*
+ * PR0: File Pointer DR0: User reference DR1: User Pointer
+ */
+void
+Ndbfs::execFSSUSPENDORD(Signal * signal)
+{
+  jamEntry();
+  Uint16 filePointer =  (Uint16)signal->theData[0];
+  Uint32 millis = signal->theData[1];
+  AsyncFile* openFile = theOpenFiles.find(filePointer);
+
+  if (openFile == NULL)
+  {
+    jam(); //file not open
+    return;
+  }
+
+  Request *request = theRequestPool->get();
+  request->error = 0;
+  request->action = Request::suspend;
+  request->set(0, 0, filePointer);
+  request->file = openFile;
+  request->theTrace = signal->getTrace();
+  request->par.suspend.milliseconds = millis;
+
+  ndbrequire(forward(openFile,request));
+}
+
 void 
 Ndbfs::execFSAPPENDREQ(Signal * signal)
 {
@@ -887,6 +915,46 @@ Ndbfs::execALLOC_MEM_REQ(Signal* signal)
   ndbrequire(forward(file, request));
 }
 
+#include <signaldata/BuildIndxImpl.hpp>
+
+void
+Ndbfs::execBUILD_INDX_IMPL_REQ(Signal* signal)
+{
+  jamEntry();
+  mt_BuildIndxReq * req = (mt_BuildIndxReq*)signal->getDataPtr();
+
+  AsyncFile* file = getIdleFile(true);
+  ndbrequire(file != NULL);
+
+  Request *request = theRequestPool->get();
+  request->error = 0;
+  request->set(req->senderRef, req->senderData, 0);
+  request->file = file;
+  request->theTrace = signal->getTrace();
+
+  Uint32 cnt = (req->buffer_size + 32768 - 1) / 32768;
+  Uint32 save = cnt;
+  Ptr<GlobalPage> page_ptr;
+  m_ctx.m_mm.alloc_pages(RT_DBTUP_PAGE, &page_ptr.i, &cnt, cnt);
+  if(cnt == 0)
+  {
+    file->m_page_ptr.setNull();
+    file->m_page_cnt = 0;
+
+    ndbrequire(false); // TODO
+    return;
+  }
+
+  ndbrequire(cnt == save);
+
+  m_shared_page_pool.getPtr(page_ptr);
+  file->set_buffer(RT_DBTUP_PAGE, page_ptr, cnt);
+
+  memcpy(&request->par.build.m_req, req, sizeof(* req));
+  request->action = Request::buildindx;
+  ndbrequire(forward(file, request));
+}
+
 Uint16
 Ndbfs::newId()
 {
@@ -1012,27 +1080,18 @@ Ndbfs::report(Request * request, Signal* signal)
   signal->setTrace(request->theTrace);
   const BlockReference ref = request->theUserReference;
 
-  if(request->file->has_buffer())
+  if (request->file->has_buffer())
   {
-    Uint32 cnt;
-    Ptr<GlobalPage> ptr;
-    if (request->file->m_open_flags & FsOpenReq::OM_INIT)
+    if ((request->action == Request::open && request->error) ||
+        request->action == Request::close ||
+        request->action == Request::closeRemove ||
+        request->action == Request::buildindx)
     {
-      jam();
-      request->file->clear_buffer(ptr, cnt);
-      m_ctx.m_mm.release_pages(RT_DBTUP_PAGE, ptr.i, cnt);
-    }
-    else if (request->file->m_open_flags & FsOpenReq::OM_WRITE_BUFFER)
-    {
-      jam();
-      if ((request->action == Request::open && request->error) ||
-          (request->action == Request::close ||
-           request->action == Request::closeRemove))
-      {
-        jam();
-        request->file->clear_buffer(ptr, cnt);
-        m_ctx.m_mm.release_pages(RT_FILE_BUFFER, ptr.i, cnt);
-      }
+      Uint32 rg;
+      Uint32 cnt;
+      Ptr<GlobalPage> ptr;
+      request->file->clear_buffer(rg, ptr, cnt);
+      m_ctx.m_mm.release_pages(rg, ptr.i, cnt);
     }
   }
   
@@ -1101,6 +1160,7 @@ Ndbfs::report(Request * request, Signal* signal)
     }
     
     case Request:: end: {
+    case Request:: suspend:
       // Report nothing
       break;
     }
@@ -1115,6 +1175,17 @@ Ndbfs::report(Request * request, Signal* signal)
       pushIdleFile(request->file);
       break;
     }
+    case Request::buildindx: {
+      jam();
+      BuildIndxImplRef* rep = (BuildIndxImplRef*)signal->getDataPtrSend();
+      rep->senderRef = reference();
+      rep->senderData = request->theUserPointer;
+      rep->errorCode = (BuildIndxImplRef::ErrorCode)request->error;
+      sendSignal(ref, GSN_BUILD_INDX_IMPL_REF, signal,
+                 BuildIndxImplRef::SignalLength, JBB);
+      pushIdleFile(request->file);
+      break;
+    }
     }//switch
   } else {
     jam();
@@ -1159,7 +1230,7 @@ Ndbfs::report(Request * request, Signal* signal)
     }
     case Request:: readPartial: {
       jam();
-      fsConf->bytes_read = request->par.readWrite.pages[0].size;
+      fsConf->bytes_read = Uint32(request->par.readWrite.pages[0].size);
       sendSignal(ref, GSN_FSREADCONF, signal, 2, JBA);
       break;
     }
@@ -1172,7 +1243,7 @@ Ndbfs::report(Request * request, Signal* signal)
     case Request::append_synch:
     {
       jam();
-      signal->theData[1] = request->par.append.size;
+      signal->theData[1] = Uint32(request->par.append.size);
       sendSignal(ref, GSN_FSAPPENDCONF, signal, 2, JBA);
       break;
     }
@@ -1184,6 +1255,7 @@ Ndbfs::report(Request * request, Signal* signal)
       break;
     }
     case Request:: end: {
+    case Request:: suspend:
       // Report nothing
       break;
     }
@@ -1197,6 +1269,16 @@ Ndbfs::report(Request * request, Signal* signal)
       pushIdleFile(request->file);
       break;
     }
+    case Request::buildindx: {
+      jam();
+      BuildIndxImplConf* rep = (BuildIndxImplConf*)signal->getDataPtrSend();
+      rep->senderRef = reference();
+      rep->senderData = request->theUserPointer;
+      sendSignal(ref, GSN_BUILD_INDX_IMPL_CONF, signal,
+                 BuildIndxImplConf::SignalLength, JBB);
+      pushIdleFile(request->file);
+      break;
+    }
     }    
   }//if
   signal->setTrace(orgTrace);
