@@ -50,6 +50,10 @@ Pgman::Pgman(Block_context& ctx, Uint32 instanceNumber) :
   m_page_hashlist(m_page_entry_pool),
   m_page_stack(m_page_entry_pool),
   m_page_queue(m_page_entry_pool)
+#ifdef VM_TRACE
+  ,debugFlag(false)
+  ,debugSummaryFlag(false)
+#endif
 {
   BLOCK_CONSTRUCTOR(Pgman);
 
@@ -68,14 +72,15 @@ Pgman::Pgman(Block_context& ctx, Uint32 instanceNumber) :
 
   addRecSignal(GSN_DATA_FILE_ORD, &Pgman::execDATA_FILE_ORD);
   addRecSignal(GSN_RELEASE_PAGES_REQ, &Pgman::execRELEASE_PAGES_REQ);
+  addRecSignal(GSN_DBINFO_SCANREQ, &Pgman::execDBINFO_SCANREQ);
   
   // loop status
   m_stats_loop_on = false;
   m_busy_loop_on = false;
   m_cleanup_loop_on = false;
-  m_lcp_loop_on = false;
 
   // LCP variables
+  m_lcp_state = LS_LCP_OFF;
   m_last_lcp = 0;
   m_last_lcp_complete = 0;
   m_lcp_curr_bucket = ~(Uint32)0;
@@ -147,9 +152,17 @@ Pgman::execREAD_CONFIG_REQ(Signal* signal)
     }
     // convert to pages
     Uint32 page_cnt = Uint32((page_buffer + GLOBAL_PAGE_SIZE - 1) / GLOBAL_PAGE_SIZE);
+
+    if (ERROR_INSERTED(11009))
+    {
+      page_cnt = 25;
+      ndbout_c("Setting page_cnt = %u", page_cnt);
+    }
+
     m_param.m_max_pages = page_cnt;
     m_page_entry_pool.setSize(m_param.m_lirs_stack_mult * page_cnt);
     m_param.m_max_hot_pages = (page_cnt * 9) / 10;
+    ndbrequire(m_param.m_max_hot_pages >= 1);
   }
 
   Pool_context pc;
@@ -175,14 +188,6 @@ Pgman::Param::Param() :
 {
 }
 
-Pgman::Stats::Stats() :
-  m_num_pages(0),
-  m_page_hits(0),
-  m_page_faults(0),
-  m_current_io_waits(0)
-{
-}
-
 void
 Pgman::execSTTOR(Signal* signal)
 {
@@ -266,11 +271,13 @@ Pgman::execCONTINUEB(Signal* signal)
     Page_sublist& pl = *m_page_sublist[Page_entry::SL_LOCKED];
     if (data1 != RNIL)
     {
+      jam();
       pl.getPtr(ptr, data1);
       process_lcp_locked(signal, ptr);
     }
     else
     {
+      jam();
       if (ERROR_INSERTED(11007))
       {
         ndbout << "No more writes..." << endl;
@@ -283,6 +290,7 @@ Pgman::execCONTINUEB(Signal* signal)
       conf->senderRef = reference();
       sendSignal(m_end_lcp_req.senderRef, GSN_END_LCP_CONF,
                  signal, EndLcpConf::SignalLength, JBB);
+      m_lcp_state = LS_LCP_OFF;
     }
     return;
   }
@@ -313,10 +321,6 @@ Pgman::Page_entry::Page_entry(Uint32 file_no, Uint32 page_no) :
 Uint32
 Pgman::get_sublist_no(Page_state state)
 {
-  if (state == 0)
-  {
-    return ZNIL;
-  }
   if (state & Page_entry::REQUEST)
   {
     if (! (state & Page_entry::BOUND))
@@ -348,7 +352,11 @@ Pgman::get_sublist_no(Page_state state)
   if (state == Page_entry::ONSTACK) {
     return Page_entry::SL_IDLE;
   }
-  return Page_entry::SL_OTHER;
+  if (state != 0)
+  {
+    return Page_entry::SL_OTHER;
+  }
+  return ZNIL;
 }
 
 void
@@ -381,9 +389,26 @@ Pgman::set_page_state(Ptr<Page_entry> ptr, Page_state new_state)
       }
     }
     ptr.p->m_state = new_state;
+
+    bool old_hot = (old_state & Page_entry::HOT);
+    bool new_hot = (new_state & Page_entry::HOT);
+    if (! old_hot && new_hot)
+    {
+      jam();
+      m_stats.m_num_hot_pages++;
+    }
+    if (old_hot && ! new_hot)
+    {
+      jam();
+      ndbrequire(m_stats.m_num_hot_pages != 0);
+      m_stats.m_num_hot_pages--;
+    }
   }
 
   D(ptr << ": after");
+#ifdef VM_TRACE
+  verify_page_entry(ptr);
+#endif
   D("<set_page_state");
 }
 
@@ -520,7 +545,9 @@ Pgman::release_page_entry(Ptr<Page_entry>& ptr)
   ndbrequire(ptr.p->m_real_page_i == RNIL);
 
   if (! (state & Page_entry::LOCKED))
+  {
     ndbrequire(! (state & Page_entry::REQUEST));
+  }
 
   if (ptr.p->m_copy_page_i != RNIL)
   {
@@ -641,8 +668,10 @@ Pgman::lirs_reference(Ptr<Page_entry> ptr)
   Page_state state = ptr.p->m_state;
   ndbrequire(! (state & Page_entry::LOCKED));
 
-  // even non-LIRS cache pages are counted on l.h.s.
-  if (m_stats.m_num_pages >= m_param.m_max_hot_pages)
+  ndbrequire(m_stats.m_num_hot_pages <= m_param.m_max_hot_pages);
+
+  // LIRS kicks in when we have max hot pages
+  if (m_stats.m_num_hot_pages == m_param.m_max_hot_pages)
   {
     if (state & Page_entry::HOT)
     {
@@ -684,6 +713,12 @@ Pgman::lirs_reference(Ptr<Page_entry> ptr)
       jam();
       pl_stack.add(ptr);
       state |= Page_entry::ONSTACK;
+      /*
+       * bug#48910.  Using hot page count (not total page count)
+       * guarantees that stack is not empty here.  Therefore the new
+       * entry (added to top) is not at bottom and need not be hot.
+       */
+      ndbrequire(pl_stack.hasPrev(ptr));
       if (state & Page_entry::ONQUEUE)
       {
         jam();
@@ -706,7 +741,8 @@ Pgman::lirs_reference(Ptr<Page_entry> ptr)
   }
   else
   {
-    D("hot: " << m_stats.m_num_pages << " of " << m_param.m_max_hot_pages);
+    D("filling up hot pages: " << m_stats.m_num_hot_pages << "/"
+                               << m_param.m_max_hot_pages);
     jam();
     if (state & Page_entry::ONSTACK)
     {
@@ -797,35 +833,30 @@ Pgman::do_cleanup_loop(Signal* signal)
 }
 
 void
-Pgman::do_lcp_loop(Signal* signal, bool direct)
+Pgman::do_lcp_loop(Signal* signal)
 {
-  D(">do_lcp_loop on=" << m_lcp_loop_on << " direct=" << direct);
-  Uint32 restart = false;
-  if (direct)
-  {
-    ndbrequire(! m_lcp_loop_on);
-    restart = true;
-    m_lcp_loop_on = true;
-  }
-  else
-  {
-    ndbrequire(m_lcp_loop_on);
-    restart += process_lcp(signal);
-    if (! restart)
-    {
-      m_lcp_loop_on = false;
-    }
-  }
-  if (restart)
-  {
-    Uint32 delay = m_param.m_lcp_loop_delay;
+  D(">do_lcp_loop m_lcp_state=" << Uint32(m_lcp_state));
+  ndbrequire(m_lcp_state != LS_LCP_OFF);
+  LCP_STATE newstate = process_lcp(signal);
+
+  switch(newstate) {
+  case LS_LCP_OFF:
+    jam();
+    break;
+  case LS_LCP_ON:
+    jam();
     signal->theData[0] = PgmanContinueB::LCP_LOOP;
-    if (delay)
-      sendSignalWithDelay(reference(), GSN_CONTINUEB, signal, delay, 1);
-    else
-      sendSignal(reference(), GSN_CONTINUEB, signal, 1, JBB);
+    sendSignal(reference(), GSN_CONTINUEB, signal, 1, JBB);
+    break;
+  case LS_LCP_MAX_LCP_OUTSTANDING: // wait until io is completed
+    jam();
+    break;
+  case LS_LCP_LOCKED:
+    jam();
+    break;
   }
-  D("<do_lcp_loop on=" << m_lcp_loop_on << " restart=" << restart);
+  m_lcp_state = newstate;
+  D("<do_lcp_loop m_lcp_state=" << Uint32(m_lcp_state));
 }
 
 // busy loop
@@ -1005,11 +1036,11 @@ Pgman::process_callback(Signal* signal, Ptr<Page_entry> ptr)
 {
   D(ptr << " : process_callback");
   int max_count = 1;
-  Page_state state = ptr.p->m_state;
 
   while (! ptr.p->m_requests.isEmpty() && --max_count >= 0)
   {
     jam();
+    Page_state state = ptr.p->m_state;
     SimulatedBlock* b;
     Callback callback;
     {
@@ -1051,19 +1082,18 @@ Pgman::process_callback(Signal* signal, Ptr<Page_entry> ptr)
     }
     ndbrequire(state & Page_entry::BOUND);
     ndbrequire(state & Page_entry::MAPPED);
+
+    // make REQUEST state consistent before set_page_state()
+    if (ptr.p->m_requests.isEmpty())
+    {
+      jam();
+      state &= ~ Page_entry::REQUEST;
+    }
     
     // callback may re-enter PGMAN and change page state
     set_page_state(ptr, state);
     b->execute(signal, callback, ptr.p->m_real_page_i);
-    state = ptr.p->m_state;
   }
-  
-  if (ptr.p->m_requests.isEmpty())
-  {
-    jam();
-    state &= ~ Page_entry::REQUEST;
-  }
-  set_page_state(ptr, state);
   return true;
 }
 
@@ -1186,17 +1216,20 @@ Pgman::execEND_LCP_REQ(Signal* signal)
     << " outstanding=" << m_lcp_outstanding);
 
   m_last_lcp_complete = m_last_lcp;
-  
-  do_lcp_loop(signal, true);
+  ndbrequire(m_lcp_state == LS_LCP_OFF);
+  m_lcp_state = LS_LCP_ON;
+  do_lcp_loop(signal);
 }
 
-bool
+Pgman::LCP_STATE
 Pgman::process_lcp(Signal* signal)
 {
   Page_hashlist& pl_hash = m_page_hashlist;
 
   int max_count = 0;
-  if (m_param.m_max_io_waits > m_stats.m_current_io_waits) {
+  if (m_param.m_max_io_waits > m_stats.m_current_io_waits)
+  {
+    jam();
     max_count = m_param.m_max_io_waits - m_stats.m_current_io_waits;
     max_count = max_count / 2 + 1;
   }
@@ -1210,6 +1243,7 @@ Pgman::process_lcp(Signal* signal)
   // start or re-start from beginning of current hash bucket
   if (m_lcp_curr_bucket != ~(Uint32)0)
   {
+    jam();
     Page_hashlist::Iterator iter;
     pl_hash.next(m_lcp_curr_bucket, iter);
     Uint32 loop = 0;
@@ -1217,6 +1251,7 @@ Pgman::process_lcp(Signal* signal)
 	   m_lcp_outstanding < (Uint32) max_count &&
 	   (loop ++ < 32 || iter.bucket == m_lcp_curr_bucket))
     {
+      jam();
       Ptr<Page_entry>& ptr = iter.curr;
       Page_state state = ptr.p->m_state;
       
@@ -1226,6 +1261,7 @@ Pgman::process_lcp(Signal* signal)
           (state & Page_entry::DIRTY) &&
 	  (! (state & Page_entry::LOCKED)))
       {
+        jam();
         if(! (state & Page_entry::BOUND))
         {
           ndbout << ptr << endl;
@@ -1233,17 +1269,20 @@ Pgman::process_lcp(Signal* signal)
         }
         if (state & Page_entry::BUSY)
         {
-	  DBG_LCP(" BUSY" << endl);
+          jam();
+          DBG_LCP(" BUSY" << endl);
           break;  // wait for it
         } 
 	else if (state & Page_entry::PAGEOUT)
         {
-	  DBG_LCP(" PAGEOUT -> state |= LCP" << endl);
+          jam();
+          DBG_LCP(" PAGEOUT -> state |= LCP" << endl);
           set_page_state(ptr, state | Page_entry::LCP);
         }
         else
         {
-	  DBG_LCP(" pageout()" << endl);
+          jam();
+          DBG_LCP(" pageout()" << endl);
           ptr.p->m_state |= Page_entry::LCP;
           if (c_tup != 0)
             c_tup->disk_page_unmap_callback(0,
@@ -1256,7 +1295,8 @@ Pgman::process_lcp(Signal* signal)
       }
       else
       {
-	DBG_LCP(" NOT DIRTY" << endl);
+        jam();
+        DBG_LCP(" NOT DIRTY" << endl);
       }	
       pl_hash.next(iter);
     }
@@ -1273,6 +1313,7 @@ Pgman::process_lcp(Signal* signal)
     {
       jam();
       process_lcp_locked(signal, ptr);
+      return LS_LCP_LOCKED;
     }
     else
     {
@@ -1289,11 +1330,17 @@ Pgman::process_lcp(Signal* signal)
       conf->senderRef = reference();
       sendSignal(m_end_lcp_req.senderRef, GSN_END_LCP_CONF,
                  signal, EndLcpConf::SignalLength, JBB);
+      return LS_LCP_OFF;
     }
-    return false;
+  }
+
+  if (m_lcp_outstanding >= (Uint32) max_count)
+  {
+    jam();
+    return LS_LCP_MAX_LCP_OUTSTANDING;
   }
   
-  return true;
+  return LS_LCP_ON;
 }
 
 void
@@ -1393,6 +1440,7 @@ Pgman::fsreadconf(Signal* signal, Ptr<Page_entry> ptr)
   
   ndbrequire(m_stats.m_current_io_waits > 0);
   m_stats.m_current_io_waits--;
+  m_stats.m_pages_read++;
 
   ptr.p->m_last_lcp = m_last_lcp_complete;
   do_busy_loop(signal, true);
@@ -1435,6 +1483,7 @@ Pgman::pageout(Signal* signal, Ptr<Page_entry> ptr)
   else
   {
     ndbrequire(ret == 0);
+    m_stats.m_log_waits++;
     state |= Page_entry::LOGSYNC;
   }
   set_page_state(ptr, state);
@@ -1487,6 +1536,7 @@ Pgman::fswriteconf(Signal* signal, Ptr<Page_entry> ptr)
     state &= ~ Page_entry::LCP;
     ndbrequire(m_lcp_outstanding);
     m_lcp_outstanding--;
+    m_stats.m_pages_written_lcp++;
     if (ptr.p->m_copy_page_i != RNIL)
     {
       jam();
@@ -1497,9 +1547,19 @@ Pgman::fswriteconf(Signal* signal, Ptr<Page_entry> ptr)
       return;
     }
   }
+  else
+  {
+    m_stats.m_pages_written++;
+  }
   
   set_page_state(ptr, state);
   do_busy_loop(signal, true);
+
+  if (m_lcp_state == LS_LCP_MAX_LCP_OUTSTANDING)
+  {
+    jam();
+    do_lcp_loop(signal);
+  }
 }
 
 // file system interface
@@ -1608,7 +1668,7 @@ Pgman::execFSWRITEREF(Signal* signal)
 // client methods
 
 int
-Pgman::get_page(Signal* signal, Ptr<Page_entry> ptr, Page_request page_req)
+Pgman::get_page_no_lirs(Signal* signal, Ptr<Page_entry> ptr, Page_request page_req)
 {
   jamEntry();
   Ptr<Page_request> tmp = { &page_req, RNIL};
@@ -1649,21 +1709,12 @@ Pgman::get_page(Signal* signal, Ptr<Page_entry> ptr, Page_request page_req)
     jam();
   }
 
-  // update LIRS
-  if (! (state & Page_entry::LOCKED) &&
-      ! (req_flags & Page_request::CORR_REQ))
-  {
-    jam();
-    set_page_state(ptr, state);
-    lirs_reference(ptr);
-    state = ptr.p->m_state;
-  }
-
   const Page_state LOCKED = Page_entry::LOCKED | Page_entry::MAPPED;
   if ((state & LOCKED) == LOCKED && 
       ! (req_flags & Page_request::UNLOCK_PAGE))
   {
     ptr.p->m_state |= (req_flags & DIRTY_FLAGS ? Page_entry::DIRTY : 0);
+    m_stats.m_page_requests_direct_return++;
     if (ptr.p->m_copy_page_i != RNIL)
     {
       D("<get_page: immediate copy_page");
@@ -1696,6 +1747,7 @@ Pgman::get_page(Signal* signal, Ptr<Page_entry> ptr, Page_request page_req)
       D("<get_page: immediate");
 
       ndbrequire(ptr.p->m_real_page_i != RNIL);
+      m_stats.m_page_requests_direct_return++;
       return ptr.p->m_real_page_i;
     }
   }
@@ -1706,6 +1758,12 @@ Pgman::get_page(Signal* signal, Ptr<Page_entry> ptr, Page_request page_req)
   }
 
   // queue the request
+
+  if ((state & Page_entry::MAPPED) && ! (state & Page_entry::PAGEOUT))
+    m_stats.m_page_requests_wait_q++;
+  else
+    m_stats.m_page_requests_wait_io++;
+
   Ptr<Pgman::Page_request> req_ptr;
   {
     Local_page_request_list req_list(m_page_request_pool, ptr.p->m_requests);
@@ -1746,14 +1804,43 @@ Pgman::get_page(Signal* signal, Ptr<Page_entry> ptr, Page_request page_req)
   ptr.p->m_busy_count += busy_count;
   ptr.p->m_dirty_count += !!(req_flags & DIRTY_FLAGS);
   set_page_state(ptr, state);
-  
-  do_busy_loop(signal, true);
 
   D(req_ptr);
   D("<get_page: queued");
   return 0;
 }
 
+int
+Pgman::get_page(Signal* signal, Ptr<Page_entry> ptr, Page_request page_req)
+{
+  int i = get_page_no_lirs(signal, ptr, page_req);
+  if (unlikely(i == -1))
+  {
+    jam();
+    return -1;
+  }
+
+  Uint32 req_flags = page_req.m_flags;
+  Page_state state = ptr.p->m_state;
+
+  // update LIRS
+  if (! (state & Page_entry::LOCKED) &&
+      ! (req_flags & Page_request::CORR_REQ))
+  {
+    jam();
+    lirs_reference(ptr);
+  }
+
+  // start processing if request was queued
+  if (i == 0)
+  {
+    jam();
+    do_busy_loop(signal, true);
+  }
+
+  return i;
+}
+
 void
 Pgman::update_lsn(Ptr<Page_entry> ptr, Uint32 block, Uint64 lsn)
 {
@@ -1910,9 +1997,6 @@ Pgman::drop_page(Ptr<Page_entry> ptr)
   Page_state state = ptr.p->m_state;
   if (! (state & (Page_entry::PAGEIN | Page_entry::PAGEOUT)))
   {
-    ndbrequire(state & Page_entry::BOUND);
-    ndbrequire(state & Page_entry::MAPPED);
-
     if (state & Page_entry::ONSTACK)
     {
       jam();
@@ -1922,9 +2006,13 @@ Pgman::drop_page(Ptr<Page_entry> ptr)
       if (at_bottom)
       {
         jam();
-        ndbassert(state & Page_entry::HOT);
         lirs_stack_prune();
       }
+      if (state & Page_entry::HOT)
+      {
+        jam();
+        state &= ~ Page_entry::HOT;
+      }
     }
 
     if (state & Page_entry::ONQUEUE)
@@ -1934,12 +2022,37 @@ Pgman::drop_page(Ptr<Page_entry> ptr)
       state &= ~ Page_entry::ONQUEUE;
     }
 
-    ndbassert(ptr.p->m_real_page_i != RNIL);
-    if (ptr.p->m_real_page_i != RNIL)
+    if (state & Page_entry::BUSY)
     {
       jam();
+      state &= ~ Page_entry::BUSY;
+    }
+
+    if (state & Page_entry::DIRTY)
+    {
+      jam();
+      state &= ~ Page_entry::DIRTY;
+    }
+
+    if (state & Page_entry::EMPTY)
+    {
+      jam();
+      state &= ~ Page_entry::EMPTY;
+    }
+
+    if (state & Page_entry::MAPPED)
+    {
+      jam();
+      state &= ~ Page_entry::MAPPED;
+    }
+
+    if (state & Page_entry::BOUND)
+    {
+      jam();
+      ndbrequire(ptr.p->m_real_page_i != RNIL);
       release_cache_page(ptr.p->m_real_page_i);
       ptr.p->m_real_page_i = RNIL;
+      state &= ~ Page_entry::BOUND;
     }
 
     set_page_state(ptr, state);
@@ -2152,6 +2265,8 @@ Page_cache_client::free_data_file(Signal* signal, Uint32 file_no, Uint32 fd)
 void
 Pgman::verify_page_entry(Ptr<Page_entry> ptr)
 {
+  Page_stack& pl_stack = m_page_stack;
+
   Uint32 ptrI = ptr.i;
   Page_state state = ptr.p->m_state;
 
@@ -2174,6 +2289,10 @@ Pgman::verify_page_entry(Ptr<Page_entry> ptr)
   // hot entry must be on stack
   ndbrequire(! is_hot || on_stack || dump_page_lists(ptrI));
 
+  // stack bottom is hot
+  bool at_bottom = on_stack && ! pl_stack.hasPrev(ptr);
+  ndbrequire(! at_bottom || is_hot || dump_page_lists(ptrI));
+
   bool on_queue = state & Page_entry::ONQUEUE;
   // hot entry is not on queue
   ndbrequire(! is_hot || ! on_queue || dump_page_lists(ptrI));
@@ -2185,9 +2304,12 @@ Pgman::verify_page_entry(Ptr<Page_entry> ptr)
   // entries waiting to enter queue
   bool to_queue = ! is_locked && ! is_hot && ! is_bound && has_req;
 
-  // page is either LOCKED or under LIRS
+  // page is about to be released
+  bool to_release = (state == 0);
+
+  // page is either LOCKED or under LIRS or about to be released
   bool is_lirs = on_stack || to_queue || on_queue;
-  ndbrequire(is_locked == ! is_lirs || dump_page_lists(ptrI));
+  ndbrequire(to_release || is_locked == ! is_lirs || dump_page_lists(ptrI));
 
   bool pagein = state & Page_entry::PAGEIN;
   bool pageout = state & Page_entry::PAGEOUT;
@@ -2197,19 +2319,19 @@ Pgman::verify_page_entry(Ptr<Page_entry> ptr)
   Uint32 no = get_sublist_no(state);
   switch (no) {
   case Page_entry::SL_BIND:
-    ndbrequire(! pagein && ! pageout || dump_page_lists(ptrI));
+    ndbrequire((! pagein && ! pageout) || dump_page_lists(ptrI));
     break;
   case Page_entry::SL_MAP:
-    ndbrequire(! pagein && ! pageout || dump_page_lists(ptrI));
+    ndbrequire((! pagein && ! pageout) || dump_page_lists(ptrI));
     break;
   case Page_entry::SL_MAP_IO:
-    ndbrequire(pagein && ! pageout || dump_page_lists(ptrI));
+    ndbrequire((pagein && ! pageout) || dump_page_lists(ptrI));
     break;
   case Page_entry::SL_CALLBACK:
-    ndbrequire(! pagein && ! pageout || dump_page_lists(ptrI));
+    ndbrequire((! pagein && ! pageout) || dump_page_lists(ptrI));
     break;
   case Page_entry::SL_CALLBACK_IO:
-    ndbrequire(! pagein && pageout || dump_page_lists(ptrI));
+    ndbrequire((! pagein && pageout) || dump_page_lists(ptrI));
     break;
   case Page_entry::SL_BUSY:
     break;
@@ -2219,6 +2341,9 @@ Pgman::verify_page_entry(Ptr<Page_entry> ptr)
     break;
   case Page_entry::SL_OTHER:
     break;
+  case ZNIL:
+    ndbrequire(to_release || dump_page_lists(ptrI));
+    break;
   default:
     ndbrequire(false || dump_page_lists(ptrI));
     break;
@@ -2228,124 +2353,116 @@ Pgman::verify_page_entry(Ptr<Page_entry> ptr)
 void
 Pgman::verify_page_lists()
 {
+  const Stats& stats = m_stats;
+  const Param& param = m_param;
   Page_hashlist& pl_hash = m_page_hashlist;
   Page_stack& pl_stack = m_page_stack;
   Page_queue& pl_queue = m_page_queue;
   Ptr<Page_entry> ptr;
 
-  Uint32 stack_count = 0;
-  Uint32 queue_count = 0;
-  Uint32 queuewait_count = 0;
-  Uint32 locked_bound_count = 0;
+  Uint32 is_locked = 0;
+  Uint32 is_bound = 0;
+  Uint32 is_mapped = 0;
+  Uint32 is_hot = 0;
+  Uint32 on_stack = 0;
+  Uint32 on_queue = 0;
+  Uint32 to_queue = 0;
 
   Page_hashlist::Iterator iter;
   pl_hash.next(0, iter);
   while (iter.curr.i != RNIL)
   {
-    verify_page_entry(iter.curr);
+    ptr = iter.curr;
+    Page_state state = ptr.p->m_state;
+    // (state == 0) occurs only within a time-slice
+    ndbrequire(state != 0);
+    verify_page_entry(ptr);
 
-    Page_state state = iter.curr.p->m_state;
+    if (state & Page_entry::LOCKED)
+      is_locked++;
+    if (state & Page_entry::BOUND)
+      is_bound++;
+    if (state & Page_entry::MAPPED)
+      is_mapped++;
+    if (state & Page_entry::HOT)
+      is_hot++;
     if (state & Page_entry::ONSTACK)
-      stack_count++;
+      on_stack++;
     if (state & Page_entry::ONQUEUE)
-      queue_count++;
+      on_queue++;
     if (! (state & Page_entry::LOCKED) &&
         ! (state & Page_entry::HOT) &&
         (state & Page_entry::REQUEST) &&
         ! (state & Page_entry::BOUND))
-      queuewait_count++;
-    if (state & Page_entry::LOCKED &&
-        state & Page_entry::BOUND)
-      locked_bound_count++;
+      to_queue++;
     pl_hash.next(iter);
   }
 
-  ndbrequire(stack_count == pl_stack.count() || dump_page_lists());
-  ndbrequire(queue_count == pl_queue.count() || dump_page_lists());
-
-  Uint32 hot_count = 0;
-  Uint32 hot_bound_count = 0;
-  Uint32 cold_bound_count = 0;
-  Uint32 stack_request_count = 0;
-  Uint32 queue_request_count = 0;
-
-  Uint32 i1 = RNIL;
   for (pl_stack.first(ptr); ptr.i != RNIL; pl_stack.next(ptr))
   {
-    ndbrequire(i1 != ptr.i);
-    i1 = ptr.i;
     Page_state state = ptr.p->m_state;
-    ndbrequire(state & Page_entry::ONSTACK || dump_page_lists());
+    ndbrequire(state & Page_entry::ONSTACK || dump_page_lists(ptr.i));
     if (! pl_stack.hasPrev(ptr))
-      ndbrequire(state & Page_entry::HOT || dump_page_lists());
-    if (state & Page_entry::HOT) {
-      hot_count++;
-      if (state & Page_entry::BOUND)
-        hot_bound_count++;
+    {
+      ndbrequire(state & Page_entry::HOT || dump_page_lists(ptr.i));
     }
-    if (state & Page_entry::REQUEST)
-      stack_request_count++;
   }
 
-  Uint32 i2 = RNIL;
   for (pl_queue.first(ptr); ptr.i != RNIL; pl_queue.next(ptr))
   {
-    ndbrequire(i2 != ptr.i);
-    i2 = ptr.i;
     Page_state state = ptr.p->m_state;
-    ndbrequire(state & Page_entry::ONQUEUE || dump_page_lists());
-    ndbrequire(state & Page_entry::BOUND || dump_page_lists());
-    cold_bound_count++;
-    if (state & Page_entry::REQUEST)
-      queue_request_count++;
+    ndbrequire(state & Page_entry::ONQUEUE || dump_page_lists(ptr.i));
+    ndbrequire(state & Page_entry::BOUND || dump_page_lists(ptr.i));
+    ndbrequire(! (state & Page_entry::HOT) || dump_page_lists(ptr.i));
   }
 
-  Uint32 tot_bound_count =
-    locked_bound_count + hot_bound_count + cold_bound_count;
-  ndbrequire(m_stats.m_num_pages == tot_bound_count || dump_page_lists());
+  ndbrequire(is_bound == stats.m_num_pages || dump_page_lists());
+  ndbrequire(is_hot == stats.m_num_hot_pages || dump_page_lists());
+  ndbrequire(on_stack == pl_stack.count() || dump_page_lists());
+  ndbrequire(on_queue == pl_queue.count() || dump_page_lists());
 
   Uint32 k;
   Uint32 entry_count = 0;
-
+  char sublist_info[200] = "";
   for (k = 0; k < Page_entry::SUBLIST_COUNT; k++)
   {
     const Page_sublist& pl = *m_page_sublist[k];
     for (pl.first(ptr); ptr.i != RNIL; pl.next(ptr))
-    {
-      ndbrequire(get_sublist_no(ptr.p->m_state) == k || dump_page_lists());
-      entry_count++;
-    }
+      ndbrequire(get_sublist_no(ptr.p->m_state) == k || dump_page_lists(ptr.i));
+    entry_count += pl.count();
+    sprintf(sublist_info + strlen(sublist_info),
+            " %s:%u", get_sublist_name(k), pl.count());
   }
-
   ndbrequire(entry_count == pl_hash.count() || dump_page_lists());
 
+  Uint32 hit_pct = 0;
+  char hit_pct_str[20];
+  if (stats.m_page_hits + stats.m_page_faults != 0)
+    hit_pct = 10000 * stats.m_page_hits /
+              (stats.m_page_hits + stats.m_page_faults);
+  sprintf(hit_pct_str, "%u.%02u", hit_pct / 100, hit_pct % 100);
+
   D("loop"
-    << " stats=" << m_stats_loop_on
-    << " busy=" << m_busy_loop_on
-    << " cleanup=" << m_cleanup_loop_on
-    << " lcp=" << m_lcp_loop_on);
+    << " stats:" << m_stats_loop_on
+    << " busy:" << m_busy_loop_on
+    << " cleanup:" << m_cleanup_loop_on
+    << " lcp:" << Uint32(m_lcp_state));
 
-  D("stat"
-    << " entry:" << pl_hash.count()
-    << " cache:" << m_stats.m_num_pages
-    << "(" << locked_bound_count << "L)"
+  D("page"
+    << " entries:" << pl_hash.count()
+    << " pages:" << stats.m_num_pages << "/" << param.m_max_pages
+    << " mapped:" << is_mapped
+    << " hot:" << is_hot
+    << " io:" << stats.m_current_io_waits << "/" << param.m_max_io_waits
+    << " hit pct:" << hit_pct_str);
+
+  D("list"
+    << " locked:" << is_locked
     << " stack:" << pl_stack.count()
-    << " hot:" << hot_count
-    << " hot_bound:" << hot_bound_count
-    << " stack_request:" << stack_request_count
     << " queue:" << pl_queue.count()
-    << " queue_request:" << queue_request_count
-    << " queuewait:" << queuewait_count);
+    << " to queue:" << to_queue);
 
-  char countbuf[200];
-  countbuf[0] = 0;
-  for (k = 0; k < Page_entry::SUBLIST_COUNT; k++)
-  {
-    const Page_sublist& pl = *m_page_sublist[k];
-    sprintf(countbuf + strlen(countbuf), " %s:%u",
-            get_sublist_name(k), pl.count());
-  }
-  D("list" << countbuf);
+  D(sublist_info);
 }
 
 void
@@ -2368,54 +2485,34 @@ Pgman::dump_page_lists(Uint32 ptrI)
   // use debugOut directly
   debugOut << "PGMAN: page list dump" << endl;
   if (ptrI != RNIL)
-    debugOut << "PGMAN: error on PE [" << ptrI << "]" << endl;
+    debugOut << "PGMAN: error on PE [" << ptrI << "]" << "\n";
 
-  Page_hashlist& pl_hash = m_page_hashlist;
   Page_stack& pl_stack = m_page_stack;
   Page_queue& pl_queue = m_page_queue;
   Ptr<Page_entry> ptr;
   Uint32 n;
-  char buf[40];
 
-  debugOut << "hash:" << endl;
-  Page_hashlist::Iterator iter;
-  pl_hash.next(0, iter);
-  n = 0;
-  while (iter.curr.i != RNIL)
-  {
-    sprintf(buf, "%03d", n++);
-    debugOut << buf << " " << iter.curr << endl;
-    pl_hash.next(iter);
-  }
-
-  debugOut << "stack:" << endl;
+  debugOut << "stack:" << "\n";
   n = 0;
   for (pl_stack.first(ptr); ptr.i != RNIL; pl_stack.next(ptr))
-  {
-    sprintf(buf, "%03d", n++);
-    debugOut << buf << " " << ptr << endl;
-  }
+    debugOut << n++ << " " << ptr << "\n";
 
-  debugOut << "queue:" << endl;
+  debugOut << "queue:" << "\n";
   n = 0;
   for (pl_queue.first(ptr); ptr.i != RNIL; pl_queue.next(ptr))
-  {
-    sprintf(buf, "%03d", n++);
-    debugOut << buf << " " << ptr << endl;
-  }
+    debugOut << n++ << " " << ptr << "\n";
 
   Uint32 k;
   for (k = 0; k < Page_entry::SUBLIST_COUNT; k++)
   {
-    debugOut << get_sublist_name(k) << ":" << endl;
+    debugOut << get_sublist_name(k) << ":" << "\n";
     const Page_sublist& pl = *m_page_sublist[k];
+    n = 0;
     for (pl.first(ptr); ptr.i != RNIL; pl.next(ptr))
-    {
-      sprintf(buf, "%03d", n++);
-    debugOut << buf << " " << ptr << endl;
-    }
+      debugOut << n++ << " " << ptr << "\n";
   }
 
+  debugOut.flushline();
   return false;
 }
 
@@ -2432,9 +2529,9 @@ Pgman::get_sublist_name(Uint32 list_no)
   case Page_entry::SL_MAP_IO:
     return "map_io";
   case Page_entry::SL_CALLBACK:
-    return "callback";
+    return "cb";
   case Page_entry::SL_CALLBACK_IO:
-    return "callback_io";
+    return "cb_io";
   case Page_entry::SL_BUSY:
     return "busy";
   case Page_entry::SL_LOCKED:
@@ -2580,6 +2677,15 @@ Pgman::execDUMP_STATE_ORD(Signal* signal)
 {
   jamEntry();
   Page_hashlist& pl_hash = m_page_hashlist;
+#ifdef VM_TRACE
+  if (signal->theData[0] == 11000 && signal->getLength() == 2)
+  {
+    // has no effect currently
+    Uint32 flag = signal->theData[1];
+    debugFlag = flag & 1;
+    debugSummaryFlag = flag & 2;
+  }
+#endif
 
   if (signal->theData[0] == 11001)
   {
@@ -2675,4 +2781,42 @@ Pgman::execDUMP_STATE_ORD(Signal* signal)
   {
     SET_ERROR_INSERT_VALUE(11008);
   }
+
+  if (signal->theData[0] == 11009)
+  {
+    SET_ERROR_INSERT_VALUE(11009);
+  }
+}
+
+void
+Pgman::execDBINFO_SCANREQ(Signal *signal)
+{
+  DbinfoScanReq req= *(DbinfoScanReq*)signal->theData;
+  const Ndbinfo::ScanCursor* cursor =
+    reinterpret_cast<const Ndbinfo::ScanCursor*>(DbinfoScan::getCursorPtr(&req));
+  Ndbinfo::Ratelimit rl;
+
+  jamEntry();
+  switch(req.tableId) {
+  case Ndbinfo::DISKPAGEBUFFER_TABLEID:
+  {
+    jam();
+    BlockNumber bn = blockToMain(number());
+    Ndbinfo::Row row(signal, req);
+    row.write_uint32(getOwnNodeId());
+    row.write_uint32(instance());   // block instance
+    row.write_uint64(m_stats.m_pages_written);
+    row.write_uint64(m_stats.m_pages_written_lcp);
+    row.write_uint64(m_stats.m_pages_read);
+    row.write_uint64(m_stats.m_log_waits);
+    row.write_uint64(m_stats.m_page_requests_direct_return);
+    row.write_uint64(m_stats.m_page_requests_wait_q);
+    row.write_uint64(m_stats.m_page_requests_wait_io);
+
+    ndbinfo_send_row(signal, req, row, rl);
+  }
+  default:
+    break;
+  }
+  ndbinfo_send_scan_conf(signal, req, rl);
 }
