@@ -54,7 +54,7 @@
  * Updating a resident cache page makes it "dirty".  A background
  * clean-up process makes dirty pages "clean" via "pageout" to disk.
  * Write ahead logging (WAL) of the page is done first i.e. UNDO log is
- * flushed up to the page log sequence number (LSN) by calling a TSMAN
+ * flushed up to the page log sequence number (LSN) by calling a LGMAN
  * method.  The reason for this is obvious but not relevant to PGMAN.
  *
  * A local check point (LCP) periodically performs a complete pageout of
@@ -378,9 +378,15 @@ private:
   bool m_stats_loop_on;
   bool m_busy_loop_on;
   bool m_cleanup_loop_on;
-  bool m_lcp_loop_on;
 
   // LCP variables
+  enum LCP_STATE
+  {
+    LS_LCP_OFF = 0
+    ,LS_LCP_ON = 1
+    ,LS_LCP_MAX_LCP_OUTSTANDING = 2
+    ,LS_LCP_LOCKED = 3
+  } m_lcp_state;
   Uint32 m_last_lcp;
   Uint32 m_last_lcp_complete;
   Uint32 m_lcp_curr_bucket;
@@ -418,11 +424,32 @@ private:
 
   // runtime sizes and statistics
   struct Stats {
-    Stats();
+    Stats() :
+      m_num_pages(0),
+      m_num_hot_pages(0),
+      m_current_io_waits(0),
+      m_page_hits(0),
+      m_page_faults(0),
+      m_pages_written(0),
+      m_pages_written_lcp(0),
+      m_pages_read(0),
+      m_log_waits(0),
+      m_page_requests_direct_return(0),
+      m_page_requests_wait_q(0),
+      m_page_requests_wait_io(0)
+    {}
     Uint32 m_num_pages;         // current number of cache pages
-    Uint32 m_page_hits;
-    Uint32 m_page_faults;
+    Uint32 m_num_hot_pages;
     Uint32 m_current_io_waits;
+    Uint64 m_page_hits;
+    Uint64 m_page_faults;
+    Uint64 m_pages_written;
+    Uint64 m_pages_written_lcp;
+    Uint64 m_pages_read;
+    Uint64 m_log_waits; // wait for undo WAL to flush the log recs
+    Uint64 m_page_requests_direct_return;
+    Uint64 m_page_requests_wait_q;
+    Uint64 m_page_requests_wait_io;
   } m_stats;
 
   enum CallbackIndex {
@@ -452,6 +479,8 @@ protected:
 
   void execDATA_FILE_ORD(Signal*);
 
+  void execDBINFO_SCANREQ(Signal*);
+
 private:
   static Uint32 get_sublist_no(Page_state state);
   void set_page_state(Ptr<Page_entry> ptr, Page_state new_state);
@@ -471,7 +500,7 @@ private:
   void do_stats_loop(Signal*);
   void do_busy_loop(Signal*, bool direct = false);
   void do_cleanup_loop(Signal*);
-  void do_lcp_loop(Signal*, bool direct = false);
+  void do_lcp_loop(Signal*);
 
   bool process_bind(Signal*);
   bool process_bind(Signal*, Ptr<Page_entry> ptr);
@@ -483,7 +512,7 @@ private:
   bool process_cleanup(Signal*);
   void move_cleanup_ptr(Ptr<Page_entry> ptr);
 
-  bool process_lcp(Signal*);
+  LCP_STATE process_lcp(Signal*);
   void process_lcp_locked(Signal* signal, Ptr<Page_entry> ptr);
   void process_lcp_locked_fswriteconf(Signal* signal, Ptr<Page_entry> ptr);
 
@@ -495,6 +524,7 @@ private:
   void fswritereq(Signal*, Ptr<Page_entry>);
   void fswriteconf(Signal*, Ptr<Page_entry>);
 
+  int get_page_no_lirs(Signal*, Ptr<Page_entry>, Page_request page_req);
   int get_page(Signal*, Ptr<Page_entry>, Page_request page_req);
   void update_lsn(Ptr<Page_entry>, Uint32 block, Uint64 lsn);
   Uint32 create_data_file();
@@ -504,6 +534,8 @@ private:
   int drop_page(Ptr<Page_entry>);
   
 #ifdef VM_TRACE
+  bool debugFlag;        // not yet in use in 7.0
+  bool debugSummaryFlag; // loop summary to signal log even if ! debugFlag
   void verify_page_entry(Ptr<Page_entry> ptr);
   void verify_page_lists();
   void verify_all();
