@@ -362,6 +362,11 @@ Restore::release_file(FilePtr file_ptr)
 void
 Restore::open_file(Signal* signal, FilePtr file_ptr)
 {
+  signal->theData[0] = NDB_LE_StartReadLCP;
+  signal->theData[1] = file_ptr.p->m_table_id;
+  signal->theData[2] = file_ptr.p->m_fragment_id;
+  sendSignal(CMVMI_REF, GSN_EVENT_REP, signal, 3, JBB);
+
   FsOpenReq * req = (FsOpenReq *)signal->getDataPtrSend();
   req->userReference = reference();
   req->fileFlags = FsOpenReq::OM_READONLY | FsOpenReq::OM_GZ;
@@ -489,7 +494,7 @@ Restore::restore_next(Signal* signal, FilePtr file_ptr)
       
       if (unlikely((status & File:: FILE_THREAD_RUNNING) == 0))
       {
-	ndbrequire(false);
+        crash_during_restore(file_ptr, __LINE__, 0);
       }
       len= 0;
       break;
@@ -1047,8 +1052,8 @@ Restore::parse_record(Signal* signal, FilePtr file_ptr,
 
     ndbrequire(disk == false); // Not supported...
     ndbrequire(rowid == true);
-    keyLen = keyData - key_start;
-    attrLen = attrData - attr_start;
+    keyLen = Uint32(keyData - key_start);
+    attrLen = Uint32(attrData - attr_start);
     if (desc->noOfKeyAttr != desc->noOfVarKeys)
     {
       reorder_key(desc, key_start, keyLen);
@@ -1181,18 +1186,32 @@ Restore::execLQHKEYREF(Signal* signal)
   LqhKeyRef* ref = (LqhKeyRef*)signal->getDataPtr();
   m_file_pool.getPtr(file_ptr, ref->connectPtr);
   
+  crash_during_restore(file_ptr, __LINE__, ref->errorCode);
+  ndbrequire(false);
+}
+
+void
+Restore::crash_during_restore(FilePtr file_ptr, Uint32 line, Uint32 errCode)
+{
   char buf[255], name[100];
   BaseString::snprintf(name, sizeof(name), "%u/T%dF%d",
 		       file_ptr.p->m_lcp_no,
 		       file_ptr.p->m_table_id,
 		       file_ptr.p->m_fragment_id);
   
-  BaseString::snprintf(buf, sizeof(buf),
-		       "Error %d during restore of  %s",
-		       ref->errorCode, name);
-  
+  if (errCode)
+  {
+    BaseString::snprintf(buf, sizeof(buf),
+                         "Error %d (line: %u) during restore of  %s",
+                         errCode, line, name);
+  }
+  else
+  {
+    BaseString::snprintf(buf, sizeof(buf),
+                         "Error (line %u) during restore of  %s",
+                         line, name);
+  }
   progError(__LINE__, NDBD_EXIT_INVALID_LCP_FILE, buf);  
-  ndbrequire(false);
 }
 
 void
@@ -1237,6 +1256,14 @@ Restore::restore_lcp_conf(Signal* signal, FilePtr file_ptr)
                GSN_RESTORE_LCP_CONF, signal, 
                RestoreLcpConf::SignalLength, JBB);
   }
+
+  signal->theData[0] = NDB_LE_ReadLCPComplete;
+  signal->theData[1] = file_ptr.p->m_table_id;
+  signal->theData[2] = file_ptr.p->m_fragment_id;
+  signal->theData[3] = Uint32(file_ptr.p->m_rows_restored >> 32);
+  signal->theData[4] = Uint32(file_ptr.p->m_rows_restored);
+  sendSignal(CMVMI_REF, GSN_EVENT_REP, signal, 5, JBB);
+
   release_file(file_ptr);
 }
 
