@@ -87,6 +87,7 @@ public:
    */
   void execTRIG_ATTRINFO(Signal* signal);
   void execFIRE_TRIG_ORD(Signal* signal);
+  void execFIRE_TRIG_ORD_L(Signal* signal);
   void execSUB_GCP_COMPLETE_REP(Signal* signal);
   
   /**
@@ -246,7 +247,8 @@ public:
     enum Options {
       REPORT_ALL       = 0x1,
       REPORT_SUBSCRIBE = 0x2,
-      MARKED_DROPPED   = 0x4
+      MARKED_DROPPED   = 0x4,
+      NO_REPORT_DDL    = 0x8
     };
 
     enum State {
@@ -410,6 +412,7 @@ public:
   void check_remove_queue(Signal*, Ptr<Subscription>,
                           Ptr<SubOpRecord>,bool,bool);
   void check_release_subscription(Signal* signal, Ptr<Subscription>);
+  void get_tabinfo_ref_release(Signal*, Ptr<Table>);
 
   /**
    * Public interface
@@ -471,6 +474,8 @@ public:
   void execSUMA_START_ME_REF(Signal* signal);
   void execSUMA_START_ME_CONF(Signal* signal);
 
+  void execSTOP_ME_REQ(Signal*);
+
   void copySubscription(Signal* signal, DLHashTable<Subscription>::Iterator);
   void sendSubCreateReq(Signal* signal, Ptr<Subscription>);
   void copySubscriber(Signal*, Ptr<Subscription>, Ptr<Subscriber>);
@@ -515,6 +520,16 @@ private:
     NdbNodeBitmask m_handover_nodes;
   } c_startup;
 
+  /**
+   * for graceful shutdown
+   */
+  struct Shutdown
+  {
+    bool m_wait_handover;
+    Uint32 m_senderRef;
+    Uint32 m_senderData;
+  } c_shutdown;
+
   struct Restart
   {
     Uint16 m_abort;
@@ -543,7 +558,7 @@ private:
   void send_dict_unlock_ord(Signal* signal, Uint32 state);
   void send_start_me_req(Signal* signal);
   void check_start_handover(Signal* signal);
-  void send_handover_req(Signal* signal);
+  void send_handover_req(Signal* signal, Uint32 type);
 
   Uint32 get_responsible_node(Uint32 B) const;
   Uint32 get_responsible_node(Uint32 B, const NdbNodeBitmask& mask) const;
@@ -568,8 +583,14 @@ private:
       ,BUCKET_HANDOVER = 0x2 // On running node
       ,BUCKET_TAKEOVER = 0x4 // On takeing over node
       ,BUCKET_RESEND   = 0x8 // On takeing over node
-      ,BUCKET_CREATED  = 0x10 // New nodegroup (me)
-      ,BUCKET_DROPPED  = 0x20 // New nodegroup (me) uses hi 8 bit for cnt
+      ,BUCKET_CREATED_SELF  = 0x10 // New nodegroup (me)
+      ,BUCKET_CREATED_OTHER = 0x20 // New nodegroup (not me)
+      ,BUCKET_CREATED_MASK  = (BUCKET_CREATED_SELF | BUCKET_CREATED_OTHER)
+      ,BUCKET_DROPPED_SELF  = 0x40 // New nodegroup (me) uses hi 8 bit for cnt
+      ,BUCKET_DROPPED_OTHER = 0x80 // New nodegroup (not me)
+      ,BUCKET_DROPPED_MASK  = (BUCKET_DROPPED_SELF | BUCKET_DROPPED_OTHER)
+      ,BUCKET_SHUTDOWN = 0x100 // Graceful shutdown
+      ,BUCKET_SHUTDOWN_TO = 0x200 // Graceful shutdown
     };
     Uint16 m_state;
     Uint16 m_switchover_node;
