@@ -39,9 +39,6 @@
 
 #include <signaldata/DbinfoScan.hpp>
 #include <signaldata/TransIdAI.hpp>
-#include <ndbinfo.h>
-#include <dbinfo/ndbinfo_tableids.h>
-
 #include <signaldata/WaitGCP.hpp>
 
 #define CONSTRAINT_VIOLATION 893
@@ -495,49 +492,66 @@ Trix::execDUMP_STATE_ORD(Signal* signal)
 void Trix::execDBINFO_SCANREQ(Signal *signal)
 {
   DbinfoScanReq req= *(DbinfoScanReq*)signal->theData;
-  char buf[512];
-  struct dbinfo_row r;
-  struct dbinfo_ratelimit rl;
-
-  dbinfo_ratelimit_init(&rl, &req);
+  const Ndbinfo::ScanCursor* cursor =
+    CAST_CONSTPTR(Ndbinfo::ScanCursor, DbinfoScan::getCursorPtr(&req));
+  Ndbinfo::Ratelimit rl;
 
   jamEntry();
 
-  if(req.tableId == NDBINFO_POOLS_TABLEID)
+  switch(req.tableId){
+  case Ndbinfo::POOLS_TABLEID:
   {
-    struct {
-      const char* poolname;
-      Uint32 free;
-      Uint32 size;
-    } pools[] =
-        {
-          {"Attribute Order Buffer",
-           c_theAttrOrderBufferPool.getNoOfFree(),
-           c_theAttrOrderBufferPool.getSize() },
-          {"Subscription Record",
-           c_theSubscriptionRecPool.getNoOfFree(),
-           c_theSubscriptionRecPool.getSize() },
-          { NULL, 0, 0}
-        };
-
-    for(int i=0; pools[i].poolname; i++)
+    Ndbinfo::pool_entry pools[] =
     {
-      dbinfo_write_row_init(&r, buf, sizeof(buf));
-      dbinfo_write_row_column_uint32(&r, getOwnNodeId());
-      const char *blockname= "TRIX";
-      dbinfo_write_row_column(&r, blockname, strlen(blockname));
-      dbinfo_write_row_column(&r, pools[i].poolname, strlen(pools[i].poolname));
-      dbinfo_write_row_column_uint32(&r, pools[i].free);
-      dbinfo_write_row_column_uint32(&r, pools[i].size);
-      dbinfo_send_row(signal, r, rl, req.apiTxnId, req.senderRef);
+      { "Attribute Order Buffer",
+        c_theAttrOrderBufferPool.getUsed(),
+        c_theAttrOrderBufferPool.getSize(),
+        c_theAttrOrderBufferPool.getEntrySize(),
+        c_theAttrOrderBufferPool.getUsedHi(),
+        { 0,0,0,0 }},
+      { "Subscription Record",
+        c_theSubscriptionRecPool.getUsed(),
+        c_theSubscriptionRecPool.getSize(),
+        c_theSubscriptionRecPool.getEntrySize(),
+        c_theSubscriptionRecPool.getUsedHi(),
+        { 0,0,0,0 }},
+      { NULL, 0,0,0,0,{0,0,0,0}}
+    };
+
+    const size_t num_config_params =
+      sizeof(pools[0].config_params) / sizeof(pools[0].config_params[0]);
+    Uint32 pool = cursor->data[0];
+    BlockNumber bn = blockToMain(number());
+    while(pools[pool].poolname)
+    {
+      jam();
+      Ndbinfo::Row row(signal, req);
+      row.write_uint32(getOwnNodeId());
+      row.write_uint32(bn);           // block number
+      row.write_uint32(instance());   // block instance
+      row.write_string(pools[pool].poolname);
+      row.write_uint64(pools[pool].used);
+      row.write_uint64(pools[pool].total);
+      row.write_uint64(pools[pool].used_hi);
+      row.write_uint64(pools[pool].entry_size);
+      for (size_t i = 0; i < num_config_params; i++)
+        row.write_uint32(pools[pool].config_params[i]);
+      ndbinfo_send_row(signal, req, row, rl);
+      pool++;
+      if (rl.need_break(req))
+      {
+        jam();
+        ndbinfo_send_scan_break(signal, req, rl, pool);
+        return;
+      }
     }
+    break;
+  }
+  default:
+    break;
   }
 
-  DbinfoScanConf *conf= (DbinfoScanConf*)signal->getDataPtrSend();
-  memcpy(conf,&req, DbinfoScanReq::SignalLengthWithCursor * sizeof(Uint32));
-  conf->requestInfo &= ~(DbinfoScanConf::MoreData);
-  sendSignal(DBINFO_REF, GSN_DBINFO_SCANCONF,
-             signal, DbinfoScanConf::SignalLengthWithCursor, JBB);
+  ndbinfo_send_scan_conf(signal, req, rl);
 }
 
 // Build index
@@ -587,6 +601,11 @@ void Trix:: execBUILD_INDX_IMPL_REQ(Signal* signal)
   subRec->m_rows_processed = 0;
   subRec->m_flags = SubscriptionRecord::RF_WAIT_GCP; // Todo make configurable
   subRec->m_gci = 0;
+  if (buildIndxReq->requestType & BuildIndxImplReq::RF_NO_DISK)
+  {
+    subRec->m_flags |= SubscriptionRecord::RF_NO_DISK;
+  }
+
   // Get column order segments
   Uint32 noOfSections = handle.m_cnt;
   if (noOfSections > 0) {
@@ -808,7 +827,7 @@ void Trix::execSUB_SYNC_CONF(Signal* signal)
     {
       jam();
       wait_gcp(signal, subRecPtr);
-      return;
+      DBUG_VOID_RETURN;
     }
     buildComplete(signal, subRecPtr);
   }
@@ -933,6 +952,18 @@ void Trix::startTableScan(Signal* signal, SubscriptionRecPtr subRecPtr)
   subSyncReq->requestInfo = 0;
   subSyncReq->fragCount = subRec->fragCount;
 
+  if (subRec->m_flags & SubscriptionRecord::RF_NO_DISK)
+  {
+    jam();
+    subSyncReq->requestInfo |= SubSyncReq::NoDisk;
+  }
+
+  if (subRec->m_flags & SubscriptionRecord::RF_TUP_ORDER)
+  {
+    jam();
+    subSyncReq->requestInfo |= SubSyncReq::TupOrder;
+  }
+  
   if (subRec->requestType == REORG_COPY)
   {
     jam();
@@ -1380,6 +1411,12 @@ Trix::execCOPY_DATA_IMPL_REQ(Signal* signal)
     ndbrequire(false);
   }
 
+  if (req->requestInfo & CopyDataReq::TupOrder)
+  {
+    jam();
+    subRec->m_flags |= SubscriptionRecord::RF_TUP_ORDER;
+  }
+
   // Get column order segments
   Uint32 noOfSections = handle.m_cnt;
   if (noOfSections > 0) {
