@@ -42,7 +42,7 @@ extern Uint32 g_start_type;
 
 bool
 Configuration::init(int _no_start, int _initial,
-                    int _initialstart, int _daemon)
+                    int _initialstart)
 {
   // Check the start flag
   if (_no_start)
@@ -54,10 +54,6 @@ Configuration::init(int _no_start, int _initial,
   if (_initial)
     _initialStart = true;
 
-  // Check daemon flag
-  if (_daemon)
-    _daemonMode = true;
-
   globalData.ownId= 0;
 
   if (_initialstart)
@@ -76,12 +72,12 @@ Configuration::init(int _no_start, int _initial,
   return true;
 }
 
-Configuration::Configuration()
+Configuration::Configuration() :
+  _executeLockCPU(NO_LOCK_CPU-1)
 {
   _fsPath = 0;
   _backupPath = 0;
   _initialStart = false;
-  _daemonMode = false;
   m_config_retriever= 0;
   m_clusterConfig= 0;
   m_clusterConfigIter= 0;
@@ -116,7 +112,10 @@ Configuration::closeConfiguration(bool end_session){
 
 void
 Configuration::fetch_configuration(const char* _connect_string,
-                                   const char* _bind_address){
+                                   int force_nodeid,
+                                   const char* _bind_address,
+                                   NodeId allocated_nodeid)
+{
   /**
    * Fetch configuration from management server
    */
@@ -125,14 +124,20 @@ Configuration::fetch_configuration(const char* _connect_string,
   }
 
   m_config_retriever= new ConfigRetriever(_connect_string,
+                                          force_nodeid,
                                           NDB_VERSION,
                                           NDB_MGM_NODE_TYPE_NDB,
 					  _bind_address);
+  if (!m_config_retriever)
+  {
+    ERROR_SET(fatal, NDBD_EXIT_MEMALLOC,
+              "Failed to create ConfigRetriever", "");
+  }
 
   if (m_config_retriever->hasError())
   {
     ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG,
-	      "Could not connect initialize handle to management server",
+	      "Could not initialize handle to management server",
 	      m_config_retriever->getErrorString());
   }
 
@@ -147,22 +152,28 @@ Configuration::fetch_configuration(const char* _connect_string,
   }
 
   ConfigRetriever &cr= *m_config_retriever;
-  
-  /**
-   * if we have a nodeid set (e.g in a restart situation)
-   * reuse it
-   */
-  if (globalData.ownId)
-    cr.setNodeId(globalData.ownId);
 
-  globalData.ownId = cr.allocNodeId(globalData.ownId ? 10 : 2 /*retry*/,
-                                    3 /*delay*/);
-  
-  if(globalData.ownId == 0){
-    ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG, 
-	      "Unable to alloc node id", m_config_retriever->getErrorString());
+  if (allocated_nodeid)
+  {
+    // The angel has already allocated the nodeid, no need to
+    // allocate it
+    globalData.ownId = allocated_nodeid;
   }
-  
+  else
+  {
+
+    const int alloc_retries = 2;
+    const int alloc_delay = 3;
+    globalData.ownId = cr.allocNodeId(alloc_retries, alloc_delay);
+    if(globalData.ownId == 0)
+    {
+      ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG,
+                "Unable to alloc node id",
+                m_config_retriever->getErrorString());
+    }
+  }
+  assert(globalData.ownId);
+
   ndb_mgm_configuration * p = cr.getConfig(globalData.ownId);
   if(p == 0){
     const char * s = cr.getErrorString();
@@ -184,15 +195,19 @@ Configuration::fetch_configuration(const char* _connect_string,
   {
     Uint32 generation;
     ndb_mgm_configuration_iterator sys_iter(*p, CFG_SECTION_SYSTEM);
-    if (sys_iter.get(CFG_SYS_CONFIG_GENERATION, &generation)) {
-      ERROR_SET(fatal, NDBD_EXIT_INVALID_CONFIG,
-                "Invalid configuration fetched", "generation missing");
+    if (sys_iter.get(CFG_SYS_CONFIG_GENERATION, &generation))
+    {
+      g_eventLogger->info("Configuration fetched from '%s:%d', unknown generation!! (likely older ndb_mgmd)",
+                          m_config_retriever->get_mgmd_host(),
+                          m_config_retriever->get_mgmd_port());
+    }
+    else
+    {
+      g_eventLogger->info("Configuration fetched from '%s:%d', generation: %d",
+                          m_config_retriever->get_mgmd_host(),
+                          m_config_retriever->get_mgmd_port(),
+                          generation);
     }
-
-    g_eventLogger->info("Configuration fetched from '%s:%d', generation: %d",
-                        m_config_retriever->get_mgmd_host(),
-                        m_config_retriever->get_mgmd_port(),
-                        generation);
   }
 
   ndb_mgm_configuration_iterator iter(* p, CFG_SECTION_NODE);
@@ -212,28 +227,6 @@ Configuration::fetch_configuration(const char* _connect_string,
   }
   NdbConfig_SetPath(datadir);
 
-  m_mgmds.clear();
-  for(ndb_mgm_first(&iter); ndb_mgm_valid(&iter); ndb_mgm_next(&iter))
-  {
-    Uint32 nodeType, port;
-    char const *hostname;
-
-    ndb_mgm_get_int_parameter(&iter,CFG_TYPE_OF_SECTION,&nodeType);
-
-    if (nodeType != NodeInfo::MGM)
-      continue;
-
-    if (ndb_mgm_get_string_parameter(&iter,CFG_NODE_HOST, &hostname) ||
-	ndb_mgm_get_int_parameter(&iter,CFG_MGM_PORT, &port) ||
-	hostname == 0 || hostname[0] == 0)
-    {
-      continue;
-    }
-    BaseString connectstring(hostname);
-    connectstring.appfmt(":%d", port);
-
-    m_mgmds.push_back(connectstring);
-  }
 }
 
 static char * get_and_validate_path(ndb_mgm_configuration_iterator &iter,
@@ -273,6 +266,8 @@ static char * get_and_validate_path(ndb_mgm_configuration_iterator &iter,
   return strdup(buf2);
 }
 
+#include "../../common/util/parse_mask.hpp"
+
 void
 Configuration::setupConfiguration(){
 
@@ -344,14 +339,22 @@ Configuration::setupConfiguration(){
   const char * mask;
   if(iter.get(CFG_DB_EXECUTE_LOCK_CPU, &mask) == 0)
   {
-    if (_executeLockCPU.parseMask(mask) < 0)
+
+    int res = parse_mask(mask, _executeLockCPU);
+    if (res < 0)
     {
+      // Could not parse LockExecuteThreadToCPU mask
+      g_eventLogger->warning("Failed to parse 'LockExecuteThreadToCPU=%s' "
+                             "(error: %d), ignoring it!",
+                             mask, res);
       _executeLockCPU.clear();
     }
   }
 
   _maintLockCPU = NO_LOCK_CPU;
   iter.get(CFG_DB_MAINT_LOCK_CPU, &_maintLockCPU);
+  if (_maintLockCPU == 65535)
+    _maintLockCPU = NO_LOCK_CPU; // Ignore old default(may come from old mgmd)
 
   if(iter.get(CFG_DB_WATCHDOG_INTERVAL_INITIAL, 
               &_timeBetweenWatchDogCheckInitial)){
@@ -454,7 +457,7 @@ Configuration::executeLockCPU() const
 void
 Configuration::executeLockCPU(Uint32 value)
 {
-  if (value >= NDB_CPU_MASK_SZ)
+  if (value >= _executeLockCPU.max_size())
   {
     value = NO_LOCK_CPU;
   }
@@ -567,6 +570,10 @@ Configuration::calcSizeAlt(ConfigValues * ownConfig){
   unsigned int noOfLocalScanRecords = 0;
   unsigned int noBatchSize = 0;
   m_logLevel = new LogLevel();
+  if (!m_logLevel)
+  {
+    ERROR_SET(fatal, NDBD_EXIT_MEMALLOC, "Failed to create LogLevel", "");
+  }
   
   struct AttribStorage { int paramId; Uint32 * storage; bool computable; };
   AttribStorage tmp[] = {
@@ -727,7 +734,9 @@ Configuration::calcSizeAlt(ConfigValues * ownConfig){
 
 
   if (noOfLocalScanRecords == 0) {
-    noOfLocalScanRecords = (noOfDBNodes * noOfScanRecords) + 1;
+    noOfLocalScanRecords = (noOfDBNodes * noOfScanRecords) + 
+      1 /* NR */ + 
+      1 /* LCP */; 
   }
   if (noOfLocalOperations == 0) {
     noOfLocalOperations= (11 * noOfOperations) / 10;
@@ -786,23 +795,14 @@ Configuration::calcSizeAlt(ConfigValues * ownConfig){
     cfg.put(CFG_DIH_API_CONNECT, 
 	    2 * noOfTransactions);
     
-    Uint32 noFragPerTable= ((noOfDBNodes + NO_OF_FRAGS_PER_CHUNK - 1) >>
-                           LOG_NO_OF_FRAGS_PER_CHUNK) <<
-                           LOG_NO_OF_FRAGS_PER_CHUNK;
+    Uint32 noFragPerTable= (((noOfDBNodes * lqhInstances) + 
+                             NO_OF_FRAGS_PER_CHUNK - 1) >>
+                            LOG_NO_OF_FRAGS_PER_CHUNK) <<
+      LOG_NO_OF_FRAGS_PER_CHUNK;
 
     cfg.put(CFG_DIH_FRAG_CONNECT, 
 	    noFragPerTable *  noOfMetaTables);
     
-    int temp;
-    temp = noOfReplicas - 2;
-    if (temp < 0)
-      temp = 1;
-    else
-      temp++;
-    cfg.put(CFG_DIH_MORE_NODES, 
-	    temp * NO_OF_FRAG_PER_NODE *
-	    noOfMetaTables *  noOfDBNodes);
-
     cfg.put(CFG_DIH_REPLICAS, 
 	    NO_OF_FRAG_PER_NODE * noOfMetaTables *
 	    noOfDBNodes * noOfReplicas * lqhInstances);
@@ -889,11 +889,6 @@ Configuration::calcSizeAlt(ConfigValues * ownConfig){
     (m_ownConfig, 0);
 }
 
-void
-Configuration::setInitialStart(bool val){
-  _initialStart = val;
-}
-
 void
 Configuration::setAllRealtimeScheduler()
 {
