@@ -58,6 +58,8 @@
 #include "ndbd_malloc_impl.hpp"
 #include <blocks/record_types.hpp>
 
+#include "Ndbinfo.hpp"
+
 #ifdef VM_TRACE
 #define D(x) \
   do { \
@@ -205,6 +207,13 @@ public:
   void synchronize_threads_for_blocks(Signal*, const Uint32 blocks[],
                                       const Callback&, JobBufferLevel = JBB);
   
+  /**
+   * This method make sure that the path specified in blocks[]
+   *   will be traversed before returning
+   */
+  void synchronize_path(Signal*, const Uint32 blocks[],
+                        const Callback&, JobBufferLevel = JBB);
+
 private:
   struct SyncThreadRecord
   {
@@ -216,6 +225,10 @@ private:
   void execSYNC_THREAD_REQ(Signal*);
   void execSYNC_THREAD_CONF(Signal*);
 
+  void execSYNC_REQ(Signal*);
+
+  void execSYNC_PATH_REQ(Signal*);
+  void execSYNC_PATH_CONF(Signal*);
 public:
   virtual const char* get_filename(Uint32 fd) const { return "";}
 protected:
@@ -328,6 +341,9 @@ protected:
   void handle_invalid_sections_in_send_signal(Signal*) const;
   void handle_lingering_sections_after_execute(Signal*) const;
   void handle_lingering_sections_after_execute(SectionHandle*) const;
+  void handle_invalid_fragmentInfo(Signal*) const;
+  void handle_send_failed(SendStatus, Signal*) const;
+  void handle_out_of_longsignal_memory(Signal*) const;
 
   /**
    * Send routed signals (ONLY LOCALLY)
@@ -589,9 +605,11 @@ protected:
    * If the cause of the shutdown is known use extradata to add an 
    * errormessage describing the problem
    */
-  void progError(int line, int err_code, const char* extradata=NULL) const ;
+  void progError(int line, int err_code, const char* extradata=NULL) const
+    ATTRIBUTE_NORETURN;
 private:
-  void  signal_error(Uint32, Uint32, Uint32, const char*, int) const ;
+  void  signal_error(Uint32, Uint32, Uint32, const char*, int) const
+    ATTRIBUTE_NORETURN;
   const NodeId         theNodeId;
   const BlockNumber    theNumber;
   const Uint32 theInstance;
@@ -684,6 +702,25 @@ protected:
    */
   void deallocRecord(void **, const char * type, size_t s, size_t n);
   
+  /**
+   * Allocate memory from global pool,
+   *   returns #chunks used
+   *
+   * Typically used by part of code, not converted to use global pool
+   *   directly, but allocates everything during startup
+   */
+  struct AllocChunk
+  {
+    Uint32 ptrI;
+    Uint32 cnt;
+  };
+  Uint32 allocChunks(AllocChunk dst[], Uint32 /* size of dst */ arraysize,
+                     Uint32 /* resource group */ rg,
+                     Uint32 /* no of pages to allocate */ pages,
+                     Uint32 paramId /* for error message if failing */);
+
+  static int sortchunks(const void*, const void*);
+
   /**
    * General info event (sent to cluster log)
    */
@@ -930,6 +967,22 @@ public:
   void debugOutUnlock() { globalSignalLoggers.unlock(); }
   const char* debugOutTag(char* buf, int line);
 #endif
+
+  void ndbinfo_send_row(Signal* signal,
+                        const DbinfoScanReq& req,
+                        const Ndbinfo::Row& row,
+                        Ndbinfo::Ratelimit& rl) const;
+
+  void ndbinfo_send_scan_break(Signal* signal,
+                               DbinfoScanReq& req,
+                               const Ndbinfo::Ratelimit& rl,
+                               Uint32 data1, Uint32 data2 = 0,
+                               Uint32 data3 = 0, Uint32 data4 = 0) const;
+
+  void ndbinfo_send_scan_conf(Signal* signal,
+                              DbinfoScanReq& req,
+                              const Ndbinfo::Ratelimit& rl) const;
+
 };
 
 // outside blocks e.g. within a struct
