@@ -46,6 +46,7 @@ static const char * f_method = "MSms";
  * For single-threaded ndbd, they are no-ops (but still called, to avoid
  * having to compile this file twice).
  */
+extern void mt_mem_manager_init();
 extern void mt_mem_manager_lock();
 extern void mt_mem_manager_unlock();
 
@@ -66,6 +67,8 @@ struct InitChunk
 
 #include <NdbOut.hpp>
 
+extern void ndbd_alloc_touch_mem(void * p, size_t sz, volatile Uint32 * watchCounter);
+
 static
 bool
 do_malloc(Uint32 pages, InitChunk* chunk, Uint32 *watchCounter = 0)
@@ -88,6 +91,9 @@ retry:
     ptr = 0;
     while (ptr == 0)
     {
+      if (watchCounter)
+        *watchCounter = 9;
+
       ptr = sbrk(sizeof(Alloc_page) * sz);
       
       if (ptr == (void*)-1)
@@ -116,6 +122,9 @@ retry:
     ptr = 0;
     while (ptr == 0)
     {
+      if (watchCounter)
+        *watchCounter = 9;
+
       ptr = malloc(sizeof(Alloc_page) * sz);
       if (ptr == 0)
       {
@@ -148,6 +157,8 @@ retry:
     chunk->m_ptr = (Alloc_page*)((UintPtr(ptr) + align) & ~align);
   }
 
+  ndbd_alloc_touch_mem(chunk->m_ptr, chunk->m_cnt * sizeof(Alloc_page), watchCounter);
+
 #ifdef UNIT_TEST
   ndbout_c("do_malloc(%d) -> %p %d", pages, ptr, chunk->m_cnt);
   if (1)
@@ -174,6 +185,8 @@ retry:
 Uint32
 Ndbd_mem_manager::ndb_log2(Uint32 input)
 {
+  if (input > 65535)
+    return 16;
   input = input | (input >> 8);
   input = input | (input >> 4);
   input = input | (input >> 2);
@@ -196,6 +209,7 @@ Ndbd_mem_manager::Ndbd_mem_manager()
     g_eventLogger->error("Invalid build, ndbd_malloc_impl.cpp:%d", __LINE__);
     abort();
   }
+  mt_mem_manager_init();
 }
 
 /**
@@ -292,16 +306,26 @@ Ndbd_mem_manager::init(Uint32 *watchCounter, bool alloc_less_memory)
     m_resource_limit[0].m_min = pages;
   }
   
-  g_eventLogger->info("Ndbd_mem_manager::init(%d) min: %ldMb initial: %ldMb",
+  const Uint64 pg = Uint64(sizeof(Alloc_page));
+  g_eventLogger->info("Ndbd_mem_manager::init(%d) min: %lluMb initial: %lluMb",
                       alloc_less_memory,
-                      (sizeof(Alloc_page)*m_resource_limit[0].m_min)>>20,
-                      (sizeof(Alloc_page)*pages)>>20);
+                      (pg*m_resource_limit[0].m_min)>>20,
+                      (pg*pages) >> 20);
 
   if (pages == 0)
   {
-    return 0;
+    return false;
   }
 
+#if SIZEOF_CHARP == 4
+  Uint64 sum = (pg*pages); 
+  if (sum >= (Uint64(1) << 32))
+  {
+    g_eventLogger->error("Trying to allocate more that 4Gb with 32-bit binary!!");
+    return false;
+  }
+#endif
+
   /**
    * Do malloc
    */
@@ -313,13 +337,9 @@ Ndbd_mem_manager::init(Uint32 *watchCounter, bool alloc_less_memory)
   Uint32 allocated = 0;
   while (cnt < MAX_CHUNKS && allocated < pages)
   {
-    InitChunk chunk; 
-    LINT_INIT(chunk.m_start);
+    InitChunk chunk;
+    bzero(&chunk, sizeof(chunk));
     
-#if defined(_lint) || defined(FORCE_INIT_OF_VARS)
-    memset((char*) &chunk, 0 , sizeof(chunk));
-#endif
-
     if (do_malloc(pages - allocated, &chunk, watchCounter))
     {
       if (watchCounter)
@@ -372,10 +392,12 @@ Ndbd_mem_manager::init(Uint32 *watchCounter, bool alloc_less_memory)
   {
     UintPtr start = UintPtr(chunks[i].m_ptr) - UintPtr(m_base_page);
     start >>= (2 + BMW_2LOG);
-    UintPtr last = start + chunks[i].m_cnt;
-    chunks[i].m_start = start;
     assert((Uint64(start) >> 32) == 0);
-    
+    chunks[i].m_start = Uint32(start);
+    Uint64 last64 = start + chunks[i].m_cnt;
+    assert((last64 >> 32) == 0);
+    Uint32 last = Uint32(last64);
+
     if (last > max_page)
       max_page = last;
   }
@@ -811,7 +833,7 @@ Ndbd_mem_manager::alloc_pages(Uint32 type, Uint32* i, Uint32 *cnt, Uint32 min)
   if (likely(req))
   {
     // Hi order allocations can always use any zone
-    alloc(NDB_ZONE_ANY, i, &req, 1); 
+    alloc(NDB_ZONE_ANY, i, &req, min); 
     * cnt = req;
     if (unlikely(req < res0)) // Got min than what was reserved :-(
     {
