@@ -17,8 +17,8 @@
 */
 
 #include <ndb_global.h>
-#include <my_sys.h>
 #include <mgmapi.h>
+#include <mgmapi_internal.h>
 
 #include <NdbOut.hpp>
 #include <Properties.hpp>
@@ -31,7 +31,8 @@
 #include "ndb_logevent.hpp"
 
 extern
-int ndb_mgm_listen_event_internal(NdbMgmHandle, const int filter[], int, my_socket*);
+int ndb_mgm_listen_event_internal(NdbMgmHandle, const int filter[],
+                                  int, NDB_SOCKET_TYPE*);
 
 struct ndb_logevent_error_msg {
   enum ndb_logevent_handle_error code;
@@ -52,17 +53,40 @@ struct ndb_logevent_handle {
   enum ndb_logevent_handle_error m_error;
 };
 
+/*
+  Create a new NdbLogEventHandle for reading events from
+  the same socket as the NdbMgmHandle
+*/
+
+NdbLogEventHandle
+ndb_mgm_create_logevent_handle_same_socket(NdbMgmHandle mh)
+{
+  NdbLogEventHandle h=
+    (NdbLogEventHandle)malloc(sizeof(ndb_logevent_handle));
+  if (!h)
+    return NULL;
+
+  h->socket= _ndb_mgm_get_socket(mh);
+
+  return h;
+}
+
 extern "C"
 NdbLogEventHandle
 ndb_mgm_create_logevent_handle(NdbMgmHandle mh,
 			       const int filter[])
 {
-  my_socket sock;
-  if(ndb_mgm_listen_event_internal(mh, filter, 1, &sock) < 0)
-    return 0;
-
   NdbLogEventHandle h=
-    (NdbLogEventHandle)my_malloc(sizeof(ndb_logevent_handle),MYF(MY_WME));
+    (NdbLogEventHandle)malloc(sizeof(ndb_logevent_handle));
+  if (!h)
+    return NULL;
+
+  NDB_SOCKET_TYPE sock;
+  if(ndb_mgm_listen_event_internal(mh, filter, 1, &sock) < 0)
+  {
+    free(h);
+    return 0;
+  }
 
   h->socket= sock;
 
@@ -93,7 +117,7 @@ void ndb_mgm_destroy_logevent_handle(NdbLogEventHandle * h)
   if ( *h )
     my_socket_close((*h)->socket);
 
-  my_free((char*)* h,MYF(MY_ALLOW_ZERO_PTR));
+  free(*h);
   * h = 0;
 }
 
@@ -470,7 +494,7 @@ int ndb_logevent_get_next(const NdbLogEventHandle h,
     or timeout expired. The MGM server will continusly
     send <PING>'s that should be ignored.
   */
-  char buf[256];
+  char buf[1024];
   NDB_TICKS start = NdbTick_CurrentMillisecond();
   while(1)
   {
@@ -562,18 +586,40 @@ int ndb_logevent_get_next(const NdbLogEventHandle h,
   /* fill in header info from p */
   for (i= 0; ndb_logevent_body[i].token; i++)
   {
-    if ( ndb_logevent_body[i].type != dst->type )
-      continue;
-    if ( p.get(ndb_logevent_body[i].token, &val) == 0 )
+    if ( ndb_logevent_body[i].type == dst->type )
+      break;
+  }
+
+  if (ndb_logevent_body[i].token)
+  {
+    do {
+
+      if ( p.get(ndb_logevent_body[i].token, &val) == 0 )
+      {
+        h->m_error= NDB_LEH_UNKNOWN_EVENT_VARIABLE;
+        return -1;
+      }
+      if ( memcpy_atoi((char *)dst+ndb_logevent_body[i].offset, val,
+                       ndb_logevent_body[i].size) )
+      {
+        h->m_error= NDB_LEH_INTERNAL_ERROR;
+        return -1;
+      }
+    } while (ndb_logevent_body[++i].type == dst->type);
+  }
+  else
+  {
+    if (!p.get("data", &val))
     {
       h->m_error= NDB_LEH_UNKNOWN_EVENT_VARIABLE;
       return -1;
     }
-    if ( memcpy_atoi((char *)dst+ndb_logevent_body[i].offset, val,
-		     ndb_logevent_body[i].size) )
+    BaseString tmp(val);
+    Vector<BaseString> list;
+    tmp.split(list);
+    for (size_t j = 0; j<list.size(); j++)
     {
-      h->m_error= NDB_LEH_INTERNAL_ERROR;
-      return -1;
+      dst->Data[j] = atoi(list[j].c_str());
     }
   }
   return 1;
