@@ -33,11 +33,11 @@
 #include "Services.hpp"
 #include "../mgmapi/ndb_logevent.hpp"
 
+#include "ndb_mgmd_error.h"
+
 #include <base64.h>
 #include <ndberror.h>
 
-#include <ndbinfo.h>
-
 extern bool g_StopServer;
 extern bool g_RestartServer;
 extern EventLogger * g_eventLogger;
@@ -160,6 +160,7 @@ ParserRow<MgmApiSession> commands[] = {
     MGM_ARG("initialstart", Int, Optional, "Initial start"),
     MGM_ARG("nostart", Int, Optional, "No start"),
     MGM_ARG("abort", Int, Optional, "Abort"),
+    MGM_ARG("force", Int, Optional, "Force"),
 
   MGM_CMD("restart all", &MgmApiSession::restartAll, ""),
     MGM_ARG("initialstart", Int, Optional, "Initial start"),
@@ -205,6 +206,7 @@ ParserRow<MgmApiSession> commands[] = {
   MGM_CMD("stop v2", &MgmApiSession::stop_v2, ""),
     MGM_ARG("node", String, Mandatory, "Node"),
     MGM_ARG("abort", Int, Mandatory, "Node"),
+    MGM_ARG("force", Int, Optional, "Force"),
 
   MGM_CMD("stop all", &MgmApiSession::stopAll, ""),
     MGM_ARG("abort", Int, Mandatory, "Node"),
@@ -292,9 +294,6 @@ ParserRow<MgmApiSession> commands[] = {
   MGM_CMD("drop nodegroup", &MgmApiSession::drop_nodegroup, ""),
     MGM_ARG("ng", Int, Mandatory, "Nodegroup"),
 
-  MGM_CMD("ndbinfo", &MgmApiSession::getNdbInfo, ""),
-    MGM_ARG("query", String, Mandatory, "SQL-Like Query"),
-
   MGM_CMD("show config", &MgmApiSession::showConfig, ""),
     MGM_ARG("Section", String, Optional, "Section name"),
     MGM_ARG("NodeId", Int, Optional, "Nodeid"),
@@ -307,6 +306,10 @@ ParserRow<MgmApiSession> commands[] = {
 
   MGM_CMD("show variables", &MgmApiSession::show_variables, ""),
 
+  MGM_CMD("dump events", &MgmApiSession::dump_events, ""),
+    MGM_ARG("type", Int, Mandatory, "Type of event"),
+    MGM_ARG("nodes", String, Optional, "Nodes to include"),
+
   MGM_END()
 };
 
@@ -433,7 +436,7 @@ MgmApiSession::runSession()
         // Send result to client
         m_output->println("result: %s, '%s'",
                           msg, ctx.m_currentToken);
-        m_output->println("");
+        m_output->print("\n");
       }
     }
 
@@ -495,7 +498,7 @@ MgmApiSession::get_nodeid(Parser_t::Context &,
   if(endian 
      && strcmp(endian,(endian_check.c[sizeof(long)-1])?"big":"little")!=0) {
     m_output->println("result: Node does not have the same endianness as the management server.");
-    m_output->println("");
+    m_output->println("%s", "");
     return;
   }
 
@@ -510,7 +513,7 @@ MgmApiSession::get_nodeid(Parser_t::Context &,
     break;
   default:
     m_output->println("result: unknown nodetype %d", nodetype);
-    m_output->println("");
+    m_output->println("%s", "");
     return;
   }
 
@@ -521,7 +524,15 @@ MgmApiSession::get_nodeid(Parser_t::Context &,
     m_output->println("result: getpeername(" MY_SOCKET_FORMAT   \
                       ") failed, err= %d",
                       MY_SOCKET_FORMAT_VALUE(m_socket), r);
-    m_output->println("");
+    m_output->println("%s", "");
+    return;
+  }
+
+  /* Check nodeid parameter */
+  if (nodeid > MAX_NODES_ID)
+  {
+    m_output->println("result: illegal nodeid %u", nodeid);
+    m_output->println("%s", "");
     return;
   }
 
@@ -562,7 +573,7 @@ MgmApiSession::get_nodeid(Parser_t::Context &,
       /* only use error_code protocol if client knows about it */
       if (log_event_version)
         m_output->println("error_code: %d", error_code);
-      m_output->println("");
+      m_output->println("%s", "");
       return;
     }
   }    
@@ -572,14 +583,14 @@ MgmApiSession::get_nodeid(Parser_t::Context &,
     m_output->println(cmd);
     m_output->println("result: incompatible version mgmt 0x%x and node 0x%x",
 		      NDB_VERSION, version);
-    m_output->println("");
+    m_output->println("%s", "");
     return;
   }
 #endif
   
   m_output->println("nodeid: %u", tmp);
   m_output->println("result: Ok");
-  m_output->println("");
+  m_output->println("%s", "");
   m_allocated_resources->reserve_node(tmp, timeout*1000);
   
   if (name)
@@ -608,7 +619,7 @@ MgmApiSession::getConfig(Parser_t::Context &,
                                   pack64, error))
   {
     m_output->println("result: %s", error.c_str());
-    m_output->println("");
+    m_output->print("\n");
     return;
   }
 
@@ -657,7 +668,7 @@ MgmApiSession::insertError(Parser<MgmApiSession>::Context &,
     m_output->println("result: %s", get_error_text(result));
   else
     m_output->println("result: Ok");
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -675,22 +686,22 @@ MgmApiSession::setTrace(Parser<MgmApiSession>::Context &,
     m_output->println("result: %s", get_error_text(result));
   else
     m_output->println("result: Ok");
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
 MgmApiSession::getVersion(Parser<MgmApiSession>::Context &,
 			  Properties const &) {
   m_output->println("version");
-  m_output->println("id: %d", NDB_VERSION);
-  m_output->println("major: %d", getMajor(NDB_VERSION));
-  m_output->println("minor: %d", getMinor(NDB_VERSION));
-  m_output->println("build: %d", getBuild(NDB_VERSION));
-  m_output->println("string: %s", NDB_VERSION_STRING);
-  m_output->println("mysql_major: %d", getMajor(NDB_MYSQL_VERSION_D));
-  m_output->println("mysql_minor: %d", getMinor(NDB_MYSQL_VERSION_D));
-  m_output->println("mysql_build: %d", getBuild(NDB_MYSQL_VERSION_D));
-  m_output->println("");
+  m_output->println("id: %d", NDB_VERSION_D);
+  m_output->println("major: %d", NDB_VERSION_MAJOR);
+  m_output->println("minor: %d", NDB_VERSION_MINOR);
+  m_output->println("build: %d", NDB_VERSION_BUILD);
+  m_output->println("string: %s", m_mgmsrv.get_version_string());
+  m_output->println("mysql_major: %d", NDB_MYSQL_VERSION_MAJOR);
+  m_output->println("mysql_minor: %d", NDB_MYSQL_VERSION_MINOR);
+  m_output->println("mysql_build: %d", NDB_MYSQL_VERSION_BUILD);
+  m_output->println("%s", "");
 }
 
 void
@@ -722,7 +733,7 @@ MgmApiSession::startBackup(Parser<MgmApiSession>::Context &,
     if (completed)
       m_output->println("id: %d", backupId);
   }
-  m_output->println("");
+  m_output->println("%s", "");
   DBUG_VOID_RETURN;
 }
 
@@ -740,7 +751,7 @@ MgmApiSession::abortBackup(Parser<MgmApiSession>::Context &,
     m_output->println("result: %s", get_error_text(result));
   else
     m_output->println("result: Ok");
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 /*****************************************************************************/
@@ -760,7 +771,7 @@ MgmApiSession::dumpState(Parser<MgmApiSession>::Context &,
     m_output->println("result: %s", get_error_text(result));
   else
     m_output->println("result: Ok");
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 
@@ -805,7 +816,7 @@ MgmApiSession::getClusterLogLevel(Parser<MgmApiSession>::Context &			, Propertie
     category = (LogLevel::EventCategory) i;
     m_output->println("%s: %d", names[i], m_mgmsrv.m_event_listner[0].m_logLevel.getLogLevel(category));
   }
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -824,7 +835,7 @@ MgmApiSession::setClusterLogLevel(Parser<MgmApiSession>::Context &,
 
   if(level > NDB_MGM_MAX_LOGLEVEL) {
     m_output->println("result: Invalid loglevel %d", level);
-    m_output->println("");
+    m_output->println("%s", "");
     DBUG_VOID_RETURN;
   }
 
@@ -835,7 +846,7 @@ MgmApiSession::setClusterLogLevel(Parser<MgmApiSession>::Context &,
   if (m_mgmsrv.m_event_listner[0].m_logLevel.setLogLevel(category,level))
   {
     m_output->println("result: Invalid category %d", category);
-    m_output->println("");
+    m_output->println("%s", "");
     m_mgmsrv.m_event_listner.unlock();
     DBUG_VOID_RETURN;
   }
@@ -847,7 +858,7 @@ MgmApiSession::setClusterLogLevel(Parser<MgmApiSession>::Context &,
   }
 
   m_output->println("result: Ok");
-  m_output->println("");
+  m_output->println("%s", "");
   DBUG_VOID_RETURN;
 }
 
@@ -865,7 +876,7 @@ MgmApiSession::setLogLevel(Parser<MgmApiSession>::Context &,
   if(level > NDB_MGM_MAX_LOGLEVEL) {
     m_output->println("set loglevel reply");
     m_output->println("result: Invalid loglevel: %s", errorString.c_str());
-    m_output->println("");
+    m_output->println("%s", "");
     return;
   }
 
@@ -880,7 +891,7 @@ MgmApiSession::setLogLevel(Parser<MgmApiSession>::Context &,
 
   m_output->println("set loglevel reply");
   m_output->println("result: Ok");
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -897,7 +908,7 @@ MgmApiSession::stopSignalLog(Parser<MgmApiSession>::Context &,
     m_output->println("result: %s", get_error_text(result));
   else
     m_output->println("result: Ok");
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -917,7 +928,7 @@ MgmApiSession::restart(Properties const &args, int version) {
   Uint32
     nostart = 0,
     initialstart = 0,
-    abort = 0;
+    abort = 0, force = 0;
   char *nodes_str;
   Vector<NodeId> nodes;
     
@@ -925,6 +936,7 @@ MgmApiSession::restart(Properties const &args, int version) {
   args.get("nostart", &nostart);
   args.get("abort", &abort);
   args.get("node", (const char **)&nodes_str);
+  args.get("force", &force);
 
   char *p, *last;
   for((p = strtok_r(nodes_str, " ", &last));
@@ -940,7 +952,15 @@ MgmApiSession::restart(Properties const &args, int version) {
                                     initialstart != 0,
                                     abort != 0,
                                     &m_stopSelf);
-  
+
+  if (force &&
+      (result == NODE_SHUTDOWN_WOULD_CAUSE_SYSTEM_CRASH ||
+       result == UNSUPPORTED_NODE_SHUTDOWN))
+  {
+    // Force restart by restarting all nodes
+    result = m_mgmsrv.restartDB(nostart, initialstart, false, &restarted);
+  }
+
   m_output->println("restart reply");
   if(result != 0){
     m_output->println("result: %d-%s", result, get_error_text(result));
@@ -949,7 +969,7 @@ MgmApiSession::restart(Properties const &args, int version) {
   m_output->println("restarted: %d", restarted);
   if(version>1)
     m_output->println("disconnect: %d", (m_stopSelf)?1:0);
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -973,7 +993,7 @@ MgmApiSession::restartAll(Parser<MgmApiSession>::Context &,
   else
     m_output->println("result: Ok");
   m_output->println("restarted: %d", count);
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 static void
@@ -1051,7 +1071,7 @@ MgmApiSession::getStatus(Parser<MgmApiSession>::Context &,
     SLEEP_ERROR_INSERTED(int(7+i));
     printNodeStatus(m_output, m_mgmsrv, types[i]);
   }
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 
@@ -1078,7 +1098,7 @@ MgmApiSession::getInfoClusterLog(Parser<MgmApiSession>::Context &,
                       names[i],
                       isEventLogFilterEnabled(i));
   }
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1095,7 +1115,7 @@ MgmApiSession::stop_v2(Parser<MgmApiSession>::Context &,
 
 void
 MgmApiSession::stop(Properties const &args, int version) {
-  Uint32 abort;
+  Uint32 abort, force = 0;
   char *nodes_str;
   Vector<NodeId> nodes;
 
@@ -1104,10 +1124,11 @@ MgmApiSession::stop(Properties const &args, int version) {
   {
     m_output->println("stop reply");
     m_output->println("result: empty node list");
-    m_output->println("");
+    m_output->println("%s", "");
     return;
   }
   args.get("abort", &abort);
+  args.get("force", &force);
 
   char *p, *last;
   for((p = strtok_r(nodes_str, " ", &last));
@@ -1119,8 +1140,18 @@ MgmApiSession::stop(Properties const &args, int version) {
   int stopped= 0;
   int result= 0;
   if (nodes.size())
+  {
     result= m_mgmsrv.stopNodes(nodes, &stopped, abort != 0, &m_stopSelf);
 
+    if (force &&
+        (result == NODE_SHUTDOWN_WOULD_CAUSE_SYSTEM_CRASH ||
+         result == UNSUPPORTED_NODE_SHUTDOWN))
+    {
+      // Force stop and shutdown all remaining nodes
+      result = m_mgmsrv.shutdownDB(&stopped, false);
+    }
+  }
+
   m_output->println("stop reply");
   if(result != 0)
     m_output->println("result: %s", get_error_text(result));
@@ -1129,7 +1160,7 @@ MgmApiSession::stop(Properties const &args, int version) {
   m_output->println("stopped: %d", stopped);
   if(version>1)
     m_output->println("disconnect: %d", (m_stopSelf)?1:0);
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1162,7 +1193,7 @@ MgmApiSession::stopAll(Parser<MgmApiSession>::Context &,
   m_output->println("stopped: %d", stopped[0]+stopped[1]);
   if(ver >1)
     m_output->println("disconnect: %d", (m_stopSelf)?1:0);
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1181,7 +1212,7 @@ MgmApiSession::enterSingleUser(Parser<MgmApiSession>::Context &,
   else {
     m_output->println("result: Ok");
   }
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1194,7 +1225,7 @@ MgmApiSession::exitSingleUser(Parser<MgmApiSession>::Context &,
     m_output->println("result: %s", get_error_text(result));
   else
     m_output->println("result: Ok");
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 
@@ -1212,7 +1243,7 @@ MgmApiSession::startSignalLog(Parser<MgmApiSession>::Context &,
     m_output->println("result: %s", get_error_text(result));
   else
     m_output->println("result: Ok");
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1257,7 +1288,7 @@ MgmApiSession::logSignals(Parser<MgmApiSession>::Context &,
     m_output->println("result: %s", get_error_text(result));
   else
     m_output->println("result: Ok");
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1274,7 +1305,7 @@ MgmApiSession::start(Parser<MgmApiSession>::Context &,
     m_output->println("result: %s", get_error_text(result));
   else
     m_output->println("result: Ok");
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1290,7 +1321,7 @@ MgmApiSession::startAll(Parser<MgmApiSession>::Context &,
   m_output->println("start reply");
   m_output->println("result: Ok");
   m_output->println("started: %d", started);
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 
@@ -1328,7 +1359,7 @@ MgmApiSession::setLogFilter(Parser_t::Context &ctx,
 
   m_output->println("set logfilter reply");
   m_output->println("result: %d", result);
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 #ifdef NOT_USED
@@ -1344,6 +1375,54 @@ operator<<(NdbOut& out, const LogLevel & ll)
 }
 #endif
 
+
+static void
+logevent2str(BaseString& str, int eventType,
+             const Uint32* theData,
+             Uint32 len,
+             NodeId nodeId, Uint32 timeval,
+             char* pretty_text, size_t pretty_text_size)
+{
+  str.assign("log event reply\n");
+  str.appfmt("type=%d\n", eventType);
+  str.appfmt("time=%d\n", timeval);
+  str.appfmt("source_nodeid=%d\n", nodeId);
+  unsigned i;
+  for (i = 0; ndb_logevent_body[i].token; i++)
+  {
+    if ( ndb_logevent_body[i].type == eventType)
+      break;
+  }
+
+  if (ndb_logevent_body[i].token)
+  {
+    do {
+      int val= theData[ndb_logevent_body[i].index];
+      if (ndb_logevent_body[i].index_fn)
+        val= (*(ndb_logevent_body[i].index_fn))(val);
+      str.appfmt("%s=%d\n",ndb_logevent_body[i].token, val);
+      if(strcmp(ndb_logevent_body[i].token,"error") == 0)
+      {
+        int pretty_text_len= strlen(pretty_text);
+        if(pretty_text_size-pretty_text_len-3 > 0)
+        {
+          BaseString::snprintf(pretty_text+pretty_text_len, 4 , " - ");
+          ndb_error_string(val, pretty_text+(pretty_text_len+3),
+                           pretty_text_size-pretty_text_len-3);
+        }
+      }
+    } while (ndb_logevent_body[++i].type == eventType);
+  }
+  else
+  {
+    str.append("data=");
+    for (i = 1; i<len; i++)
+      str.appfmt("%u ", theData[i]);
+    str.append("\n");
+  }
+}
+
+
 void
 Ndb_mgmd_event_service::log(int eventType, const Uint32* theData, 
 			    Uint32 len, NodeId nodeId){
@@ -1359,33 +1438,18 @@ Ndb_mgmd_event_service::log(int eventType, const Uint32* theData,
   if (EventLoggerBase::event_lookup(eventType,cat,threshold,severity,textF))
     DBUG_VOID_RETURN;
 
-  char m_text[512];
-  EventLogger::getText(m_text, sizeof(m_text),
+  // Generate the message for pretty format clients
+  char pretty_text[512];
+  EventLogger::getText(pretty_text, sizeof(pretty_text),
 		       textF, theData, len, nodeId);
 
-  BaseString str("log event reply\n");
-  str.appfmt("type=%d\n", eventType);
-  str.appfmt("time=%d\n", 0);
-  str.appfmt("source_nodeid=%d\n", nodeId);
-  for (i= 0; ndb_logevent_body[i].token; i++)
-  {
-    if ( ndb_logevent_body[i].type != eventType)
-      continue;
-    int val= theData[ndb_logevent_body[i].index];
-    if (ndb_logevent_body[i].index_fn)
-      val= (*(ndb_logevent_body[i].index_fn))(val);
-    str.appfmt("%s=%d\n",ndb_logevent_body[i].token, val);
-    if(strcmp(ndb_logevent_body[i].token,"error") == 0)
-    {
-      int m_text_len= strlen(m_text);
-      if(sizeof(m_text)-m_text_len-3 > 0)
-      {
-        BaseString::snprintf(m_text+m_text_len, 4 , " - ");
-        ndb_error_string(val, m_text+(m_text_len+3), sizeof(m_text)-m_text_len-3);
-      }
-    }
-  }
-  
+  // Generate the message for parseable format clients
+  // and if there is a field named "error" append the ndb_error_string
+  // for that error number to the end of the pretty format message
+  BaseString str;
+  logevent2str(str, eventType, theData, len, nodeId, 0,
+               pretty_text, sizeof(pretty_text));
+
   Vector<NDB_SOCKET_TYPE> copy;
   m_clients.lock();
   for(i = m_clients.size() - 1; i >= 0; i--)
@@ -1401,7 +1465,7 @@ Ndb_mgmd_event_service::log(int eventType, const Uint32* theData,
       if (m_clients[i].m_parsable)
         r= out.println(str.c_str());
       else
-        r= out.println(m_text);
+        r= out.println(pretty_text);
 
       if (r<0)
       {
@@ -1443,7 +1507,7 @@ Ndb_mgmd_event_service::update_log_level(const LogLevel &tmp)
 {
   m_logLevel = tmp;
   EventSubscribeReq req;
-  req = tmp;
+  req.assign(tmp);
   // send update to all nodes
   req.blockRef = 0;
   m_mgmsrv->m_log_level_requests.push_back(req);
@@ -1531,7 +1595,7 @@ MgmApiSession::setParameter(Parser_t::Context &,
   m_output->println("set parameter reply");
   m_output->println("message: %s", result.c_str());
   m_output->println("result: %d", ret);
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1554,7 +1618,7 @@ MgmApiSession::setConnectionParameter(Parser_t::Context &ctx,
   m_output->println("set connection parameter reply");
   m_output->println("message: %s", result.c_str());
   m_output->println("result: %s", (ret>0)?"Ok":"Failed");
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1578,7 +1642,7 @@ MgmApiSession::getConnectionParameter(Parser_t::Context &ctx,
   m_output->println("get connection parameter reply");
   m_output->println("value: %d", value);
   m_output->println("result: %s", (ret>0)?"Ok":result.c_str());
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1644,7 +1708,7 @@ done:
   m_output->println("result: %d", result);
   if(result != 0)
     m_output->println("msg: %s", msg.c_str());
-  m_output->println("");
+  m_output->println("%s", "");
 
   if(result==0)
   {
@@ -1697,7 +1761,7 @@ MgmApiSession::purge_stale_sessions(Parser_t::Context &ctx,
   if (str.length() > 0)
     m_output->println("purged:%s",str.c_str());
   m_output->println("result: Ok");
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1709,7 +1773,7 @@ MgmApiSession::check_connection(Parser_t::Context &ctx,
   SLEEP_ERROR_INSERTED(2);
   m_output->println("result: Ok");
   SLEEP_ERROR_INSERTED(3);
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1747,7 +1811,7 @@ MgmApiSession::get_mgmd_nodeid(Parser_t::Context &ctx,
   m_output->println("nodeid:%u",m_mgmsrv.getOwnNodeId());
   SLEEP_ERROR_INSERTED(1);
 
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1772,7 +1836,7 @@ MgmApiSession::report_event(Parser_t::Context &ctx,
   m_mgmsrv.eventReport(data, length);
   m_output->println("report event reply");
   m_output->println("result: ok");
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1815,7 +1879,7 @@ MgmApiSession::create_nodegroup(Parser_t::Context &ctx,
   {
     m_output->println("result: Ok");
   }
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1836,7 +1900,7 @@ MgmApiSession::drop_nodegroup(Parser_t::Context &ctx,
 //end:
   m_output->println("drop nodegroup reply");
   m_output->println("result: %s", result.c_str());
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1886,7 +1950,7 @@ MgmApiSession::listSessions(Parser_t::Context &ctx,
                             Properties const &args) {
   m_mgmsrv.get_socket_server()->foreachSession(list_session,(void*)this);
 
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 void
@@ -1894,7 +1958,7 @@ MgmApiSession::getSessionId(Parser_t::Context &ctx,
                                  Properties const &args) {
   m_output->println("get session id reply");
   m_output->println("id: %llu",m_session_id);
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 struct get_session_param {
@@ -1953,7 +2017,7 @@ MgmApiSession::getSession(Parser_t::Context &ctx,
   if(p.found==false)
     m_output->println("id: 0");
 
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 
@@ -2025,7 +2089,7 @@ void MgmApiSession::setConfig(Parser_t::Context &ctx, Properties const &args)
                          &buf64[start],
                          len64-start)) < 1)
       {
-        delete buf64;
+        delete[] buf64;
         result.assfmt("read_socket failed, errno: %d", errno);
         goto done;
       }
@@ -2034,16 +2098,16 @@ void MgmApiSession::setConfig(Parser_t::Context &ctx, Properties const &args)
 
     char* decoded = new char[base64_needed_decoded_length((size_t)len64 - 1)];
     int decoded_len= base64_decode(buf64, len64-1, decoded, NULL);
-    delete buf64;
+    delete[] buf64;
 
     ConfigValuesFactory cvf;
     if(!cvf.unpack(decoded, decoded_len))
     {
-      delete decoded;
+      delete[] decoded;
       result.assfmt("Failed to unpack config");
       goto done;
     }
-    delete decoded;
+    delete[] decoded;
 
     Config new_config(cvf.getConfigValues());
 
@@ -2062,43 +2126,7 @@ done:
 
   m_output->println("set config reply");
   m_output->println("result: %s", result.c_str());
-  m_output->println("");
-}
-
-int print_ndbinfo_table_mgm(struct ndbinfo_table* t, BaseString &out);
-
-void MgmApiSession::getNdbInfo(Parser_t::Context &ctx, Properties const &args)
-{
-  BaseString query;
-  args.get("query", query);
-
-  Vector<BaseString> columns;
-  Vector<BaseString> rows;
-
-  m_output->println("ndbinfo reply");
-
-  int r= m_mgmsrv.ndbinfo(query, &columns, &rows);
-
-  if(r)
-  {
-    m_output->println("error: %d",r);
-    m_output->println("");
-    return;
-  }
-
-  m_output->println("error: 0");
-  m_output->println("rows: %d",rows.size());
-  m_output->println("");
-
-  m_output->println("%d",columns.size());
-
-  for(unsigned i = 0; i < columns.size(); i++)
-    m_output->println(columns[i].c_str());
-
-  for(unsigned i = 0; i < rows.size(); i++)
-    m_output->println(rows[i].c_str());
-
-  return ;
+  m_output->println("%s", "");
 }
 
 
@@ -2116,7 +2144,7 @@ void MgmApiSession::showConfig(Parser_t::Context &ctx, Properties const &args)
   m_output->println("show config reply");
   m_mgmsrv.print_config(section, nodeid, name,
                         socket_out);
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 
@@ -2142,7 +2170,7 @@ MgmApiSession::reloadConfig(Parser_t::Context &,
   else
     m_output->println("result: Ok");
 
-  m_output->println("");
+  m_output->println("%s", "");
 }
 
 
@@ -2153,11 +2181,144 @@ MgmApiSession::show_variables(Parser_t::Context &,
   m_output->println("show variables reply");
   NdbOut socket_out(*m_output, false /* turn off autoflush */);
   m_mgmsrv.show_variables(socket_out);
-  m_output->println("");
+  m_output->println("%s", "");
+}
 
+
+static bool
+valid_nodes(const NdbNodeBitmask& nodes, unsigned max_nodeid)
+{
+  unsigned nodeid = 0;
+  while((nodeid = nodes.find(nodeid)) != NdbNodeBitmask::NotFound)
+  {
+    if (nodeid == 0 || nodeid > max_nodeid)
+      return false;
+    nodeid++;
+  }
+  return true;
+}
+
+
+#include <signaldata/DumpStateOrd.hpp>
+#include "../common/util/parse_mask.hpp"
+
+static const
+struct dump_request {
+  Ndb_logevent_type type;
+  DumpStateOrd::DumpStateType dump_type;
+  // Number of reports to wait for from each node
+  Uint32 reports_per_node;
+} dump_requests [] =
+{
+  { NDB_LE_BackupStatus,
+    DumpStateOrd::BackupStatus,
+    1 },
+
+  { NDB_LE_MemoryUsage,
+    DumpStateOrd::DumpPageMemory,
+    2},
+
+  { NDB_LE_SavedEvent,
+    DumpStateOrd::DumpEventLog,
+    0},
+
+  { NDB_LE_ILLEGAL_TYPE, (DumpStateOrd::DumpStateType)0, 0 }
+};
+
+void
+MgmApiSession::dump_events(Parser_t::Context &,
+                           const class Properties &args)
+{
+  m_output->println("dump events reply");
+
+  // Check "type" argument
+  Uint32 type;
+  args.get("type", &type);
+
+  const dump_request* request = dump_requests;
+
+  for (; request->type != NDB_LE_ILLEGAL_TYPE; request++)
+  {
+    if (request->type == (Ndb_logevent_type)type)
+      break;
+  }
+
+  if (request->type == NDB_LE_ILLEGAL_TYPE)
+  {
+    m_output->println("result: ndb_logevent_type %u not supported", type);
+    m_output->println("%s", "");
+    return;
+  }
+
+  // Check "nodes" argument
+  NdbNodeBitmask nodes;
+  const char* nodes_str = NULL;
+  args.get("nodes", &nodes_str);
+  if (nodes_str)
+  {
+    int res = parse_mask(nodes_str, nodes);
+    if (res < 0 || !valid_nodes(nodes, MAX_NDB_NODES-1))
+    {
+      m_output->println("result: invalid nodes: '%s'", nodes_str);
+      m_output->println("%s", "");
+      return;
+    }
+  }
+
+  // Request the events
+  Vector<SimpleSignal> events;
+  if (!m_mgmsrv.request_events(nodes,
+                               request->reports_per_node,
+                               request->dump_type,
+                               events))
+  {
+    m_output->println("result: failed to dump events");
+    m_output->println("%s", "");
+    return;
+  }
+
+  // Return result
+  m_output->println("result: Ok");
+  m_output->println("events: %u", events.size());
+  m_output->println("%s", ""); // Empty line between header and first event
+  for (unsigned i = 0; i < events.size(); i++)
+  {
+    const EventReport * const event =
+      (const EventReport*)events[i].getDataPtrSend();
+    const NodeId nodeid = refToNode(events[i].header.theSendersBlockRef);
+
+    // Check correct EVENT_REP type returned
+    assert(event->getEventType() == request->type);
+
+    BaseString str;
+    char pretty_text[512];
+    Uint32 tmpData[256];
+    const Uint32 * dataPtr = events[i].getDataPtr();
+    Uint32 dataLen = events[i].getLength();
+    if (events[i].header.m_noOfSections == 1)
+    {
+      if (dataLen + events[i].ptr[0].sz > NDB_ARRAY_SIZE(tmpData))
+      {
+        events[i].ptr[0].sz = NDB_ARRAY_SIZE(tmpData) - dataLen;
+      }
+      memcpy(tmpData, dataPtr, 4 * dataLen);
+      memcpy(tmpData + dataLen, events[i].ptr[0].p, 4*events[i].ptr[0].sz);
+      dataPtr = tmpData;
+      dataLen += events[i].ptr[0].sz;
+    }
+    logevent2str(str,
+                 event->getEventType(),
+                 dataPtr,
+                 dataLen,
+                 nodeid, 0,
+                 pretty_text, sizeof(pretty_text));
+
+    m_output->println("%s", str.c_str());
+  }
 }
 
 
 template class MutexVector<int>;
 template class Vector<ParserRow<MgmApiSession> const*>;
 template class Vector<NDB_SOCKET_TYPE>;
+template class Vector<SimpleSignal>;
