@@ -21,10 +21,11 @@
 #include <util/version.h>
 
 #include "TransporterFacade.hpp"
+#include <kernel/GlobalSignalNumbers.h>
+
 #include "ClusterMgr.hpp"
 #include <IPCConfig.hpp>
 #include "NdbApiSignal.hpp"
-#include "API.hpp"
 #include <NdbSleep.h>
 #include <NdbOut.hpp>
 #include <NdbTick.h>
@@ -33,6 +34,8 @@
 #include <signaldata/NodeFailRep.hpp>
 #include <signaldata/NFCompleteRep.hpp>
 #include <signaldata/ApiRegSignalData.hpp>
+#include <signaldata/AlterTable.hpp>
+#include <signaldata/SumaImpl.hpp>
 
 #include <mgmapi.h>
 #include <mgmapi_configuration.hpp>
@@ -52,25 +55,30 @@ runClusterMgr_C(void * me)
   return NULL;
 }
 
-extern "C" {
-  void ndbSetOwnVersion();
-}
 ClusterMgr::ClusterMgr(TransporterFacade & _facade):
   theStop(0),
   theFacade(_facade),
+  theArbitMgr(NULL),
   m_connect_count(0),
   m_max_api_reg_req_interval(~0),
   noOfAliveNodes(0),
   noOfConnectedNodes(0),
+  minDbVersion(0),
   theClusterMgrThread(NULL),
   waitingForHB(false),
   m_cluster_state(CS_waiting_for_clean_cache)
 {
   DBUG_ENTER("ClusterMgr::ClusterMgr");
-  ndbSetOwnVersion();
   clusterMgrThreadMutex = NdbMutex_Create();
   waitForHBCond= NdbCondition_Create();
   m_auto_reconnect = -1;
+
+  int ret = this->open(&theFacade, API_CLUSTERMGR);
+  if (unlikely(ret < 0))
+  {
+    ndbout_c("Failed to register ClusterMgr! ret: %d", ret);
+    abort();
+  }
   DBUG_VOID_RETURN;
 }
 
@@ -78,13 +86,20 @@ ClusterMgr::~ClusterMgr()
 {
   DBUG_ENTER("ClusterMgr::~ClusterMgr");
   doStop();
+  if (theArbitMgr != 0)
+  {
+    delete theArbitMgr;
+    theArbitMgr = 0;
+  }
   NdbCondition_Destroy(waitForHBCond);
   NdbMutex_Destroy(clusterMgrThreadMutex);
   DBUG_VOID_RETURN;
 }
 
 void
-ClusterMgr::configure(const ndb_mgm_configuration* config){
+ClusterMgr::configure(Uint32 nodeId,
+                      const ndb_mgm_configuration* config)
+{
   ndb_mgm_configuration_iterator iter(* config, CFG_SECTION_NODE);
   for(iter.first(); iter.valid(); iter.next()){
     Uint32 nodeId = 0;
@@ -127,12 +142,37 @@ ClusterMgr::configure(const ndb_mgm_configuration* config){
   /* Init own node info */
   Node &node= theNodes[theFacade.ownId()];
   assert(node.defined);
-  node.m_api_reg_conf= true;
+  node.set_connected(true);
+  node.set_confirmed(true);
 
 #if 0
   print_nodes("init");
 #endif
 
+  // Configure arbitrator
+  Uint32 rank = 0;
+  iter.first();
+  iter.find(CFG_NODE_ID, nodeId); // let not found in config mean rank=0
+  iter.get(CFG_NODE_ARBIT_RANK, &rank);
+
+  if (rank > 0)
+  {
+    // The arbitrator should be active
+    if (!theArbitMgr)
+      theArbitMgr = new ArbitMgr(theFacade);
+    theArbitMgr->setRank(rank);
+
+    Uint32 delay = 0;
+    iter.get(CFG_NODE_ARBIT_DELAY, &delay);
+    theArbitMgr->setDelay(delay);
+  }
+  else if (theArbitMgr)
+  {
+    // No arbitrator should be started
+    theArbitMgr->doStop(NULL);
+    delete theArbitMgr;
+    theArbitMgr= NULL;
+  }
 }
 
 void
@@ -144,7 +184,7 @@ ClusterMgr::startThread() {
                                          (void**)this,
                                          0, // default stack size
                                          "ndb_clustermgr",
-                                         NDB_THREAD_PRIO_LOW);
+                                         NDB_THREAD_PRIO_HIGH);
 }
 
 void
@@ -162,6 +202,11 @@ ClusterMgr::doStop( ){
     NdbThread_Destroy(&theClusterMgrThread);
   }
 
+  if (theArbitMgr != NULL)
+  {
+    theArbitMgr->doStop(NULL);
+  }
+
   DBUG_VOID_RETURN;
 }
 
@@ -184,7 +229,7 @@ ClusterMgr::forceHB()
     waitForHBFromNodes.clear();
     for(Uint32 i = 1; i < MAX_NDB_NODES; i++)
     {
-      const Node &node= getNodeInfo(i);
+      const trp_node &node= getNodeInfo(i);
       if(!node.defined)
         continue;
       if(node.m_info.getType() == NodeInfo::DB)
@@ -285,7 +330,7 @@ ClusterMgr::threadMain( ){
       if (!theNode.defined)
 	continue;
 
-      if (theNode.connected == false){
+      if (theNode.is_connected() == false){
 	theFacade.doConnect(nodeId);
 	continue;
       }
@@ -324,12 +369,181 @@ ClusterMgr::threadMain( ){
   }
 }
 
+void
+ClusterMgr::trp_deliver_signal(const NdbApiSignal* sig,
+                               const LinearSectionPtr ptr[3])
+{
+  const Uint32 gsn = sig->theVerId_signalNumber;
+  const Uint32 * theData = sig->getDataPtr();
+
+  switch (gsn){
+  case GSN_API_REGREQ:
+    execAPI_REGREQ(theData);
+    break;
+
+  case GSN_API_REGCONF:
+  {
+    execAPI_REGCONF(theData);
+
+    // Distribute signal to all threads/blocks
+    theFacade.for_each(this, sig, ptr);
+    break;
+  }
+
+  case GSN_API_REGREF:
+    execAPI_REGREF(theData);
+    break;
+
+  case GSN_NODE_FAILREP:
+    execNODE_FAILREP(theData);
+    break;
+
+  case GSN_NF_COMPLETEREP:
+    execNF_COMPLETEREP(theData);
+    break;
+
+  case GSN_ARBIT_STARTREQ:
+    if (theArbitMgr != NULL)
+      theArbitMgr->doStart(theData);
+    break;
+
+  case GSN_ARBIT_CHOOSEREQ:
+    if (theArbitMgr != NULL)
+      theArbitMgr->doChoose(theData);
+    break;
+
+  case GSN_ARBIT_STOPORD:
+    if(theArbitMgr != NULL)
+      theArbitMgr->doStop(theData);
+    break;
+
+  case GSN_ALTER_TABLE_REP:
+  {
+    if (theFacade.m_globalDictCache == NULL)
+      break;
+    const AlterTableRep* rep = (const AlterTableRep*)theData;
+    theFacade.m_globalDictCache->lock();
+    theFacade.m_globalDictCache->
+      alter_table_rep((const char*)ptr[0].p,
+                      rep->tableId,
+                      rep->tableVersion,
+                      rep->changeType == AlterTableRep::CT_ALTERED);
+    theFacade.m_globalDictCache->unlock();
+    break;
+  }
+  case GSN_SUB_GCP_COMPLETE_REP:
+  {
+    /**
+     * Report
+     */
+    theFacade.for_each(this, sig, ptr);
+
+    /**
+     * Reply
+     */
+    {
+      BlockReference ownRef = numberToRef(API_CLUSTERMGR, theFacade.ownId());
+      NdbApiSignal tSignal(* sig);
+      Uint32* send= tSignal.getDataPtrSend();
+      memcpy(send, theData, tSignal.getLength() << 2);
+      CAST_PTR(SubGcpCompleteAck, send)->rep.senderRef = ownRef;
+      Uint32 ref= sig->theSendersBlockRef;
+      Uint32 aNodeId= refToNode(ref);
+      tSignal.theReceiversBlockNumber= refToBlock(ref);
+      tSignal.theVerId_signalNumber= GSN_SUB_GCP_COMPLETE_ACK;
+      tSignal.theSendersBlockRef = API_CLUSTERMGR;
+      theFacade.sendSignalUnCond(&tSignal, aNodeId);
+    }
+    break;
+  }
+  case GSN_TAKE_OVERTCCONF:
+  {
+    /**
+     * Report
+     */
+    theFacade.for_each(this, sig, ptr);
+    return;
+  }
+  default:
+    break;
+
+  }
+  return;
+}
+
+void
+ClusterMgr::trp_node_status(Uint32 nodeId, Uint32 event)
+{
+}
 
 ClusterMgr::Node::Node()
-  : m_state(NodeState::SL_NOTHING) { 
-  compatible = nfCompleteRep = true;
-  connected = defined = m_alive = m_api_reg_conf = false; 
-  m_state.m_connected_nodes.clear();
+  : hbFrequency(0), hbCounter(0)
+{
+}
+
+/**
+ * recalcMinDbVersion
+ *
+ * This method is called whenever the 'minimum DB node
+ * version' data for the connected DB nodes changes
+ * It calculates the minimum version of all the connected
+ * DB nodes.
+ * This information is cached by Ndb object instances.
+ * This information is useful when implementing API compatibility
+ * with older DB nodes
+ */
+void
+ClusterMgr::recalcMinDbVersion()
+{
+  Uint32 newMinDbVersion = ~ (Uint32) 0;
+  
+  for (Uint32 i = 0; i < MAX_NODES; i++)
+  {
+    Node& node = theNodes[i];
+
+    if (node.is_connected() &&
+        node.is_confirmed() &&
+        node.m_info.getType() == NodeInfo::DB)
+    {
+      /* Include this node in the set of nodes used to
+       * compute the lowest current DB node version
+       */
+      assert(node.m_info.m_version);
+
+      if (node.minDbVersion < newMinDbVersion)
+      {
+        newMinDbVersion = node.minDbVersion;
+      }
+    }
+  }
+
+  /* Now update global min Db version if we have one.
+   * Otherwise set it to 0
+   */
+  newMinDbVersion = (newMinDbVersion == ~ (Uint32) 0) ?
+    0 :
+    newMinDbVersion;
+
+//#ifdef DEBUG_MINVER
+
+#ifdef DEBUG_MINVER
+  if (newMinDbVersion != minDbVersion)
+  {
+    ndbout << "Previous min Db node version was "
+           << NdbVersion(minDbVersion)
+           << " new min is "
+           << NdbVersion(newMinDbVersion)
+           << endl;
+  }
+  else
+  {
+    ndbout << "MinDbVersion recalculated, but is same : "
+           << NdbVersion(minDbVersion)
+           << endl;
+  }
+#endif
+
+  minDbVersion = newMinDbVersion;
 }
 
 /******************************************************************************
@@ -349,7 +563,7 @@ ClusterMgr::execAPI_REGREQ(const Uint32 * theData){
 
   Node & node = theNodes[nodeId];
   assert(node.defined == true);
-  assert(node.connected == true);
+  assert(node.is_connected() == true);
 
   if(node.m_info.m_version != apiRegReq->version){
     node.m_info.m_version = apiRegReq->version;
@@ -378,7 +592,7 @@ ClusterMgr::execAPI_REGREQ(const Uint32 * theData){
   conf->nodeState= node.m_state;
 
   if (theFacade.sendSignalUnCond(&signal, nodeId) == SEND_OK)
-    node.m_api_reg_conf= true;
+    node.set_confirmed(true);
 }
 
 void
@@ -394,7 +608,7 @@ ClusterMgr::execAPI_REGCONF(const Uint32 * theData){
   
   Node & node = theNodes[nodeId];
   assert(node.defined == true);
-  assert(node.connected == true);
+  assert(node.is_connected() == true);
 
   if(node.m_info.m_version != apiRegConf->version){
     node.m_info.m_version = apiRegConf->version;
@@ -410,7 +624,13 @@ ClusterMgr::execAPI_REGCONF(const Uint32 * theData){
 					      node.m_info.m_version);
   }
 
-  node.m_api_reg_conf = true;
+  node.set_confirmed(true);
+
+  if (node.minDbVersion != apiRegConf->minDbVersion)
+  {
+    node.minDbVersion = apiRegConf->minDbVersion;
+    recalcMinDbVersion();
+  }
 
   if (node.m_info.m_version >= NDBD_255_NODES_VERSION)
   {
@@ -466,7 +686,7 @@ ClusterMgr::execAPI_REGREF(const Uint32 * theData){
   assert(nodeId > 0 && nodeId < MAX_NODES);
 
   Node & node = theNodes[nodeId];
-  assert(node.connected == true);
+  assert(node.is_connected() == true);
   assert(node.defined == true);
   /* Only DB nodes will send API_REGREF */
   assert(node.m_info.getType() == NodeInfo::DB);
@@ -509,7 +729,7 @@ ClusterMgr::execNF_COMPLETEREP(const Uint32 * theData){
   Node & node = theNodes[nodeId];
   if (node.nfCompleteRep == false)
   {
-    theFacade.ReportNodeFailureComplete(nodeId);
+    theFacade.trp_node_status(nodeId, NS_NODE_NF_COMPLETE);
     node.nfCompleteRep = true;
   }
 }
@@ -528,7 +748,7 @@ ClusterMgr::reportConnected(NodeId nodeId){
   noOfConnectedNodes++;
 
   Node & theNode = theNodes[nodeId];
-  theNode.connected = true;
+  theNode.set_connected(true);
   theNode.m_info.m_heartbeat_cnt = 0;
   theNode.hbCounter = 0;
 
@@ -542,8 +762,10 @@ ClusterMgr::reportConnected(NodeId nodeId){
   theNode.compatible = true;
   theNode.nfCompleteRep = true;
   theNode.m_state.startLevel = NodeState::SL_NOTHING;
+
+  theNode.minDbVersion = 0;
   
-  theFacade.ReportNodeAlive(nodeId);
+  theFacade.trp_node_status(nodeId, NS_NODE_ALIVE);
   DBUG_VOID_RETURN;
 }
 
@@ -553,8 +775,8 @@ ClusterMgr::reportDisconnected(NodeId nodeId){
   assert(noOfConnectedNodes > 0);
 
   noOfConnectedNodes--;
-  theNodes[nodeId].connected = false;
-  theNodes[nodeId].m_api_reg_conf = false;
+  theNodes[nodeId].set_confirmed(false);
+  theNodes[nodeId].set_connected(false);
   theNodes[nodeId].m_state.m_connected_nodes.clear();
 
   reportNodeFailed(nodeId, true);
@@ -570,17 +792,20 @@ ClusterMgr::reportNodeFailed(NodeId nodeId, bool disconnect){
   set_node_alive(theNode, false);
   theNode.m_info.m_connectCount ++;
   
-  if(theNode.connected)
+  if(theNode.is_connected())
   {
     theFacade.doDisconnect(nodeId);
   }
+
+  if (theNode.m_info.getType() == NodeInfo::DB)
+    recalcMinDbVersion();
   
   const bool report = (theNode.m_state.startLevel != NodeState::SL_NOTHING);  
   theNode.m_state.startLevel = NodeState::SL_NOTHING;
   
   if(disconnect || report)
   {
-    theFacade.ReportNodeDead(nodeId);
+    theFacade.trp_node_status(nodeId, NS_NODE_FAILED);
   }
   
   if (noOfConnectedNodes == 0)
@@ -620,16 +845,16 @@ ClusterMgr::print_nodes(const char* where, NdbOut& out)
   out << where << " >>" << endl;
   for (NodeId n = 1; n < MAX_NODES ; n++)
   {
-    const Node node = getNodeInfo(n);
+    const trp_node node = getNodeInfo(n);
     if (!node.defined)
       continue;
     out << "node: " << n << endl;
     out << " -";
-    out << " connected: " << node.connected;
+    out << " connected: " << node.is_connected();
     out << ", compatible: " << node.compatible;
     out << ", nf_complete_rep: " << node.nfCompleteRep;
     out << ", alive: " << node.m_alive;
-    out << ", api_reg_conf: " << node.m_api_reg_conf;
+    out << ", confirmed: " << node.is_confirmed();
     out << endl;
 
     out << " - " << node.m_info << endl;
