@@ -25,19 +25,11 @@ Name:          Ndb.cpp
 
 #include <ndb_global.h>
 
-
-#include "NdbApiSignal.hpp"
-#include "NdbImpl.hpp"
-#include <NdbOperation.hpp>
-#include <NdbTransaction.hpp>
-#include <NdbEventOperation.hpp>
-#include <NdbEventOperationImpl.hpp>
-#include <NdbRecAttr.hpp>
+#include "API.hpp"
 #include <md5_hash.hpp>
 #include <NdbSleep.h>
 #include <NdbOut.hpp>
 #include <ndb_limits.h>
-#include "API.hpp"
 #include <NdbEnv.h>
 #include <BaseString.hpp>
 #include <NdbSqlUtil.hpp>
@@ -151,6 +143,14 @@ Ndb::NDB_connect(Uint32 tNode)
 
   DBUG_ENTER("Ndb::NDB_connect");
 
+  {
+    TransporterFacade *tp = theImpl->m_transporter_facade;
+    if (tp->get_node_stopping(tNode))
+    {
+      DBUG_RETURN(0);
+    }
+  }
+
   NdbTransaction * tConArray = theConnectionArray[tNode];
   if (tConArray != NULL) {
     DBUG_RETURN(2);
@@ -990,171 +990,6 @@ Ndb::closeTransaction(NdbTransaction* aConnection)
   DBUG_VOID_RETURN;
 }//Ndb::closeTransaction()
 
-/*****************************************************************************
-int* NdbTamper(int aAction, int aNode);
-
-Parameters: aAction     Specifies what action to be taken
-            1: Lock global checkpointing    Can only be sent to master DIH, Parameter aNode ignored.
-            2: UnLock global checkpointing    Can only be sent to master DIH, Parameter aNode ignored.
-	    3: Crash node
-
-           aNode        Specifies which node the action will be taken
-     	  -1: Master DIH 
-       	0-16: Nodnumber
-
-Return Value: -1 Error  .
-                
-Remark:         Sends a signal to DIH.
-*****************************************************************************/ 
-int 
-Ndb::NdbTamper(TamperType aAction, int aNode)
-{
-  NdbTransaction*	tNdbConn;
-  NdbApiSignal		tSignal(theMyRef);
-  int			tNode;
-  int                   tAction;
-  int			ret_code;
-
-#ifdef CUSTOMER_RELEASE
-  return -1;
-#else
-  DBUG_ENTER("Ndb::NdbTamper");
-  CHECK_STATUS_MACRO;
-  checkFailedNode();
-
-  theRestartGCI = 0;
-  switch (aAction) {
-// Translate enum to integer. This is done because the SCI layer
-// expects integers. 
-     case LockGlbChp:
-        tAction = 1;
-        break;
-     case UnlockGlbChp:
-        tAction = 2;
-	break;
-     case CrashNode:
-        tAction = 3;
-        break;
-     case ReadRestartGCI:
-	tAction = 4;
-	break;
-     default:
-        theError.code = 4102;
-        DBUG_RETURN(-1);
-  }
-
-  tNdbConn = getNdbCon();	// Get free connection object
-  if (tNdbConn == NULL) {
-    theError.code = 4000;
-    DBUG_RETURN(-1);
-  }
-  tSignal.setSignal(GSN_DIHNDBTAMPER);
-  tSignal.setData (tAction, 1);
-  tSignal.setData(tNdbConn->ptr2int(),2);
-  tSignal.setData(theMyRef,3);		// Set return block reference
-  tNdbConn->Status(NdbTransaction::Connecting); // Set status to connecting
-  TransporterFacade *tp = theImpl->m_transporter_facade;
-  if (tAction == 3) {
-    tp->lock_mutex();
-    tp->sendSignal(&tSignal, aNode);
-    tp->unlock_mutex();
-    releaseNdbCon(tNdbConn);
-  } else if ( (tAction == 2) || (tAction == 1) ) {
-    tp->lock_mutex();
-    tNode = tp->get_an_alive_node();
-    if (tNode == 0) {
-      theError.code = 4002;
-      releaseNdbCon(tNdbConn);
-      DBUG_RETURN(-1);
-    }//if
-    ret_code = tp->sendSignal(&tSignal,aNode);
-    tp->unlock_mutex();
-    releaseNdbCon(tNdbConn);
-    DBUG_RETURN(ret_code);
-  } else {
-    do {
-      tp->lock_mutex();
-      // Start protected area
-      tNode = tp->get_an_alive_node();
-      tp->unlock_mutex();
-      // End protected area
-      if (tNode == 0) {
-        theError.code = 4009;
-        releaseNdbCon(tNdbConn);
-        DBUG_RETURN(-1);
-      }//if
-      ret_code = sendRecSignal(tNode, WAIT_NDB_TAMPER, &tSignal, 0);
-      if (ret_code == 0) {  
-        if (tNdbConn->Status() != NdbTransaction::Connected) {
-          theRestartGCI = 0;
-        }//if
-        releaseNdbCon(tNdbConn);
-        DBUG_RETURN(theRestartGCI);
-      } else if ((ret_code == -5) || (ret_code == -2)) {
-        TRACE_DEBUG("Continue DIHNDBTAMPER when node failed/stopping");
-      } else {
-        DBUG_RETURN(-1);
-      }//if
-    } while (1);
-  }
-  DBUG_RETURN(0);
-#endif
-}
-#if 0
-/****************************************************************************
-NdbSchemaCon* startSchemaTransaction();
-
-Return Value:   Returns a pointer to a schema connection object.
-                Return NULL otherwise.
-Remark:         Start schema transaction. Synchronous.
-****************************************************************************/ 
-NdbSchemaCon* 
-Ndb::startSchemaTransaction()
-{
-  NdbSchemaCon* tSchemaCon;
-  if (theSchemaConToNdbList != NULL) {
-    theError.code = 4321;
-    return NULL;
-  }//if
-  tSchemaCon = new NdbSchemaCon(this);
-  if (tSchemaCon == NULL) {
-    theError.code = 4000;
-    return NULL;
-  }//if 
-  theSchemaConToNdbList = tSchemaCon;
-  return tSchemaCon;  
-}
-/*****************************************************************************
-void closeSchemaTransaction(NdbSchemaCon* aSchemaCon);
-
-Parameters:     aSchemaCon: the schemacon used in the transaction.
-Remark:         Close transaction by releasing the schemacon and all schemaop.
-*****************************************************************************/
-void
-Ndb::closeSchemaTransaction(NdbSchemaCon* aSchemaCon)
-{
-  if (theSchemaConToNdbList != aSchemaCon) {
-    abort();
-    return;
-  }//if
-  aSchemaCon->release();
-  delete aSchemaCon;
-  theSchemaConToNdbList = NULL;
-  return;
-}//Ndb::closeSchemaTransaction()
-#endif
-
-/*****************************************************************************
-void RestartGCI(int aRestartGCI);
-
-Remark:		Set theRestartGCI on the NDB object
-*****************************************************************************/
-void
-Ndb::RestartGCI(int aRestartGCI)
-{
-  theRestartGCI = aRestartGCI;
-}
-
 /****************************************************************************
 int getBlockNumber(void);
 
@@ -2048,11 +1883,6 @@ int Ndb::dropEventOperation(NdbEventOperation* tOp)
   DBUG_ENTER("Ndb::dropEventOperation");
   DBUG_PRINT("info", ("name: %s", tOp->getEvent()->getTable()->getName()));
   // remove it from list
-
-#ifdef REMOVED_WARNING_SURROUNDING_WEIRD_CODE
-  NdbEventOperationImpl *op=
-    NdbEventBuffer::getEventOperationImpl(tOp);
-#endif  
   
   theEventBuffer->dropEventOperation(tOp);
   DBUG_RETURN(0);
@@ -2206,6 +2036,160 @@ Ndb::printState(const char* fmt, ...)
     theCompletedTransactionsArray[i]->printState();
   NdbMutex_Unlock(ndb_print_state_mutex);
 }
+
 #endif
 
+const char*
+Ndb::getNdbErrorDetail(const NdbError& err, char* buff, Uint32 buffLen) const
+{
+  DBUG_ENTER("Ndb::getNdbErrorDetail");
+  /* If err has non-null details member, prepare a string containing
+   * those details
+   */
+  if (!buff)
+    DBUG_RETURN(NULL);
 
+  if (err.details != NULL)
+  {
+    DBUG_PRINT("info", ("err.code is %u", err.code));
+    switch (err.code) {
+    case 893: /* Unique constraint violation */
+    {
+      /* err.details contains the violated Index's object id
+       * We'll map it to a name, then map the name to a 
+       * base table, schema and database, and put that in
+       * string form into the caller's buffer
+       */
+      UintPtr uip = (UintPtr) err.details;
+      Uint32 indexObjectId = (Uint32) (uip - (UintPtr(0)));
+      Uint32 primTableObjectId = ~ (Uint32) 0;
+      BaseString indexName;
+      char splitChars[2] = {table_name_separator, 0};
+      BaseString splitString(&splitChars[0]);
+      
+      {
+        DBUG_PRINT("info", ("Index object id is %u", indexObjectId));
+        NdbDictionary::Dictionary::List allIndices;
+        int rc = theDictionary->listObjects(allIndices, 
+                                            NdbDictionary::Object::UniqueHashIndex,
+                                            false); // FullyQualified names
+        if (rc)
+        {
+          DBUG_PRINT("info", ("listObjects call 1 failed with rc %u", rc));
+          DBUG_RETURN(NULL);
+        }
+        
+        DBUG_PRINT("info", ("Retrieved details for %u indices", allIndices.count));
+        
+        for (unsigned i = 0; i < allIndices.count; i++)
+        {
+          if (allIndices.elements[i].id == indexObjectId)
+          {
+            /* Found the index in question
+             * Expect fully qualified name to be in the form :
+             * <db>/<schema>/<primTabId>/<IndexName>
+             */
+            Vector<BaseString> idxNameComponents;
+            BaseString idxName(allIndices.elements[i].name);
+            
+            Uint32 components = idxName.split(idxNameComponents,
+                                              splitString);
+            
+            assert(components == 4);
+            
+            primTableObjectId = atoi(idxNameComponents[2].c_str());
+            indexName = idxNameComponents[3];
+            
+            DBUG_PRINT("info", ("Found index name : %s, primary table id : %u",
+                                indexName.c_str(), primTableObjectId));
+            
+            break;
+          }
+        }
+      }
+
+      if (primTableObjectId != (~(Uint32) 0))
+      {
+        NdbDictionary::Dictionary::List allTables;
+        int rc = theDictionary->listObjects(allTables,
+                                            NdbDictionary::Object::UserTable,
+                                            false); // FullyQualified names
+        
+        if (rc)
+        {
+          DBUG_PRINT("info", ("listObjects call 2 failed with rc %u", rc));
+          DBUG_RETURN(NULL);
+        }
+
+        DBUG_PRINT("info", ("Retrieved details for %u tables", allTables.count));
+
+        for (Uint32 t = 0; t < allTables.count; t++)
+        {
+          
+          if (allTables.elements[t].id == primTableObjectId)
+          {
+            /* Found table, name should be in format :
+             * <db>/<schema>/<tablename>
+             */
+            Vector<BaseString> tabNameComponents;
+            BaseString tabName(allTables.elements[t].name);
+            
+            Uint32 components = tabName.split(tabNameComponents,
+                                              splitString);
+            assert (components == 3);
+            
+            /* Now we generate a string of the format
+             * <dbname>/<schemaname>/<tabname>/<idxname>
+             * which should be usable by end users
+             */
+            BaseString result;
+            result.assfmt("%s/%s/%s/%s",
+                          tabNameComponents[0].c_str(),
+                          tabNameComponents[1].c_str(),
+                          tabNameComponents[2].c_str(),
+                          indexName.c_str());
+            
+            DBUG_PRINT("info", ("Found full index details : %s",
+                                result.c_str()));
+            
+            memcpy(buff, result.c_str(), 
+                   MIN(buffLen, 
+                       (result.length() + 1)));
+            buff[buffLen] = 0;
+
+            DBUG_RETURN(buff);
+          }
+        }
+
+        /* Primary table not found!  
+         * Strange - perhaps it's been dropped?
+         */          
+        DBUG_PRINT("info", ("Table id %u not found", primTableObjectId));
+        DBUG_RETURN(NULL);
+      }
+      else
+      {
+        /* Index not found from id - strange.
+         * Perhaps it has been dropped?
+         */
+        DBUG_PRINT("info", ("Index id %u not found", indexObjectId));
+        DBUG_RETURN(NULL);
+      }
+    }
+    default:
+    {
+      /* Unhandled details type */
+    }
+    }
+  }
+  
+  DBUG_PRINT("info", ("No details string for this error"));
+  DBUG_RETURN(NULL);
+}
+
+
+Uint32
+Ndb::getMinDbNodeVersion() const
+{
+  return theCachedMinDbNodeVersion;
+}
