@@ -31,9 +31,9 @@
 #include <Bitmask.hpp>
 #include <AttributeList.hpp>
 #include <Ndb.hpp>
-#include "NdbWaiter.hpp"
 #include "DictCache.hpp"
 #include <signaldata/DictSignal.hpp>
+#include "NdbWaiter.hpp"
 
 class ListTablesReq;
 
@@ -96,7 +96,7 @@ public:
   bool m_nullable;
   bool m_autoIncrement;
   Uint64 m_autoIncrementInitialValue;
-  BaseString m_defaultValue;
+  UtilBuffer m_defaultValue;
   /*
    * Table holding the blob parts.
    *
@@ -174,9 +174,15 @@ public:
   const Int32 * getRangeListData() const;
   Uint32 getRangeListDataLen() const;
 
+  Uint32 getFragmentNodes(Uint32 fragmentId, 
+                          Uint32* nodeIdArrayPtr,
+                          Uint32 arraySize) const;
+
   const char * getMysqlName() const;
   int updateMysqlName();
 
+  bool matchDb(const char * name, size_t len) const;
+
   int aggregate(NdbError& error);
   int validate(NdbError& error);
 
@@ -201,6 +207,7 @@ public:
     if iterating over columns.
   */
   Vector<NdbColumnImpl *> m_columns;
+  Uint32 m_noOfAutoIncColumns;
   void computeAggregates();
   int buildColumnHash(); 
 
@@ -221,6 +228,7 @@ public:
   bool m_row_gci;
   bool m_row_checksum;
   bool m_force_var_part;
+  bool m_has_default_values; 
   int m_kvalue;
   int m_minLoadFactor;
   int m_maxLoadFactor;
@@ -589,16 +597,16 @@ public:
     }
   };
 
-  NdbDictInterface(Tx& tx, NdbError& err) : m_tx(tx), m_error(err) {
+  NdbDictInterface(Tx& tx, NdbError& err, int& warn) :
+    m_tx(tx), m_error(err), m_warn(warn) {
     m_reference = 0;
     m_masterNodeId = 0;
-    m_transporter= NULL;
+    m_impl = 0;
   }
   ~NdbDictInterface();
   
-  bool setTransporter(class Ndb * ndb, class TransporterFacade * tf);
-  bool setTransporter(class TransporterFacade * tf);
-  class TransporterFacade *getTransporter();
+  bool setTransporter(class Ndb * ndb);
+  class TransporterFacade *getTransporter() const;
   
   // To abstract the stuff thats made in all create/drop/lists below
   int dictSignal(NdbApiSignal* signal, LinearSectionPtr ptr[3], int secs,
@@ -625,7 +633,8 @@ public:
 
   int dropTable(const NdbTableImpl &);
 
-  int createIndex(class Ndb & ndb, const NdbIndexImpl &, const NdbTableImpl &);
+  int createIndex(class Ndb & ndb, const NdbIndexImpl &, const NdbTableImpl &,
+                  bool offline);
   int dropIndex(const NdbIndexImpl &, const NdbTableImpl &);
   
   int createEvent(class Ndb & ndb, NdbEventImpl &, int getFlag);
@@ -650,7 +659,8 @@ public:
 			  LinearSectionPtr ptr[3],
 			  Uint32 noOfSections, bool fullyQualifiedNames);
 
-  int forceGCPWait();
+  int forceGCPWait(int type);
+  int getRestartGCI(Uint32 *);
 
   static int parseTableInfo(NdbTableImpl ** dst, 
 			    const Uint32 * data, Uint32 len,
@@ -689,73 +699,85 @@ public:
   int endSchemaTrans(Uint32 flags);
   Tx & m_tx; // shared with NdbDictionaryImpl
 
+  bool checkAllNodeVersionsMin(Uint32 minNdbVersion) const;
+
   const NdbError &getNdbError() const;  
   NdbError & m_error;
+  int & m_warn;
 private:
   Uint32 m_reference;
   Uint32 m_masterNodeId;
-  
-  NdbWaiter m_waiter;
-  class TransporterFacade * m_transporter;
+
+  class NdbImpl * m_impl;
   
   friend class Ndb;
+  friend class NdbImpl;
   friend class NdbDictionaryImpl;
   static void execSignal(void* dictImpl, 
-			 class NdbApiSignal* signal, 
-			 struct LinearSectionPtr ptr[3]);
+			 const class NdbApiSignal* signal,
+			 const struct LinearSectionPtr ptr[3]);
   
-  static void execNodeStatus(void* dictImpl, Uint32, 
-			     bool alive, bool nfCompleted);  
+  static void execNodeStatus(void* dictImpl, Uint32, Uint32);
   
-  void execGET_TABINFO_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execGET_TABINFO_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execCREATE_TABLE_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execCREATE_TABLE_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execALTER_TABLE_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execALTER_TABLE_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
+  void execGET_TABINFO_REF(const NdbApiSignal *, const LinearSectionPtr p[3]);
+  void execGET_TABINFO_CONF(const NdbApiSignal *, const LinearSectionPtr p[3]);
+  void execCREATE_TABLE_REF(const NdbApiSignal *, const LinearSectionPtr p[3]);
+  void execCREATE_TABLE_CONF(const NdbApiSignal *, const LinearSectionPtr [3]);
+  void execALTER_TABLE_REF(const NdbApiSignal *, const LinearSectionPtr pr[3]);
+  void execALTER_TABLE_CONF(const NdbApiSignal *, const LinearSectionPtr p[3]);
 
-  void execCREATE_INDX_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execCREATE_INDX_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execDROP_INDX_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execDROP_INDX_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
+  void execCREATE_INDX_REF(const NdbApiSignal *, const LinearSectionPtr pr[3]);
+  void execCREATE_INDX_CONF(const NdbApiSignal *, const LinearSectionPtr p[3]);
+  void execDROP_INDX_REF(const NdbApiSignal *, const LinearSectionPtr ptr[3]);
+  void execDROP_INDX_CONF(const NdbApiSignal *, const LinearSectionPtr ptr[3]);
 
-  void execCREATE_EVNT_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execCREATE_EVNT_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execSUB_START_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execSUB_START_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execSUB_STOP_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execSUB_STOP_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execDROP_EVNT_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execDROP_EVNT_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
+  void execCREATE_EVNT_REF(const NdbApiSignal *, const LinearSectionPtr pr[3]);
+  void execCREATE_EVNT_CONF(const NdbApiSignal *, const LinearSectionPtr p[3]);
+  void execSUB_START_CONF(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
+  void execSUB_START_REF(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
+  void execSUB_STOP_CONF(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
+  void execSUB_STOP_REF(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
+  void execDROP_EVNT_REF(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
+  void execDROP_EVNT_CONF(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
 
-  void execDROP_TABLE_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execDROP_TABLE_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execOLD_LIST_TABLES_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execLIST_TABLES_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
+  void execDROP_TABLE_REF(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
+  void execDROP_TABLE_CONF(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
+  void execOLD_LIST_TABLES_CONF(const NdbApiSignal*,
+				const LinearSectionPtr ptr[3]);
+  void execLIST_TABLES_CONF(const NdbApiSignal*, const LinearSectionPtr pt[3]);
 
-  void execCREATE_FILE_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execCREATE_FILE_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  
-  void execCREATE_FILEGROUP_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execCREATE_FILEGROUP_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
+  void execCREATE_FILE_REF(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
+  void execCREATE_FILE_CONF(const NdbApiSignal*, const LinearSectionPtr pr[3]);
 
-  void execDROP_FILE_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execDROP_FILE_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  
-  void execDROP_FILEGROUP_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execDROP_FILEGROUP_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
+  void execCREATE_FILEGROUP_REF(const NdbApiSignal*,
+				const LinearSectionPtr ptr[3]);
+  void execCREATE_FILEGROUP_CONF(const NdbApiSignal*,
+				 const LinearSectionPtr ptr[3]);
 
-  void execSCHEMA_TRANS_BEGIN_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execSCHEMA_TRANS_BEGIN_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execSCHEMA_TRANS_END_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execSCHEMA_TRANS_END_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execSCHEMA_TRANS_END_REP(NdbApiSignal *, LinearSectionPtr ptr[3]);
+  void execDROP_FILE_REF(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
+  void execDROP_FILE_CONF(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
 
-  void execWAIT_GCP_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execWAIT_GCP_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
+  void execDROP_FILEGROUP_REF(const NdbApiSignal*, const LinearSectionPtr [3]);
+  void execDROP_FILEGROUP_CONF(const NdbApiSignal*,
+			       const LinearSectionPtr ptr[3]);
 
-  void execCREATE_HASH_MAP_REF(NdbApiSignal *, LinearSectionPtr ptr[3]);
-  void execCREATE_HASH_MAP_CONF(NdbApiSignal *, LinearSectionPtr ptr[3]);
+  void execSCHEMA_TRANS_BEGIN_CONF(const NdbApiSignal*,
+				   const LinearSectionPtr ptr[3]);
+  void execSCHEMA_TRANS_BEGIN_REF(const NdbApiSignal*,
+				  const LinearSectionPtr ptr[3]);
+  void execSCHEMA_TRANS_END_CONF(const NdbApiSignal*,
+				 const LinearSectionPtr ptr[3]);
+  void execSCHEMA_TRANS_END_REF(const NdbApiSignal*,
+				const LinearSectionPtr ptr[3]);
+  void execSCHEMA_TRANS_END_REP(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
+
+  void execWAIT_GCP_CONF(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
+  void execWAIT_GCP_REF(const NdbApiSignal*, const LinearSectionPtr ptr[3]);
+
+  void execCREATE_HASH_MAP_REF(const NdbApiSignal*,
+			       const LinearSectionPtr ptr[3]);
+  void execCREATE_HASH_MAP_CONF(const NdbApiSignal*,
+				const LinearSectionPtr ptr[3]);
 
   Uint32 m_fragmentId;
   UtilBuffer m_buffer;
@@ -764,9 +786,15 @@ private:
   UtilBuffer m_tableData;
   UtilBuffer m_tableNames;
 
-  struct {
-    Uint32 m_buckets;
-  } m_sub_start_conf;
+  union {
+    struct SubStartConfData {
+      Uint32 m_buckets;
+    } m_sub_start_conf;
+    struct WaitGcpData {
+      Uint32 gci_hi;
+      Uint32 gci_lo;
+    } m_wait_gcp_conf;
+  } m_data;
 };
 
 class NdbDictionaryImpl;
@@ -804,8 +832,8 @@ public:
   int invalidateObject(NdbTableImpl &);
   int removeCachedObject(NdbTableImpl &);
 
-  int createIndex(NdbIndexImpl &ix);
-  int createIndex(NdbIndexImpl &ix, NdbTableImpl & tab);
+  int createIndex(NdbIndexImpl &ix, bool offline);
+  int createIndex(NdbIndexImpl &ix, NdbTableImpl & tab, bool offline);
   int dropIndex(const char * indexName, 
 		const char * tableName);
   int dropIndex(NdbIndexImpl &, const char * tableName);
@@ -822,9 +850,11 @@ public:
   int executeSubscribeEvent(NdbEventOperationImpl &, Uint32 & buckets);
   int stopSubscribeEvent(NdbEventOperationImpl &);
 
-  int forceGCPWait();
+  int forceGCPWait(int type);
+  int getRestartGCI(Uint32*);
 
-  int listObjects(List& list, NdbDictionary::Object::Type type);
+  int listObjects(List& list, NdbDictionary::Object::Type type, 
+                  bool fullyQualified);
   int listIndexes(List& list, Uint32 indexId);
 
   NdbTableImpl * getTableGlobal(const char * tableName);
@@ -871,6 +901,7 @@ public:
 
   const NdbError & getNdbError() const;
   NdbError m_error;
+  int m_warn;
   Uint32 m_local_table_data_size;
 
   LocalDictCache m_localHash;
@@ -1075,6 +1106,15 @@ NdbTableImpl::getMysqlName() const
   return m_mysqlName.c_str();
 }
 
+inline
+bool
+NdbTableImpl::matchDb(const char * name, size_t len) const
+{
+  return 
+    len < m_internalName.length() &&
+    memcmp(name, m_internalName.c_str(), len) == 0;
+}
+
 inline
 Uint32
 Hash( const char* str ){
@@ -1220,6 +1260,21 @@ inline
 NdbTableImpl *
 NdbDictionaryImpl::getTableGlobal(const char * table_name)
 {
+  if (unlikely(strchr(table_name, '$') != 0)) {
+    if (is_ndb_blob_table(table_name)) 
+    {
+      /* Could attempt to get the Blob table here, but
+       * instead we will generate an error.
+       * The non-global getTable() calls can fetch Blob
+       * tables correctly if necessary.
+       *
+       * 4307 Invalid Table name
+       */
+      m_error.code = 4307;
+      return NULL;
+    }
+  }
+
   const BaseString internal_tabname(m_ndb.internalize_table_name(table_name));
   return fetchGlobalTableImplRef(InitTable(internal_tabname));
 }
