@@ -20,11 +20,9 @@
 #include <ndb_global.h>
 #include <kernel_types.h>
 
-#include "NdbDictionaryImpl.hpp"
 #include "API.hpp"
 #include <NdbOut.hpp>
-#include "NdbApiSignal.hpp"
-#include "TransporterFacade.hpp"
+
 #include <signaldata/CreateEvnt.hpp>
 #include <signaldata/SumaImpl.hpp>
 #include <SimpleProperties.hpp>
@@ -34,21 +32,14 @@
 #include <NdbError.hpp>
 #include <BaseString.hpp>
 #include <UtilBuffer.hpp>
-#include <NdbDictionary.hpp>
-#include <Ndb.hpp>
-#include "NdbImpl.hpp"
-#include "DictCache.hpp"
 #include <portlib/NdbMem.h>
-#include <NdbRecAttr.hpp>
-#include <NdbBlob.hpp>
-#include <NdbEventOperation.hpp>
-#include "NdbEventOperationImpl.hpp"
 #include <signaldata/AlterTable.hpp>
 #include "ndb_internal.hpp"
 
 #include <EventLogger.hpp>
 extern EventLogger * g_eventLogger;
 
+#define TOTAL_BUCKETS_INIT (1 << 15)
 static Gci_container_pod g_empty_gci_container;
 
 #if defined(VM_TRACE) && defined(NOT_USED)
@@ -598,6 +589,18 @@ NdbEventOperationImpl::execute_nolock()
     DBUG_RETURN(-1);
   }
 
+  bool schemaTrans = false;
+  if (m_ndb->theEventBuffer->m_total_buckets == TOTAL_BUCKETS_INIT)
+  {
+    int res = myDict->beginSchemaTrans();
+    if (res != 0)
+    {
+      m_error.code= myDict->getNdbError().code;
+      DBUG_RETURN(-1);
+    }
+    schemaTrans = true;
+  }
+
   if (theFirstPkAttrs[0] == NULL && 
       theFirstDataAttrs[0] == NULL) { // defaults to get all
   }
@@ -616,7 +619,18 @@ NdbEventOperationImpl::execute_nolock()
                                                                    buckets);
   if (r == 0) 
   {
+    /* Pre-7.0 kernel nodes do not return the number of buckets
+     * Assume it's == theNoOfDBnodes as was the case in 6.3
+     */
+    if (buckets == ~ (Uint32)0)
+      buckets = m_ndb->theImpl->theNoOfDBnodes;
+
     m_ndb->theEventBuffer->set_total_buckets(buckets);
+    if (schemaTrans)
+    {
+      schemaTrans = false;
+      myDict->endSchemaTrans(1);
+    }
 
     if (theMainOp == NULL) {
       DBUG_PRINT("info", ("execute blob ops"));
@@ -650,6 +664,13 @@ NdbEventOperationImpl::execute_nolock()
   m_magic_number= 0;
   m_error.code= myDict->getNdbError().code;
   m_ndb->theEventBuffer->remove_op();
+
+  if (schemaTrans)
+  {
+    schemaTrans = false;
+    myDict->endSchemaTrans(1);
+  }
+
   DBUG_RETURN(r);
 }
 
@@ -722,22 +743,22 @@ NdbEventOperationImpl::stop()
   DBUG_RETURN(r);
 }
 
-const bool NdbEventOperationImpl::tableNameChanged() const
+bool NdbEventOperationImpl::tableNameChanged() const
 {
   return (bool)AlterTableReq::getNameFlag(m_change_mask);
 }
 
-const bool NdbEventOperationImpl::tableFrmChanged() const
+bool NdbEventOperationImpl::tableFrmChanged() const
 {
   return (bool)AlterTableReq::getFrmFlag(m_change_mask);
 }
 
-const bool NdbEventOperationImpl::tableFragmentationChanged() const
+bool NdbEventOperationImpl::tableFragmentationChanged() const
 {
   return (bool)AlterTableReq::getFragDataFlag(m_change_mask);
 }
 
-const bool NdbEventOperationImpl::tableRangeListChanged() const
+bool NdbEventOperationImpl::tableRangeListChanged() const
 {
   return (bool)AlterTableReq::getRangeListFlag(m_change_mask);
 }
@@ -763,8 +784,8 @@ NdbEventOperationImpl::getLatestGCI()
 }
 
 bool
-NdbEventOperationImpl::execSUB_TABLE_DATA(NdbApiSignal * signal, 
-                                          LinearSectionPtr ptr[3])
+NdbEventOperationImpl::execSUB_TABLE_DATA(const NdbApiSignal * signal,
+                                          const LinearSectionPtr ptr[3])
 {
   DBUG_ENTER("NdbEventOperationImpl::execSUB_TABLE_DATA");
   const SubTableData * const sdata=
@@ -807,7 +828,8 @@ NdbEventOperationImpl::receive_event()
       // create a table object
       NdbDictInterface::Tx tx_unused;
       NdbError error;
-      NdbDictInterface dif(tx_unused, error);
+      int warn;
+      NdbDictInterface dif(tx_unused, error, warn);
       NdbTableImpl *at;
       m_change_mask = m_data_item->sdata->changeMask;
       error.code = dif.parseTableInfo(&at,
@@ -1059,8 +1081,6 @@ NdbEventOperationImpl::printAll()
  * Class NdbEventBuffer
  * Each Ndb object has a Object.
  */
-#define TOTAL_BUCKETS_INIT (1 << 15)
-
 NdbEventBuffer::NdbEventBuffer(Ndb *ndb) :
   m_total_buckets(TOTAL_BUCKETS_INIT), 
   m_min_gci_index(0),
@@ -1087,7 +1107,8 @@ NdbEventBuffer::NdbEventBuffer(Ndb *ndb) :
     ndbout_c("NdbEventHandle: NdbCondition_Create() failed");
     exit(-1);
   }
-  m_mutex= ndb->theImpl->theWaiter.m_mutex;
+  m_mutex = 0; // Set in Ndb::init()
+
   // ToDo set event buffer size
   // pre allocate event data array
   m_sz= 0;
@@ -2162,7 +2183,7 @@ NdbEventBuffer::handle_change_nodegroup(const SubGcpCompleteRep* rep)
 
     m_total_buckets += cnt;
 
-    pos = pos = (pos + 1) & mask;
+    pos = (pos + 1) & mask;
     for (; pos != maxpos; pos = (pos + 1) & mask)
     {
       assert(array[pos] > gci);
@@ -2221,15 +2242,17 @@ NdbEventBuffer::handle_change_nodegroup(const SubGcpCompleteRep* rep)
 
     m_total_buckets -= cnt;
 
-    pos = pos = (pos + 1) & mask;
+    pos = (pos + 1) & mask;
     for (; pos != maxpos; pos = (pos + 1) & mask)
     {
       assert(array[pos] > gci);
       Gci_container* tmp = find_bucket(array[pos]);
       assert((tmp->m_state & Gci_container::GC_CHANGE_CNT) == 0);
       tmp->m_gcp_complete_rep_count -= cnt;
-      ndbout_c(" - decreasing cnt on %u/%u by %u",
-               Uint32(tmp->m_gci >> 32), Uint32(tmp->m_gci), cnt);
+      ndbout_c(" - decreasing cnt on %u/%u by %u to: %u",
+               Uint32(tmp->m_gci >> 32), Uint32(tmp->m_gci), 
+               cnt,
+               tmp->m_gcp_complete_rep_count);
     }
   }
 }
