@@ -376,15 +376,15 @@ public:
   int readBlobParts(char* buf, NdbBlob* blob,
                     Uint32 part, Uint32 count, Uint16* lenLoc);
   int receive_event();
-  const bool tableNameChanged() const;
-  const bool tableFrmChanged() const;
-  const bool tableFragmentationChanged() const;
-  const bool tableRangeListChanged() const;
+  bool tableNameChanged() const;
+  bool tableFrmChanged() const;
+  bool tableFragmentationChanged() const;
+  bool tableRangeListChanged() const;
   Uint64 getGCI();
   Uint32 getAnyValue() const;
   Uint64 getLatestGCI();
-  bool execSUB_TABLE_DATA(NdbApiSignal * signal, 
-                          LinearSectionPtr ptr[3]);
+  bool execSUB_TABLE_DATA(const NdbApiSignal * signal,
+                          const LinearSectionPtr ptr[3]);
 
   NdbDictionary::Event::TableEvent getEventType();
 
