@@ -244,8 +244,12 @@ NdbInterpretedCode::load_const_u32(Uint32 RegDest, Uint32 Constant)
 int
 NdbInterpretedCode::load_const_u64(Uint32 RegDest, Uint64 Constant)
 {
-  const Uint32* p= (const Uint32 *)(&Constant);
-  return add3(Interpreter::LoadConst64(RegDest % MaxReg), p[0], p[1]);
+  union {
+    Uint64 val64;
+    Uint32 val32[2];
+  };
+  val64 = Constant;
+  return add3(Interpreter::LoadConst64(RegDest % MaxReg), val32[0], val32[1]);
 }
 
 int
@@ -558,24 +562,25 @@ NdbInterpretedCode::branch_col(Uint32 branch_type,
 
   /* Get value byte length rounded up to nearest 32-bit word */
   Uint32 len2 = Interpreter::mod4(len);
-  if((len2 == len)  &&
-     (lastWordMask == (Uint32)~0)){
+  if((len2 == len)  && (lastWordMask == (Uint32)~0))
+  {
     /* Whole number of 32-bit words */
     DBUG_RETURN(addN((Uint32*)val, len2 >> 2));
-  } else {
-    /* Partial last word */
-    len2 -= 4;
-    if (addN((Uint32*)val, len2 >> 2) != 0)
-      DBUG_RETURN(-1);
+  } 
 
-    /* Zero insignificant bytes in last word */
-    Uint32 tmp = 0;
-    for (Uint32 i = 0; i < len-len2; i++) {
-      char* p = (char*)&tmp;
-      p[i] = ((char*)val)[len2+i];
-    }
-    DBUG_RETURN(add1((tmp & lastWordMask)));
+  /* else */
+  /* Partial last word */
+  len2 -= 4;
+  if (addN((Uint32*)val, len2 >> 2) != 0)
+    DBUG_RETURN(-1);
+  
+  /* Zero insignificant bytes in last word */
+  Uint32 tmp = 0;
+  for (Uint32 i = 0; i < len-len2; i++) {
+    char* p = (char*)&tmp;
+    p[i] = ((char*)val)[len2+i];
   }
+  DBUG_RETURN(add1((tmp & lastWordMask)));
 }
 
 int 
@@ -701,7 +706,7 @@ NdbInterpretedCode::interpret_exit_nok(Uint32 ErrorCode)
 int
 NdbInterpretedCode::interpret_exit_nok()
 {
-  return add1((899 << 16) | Interpreter::EXIT_REFUSE);
+  return add1((626 << 16) | Interpreter::EXIT_REFUSE);
 }
 
 int
@@ -1068,7 +1073,7 @@ NdbInterpretedCode::finalise()
 
       assert(info.type == Label);
       
-      Uint32 currOffset = ip - firstInstruction;
+      Uint32 currOffset = Uint32(ip - firstInstruction);
       Uint32 labelOffset= info.firstInstrPos;
       
       if (labelOffset >= m_instructions_length)
