@@ -17,14 +17,7 @@
 */
 
 #include <ndb_global.h>
-#include <NdbOperation.hpp>
-#include <NdbTransaction.hpp>
-#include <NdbBlob.hpp>
-#include "NdbApiSignal.hpp"
-#include <Ndb.hpp>
-#include <NdbRecAttr.hpp>
-#include "NdbUtil.hpp"
-#include "NdbInterpretedCode.hpp"
+#include "API.hpp"
 
 #include "Interpreter.hpp"
 #include <AttributeHeader.hpp>
@@ -130,6 +123,128 @@ NdbOperation::setLastFlag(NdbApiSignal* signal, Uint32 lastFlag)
   TcKeyReq::setExecuteFlag(req->requestInfo, lastFlag);
 }
 
+int
+NdbOperation::doSendKeyReq(int aNodeId, 
+                           GenericSectionPtr* secs, 
+                           Uint32 numSecs)
+{
+  /* Send a KeyRequest - could be TCKEYREQ or TCINDXREQ
+   *
+   * Normally we send a single long signal with 1 or 2
+   * sections containing KeyInfo and AttrInfo.
+   * For backwards compatibility and testing purposes 
+   * we can send signal trains instead.
+   */
+  NdbApiSignal* request = theTCREQ;
+  TransporterFacade *tp = theNdb->theImpl->m_transporter_facade;
+  Uint32 tcNodeVersion = tp->getNodeNdbVersion(aNodeId);
+  bool forceShort = false;
+  forceShort = theNdb->theImpl->forceShortRequests;
+  bool sendLong = ( tcNodeVersion >= NDBD_LONG_TCKEYREQ ) &&
+    ! forceShort;
+  
+  if (sendLong)
+  {
+    return tp->sendSignal(request, aNodeId, secs, numSecs);
+  }
+  else
+  {
+    /* Send signal as short request - either for backwards
+     * compatibility or testing
+     */
+    Uint32 sigCount = 1;
+    Uint32 keyInfoLen  = secs[0].sz;
+    Uint32 attrInfoLen = (numSecs == 2)?
+      secs[1].sz : 
+      0;
+    
+    Uint32 keyInfoInReq = MIN(keyInfoLen, TcKeyReq::MaxKeyInfo);
+    Uint32 attrInfoInReq = MIN(attrInfoLen, TcKeyReq::MaxAttrInfo);
+    TcKeyReq* tcKeyReq = (TcKeyReq*) request->getDataPtrSend();
+    Uint32 connectPtr = tcKeyReq->apiConnectPtr;
+    Uint32 transId1 = tcKeyReq->transId1;
+    Uint32 transId2 = tcKeyReq->transId2;
+    bool indexReq = (request->theVerId_signalNumber == GSN_TCINDXREQ);
+    
+    Uint32 reqLen = request->theLength;
+    
+    /* Set TCKEYREQ flags */
+    TcKeyReq::setKeyLength(tcKeyReq->requestInfo, keyInfoLen);
+    TcKeyReq::setAIInTcKeyReq(tcKeyReq->requestInfo , attrInfoInReq);
+    TcKeyReq::setAttrinfoLen(tcKeyReq->attrLen, attrInfoLen);
+    
+    Uint32* writePtr = request->getDataPtrSend() + reqLen;
+
+    GSIReader keyInfoReader(secs[0].sectionIter);
+    GSIReader attrInfoReader(secs[1].sectionIter);
+    
+    keyInfoReader.copyNWords(writePtr, keyInfoInReq);
+    writePtr += keyInfoInReq;
+    attrInfoReader.copyNWords(writePtr, attrInfoInReq);
+
+    reqLen += keyInfoInReq + attrInfoInReq;
+    assert( reqLen <= TcKeyReq::SignalLength );
+
+    request->setLength(reqLen);
+
+    if (tp->sendSignal(request, aNodeId) == -1)
+      return -1;
+    
+    keyInfoLen -= keyInfoInReq;
+    attrInfoLen -= attrInfoInReq;
+
+    if (keyInfoLen)
+    {
+      request->theVerId_signalNumber = indexReq ? 
+        GSN_INDXKEYINFO : GSN_KEYINFO;
+      KeyInfo* keyInfo = (KeyInfo*) request->getDataPtrSend();
+      keyInfo->connectPtr = connectPtr;
+      keyInfo->transId[0] = transId1;
+      keyInfo->transId[1] = transId2;
+
+      while(keyInfoLen)
+      {
+        Uint32 dataWords = MIN(keyInfoLen, KeyInfo::DataLength);
+
+        keyInfoReader.copyNWords(&keyInfo->keyData[0], dataWords);
+        request->setLength(KeyInfo::HeaderLength + dataWords);
+
+        if (tp->sendSignal(request, aNodeId) == -1)
+          return -1;
+
+        keyInfoLen-= dataWords;
+        sigCount++;
+      }
+    }
+
+    if (attrInfoLen)
+    {
+      request->theVerId_signalNumber = indexReq ? 
+        GSN_INDXATTRINFO : GSN_ATTRINFO;
+      AttrInfo* attrInfo = (AttrInfo*) request->getDataPtrSend();
+      attrInfo->connectPtr = connectPtr;
+      attrInfo->transId[0] = transId1;
+      attrInfo->transId[1] = transId2;
+
+      while(attrInfoLen)
+      {
+        Uint32 dataWords = MIN(attrInfoLen, AttrInfo::DataLength);
+
+        attrInfoReader.copyNWords(&attrInfo->attrData[0], dataWords);
+        request->setLength(AttrInfo::HeaderLength + dataWords);
+
+        if (tp->sendSignal(request, aNodeId) == -1)
+          return -1;
+
+        attrInfoLen-= dataWords;
+        sigCount++;
+      }
+    }
+    
+    return sigCount;
+  }
+}
+
 /******************************************************************************
 int doSend()
 
@@ -144,17 +259,40 @@ NdbOperation::doSend(int aNodeId, Uint32 lastFlag)
 {
   assert(theTCREQ != NULL);
   setLastFlag(theTCREQ, lastFlag);
+  Uint32 numSecs= 1;
+  GenericSectionPtr secs[2];
 
   if (m_attribute_record != NULL)
   {
-    /* NdbRecord send - single long signal */
-    if (doSendNdbRecord(aNodeId) == -1 )
+    /*
+     * NdbRecord signal building code puts all KeyInfo and 
+     * AttrInfo into the KeyInfo and AttrInfo signal lists.
+     */
+    SignalSectionIterator keyInfoIter(theTCREQ->next());
+    SignalSectionIterator attrInfoIter(theFirstATTRINFO);
+    
+    /* KeyInfo - always present for TCKEY/INDXREQ*/
+    secs[0].sz= theTupKeyLen;
+    secs[0].sectionIter= &keyInfoIter;
+    
+    /* AttrInfo - not always needed (e.g. Delete) */
+    if (theTotalCurrAI_Len != 0)
+    {
+      secs[1].sz= theTotalCurrAI_Len;
+      secs[1].sectionIter= &attrInfoIter;
+      numSecs++;
+    }
+    
+    if (doSendKeyReq(aNodeId, &secs[0], numSecs) == -1)
       return -1;
   }
   else
   {
-    /* Old Api send - transform signal train to long sections */
-    TransporterFacade *tp = theNdb->theImpl->m_transporter_facade;
+    /* 
+     * Old Api signal building code puts first words of KeyInfo
+     * and AttrInfo into the initial request signal
+     * We use special iterators to extract this
+     */
 
     TcKeyReq* tcKeyReq= (TcKeyReq*) theTCREQ->getDataPtrSend();
     const Uint32 inlineKIOffset= Uint32(tcKeyReq->keyInfo - (Uint32*)tcKeyReq);
@@ -164,8 +302,6 @@ NdbOperation::doSend(int aNodeId, Uint32 lastFlag)
     const Uint32 inlineAILength= MIN(TcKeyReq::MaxAttrInfo, 
                                      theTotalCurrAI_Len);
     
-    Uint32 numSecs= 1;
-    GenericSectionPtr secs[2];
     /* Create iterators which use the signal train to extract
      * long sections from the short signal trains
      */
@@ -190,7 +326,7 @@ NdbOperation::doSend(int aNodeId, Uint32 lastFlag)
       numSecs++;
     }
 
-    if (tp->sendSignal(theTCREQ, aNodeId, &secs[0], numSecs) == -1)
+    if (doSendKeyReq(aNodeId, &secs[0], numSecs) == -1)
       return -1;
   }
 
@@ -204,34 +340,38 @@ NdbOperation::doSend(int aNodeId, Uint32 lastFlag)
 }//NdbOperation::doSend()
 
 
-int
-NdbOperation::doSendNdbRecord(int aNodeId)
+int 
+NdbOperation::prepareGetLockHandle()
 {
- /*
-  * Send a long signal to the kernel.
-  * KeyInfo and AttrInfo for NdbRecord are always sent as
-  * separate sections, with none in the TCKEYREQ
-  */
-  TransporterFacade *tp = theNdb->theImpl->m_transporter_facade;
+  /* Read LOCK_REF pseudo column */
+  assert(! theLockHandle->isLockRefValid() );
 
-  Uint32 numSecs= 1;
-  GenericSectionPtr secs[2];
-  SignalSectionIterator keyInfoIter(theTCREQ->next());
-  SignalSectionIterator attrInfoIter(theFirstATTRINFO);
+  theLockHandle->m_table = m_currentTable;
+
+  /* Add read of TC_LOCKREF pseudo column */
+  NdbRecAttr* ra = getValue(NdbDictionary::Column::LOCK_REF,
+                            (char*) &theLockHandle->m_lockRef);
   
-  /* KeyInfo - always present for TCKEY/INDXREQ*/
-  secs[0].sz= theTupKeyLen;
-  secs[0].sectionIter= &keyInfoIter;
-
-  /* AttrInfo - not always needed (e.g. Delete) */
-  if (theTotalCurrAI_Len != 0)
+  if (!ra)
   {
-    secs[1].sz= theTotalCurrAI_Len;
-    secs[1].sectionIter= &attrInfoIter;
-    numSecs++;
+    /* Assume error code set */
+    return -1;
   }
 
-  return tp->sendSignal(theTCREQ, aNodeId, &secs[0], numSecs);
+  theLockHandle->m_state = NdbLockHandle::PREPARED;
+  
+  /* Count Blob handles associated with this operation
+   * for LockHandle open Blob handles ref count
+   */
+  NdbBlob* blobHandle = theBlobList;
+
+  while (blobHandle != NULL)
+  {
+    theLockHandle->m_openBlobCount ++;
+    blobHandle = blobHandle->theNext;
+  }
+
+  return 0;
 }
 
 /***************************************************************************
@@ -281,6 +421,14 @@ NdbOperation::prepareSend(Uint32 aTC_ConnectPtr,
       else if (tOpType != DeleteRequest)
       {
 	assert(tOpType == ReadRequest || tOpType == ReadExclusive);
+        if (theLockHandle)
+        {
+          /* Take steps to read LockHandle info as part of read */
+          if (prepareGetLockHandle() != 0)
+            return -1;
+          
+          tTotalCurrAI_Len = theTotalCurrAI_Len;
+        }
         tTotalCurrAI_Len = repack_read(tTotalCurrAI_Len);
       }
     } else {
@@ -471,7 +619,10 @@ NdbOperation::repack_read(Uint32 len)
      * or READ_PACKED
      * (Correct NdbRecAttrs will be used when data is received)
      */
-    assert(1+ MAXNROFATTRIBUTESINWORDS <= TcKeyReq::MaxAttrInfo);
+    if (all == false && ((1 + newlen) > TcKeyReq::MaxAttrInfo))
+    {
+      return save;
+    }
     
     theNdb->releaseSignals(cnt, theFirstATTRINFO, theCurrentATTRINFO);
     theFirstATTRINFO = 0;
@@ -485,7 +636,7 @@ NdbOperation::repack_read(Uint32 len)
     else  
     {
       AttributeHeader::init(ptr, AttributeHeader::READ_PACKED, 4*newlen);
-      memcpy(ptr + 1, &mask, 4*MAXNROFATTRIBUTESINWORDS);
+      memcpy(ptr + 1, &mask, 4*newlen);
       return 1+newlen;
     }
   }
@@ -680,7 +831,8 @@ NdbOperation::prepareSendInterpreted()
 */
 int
 NdbOperation::buildSignalsNdbRecord(Uint32 aTC_ConnectPtr, 
-                                    Uint64 aTransId)
+                                    Uint64 aTransId,
+                                    const Uint32 * m_read_mask)
 {
   char buf[NdbRecord::Attr::SHRINK_VARCHAR_BUFFSIZE];
   int res;
@@ -698,6 +850,7 @@ NdbOperation::buildSignalsNdbRecord(Uint32 aTC_ConnectPtr,
   const NdbRecord *attr_rec= m_attribute_record;
   const char *updRow;
   const bool isScanTakeover= (key_rec == NULL);
+  const bool isUnlock = (theOperationType == UnlockRequest);
 
   TcKeyReq *tcKeyReq= CAST_PTR(TcKeyReq, theTCREQ->getDataPtrSend());
   Uint32 hdrSize= fillTcKeyReqHdr(tcKeyReq, aTC_ConnectPtr, aTransId);
@@ -719,8 +872,9 @@ NdbOperation::buildSignalsNdbRecord(Uint32 aTC_ConnectPtr,
     if (res)
       return res;
   }
-  else
+  else if (!isUnlock)
   {
+    /* Normal PK / unique index read */
     tcKeyReq->tableId= key_rec->tableId;
     tcKeyReq->tableSchemaVersion= key_rec->tableVersion;
     theTotalNrOfKeyWordInSignal= 0;
@@ -767,6 +921,26 @@ NdbOperation::buildSignalsNdbRecord(Uint32 aTC_ConnectPtr,
         return res;
     }
   }
+  else
+  {
+    assert(isUnlock);
+    assert(theLockHandle);
+    assert(attr_rec);
+    assert(theLockHandle->isLockRefValid());
+
+    tcKeyReq->tableId= attr_rec->tableId;
+    tcKeyReq->tableSchemaVersion= attr_rec->tableVersion;
+
+    /* Copy key data from NdbLockHandle */
+    Uint32 keyInfoWords = 0;
+    const Uint32* keyInfoSrc = theLockHandle->getKeyInfoWords(keyInfoWords);
+    assert( keyInfoWords );
+    
+    res = insertKEYINFO_NdbRecord((const char*)keyInfoSrc,
+                                  keyInfoWords << 2);
+    if (res)
+      return res;
+  }
 
   /* For long TCKEYREQ, we don't need to set the key length in the
    * header, as it is passed as the length of the KeyInfo section
@@ -1515,7 +1689,7 @@ NdbOperation::insertATTRINFOData_NdbRecord(const char *value,
 }
 
 int
-NdbOperation::checkState_TransId(NdbApiSignal* aSignal)
+NdbOperation::checkState_TransId(const NdbApiSignal* aSignal)
 {
   Uint64 tRecTransId, tCurrTransId;
   Uint32 tTmp1, tTmp2;
@@ -1550,7 +1724,7 @@ Parameters:     aSignal: the signal object that contains the TCKEYREF signal fro
 Remark:         Handles the reception of the TCKEYREF signal.
 ***************************************************************************/
 int
-NdbOperation::receiveTCKEYREF( NdbApiSignal* aSignal)
+NdbOperation::receiveTCKEYREF(const NdbApiSignal* aSignal)
 {
   if (checkState_TransId(aSignal) == -1) {
     return -1;
