@@ -16,15 +16,8 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#include <ndb_global.h>
-#include "NdbImpl.hpp"
-#include <NdbReceiver.hpp>
-#include "NdbDictionaryImpl.hpp"
-#include <NdbRecAttr.hpp>
+#include "API.hpp"
 #include <AttributeHeader.hpp>
-#include <NdbTransaction.hpp>
-#include <TransporterFacade.hpp>
-#include <NdbBlob.hpp>
 #include <signaldata/TcKeyConf.hpp>
 #include <signaldata/DictTabInfo.hpp>
 
@@ -147,10 +140,10 @@ NdbReceiver::calculate_batch_size(Uint32 key_size,
                                   Uint32& first_batch_size,
                                   const NdbRecord *record)
 {
-  TransporterFacade *tp= m_ndb->theImpl->m_transporter_facade;
-  Uint32 max_scan_batch_size= tp->get_scan_batch_size();
-  Uint32 max_batch_byte_size= tp->get_batch_byte_size();
-  Uint32 max_batch_size= tp->get_batch_size();
+  const NdbApiConfig & cfg = m_ndb->theImpl->get_ndbapi_config_parameters();
+  Uint32 max_scan_batch_size= cfg.m_scan_batch_size;
+  Uint32 max_batch_byte_size= cfg.m_batch_byte_size;
+  Uint32 max_batch_size= cfg.m_batch_size;
   Uint32 tot_size= (key_size ? (key_size + 32) : 0); //key + signal overhead
 
   if (record)
@@ -388,7 +381,7 @@ NdbReceiver::receive_packed_recattr(NdbRecAttr** recAttr,
     }
   }
   * recAttr = currRecAttr;
-  return ((Uint32*)pad(src, 0, bitPos)) - aDataPtr;
+  return (Uint32)(((Uint32*)pad(src, 0, bitPos)) - aDataPtr);
 
 err:
   abort();
@@ -574,7 +567,7 @@ NdbReceiver::receive_packed_ndbrecord(Uint32 bmlen,
     }
   }
 
-  return ((Uint32*)pad(src, 0, bitPos)) - aDataPtr;
+  return (Uint32)(((Uint32*)pad(src, 0, bitPos)) - aDataPtr);
 }
 
 
