@@ -18,16 +18,7 @@
 
 #include <ndb_global.h>
 #include <NdbOut.hpp>
-#include <NdbTransaction.hpp>
-#include <NdbOperation.hpp>
-#include <NdbScanOperation.hpp>
-#include <NdbIndexScanOperation.hpp>
-#include <NdbIndexOperation.hpp>
-#include <NdbDictionaryImpl.hpp>
-#include "NdbApiSignal.hpp"
-#include "TransporterFacade.hpp"
 #include "API.hpp"
-#include "NdbBlob.hpp"
 
 #include <AttributeHeader.hpp>
 #include <signaldata/TcKeyConf.hpp>
@@ -83,7 +74,9 @@ NdbTransaction::NdbTransaction( Ndb* aNdb ) :
   theScanningOp(NULL),
   theBuddyConPtr(0xFFFFFFFF),
   theBlobFlag(false),
-  thePendingBlobOps(0)
+  thePendingBlobOps(0),
+  m_theFirstLockHandle(NULL),
+  m_theLastLockHandle(NULL)
 {
   theListState = NotInList;
   theError.code = 0;
@@ -155,6 +148,8 @@ NdbTransaction::init()
   //
   theBlobFlag = false;
   thePendingBlobOps = 0;
+  m_theFirstLockHandle    = NULL;
+  m_theLastLockHandle     = NULL;
   if (theId == NdbObjectIdMap::InvalidId)
   {
     theId = theNdb->theImpl->theNdbObjectIdMap.map(this);
@@ -530,7 +525,7 @@ NdbTransaction::executeNoBlobs(NdbTransaction::ExecType aTypeOfExec,
 //------------------------------------------------------------------------
   Ndb* tNdb = theNdb;
 
-  Uint32 timeout = theNdb->theImpl->m_transporter_facade->m_waitfor_timeout;
+  Uint32 timeout = theNdb->theImpl->get_waitfor_timeout();
   m_waitForReply = false;
   executeAsynchPrepare(aTypeOfExec, NULL, NULL, abortOption);
   if (m_waitForReply){
@@ -805,7 +800,14 @@ void NdbTransaction::close()
   theNdb->closeTransaction(this);
 }
 
-int NdbTransaction::refresh(){
+int NdbTransaction::refresh()
+{
+  for(NdbIndexScanOperation* scan_op = m_firstExecutedScanOp;
+      scan_op != 0; scan_op = (NdbIndexScanOperation *) scan_op->theNext)
+  {
+    NdbTransaction* scan_trans = scan_op->theNdbCon;
+    scan_trans->sendTC_HBREP();
+  }
   return sendTC_HBREP();
 }
 
@@ -1028,6 +1030,7 @@ Remark:         Release all operations.
 void 
 NdbTransaction::release(){
   releaseOperations();
+  releaseLockHandles();
   if ( (theTransactionIsStarted == true) &&
        ((theCommitStatus != Committed) &&
 	(theCommitStatus != Aborted))) {
@@ -1175,6 +1178,24 @@ NdbTransaction::releaseScanOperation(NdbIndexScanOperation** listhead,
   return false;
 }
 
+void
+NdbTransaction::releaseLockHandles()
+{
+  NdbLockHandle* lh = m_theFirstLockHandle;
+
+  while (lh)
+  {
+    NdbLockHandle* next = lh->next();
+    lh->next(NULL);
+    
+    theNdb->releaseLockHandle(lh);
+    lh = next;
+  }
+
+  m_theFirstLockHandle = NULL;
+  m_theLastLockHandle = NULL;
+}
+
 /*****************************************************************************
 NdbOperation* getNdbOperation(const char* aTableName);
 
@@ -1604,26 +1625,6 @@ NdbTransaction::getNdbIndexOperation(const NdbDictionary::Index * index,
 }//NdbTransaction::getNdbIndexOperation()
 
 
-/*******************************************************************************
-int  receiveDIHNDBTAMPER(NdbApiSignal* aSignal)
-
-Return Value:  Return 0 : receiveDIHNDBTAMPER was successful.
-               Return -1: In all other case.
-Parameters:    aSignal: The signal object pointer.
-Remark:        Sets theRestartGCI in the NDB object. 
-*******************************************************************************/
-int			
-NdbTransaction::receiveDIHNDBTAMPER(NdbApiSignal* aSignal)
-{
-  if (theStatus != Connecting) {
-    return -1;
-  } else {
-    theNdb->RestartGCI((Uint32)aSignal->readData(2));
-    theStatus = Connected;
-  }//if
-  return 0;  
-}//NdbTransaction::receiveDIHNDBTAMPER()
-
 /*******************************************************************************
 int  receiveTCSEIZECONF(NdbApiSignal* aSignal);
 
@@ -1633,7 +1634,7 @@ Parameters:    aSignal: The signal object pointer.
 Remark:        Sets TC Connect pointer at reception of TCSEIZECONF. 
 *******************************************************************************/
 int			
-NdbTransaction::receiveTCSEIZECONF(NdbApiSignal* aSignal)
+NdbTransaction::receiveTCSEIZECONF(const NdbApiSignal* aSignal)
 {
   if (theStatus != Connecting)
   {
@@ -1655,7 +1656,7 @@ Parameters:    aSignal: The signal object pointer.
 Remark:        Sets TC Connect pointer. 
 *******************************************************************************/
 int			
-NdbTransaction::receiveTCSEIZEREF(NdbApiSignal* aSignal)
+NdbTransaction::receiveTCSEIZEREF(const NdbApiSignal* aSignal)
 {
   DBUG_ENTER("NdbTransaction::receiveTCSEIZEREF");
   if (theStatus != Connecting)
@@ -1681,7 +1682,7 @@ Parameters:    aSignal: The signal object pointer.
 Remark:         DisConnect TC Connect pointer to NDBAPI. 
 *******************************************************************************/
 int			
-NdbTransaction::receiveTCRELEASECONF(NdbApiSignal* aSignal)
+NdbTransaction::receiveTCRELEASECONF(const NdbApiSignal* aSignal)
 {
   if (theStatus != DisConnecting)
   {
@@ -1702,7 +1703,7 @@ Parameters:    aSignal: The signal object pointer.
 Remark:        DisConnect TC Connect pointer to NDBAPI Failure. 
 *******************************************************************************/
 int			
-NdbTransaction::receiveTCRELEASEREF(NdbApiSignal* aSignal)
+NdbTransaction::receiveTCRELEASEREF(const NdbApiSignal* aSignal)
 {
   if (theStatus != DisConnecting) {
     return -1;
@@ -1757,7 +1758,7 @@ Parameters:    aSignal: The signal object pointer.
 Remark:        
 ******************************************************************************/
 int			
-NdbTransaction::receiveTC_COMMITREF(NdbApiSignal* aSignal)
+NdbTransaction::receiveTC_COMMITREF(const NdbApiSignal* aSignal)
 {
   const TcCommitRef * ref = CAST_CONSTPTR(TcCommitRef, aSignal->getDataPtr());
   if(checkState_TransId(&ref->transId1)){
@@ -1784,7 +1785,7 @@ Parameters:    aSignal: The signal object pointer.
 Remark:        
 ******************************************************************************/
 int			
-NdbTransaction::receiveTCROLLBACKCONF(NdbApiSignal* aSignal)
+NdbTransaction::receiveTCROLLBACKCONF(const NdbApiSignal* aSignal)
 {
   if(checkState_TransId(aSignal->getDataPtr() + 1)){
     theCommitStatus = Aborted;
@@ -1808,7 +1809,7 @@ Parameters:    aSignal: The signal object pointer.
 Remark:        
 *******************************************************************************/
 int			
-NdbTransaction::receiveTCROLLBACKREF(NdbApiSignal* aSignal)
+NdbTransaction::receiveTCROLLBACKREF(const NdbApiSignal* aSignal)
 {
   if(checkState_TransId(aSignal->getDataPtr() + 1)){
     setOperationErrorCodeAbort(aSignal->readData(4));
@@ -1835,7 +1836,7 @@ Parameters:     aSignal: the signal object that contains the
 Remark:         Handles the reception of the ROLLBACKREP signal.
 *****************************************************************************/
 int
-NdbTransaction::receiveTCROLLBACKREP( NdbApiSignal* aSignal)
+NdbTransaction::receiveTCROLLBACKREP( const NdbApiSignal* aSignal)
 {
   DBUG_ENTER("NdbTransaction::receiveTCROLLBACKREP");
 
@@ -1993,6 +1994,7 @@ NdbTransaction::receiveTCKEY_FAILCONF(const TcKeyFailConf * failConf)
       case NdbOperation::InsertRequest:
       case NdbOperation::DeleteRequest:
       case NdbOperation::WriteRequest:
+      case NdbOperation::UnlockRequest:
 	tOp = tOp->next();
 	break;
       case NdbOperation::ReadRequest:
@@ -2030,7 +2032,7 @@ Parameters:     aSignal: the signal object that contains the
 Remark:         Handles the reception of the TCKEY_FAILREF signal.
 **************************************************************************/
 int
-NdbTransaction::receiveTCKEY_FAILREF(NdbApiSignal* aSignal)
+NdbTransaction::receiveTCKEY_FAILREF(const NdbApiSignal* aSignal)
 {
   /*
     Check that we are expecting signals from this transaction and
@@ -2266,7 +2268,8 @@ NdbTransaction::setupRecordOp(NdbOperation::OperationType type,
                               const char *attribute_row,
                               const unsigned char *mask,
                               const NdbOperation::OperationOptions *opts,
-                              Uint32 sizeOfOptions)
+                              Uint32 sizeOfOptions,
+                              const NdbLockHandle* lh)
 {
   NdbOperation *op;
   
@@ -2310,8 +2313,11 @@ NdbTransaction::setupRecordOp(NdbOperation::OperationType type,
   op->m_key_row= key_row;
   op->m_attribute_record= attribute_record;
   op->m_attribute_row= attribute_row;
-  attribute_record->copyMask(op->m_read_mask, mask);
   op->m_abortOption=default_ao;
+  op->theLockHandle = const_cast<NdbLockHandle*>(lh);
+  
+  AttributeMask readMask;
+  attribute_record->copyMask(readMask.rep.data, mask);
   
   /*
    * Handle options
@@ -2339,13 +2345,15 @@ NdbTransaction::setupRecordOp(NdbOperation::OperationType type,
      * been asked for
      */
     if (op->getBlobHandlesNdbRecordDelete(this,
-                                          (attribute_row != NULL)) == -1)
+                                          (attribute_row != NULL),
+                                          readMask.rep.data) == -1)
       return NULL;
   }
-  else if (unlikely(attribute_record->flags & NdbRecord::RecHasBlob))
+  else if (unlikely((attribute_record->flags & NdbRecord::RecHasBlob) &&
+                    (type != NdbOperation::UnlockRequest)))
   {
-    /* Create blob handles for non-delete operations */
-    if (op->getBlobHandlesNdbRecord(this) == -1)
+    /* Create blob handles for non-delete, non-unlock operations */
+    if (op->getBlobHandlesNdbRecord(this, readMask.rep.data) == -1)
       return NULL;
   }
 
@@ -2353,7 +2361,8 @@ NdbTransaction::setupRecordOp(NdbOperation::OperationType type,
    * Now prepare the signals to be sent...
    *
    */
-  int returnCode=op->buildSignalsNdbRecord(theTCConPtr, theTransactionId);
+  int returnCode=op->buildSignalsNdbRecord(theTCConPtr, theTransactionId,
+                                           readMask.rep.data);
 
   if (returnCode)
   {
@@ -2745,3 +2754,194 @@ NdbTransaction::report_node_failure(Uint32 id){
   }
   return 0;
 }
+
+NdbLockHandle*
+NdbTransaction::getLockHandle()
+{
+  NdbLockHandle* lh;
+
+  /* Get a LockHandle object from the Ndb pool and
+   * link it into our transaction
+   */
+  lh = theNdb->getLockHandle();
+
+  if (lh)
+  {
+    lh->thePrev = m_theLastLockHandle;
+    if (m_theLastLockHandle == NULL)
+    {
+      m_theFirstLockHandle = lh;
+      m_theLastLockHandle = lh;
+    }
+    else
+    {
+      lh->next(NULL);
+      m_theLastLockHandle->next(lh);
+      m_theLastLockHandle = lh;
+    }
+  }
+
+  return lh;
+}
+
+const NdbOperation*
+NdbTransaction::unlock(const NdbLockHandle* lockHandle,
+                       NdbOperation::AbortOption ao)
+{
+  switch(lockHandle->m_state)
+  {
+  case NdbLockHandle::FREE:
+    /* LockHandle already released */
+    setErrorCode(4551);
+    return NULL;
+  case NdbLockHandle::PREPARED:
+    if (likely(lockHandle->isLockRefValid()))
+    {
+      /* Looks ok */
+      break;
+    }
+    /* Fall through */
+  case NdbLockHandle::ALLOCATED:
+    /* NdbLockHandle original operation not executed successfully */
+    setErrorCode(4553);
+    return NULL;
+  default:
+    abort();
+    return NULL;
+  }
+
+  if (m_theFirstLockHandle == NULL)
+  {
+    /* NdbLockHandle does not belong to transaction */
+    setErrorCode(4552);
+    return NULL;
+  }
+
+#ifdef VM_TRACE
+  /* Check that this transaction 'owns' this lockhandle */
+  {
+    NdbLockHandle* tmp = m_theLastLockHandle;
+    while (tmp && (tmp != lockHandle))
+    {
+      tmp = tmp->thePrev;
+    }
+    
+    if (tmp != lockHandle)
+    {
+      /* NdbLockHandle does not belong to transaction */
+      setErrorCode(4552);
+      return NULL;
+    }
+  }
+#endif
+
+  assert(theSimpleState == 0);
+
+  /* Use the first work of the Lock reference as the unlock
+   * operation's partition id
+   * The other two words form the key.
+   */
+  NdbOperation::OperationOptions opts;
+
+  opts.optionsPresent = NdbOperation::OperationOptions::OO_PARTITION_ID;
+  opts.partitionId = lockHandle->getDistKey();
+
+  if (ao != NdbOperation::DefaultAbortOption)
+  {
+    /* User supplied a preference, pass it on */
+    opts.optionsPresent |= NdbOperation::OperationOptions::OO_ABORTOPTION;
+    opts.abortOption = ao;
+  }
+
+  NdbOperation* unlockOp = setupRecordOp(NdbOperation::UnlockRequest,
+                                         NdbOperation::LM_CommittedRead,
+                                         NdbOperation::AbortOnError, // Default
+                                         lockHandle->m_table->m_ndbrecord,
+                                         NULL, // key_row
+                                         lockHandle->m_table->m_ndbrecord,
+                                         NULL,             // attr_row
+                                         NULL,             // mask
+                                         &opts,            // opts,
+                                         sizeof(opts),     // sizeOfOptions
+                                         lockHandle);
+  
+  return unlockOp;
+}
+
+int
+NdbTransaction::releaseLockHandle(const NdbLockHandle* lockHandle)
+{
+  NdbLockHandle* prev = lockHandle->thePrev;
+  NdbLockHandle* next = lockHandle->theNext;
+
+  switch(lockHandle->m_state)
+  {
+  case NdbLockHandle::FREE:
+    /* NdbLockHandle already released */
+    setErrorCode(4551);
+    return -1;
+  case NdbLockHandle::PREPARED:
+    if (! lockHandle->isLockRefValid())
+    {
+      /* It's not safe to release the lockHandle after it's
+       * defined and before the operation's executed.
+       * The lockhandle memory is needed to receive the
+       * Lock Reference during execution
+       */
+      /* Cannot releaseLockHandle until operation executed */
+      setErrorCode(4550);
+      return -1;
+    }
+    /* Fall through */
+  case NdbLockHandle::ALLOCATED:
+    /* Ok to release */
+    break;
+  default:
+    /* Bad state */
+    abort();
+    return -1;
+  }
+
+#ifdef VM_TRACE
+  /* Check lockhandle is known to this transaction */
+  NdbLockHandle* tmp = m_theFirstLockHandle;
+  while (tmp &&
+         (tmp != lockHandle))
+  {
+    tmp = tmp->next();
+  }
+
+  if (tmp != lockHandle)
+  {
+    abort();
+    return -1;
+  }
+#endif
+
+  /* Repair list around lock handle */
+  if (prev)
+    prev->next(next);
+  
+  if (next)
+    next->thePrev = prev;
+  
+  /* Repair list head and tail ptrs */
+  if (lockHandle == m_theFirstLockHandle)
+  {
+    m_theFirstLockHandle = next;
+  }
+  if (lockHandle == m_theLastLockHandle)
+  {
+    m_theLastLockHandle = prev;
+  }
+  
+  /* Now return it to the Ndb's freelist */
+  NdbLockHandle* lh = const_cast<NdbLockHandle*>(lockHandle);
+
+  lh->thePrev = NULL;
+  lh->theNext = NULL;
+  
+  theNdb->releaseLockHandle(lh);
+
+  return 0;
+}
