@@ -19,16 +19,7 @@
 
 #include <ndb_global.h>
 
-#include "NdbApiSignal.hpp"
-#include "NdbImpl.hpp"
-#include <NdbTransaction.hpp>
-#include <NdbOperation.hpp>
-#include <NdbIndexOperation.hpp>
-#include <NdbScanOperation.hpp>
-#include <NdbRecAttr.hpp>
-#include <NdbReceiver.hpp>
 #include "API.hpp"
-#include "NdbEventOperationImpl.hpp"
 
 #include <signaldata/TcCommit.hpp>
 #include <signaldata/TcKeyFailConf.hpp>
@@ -80,19 +71,20 @@ Ndb::init(int aMaxNoOfTransactions)
   }//if
   theInitState = StartingInit;
   TransporterFacade * theFacade =  theImpl->m_transporter_facade;
-  theFacade->lock_mutex();
-  
-  const int tBlockNo = theFacade->open(this,
-                                       executeMessage, 
-                                       statusMessage);  
+  theEventBuffer->m_mutex = theFacade->theMutexPtr;
+
+  const int tBlockNo = theImpl->open(theFacade);
+
   if ( tBlockNo == -1 ) {
     theError.code = 4105;
-    theFacade->unlock_mutex();
     DBUG_RETURN(-1); // no more free blocknumbers
   }//if
   
   theNdbBlockNumber = tBlockNo;
 
+  /* Init cached min node version */
+  theFacade->lock_mutex();
+  theCachedMinDbNodeVersion = theFacade->getMinDbNodeVersion();
   theFacade->unlock_mutex();
   
   theDictionary->setTransporter(this, theFacade);
@@ -147,7 +139,7 @@ error_handler:
   ndbout << "error_handler" << endl;
   releaseTransactionArrays();
   delete theDictionary;
-  theImpl->m_transporter_facade->close(theNdbBlockNumber, 0);
+  theImpl->close();
   DBUG_RETURN(-1);
 }
 
@@ -168,12 +160,10 @@ Ndb::releaseTransactionArrays()
 }//Ndb::releaseTransactionArrays()
 
 void
-Ndb::executeMessage(void* NdbObject,
-                    NdbApiSignal * aSignal,
-                    LinearSectionPtr ptr[3])
+NdbImpl::trp_deliver_signal(const NdbApiSignal * aSignal,
+                            const LinearSectionPtr ptr[3])
 {
-  Ndb* tNdb = (Ndb*)NdbObject;
-  tNdb->handleReceivedSignal(aSignal, ptr);
+  m_ndb.handleReceivedSignal(aSignal, ptr);
 }
 
 void Ndb::connected(Uint32 ref)
@@ -196,9 +186,8 @@ void Ndb::connected(Uint32 ref)
   }
   theImpl->theNoOfDBnodes = n;
   
-  theFirstTransId = ((Uint64)tBlockNo << 52)+
+  theFirstTransId += ((Uint64)tBlockNo << 52)+
     ((Uint64)tmpTheNode << 40);
-  theFirstTransId += theFacade->m_max_trans_id;
   //      assert(0);
   DBUG_PRINT("info",("connected with ref=%x, id=%d, no_db_nodes=%d, first_trans_id: 0x%lx",
 		     theMyRef,
@@ -222,28 +211,30 @@ void Ndb::report_node_connected(Uint32 nodeId)
 }
 
 void
-Ndb::statusMessage(void* NdbObject, Uint32 a_node, bool alive, bool nfComplete)
+NdbImpl::trp_node_status(Uint32 a_node, Uint32 _event)
 {
   DBUG_ENTER("Ndb::statusMessage");
-  DBUG_PRINT("info", ("a_node: %u  alive: %u  nfComplete: %u",
-                      a_node, alive, nfComplete));
-  Ndb* tNdb = (Ndb*)NdbObject;
-  if (alive) {
-    if (nfComplete) {
-      // cluster connect, a_node == own reference
-      tNdb->connected(a_node);
-      DBUG_VOID_RETURN;
-    }//if
+  NS_Event event = (NS_Event)_event;
+  DBUG_PRINT("info", ("a_node: %u  event: %u",
+                      a_node, _event));
+  Ndb* tNdb = (Ndb*)&m_ndb;
+  switch(event){
+  case NS_CONNECTED:
+    // cluster connect, a_node == own reference
+    tNdb->connected(a_node);
+    break;
+  case NS_NODE_ALIVE:
     tNdb->report_node_connected(a_node);
-  } else {
-    if (nfComplete) {
-      tNdb->report_node_failure_completed(a_node);
-    } else {
-      tNdb->report_node_failure(a_node);
-    }//if
+    break;
+  case NS_NODE_FAILED:
+    tNdb->report_node_failure(a_node);
+    break;
+  case NS_NODE_NF_COMPLETE:
+    tNdb->report_node_failure_completed(a_node);
+    break;
   }//if
   NdbDictInterface::execNodeStatus(&tNdb->theDictionary->m_receiver,
-				   a_node, alive, nfComplete);
+				   a_node, event);
   DBUG_VOID_RETURN;
 }
 
@@ -340,19 +331,24 @@ Parameters:     aSignal: The signal object.
 Remark:         Send all operations belonging to this connection. 
 *****************************************************************************/
 void	
-Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
+Ndb::handleReceivedSignal(const NdbApiSignal* aSignal,
+			  const LinearSectionPtr ptr[3])
 {
   NdbOperation* tOp;
   NdbIndexOperation* tIndexOp;
   NdbTransaction* tCon;
   int tReturnCode = -1;
   const Uint32* tDataPtr = aSignal->getDataPtr();
-  const Uint32 tWaitState = theImpl->theWaiter.m_state;
+  const Uint32 tWaitState = theImpl->theWaiter.get_state();
   const Uint32 tSignalNumber = aSignal->readSignalNumber();
   const Uint32 tFirstData = *tDataPtr;
   const Uint32 tLen = aSignal->getLength();
+  Uint32 tNewState = tWaitState;
   void * tFirstDataPtr;
-  NdbWaiter *t_waiter;
+  NdbWaiter *t_waiter = &theImpl->theWaiter;
+
+  /* Update cached Min Db node version */
+  theCachedMinDbNodeVersion = theImpl->m_transporter_facade->getMinDbNodeVersion();
 
   /*
     In order to support 64 bit processes in the application we need to use
@@ -421,8 +417,8 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 	return;
       case NdbReceiver::NDB_SCANRECEIVER:
 	tCon->theScanningOp->receiver_delivered(tRec);
-	theImpl->theWaiter.m_state = (((WaitSignalType) tWaitState) == WAIT_SCAN ? 
-				      (Uint32) NO_WAIT : tWaitState);
+        tNewState = (((WaitSignalType) tWaitState) == WAIT_SCAN ? 
+                     (Uint32) NO_WAIT : tWaitState);
 	break;
       default:
 	goto InvalidSignal;
@@ -619,7 +615,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
       }//if
       tReturnCode = tCon->receiveTCSEIZECONF(aSignal);
       if (tReturnCode != -1) {
-	theImpl->theWaiter.m_state = NO_WAIT;
+        tNewState = NO_WAIT;
       } else {
 	goto InvalidSignal;
       }//if
@@ -639,7 +635,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
       }//if
       tReturnCode = tCon->receiveTCSEIZEREF(aSignal);
       if (tReturnCode != -1) {
-	theImpl->theWaiter.m_state = NO_WAIT;
+        tNewState = NO_WAIT;
       } else {
         return;
       }//if
@@ -659,7 +655,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
       }//if
       tReturnCode = tCon->receiveTCRELEASECONF(aSignal);
       if (tReturnCode != -1) {
-	theImpl->theWaiter.m_state = NO_WAIT;
+        tNewState = NO_WAIT;
       }//if
       break;
     } 
@@ -677,7 +673,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
       }//if
       tReturnCode = tCon->receiveTCRELEASEREF(aSignal);
       if (tReturnCode != -1) {
-	theImpl->theWaiter.m_state = NO_WAIT;
+        tNewState = NO_WAIT;
       }//if
       break;
     }
@@ -757,9 +753,15 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
         !op->execSUB_TABLE_DATA(aSignal, ptr))
       return;
     
-    for (int i= aSignal->m_noOfSections;i < 3; i++) {
-      ptr[i].p = NULL;
-      ptr[i].sz = 0;
+    LinearSectionPtr copy[3];
+    for (int i = 0; i<aSignal->m_noOfSections; i++)
+    {
+      copy[i] = ptr[i];
+    }
+    for (int i = aSignal->m_noOfSections; i < 3; i++)
+    {
+      copy[i].p = NULL;
+      copy[i].sz = 0;
     }
     DBUG_PRINT("info",("oid=senderData: %d, gci{hi/lo}: %d/%d, operation: %d, "
 		       "tableId: %d",
@@ -767,24 +769,9 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 		       SubTableData::getOperation(sdata->requestInfo),
 		       sdata->tableId));
 
-    theEventBuffer->insertDataL(op, sdata, tLen, ptr);
+    theEventBuffer->insertDataL(op, sdata, tLen, copy);
     return;
   }
-  case GSN_DIHNDBTAMPER:
-    {
-      tFirstDataPtr = int2void(tFirstData);
-      if (tFirstDataPtr == 0) goto InvalidSignal;
-      
-      if (tWaitState != WAIT_NDB_TAMPER)
-	return;
-      tCon = void2con(tFirstDataPtr);
-      if (tCon->checkMagicNumber() != 0)
-	return;
-      tReturnCode = tCon->receiveDIHNDBTAMPER(aSignal);
-      if (tReturnCode != -1)
-	theImpl->theWaiter.m_state = NO_WAIT;
-      break;
-    }
   case GSN_SCAN_TABCONF:
     {
       tFirstDataPtr = int2void(tFirstData);
@@ -804,7 +791,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 				      tLen - ScanTabConf::SignalLength);
 	}
 	if (tReturnCode != -1 && tWaitState == WAIT_SCAN)
-	  theImpl->theWaiter.m_state = NO_WAIT;
+          tNewState = NO_WAIT;
 	break;
       } else {
 	goto InvalidSignal;
@@ -823,7 +810,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
       if (tCon->checkMagicNumber() == 0){
 	tReturnCode = tCon->receiveSCAN_TABREF(aSignal);
 	if (tReturnCode != -1 && tWaitState == WAIT_SCAN){
-	  theImpl->theWaiter.m_state = NO_WAIT;
+          tNewState = NO_WAIT;
 	}
 	break;
       }
@@ -848,8 +835,8 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
       switch(com){
       case 1:
 	tCon->theScanningOp->receiver_delivered(tRec);
-	theImpl->theWaiter.m_state = (((WaitSignalType) tWaitState) == WAIT_SCAN ? 
-			      (Uint32) NO_WAIT : tWaitState);
+        tNewState = (((WaitSignalType) tWaitState) == WAIT_SCAN ? 
+                     (Uint32) NO_WAIT : tWaitState);
 	break;
       case 0:
 	break;
@@ -920,8 +907,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
     goto InvalidSignal;
   }//swich
 
-  t_waiter= &theImpl->theWaiter;
-  if (t_waiter->get_state() == NO_WAIT && tWaitState != NO_WAIT)
+  if (tNewState != tWaitState)
   {
     /*
       If our waiter object is the owner of the "poll rights", then we
@@ -934,20 +920,13 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
       its conditional wait. This will wake up this thread so that it
       can continue its work.
     */
-    TransporterFacade *tp= theImpl->m_transporter_facade;
-    if (tp->get_poll_owner() != t_waiter)
-    {
-      /*
-         Wake up the thread waiting for response and remove it from queue
-         of objects waiting for receive completion
-      */
-      tp->remove_from_cond_wait_queue(t_waiter);
-      t_waiter->cond_signal();
-    }
-  }//if
+    t_waiter->signal(tNewState);
+  }
+
   return;
 
- InvalidSignal:
+
+InvalidSignal:
 #ifdef VM_TRACE
   ndbout_c("Ndbif: Error Ndb::handleReceivedSignal "
 	   "(tFirstDataPtr=%p, GSN=%d, theImpl->theWaiter.m_state=%d)"
@@ -999,19 +978,7 @@ Ndb::completedTransaction(NdbTransaction* aCon)
     if ((theMinNoOfEventsToWakeUp != 0) &&
         (theNoOfCompletedTransactions >= theMinNoOfEventsToWakeUp)) {
       theMinNoOfEventsToWakeUp = 0;
-      TransporterFacade *tp = theImpl->m_transporter_facade;
-      NdbWaiter *t_waiter= &theImpl->theWaiter;
-      if (tp->get_poll_owner() != t_waiter) {
-        /*
-          When we come here, this is executed by the thread owning the "poll
-          rights". This thread is not where our waiter object belongs.
-          Thus we wake up the thread owning this waiter object but first
-          we must remove it from the conditional wait queue so that we
-          don't assign it as poll owner later on.
-        */
-        tp->remove_from_cond_wait_queue(t_waiter);
-        t_waiter->cond_signal();
-      }
+      theImpl->theWaiter.signal(NO_WAIT);
       return;
     }//if
   } else {
@@ -1081,7 +1048,7 @@ Ndb::pollCompleted(NdbTransaction** aCopyArray)
 void
 Ndb::check_send_timeout()
 {
-  Uint32 timeout = theImpl->m_transporter_facade->m_waitfor_timeout;
+  Uint32 timeout = theImpl->get_ndbapi_config_parameters().m_waitfor_timeout;
   NDB_TICKS current_time = NdbTick_CurrentMillisecond();
   assert(current_time >= the_last_check_time);
   if (current_time - the_last_check_time > 1000) {
@@ -1172,18 +1139,15 @@ Ndb::sendPrepTrans(int forceSend)
   */
   Uint32 i;
   TransporterFacade* tp = theImpl->m_transporter_facade;
+  theCachedMinDbNodeVersion = theImpl->m_transporter_facade->getMinDbNodeVersion();
   Uint32 no_of_prep_trans = theNoOfPreparedTransactions;
   for (i = 0; i < no_of_prep_trans; i++) {
     NdbTransaction * a_con = thePreparedTransactionsArray[i];
     thePreparedTransactionsArray[i] = NULL;
     Uint32 node_id = a_con->getConnectedNodeId();
     if ((tp->getNodeSequence(node_id) == a_con->theNodeSequence) &&
-         tp->get_node_alive(node_id) ||
-         (tp->get_node_stopping(node_id) && 
-         ((a_con->theSendStatus == NdbTransaction::sendABORT) ||
-          (a_con->theSendStatus == NdbTransaction::sendABORTfail) ||
-          (a_con->theSendStatus == NdbTransaction::sendCOMMITstate) ||
-          (a_con->theSendStatus == NdbTransaction::sendCompleted)))) {
+        (tp->get_node_alive(node_id) || tp->get_node_stopping(node_id)))
+    {
       /*
       We will send if
       1) Node is alive and sequences are correct OR
@@ -1229,27 +1193,15 @@ Ndb::sendPrepTrans(int forceSend)
 #ifdef VM_TRACE
       a_con->printState();
 #endif
-      if ((tp->getNodeSequence(node_id) == a_con->theNodeSequence) &&
-          tp->get_node_stopping(node_id)) {
-        /*
-        The node we are connected to is currently in an early stopping phase
-        of a graceful stop. We will not send the prepared transactions. We
-        will simply refuse and let the application code handle the abort.
-        */
-        TRACE_DEBUG("Abort a transaction when stopping a node");
-        a_con->setOperationErrorCodeAbort(4023);
-        a_con->theCommitStatus = NdbTransaction::NeedAbort;
-      } else {
-        /*
+      /*
         The node is hard dead and we cannot continue. We will also release
         the connection to the free pool.
-        */
-        TRACE_DEBUG("The node was stone dead, inform about abort");
-        a_con->setOperationErrorCodeAbort(4025);
-        a_con->theReleaseOnClose = true;
-        a_con->theTransactionIsStarted = false;
-        a_con->theCommitStatus = NdbTransaction::Aborted;
-      }//if
+      */
+      TRACE_DEBUG("The node was stone dead, inform about abort");
+      a_con->setOperationErrorCodeAbort(4025);
+      a_con->theReleaseOnClose = true;
+      a_con->theTransactionIsStarted = false;
+      a_con->theCommitStatus = NdbTransaction::Aborted;
     }//if
     a_con->theReturnStatus = NdbTransaction::ReturnFailure;
     a_con->theCompletionStatus = NdbTransaction::CompletedFailure;
@@ -1277,9 +1229,11 @@ Ndb::waitCompletedTransactions(int aMilliSecondsToWait,
 			       int noOfEventsToWaitFor,
                                PollGuard *poll_guard)
 {
-  theImpl->theWaiter.m_state = NO_WAIT; 
+  theImpl->theWaiter.set_node(0);
+  theImpl->theWaiter.set_state(WAIT_TRANS);
+
   /**
-   * theImpl->theWaiter.m_state = NO_WAIT; 
+   * theImpl->theWaiter.set_node(0)
    * To ensure no messup with synchronous node fail handling
    * (see ReportFailure)
    */
@@ -1298,10 +1252,6 @@ Ndb::waitCompletedTransactions(int aMilliSecondsToWait,
   } while (waitTime > 0);
 }//Ndb::waitCompletedTransactions()
 
-void Ndb::cond_signal()
-{
-  NdbCondition_Signal(theImpl->theWaiter.m_condition);
-}
 /*****************************************************************************
 void sendPreparedTransactions(int forceSend = 0);
 
@@ -1335,8 +1285,7 @@ Ndb::sendPollNdb(int aMillisecondNumber, int minNoOfEventsToWakeup, int forceSen
     in all places where the object is out of context due to a return,
     break, continue or simply end of statement block
   */
-  PollGuard pg(theImpl->m_transporter_facade, &theImpl->theWaiter,
-               theNdbBlockNumber);
+  PollGuard pg(* theImpl);
   sendPrepTrans(forceSend);
   return poll_trans(aMillisecondNumber, minNoOfEventsToWakeup, &pg);
 }
@@ -1358,6 +1307,7 @@ Ndb::poll_trans(int aMillisecondNumber, int minNoOfEventsToWakeup,
   } else {
     tNoCompletedTransactions = pollCompleted(tConArray);
   }//if
+  theMinNoOfEventsToWakeUp = 0; // no more wakup
   pg->unlock_and_signal();
   reportCallback(tConArray, tNoCompletedTransactions);
   return tNoCompletedTransactions;
@@ -1379,8 +1329,7 @@ Ndb::pollNdb(int aMillisecondNumber, int minNoOfEventsToWakeup)
     in all places where the object is out of context due to a return,
     break, continue or simply end of statement block
   */
-  PollGuard pg(theImpl->m_transporter_facade, &theImpl->theWaiter,
-               theNdbBlockNumber);
+  PollGuard pg(* theImpl);
   return poll_trans(aMillisecondNumber, minNoOfEventsToWakeup, &pg);
 }
 
@@ -1412,7 +1361,7 @@ Ndb::sendRecSignal(Uint16 node_id,
     in all places where the object is out of context due to a return,
     break, continue or simply end of statement block
   */
-  PollGuard poll_guard(tp,&theImpl->theWaiter,theNdbBlockNumber);
+  PollGuard poll_guard(* theImpl);
   read_conn_seq= tp->getNodeSequence(node_id);
   if (ret_conn_seq)
     *ret_conn_seq= read_conn_seq;
