@@ -16,16 +16,7 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#include <NdbOut.hpp>
-#include <Ndb.hpp>
-#include <NdbOperation.hpp>
-#include <NdbIndexOperation.hpp>
-#include <NdbIndexScanOperation.hpp>
-#include "NdbApiSignal.hpp"
-#include <NdbRecAttr.hpp>
-#include "NdbUtil.hpp"
 #include "API.hpp"
-#include "NdbBlob.hpp"
 
 void
 Ndb::checkFailedNode()
@@ -265,6 +256,18 @@ Ndb::getNdbBlob()
   return tBlob;
 }
 
+NdbLockHandle*
+Ndb::getLockHandle()
+{
+  NdbLockHandle* lh = theImpl->theLockHandleList.seize(this);
+  if (lh)
+  {
+    lh->init();
+  }
+
+  return lh;
+} 
+
 /***************************************************************************
 void releaseNdbBranch(NdbBranch* aNdbBranch);
 
@@ -450,6 +453,13 @@ Ndb::releaseNdbBlob(NdbBlob* aBlob)
   theImpl->theNdbBlobIdleList.release(aBlob);
 }
 
+void
+Ndb::releaseLockHandle(NdbLockHandle* lh)
+{
+  lh->release(this);
+  theImpl->theLockHandleList.release(lh);
+};
+
 /****************************************************************************
 int releaseConnectToNdb(NdbTransaction* aConnectConnection);
 
@@ -573,6 +583,10 @@ Ndb::get_free_list_usage(Ndb::Free_list_usage* curr)
     update(curr, theImpl->theScanList, "NdbReceiver");
   }
   else if(!strcmp(curr->m_name, "NdbReceiver"))
+  {
+    update(curr, theImpl->theLockHandleList, "NdbLockHandle");
+  }
+  else if(!strcmp(curr->m_name, "NdbLockHandle"))
   {
     return 0;
   }
@@ -601,3 +615,4 @@ TI(NdbReceiver);
 TI(NdbConnection);
 TI(NdbIndexOperation);
 TI(NdbIndexScanOperation);
+TI(NdbLockHandle);
