@@ -17,19 +17,20 @@
 */
 
 #include <ndb_global.h>
-#include <my_pthread.h>
 #include <ndb_limits.h>
+#include "trp_client.hpp"
 #include "TransporterFacade.hpp"
 #include "ClusterMgr.hpp"
 #include <IPCConfig.hpp>
 #include <TransporterCallback.hpp>
 #include <TransporterRegistry.hpp>
 #include "NdbApiSignal.hpp"
+#include "NdbWaiter.hpp"
 #include <NdbOut.hpp>
 #include <NdbEnv.h>
 #include <NdbSleep.h>
 
-#include "API.hpp"
+#include <kernel/GlobalSignalNumbers.h>
 #include <mgmapi_config_parameters.h>
 #include <mgmapi_configuration.hpp>
 #include <NdbConfig.h>
@@ -210,24 +211,24 @@ TransporterFacade::deliver_signal(SignalHeader * const header,
                                   Uint8 prio, Uint32 * const theData,
                                   LinearSectionPtr ptr[3])
 {
-
-  TransporterFacade::ThreadData::Object_Execute oe; 
   Uint32 tRecBlockNo = header->theReceiversBlockNumber;
   
 #ifdef API_TRACE
   if(setSignalLog() && TRACE_GSN(header->theVerId_signalNumber)){
     signalLogger.executeSignal(* header, 
-			       prio,
+                               prio,
                                theData,
-			       ownId(),
+                               ownId(),
                                ptr, header->m_noOfSections);
     signalLogger.flushSignalLog();
   }
 #endif  
 
-  if (tRecBlockNo >= MIN_API_BLOCK_NO) {
-    oe = m_threads.get(tRecBlockNo);
-    if (oe.m_object != 0 && oe.m_executeFunction != 0) {
+  if (tRecBlockNo >= MIN_API_BLOCK_NO)
+  {
+    trp_client * clnt = m_threads.get(tRecBlockNo);
+    if (clnt != 0)
+    {
       /**
        * Handle received signal immediately to avoid any unnecessary
        * copying of data, allocation of memory and other things. Copying
@@ -242,9 +243,11 @@ TransporterFacade::deliver_signal(SignalHeader * const header,
       NdbApiSignal tmpSignal(*header);
       NdbApiSignal * tSignal = &tmpSignal;
       tSignal->setDataPtr(theData);
-      (* oe.m_executeFunction) (oe.m_object, tSignal, ptr);
+      clnt->trp_deliver_signal(tSignal, ptr);
     }//if
-  } else if (tRecBlockNo == API_PACKED) {
+  }
+  else if (tRecBlockNo == API_PACKED)
+  {
     /**
      * Block number == 2047 is used to signal a signal that consists of
      * multiple instances of the same signal. This is an effort to
@@ -262,150 +265,55 @@ TransporterFacade::deliver_signal(SignalHeader * const header,
       Tsent++;
       Uint32 TpacketLen = (Theader & 0x1F) + 3;
       tRecBlockNo = Theader >> 16;
-      if (TpacketLen <= 25) {
-	if ((TpacketLen + Tsent) <= Tlength) {
-	  /**
-	   * Set the data length of the signal and the receivers block
-	   * reference and then call the API.
-	   */
-	  header->theLength = TpacketLen;
-	  header->theReceiversBlockNumber = tRecBlockNo;
-	  Uint32* tDataPtr = &theData[Tsent];
-	  Tsent += TpacketLen;
-	  if (tRecBlockNo >= MIN_API_BLOCK_NO) {
-	    oe = m_threads.get(tRecBlockNo);
-	    if(oe.m_object != 0 && oe.m_executeFunction != 0){
-	      NdbApiSignal tmpSignal(*header);
-	      NdbApiSignal * tSignal = &tmpSignal;
-	      tSignal->setDataPtr(tDataPtr);
-	      (*oe.m_executeFunction)(oe.m_object, tSignal, 0);
-	    }
-	  }
-	}
+      if (TpacketLen <= 25)
+      {
+        if ((TpacketLen + Tsent) <= Tlength)
+        {
+          /**
+           * Set the data length of the signal and the receivers block
+           * reference and then call the API.
+           */
+          header->theLength = TpacketLen;
+          header->theReceiversBlockNumber = tRecBlockNo;
+          Uint32* tDataPtr = &theData[Tsent];
+          Tsent += TpacketLen;
+          if (tRecBlockNo >= MIN_API_BLOCK_NO)
+          {
+            trp_client * clnt = m_threads.get(tRecBlockNo);
+            if(clnt != 0)
+            {
+              NdbApiSignal tmpSignal(*header);
+              NdbApiSignal * tSignal = &tmpSignal;
+              tSignal->setDataPtr(tDataPtr);
+              clnt->trp_deliver_signal(tSignal, 0);
+            }
+          }
+        }
       }
     }
     return;
-  } else if (tRecBlockNo == API_CLUSTERMGR) {
-     /**
-      * The signal was aimed for the Cluster Manager. 
-      * We handle it immediately here.
-      */     
-     ClusterMgr * clusterMgr = theClusterMgr;
-     const Uint32 gsn = header->theVerId_signalNumber;
-
-     switch (gsn){
-     case GSN_API_REGREQ:
-       clusterMgr->execAPI_REGREQ(theData);
-       break;
-
-     case GSN_API_REGCONF:
-     {
-       clusterMgr->execAPI_REGCONF(theData);
-
-       // Distribute signal to all threads/blocks
-       NdbApiSignal tSignal(* header);
-       tSignal.setDataPtr(theData);
-       for_each(&tSignal, ptr);
-       break;
-     }
-
-     case GSN_API_REGREF:
-       clusterMgr->execAPI_REGREF(theData);
-       break;
-
-     case GSN_NODE_FAILREP:
-       clusterMgr->execNODE_FAILREP(theData);
-       break;
-       
-     case GSN_NF_COMPLETEREP:
-       clusterMgr->execNF_COMPLETEREP(theData);
-       break;
-
-     case GSN_ARBIT_STARTREQ:
-       if (theArbitMgr != NULL)
-	 theArbitMgr->doStart(theData);
-       break;
-       
-     case GSN_ARBIT_CHOOSEREQ:
-       if (theArbitMgr != NULL)
-	 theArbitMgr->doChoose(theData);
-       break;
-       
-     case GSN_ARBIT_STOPORD:
-       if(theArbitMgr != NULL)
-	 theArbitMgr->doStop(theData);
-       break;
-
-     case GSN_ALTER_TABLE_REP:
-     {
-       if (m_globalDictCache == NULL)
-         break;
-       const AlterTableRep* rep = (const AlterTableRep*)theData;
-       m_globalDictCache->lock();
-       m_globalDictCache->
-	 alter_table_rep((const char*)ptr[0].p, 
-			 rep->tableId,
-			 rep->tableVersion,
-			 rep->changeType == AlterTableRep::CT_ALTERED);
-       m_globalDictCache->unlock();
-       break;
-     }
-     case GSN_SUB_GCP_COMPLETE_REP:
-     {
-       /**
-	* Report
-	*/
-       NdbApiSignal tSignal(* header);
-       tSignal.setDataPtr(theData);
-       for_each(&tSignal, ptr);
-
-       /**
-	* Reply
-	*/
-       {
-	 Uint32* send= tSignal.getDataPtrSend();
-	 memcpy(send, theData, tSignal.getLength() << 2);
-	 ((SubGcpCompleteAck*)send)->rep.senderRef = 
-	   numberToRef(API_CLUSTERMGR, theOwnId);
-	 Uint32 ref= header->theSendersBlockRef;
-	 Uint32 aNodeId= refToNode(ref);
-	 tSignal.theReceiversBlockNumber= refToBlock(ref);
-	 tSignal.theVerId_signalNumber= GSN_SUB_GCP_COMPLETE_ACK;
-	 sendSignalUnCond(&tSignal, aNodeId);
-       }
-       break;
-     }
-     case GSN_TAKE_OVERTCCONF:
-     {
-       /**
-	* Report
-	*/
-       NdbApiSignal tSignal(* header);
-       tSignal.setDataPtr(theData);
-       for_each(&tSignal, ptr);
-       return;
-     }
-     default:
-       break;
-       
-     }
-     return;
-  } else if (tRecBlockNo >= MIN_API_FIXED_BLOCK_NO &&
-             tRecBlockNo <= MAX_API_FIXED_BLOCK_NO) {
+  }
+  else if (tRecBlockNo >= MIN_API_FIXED_BLOCK_NO &&
+           tRecBlockNo <= MAX_API_FIXED_BLOCK_NO) 
+  {
     Uint32 dynamic= m_fixed2dynamic[tRecBlockNo - MIN_API_FIXED_BLOCK_NO];
-    oe = m_threads.get(dynamic);
-    if (oe.m_object != 0 && oe.m_executeFunction != 0) {
+    trp_client * clnt = m_threads.get(dynamic);
+    if (clnt != 0)
+    {
       NdbApiSignal tmpSignal(*header);
       NdbApiSignal * tSignal = &tmpSignal;
       tSignal->setDataPtr(theData);
-      (* oe.m_executeFunction) (oe.m_object, tSignal, ptr);
+      clnt->trp_deliver_signal(tSignal, ptr);
     }//if   
-  } else {
-    ; // Ignore all other block numbers.
-    if(header->theVerId_signalNumber != GSN_API_REGREQ) {
+  }
+  else
+  {
+    // Ignore all other block numbers.
+    if(header->theVerId_signalNumber != GSN_API_REGREQ)
+    {
       TRP_DEBUG( "TransporterFacade received signal to unknown block no." );
       ndbout << "BLOCK NO: "  << tRecBlockNo << " sig " 
-	     << header->theVerId_signalNumber  << endl;
+             << header->theVerId_signalNumber  << endl;
       abort();
     }
   }
@@ -439,6 +347,10 @@ TransporterFacade::start_instance(NodeId nodeId,
   assert(theOwnId == 0);
   theOwnId = nodeId;
 
+#if defined SIGPIPE && !defined _WIN32
+  (void)signal(SIGPIPE, SIG_IGN);
+#endif
+
   theTransporterRegistry = new TransporterRegistry(this);
   if (theTransporterRegistry == NULL)
     return -1;
@@ -446,7 +358,9 @@ TransporterFacade::start_instance(NodeId nodeId,
   if (!theTransporterRegistry->init(nodeId))
     return -1;
 
-  theClusterMgr = new ClusterMgr(*this);
+  if (theClusterMgr == NULL)
+    theClusterMgr = new ClusterMgr(*this);
+
   if (theClusterMgr == NULL)
     return -1;
 
@@ -470,16 +384,6 @@ TransporterFacade::start_instance(NodeId nodeId,
 
   theClusterMgr->startThread();
 
-  /**
-   * Install signal handler for SIGPIPE
-   *
-   * This due to the fact that a socket connection might have
-   * been closed in between a select and a corresponding send
-   */
-#if !defined NDB_WIN32
-  signal(SIGPIPE, SIG_IGN);
-#endif
-
   return 0;
 }
 
@@ -504,7 +408,6 @@ TransporterFacade::doStop(){
    * and also uses theFacadeInstance to lock/unlock theMutexPtr
    */
   if (theClusterMgr != NULL) theClusterMgr->doStop();
-  if (theArbitMgr != NULL) theArbitMgr->doStop(NULL);
   
   /**
    * Now stop the send and receive threads
@@ -586,10 +489,9 @@ void TransporterFacade::threadMainReceive(void)
     for(int i = 0; i<10; i++){
       NdbSleep_MilliSleep(10);
       NdbMutex_Lock(theMutexPtr);
-      if (poll_owner == NULL) {
-        const int res = theTransporterRegistry->pollReceive(0);
-        if(res > 0)
-          theTransporterRegistry->performReceive();
+      if (m_poll_owner == NULL)
+      {
+        external_poll(0);
       }
       NdbMutex_Unlock(theMutexPtr);
     }
@@ -608,139 +510,46 @@ void TransporterFacade::threadMainReceive(void)
 void TransporterFacade::external_poll(Uint32 wait_time)
 {
   NdbMutex_Unlock(theMutexPtr);
+
+#ifdef NDB_SHM_TRANSPORTER
+  /*
+    If shared memory transporters are used we need to set our sigmask
+    such that we wake up also on interrupts on the shared memory
+    interrupt signal.
+  */
+  NdbThread_set_shm_sigmask(FALSE);
+#endif
+
   const int res = theTransporterRegistry->pollReceive(wait_time);
+
+#ifdef NDB_SHM_TRANSPORTER
+  NdbThread_set_shm_sigmask(TRUE);
+#endif
+
   NdbMutex_Lock(theMutexPtr);
-  if (res > 0) {
+  if (res > 0)
+  {
     theTransporterRegistry->performReceive();
   }
 }
 
-/*
-  This Ndb object didn't get hold of the poll "right" and will wait on a
-  conditional mutex wait instead. It is put into the conditional wait
-  queue so that it is accessible to take over the poll "right" if needed.
-  The method gets a free entry in the free list and puts it first in the
-  doubly linked list. Finally it assigns the ndb object reference to the
-  entry.
-*/
-Uint32 TransporterFacade::put_in_cond_wait_queue(NdbWaiter *aWaiter)
-{
-  /*
-   Get first free entry
-  */
-  Uint32 index = first_free_cond_wait;
-  assert(index < MAX_NO_THREADS);
-  first_free_cond_wait = cond_wait_array[index].next_cond_wait;
-
-  /*
-   Put in doubly linked list
-  */
-  cond_wait_array[index].next_cond_wait = MAX_NO_THREADS;
-  cond_wait_array[index].prev_cond_wait = last_in_cond_wait;
-  if (last_in_cond_wait == MAX_NO_THREADS) {
-    first_in_cond_wait = index;
-  } else
-    cond_wait_array[last_in_cond_wait].next_cond_wait = index;
-  last_in_cond_wait = index;
-
-  cond_wait_array[index].cond_wait_object = aWaiter;
-  aWaiter->set_cond_wait_index(index);
-  return index;
-}
-
-/*
-  Somebody is about to signal the thread to wake it up, it could also
-  be that it woke up on a timeout and found himself still in the list.
-  Removes the entry from the doubly linked list.
-  Inserts the entry into the free list.
-  NULLifies the ndb object reference entry and sets the index in the
-  Ndb object to NIL (=MAX_NO_THREADS)
-*/
-void TransporterFacade::remove_from_cond_wait_queue(NdbWaiter *aWaiter)
-{
-  Uint32 index = aWaiter->get_cond_wait_index();
-  assert(index < MAX_NO_THREADS &&
-         cond_wait_array[index].cond_wait_object == aWaiter);
-  /*
-   Remove from doubly linked list
-  */
-  Uint32 prev_elem, next_elem;
-  prev_elem = cond_wait_array[index].prev_cond_wait;
-  next_elem = cond_wait_array[index].next_cond_wait;
-  if (prev_elem != MAX_NO_THREADS)
-    cond_wait_array[prev_elem].next_cond_wait = next_elem;
-  else
-    first_in_cond_wait = next_elem;
-  if (next_elem != MAX_NO_THREADS)
-    cond_wait_array[next_elem].prev_cond_wait = prev_elem;
-  else
-    last_in_cond_wait = prev_elem;
-  /*
-   Insert into free list
-  */
-  cond_wait_array[index].next_cond_wait = first_free_cond_wait;
-  cond_wait_array[index].prev_cond_wait = MAX_NO_THREADS;
-  first_free_cond_wait = index;
-
-  cond_wait_array[index].cond_wait_object = NULL;
-  aWaiter->set_cond_wait_index(MAX_NO_THREADS);
-}
-
-/*
-  Get the latest Ndb object from the conditional wait queue
-  and also remove it from the list.
-*/
-NdbWaiter* TransporterFacade::rem_last_from_cond_wait_queue()
-{
-  NdbWaiter *tWaiter;
-  Uint32 index = last_in_cond_wait;
-  if (last_in_cond_wait == MAX_NO_THREADS)
-    return NULL;
-  tWaiter = cond_wait_array[index].cond_wait_object;
-  remove_from_cond_wait_queue(tWaiter);
-  return tWaiter;
-}
-
-void TransporterFacade::init_cond_wait_queue()
-{
-  Uint32 i;
-  /*
-   Initialise the doubly linked list as empty
-  */
-  first_in_cond_wait = MAX_NO_THREADS;
-  last_in_cond_wait = MAX_NO_THREADS;
-  /*
-   Initialise free list
-  */
-  first_free_cond_wait = 0;
-  for (i = 0; i < MAX_NO_THREADS; i++) {
-    cond_wait_array[i].cond_wait_object = NULL;
-    cond_wait_array[i].next_cond_wait = i+1;
-    cond_wait_array[i].prev_cond_wait = MAX_NO_THREADS;
-  }
-}
-
 TransporterFacade::TransporterFacade(GlobalDictCache *cache) :
+  m_poll_owner(NULL),
+  m_poll_queue_head(NULL),
+  m_poll_queue_tail(NULL),
   theTransporterRegistry(0),
   theOwnId(0),
   theStartNodeId(1),
   theClusterMgr(NULL),
-  theArbitMgr(NULL),
   checkCounter(4),
   currentSendLimit(1),
-  m_scan_batch_size(MAX_SCAN_BATCH_SIZE),
-  m_batch_byte_size(SCAN_BATCH_SIZE),
-  m_batch_size(DEF_BATCH_SIZE),
   theStopReceive(0),
   theSendThread(NULL),
   theReceiveThread(NULL),
-  m_max_trans_id(0),
   m_fragmented_signal_id(0),
   m_globalDictCache(cache)
 {
   DBUG_ENTER("TransporterFacade::TransporterFacade");
-  init_cond_wait_queue();
-  poll_owner = NULL;
   theMutexPtr = NdbMutex_Create();
   sendPerformedLastInterval = 0;
 
@@ -750,6 +559,9 @@ TransporterFacade::TransporterFacade(GlobalDictCache *cache) :
 #ifdef API_TRACE
   apiSignalLog = 0;
 #endif
+
+  theClusterMgr = new ClusterMgr(*this);
+
   DBUG_VOID_RETURN;
 }
 
@@ -816,7 +628,7 @@ TransporterFacade::configure(NodeId nodeId,
     DBUG_RETURN(false);
 
   // Configure cluster manager
-  theClusterMgr->configure(conf);
+  theClusterMgr->configure(nodeId, conf);
 
   ndb_mgm_configuration_iterator iter(* conf, CFG_SECTION_NODE);
   if(iter.find(CFG_NODE_ID, nodeId))
@@ -827,45 +639,15 @@ TransporterFacade::configure(NodeId nodeId,
   iter.get(CFG_TOTAL_SEND_BUFFER_MEMORY, &total_send_buffer);
   theTransporterRegistry->allocate_send_buffers(total_send_buffer);
 
-  // Configure arbitrator
-  Uint32 rank = 0;
-  iter.get(CFG_NODE_ARBIT_RANK, &rank);
-  if (rank > 0)
-  {
-    // The arbitrator should be active
-    if (!theArbitMgr)
-      theArbitMgr = new ArbitMgr(* this);
-    theArbitMgr->setRank(rank);
-
-    Uint32 delay = 0;
-    iter.get(CFG_NODE_ARBIT_DELAY, &delay);
-    theArbitMgr->setDelay(delay);
-  }
-  else if (theArbitMgr)
-  {
-    // No arbitrator should be started
-    theArbitMgr->doStop(NULL);
-    delete theArbitMgr;
-    theArbitMgr= NULL;
-  }
-
-  // Configure scan settings
-  Uint32 scan_batch_size= 0;
-  if (!iter.get(CFG_MAX_SCAN_BATCH_SIZE, &scan_batch_size)) {
-    m_scan_batch_size= scan_batch_size;
-  }
-  Uint32 batch_byte_size= 0;
-  if (!iter.get(CFG_BATCH_BYTE_SIZE, &batch_byte_size)) {
-    m_batch_byte_size= batch_byte_size;
-  }
-  Uint32 batch_size= 0;
-  if (!iter.get(CFG_BATCH_SIZE, &batch_size)) {
-    m_batch_size= batch_size;
-  }
-
   Uint32 auto_reconnect=1;
   iter.get(CFG_AUTO_RECONNECT, &auto_reconnect);
 
+  const char * priospec = 0;
+  if (iter.get(CFG_HB_THREAD_PRIO, &priospec) == 0)
+  {
+    NdbThread_SetHighPrioProperties(priospec);
+  }
+
   /**
    * Keep value it set before connect (overriding config)
    */
@@ -874,19 +656,6 @@ TransporterFacade::configure(NodeId nodeId,
     theClusterMgr->m_auto_reconnect = auto_reconnect;
   }
   
-  // Configure timeouts
-  Uint32 timeout = 120000;
-  for (iter.first(); iter.valid(); iter.next())
-  {
-    Uint32 tmp1 = 0, tmp2 = 0;
-    iter.get(CFG_DB_TRANSACTION_CHECK_INTERVAL, &tmp1);
-    iter.get(CFG_DB_TRANSACTION_DEADLOCK_TIMEOUT, &tmp2);
-    tmp1 += tmp2;
-    if (tmp1 > timeout)
-      timeout = tmp1;
-  }
-  m_waitfor_timeout = timeout;
-
 #ifdef API_TRACE
   signalLogger.logOn(true, 0, SignalLoggerManager::LogInOut);
 #endif
@@ -899,16 +668,17 @@ TransporterFacade::configure(NodeId nodeId,
 }
 
 void
-TransporterFacade::for_each(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
+TransporterFacade::for_each(trp_client* sender,
+                            const NdbApiSignal* aSignal, 
+                            const LinearSectionPtr ptr[3])
 {
   Uint32 sz = m_threads.m_statusNext.size();
-  TransporterFacade::ThreadData::Object_Execute oe; 
   for (Uint32 i = 0; i < sz ; i ++) 
   {
-    oe = m_threads.m_objectExecute[i];
-    if (m_threads.getInUse(i))
+    trp_client * clnt = m_threads.m_objectExecute[i];
+    if (clnt != 0 && clnt != sender)
     {
-      (* oe.m_executeFunction) (oe.m_object, aSignal, ptr);
+      clnt->trp_deliver_signal(aSignal, ptr);
     }
   }
 }
@@ -918,18 +688,19 @@ TransporterFacade::connected()
 {
   DBUG_ENTER("TransporterFacade::connected");
   Uint32 sz = m_threads.m_statusNext.size();
-  for (Uint32 i = 0; i < sz ; i ++) {
-    if (m_threads.getInUse(i)){
-      void * obj = m_threads.m_objectExecute[i].m_object;
-      NodeStatusFunction RegPC = m_threads.m_statusFunction[i];
-      (*RegPC) (obj, numberToRef(indexToNumber(i), theOwnId), true, true);
+  for (Uint32 i = 0; i < sz ; i ++)
+  {
+    trp_client * clnt = m_threads.m_objectExecute[i];
+    if (clnt != 0)
+    {
+      clnt->trp_node_status(numberToRef(indexToNumber(i), theOwnId), NS_CONNECTED);
     }
   }
   DBUG_VOID_RETURN;
 }
 
 void
-TransporterFacade::ReportNodeDead(NodeId tNodeId)
+TransporterFacade::trp_node_status(NodeId tNodeId, Uint32 event)
 {
   DBUG_ENTER("TransporterFacade::ReportNodeDead");
   DBUG_PRINT("enter",("nodeid= %d", tNodeId));
@@ -942,104 +713,64 @@ TransporterFacade::ReportNodeDead(NodeId tNodeId)
    * might not have noticed the failure.
    */
   Uint32 sz = m_threads.m_statusNext.size();
-  for (Uint32 i = 0; i < sz ; i ++) {
-    if (m_threads.getInUse(i)){
-      void * obj = m_threads.m_objectExecute[i].m_object;
-      NodeStatusFunction RegPC = m_threads.m_statusFunction[i];
-      (*RegPC) (obj, tNodeId, false, false);
+  for (Uint32 i = 0; i < sz ; i ++)
+  {
+    trp_client * clnt = m_threads.m_objectExecute[i];
+    if (clnt != 0)
+    {
+      clnt->trp_node_status(tNodeId, event);
     }
   }
   DBUG_VOID_RETURN;
 }
 
-void
-TransporterFacade::ReportNodeFailureComplete(NodeId tNodeId)
-{
-  /**
-   * When a node fails we must report this to each Ndb object. 
-   * The function that is used for communicating node failures is called.
-   * This is to ensure that the Ndb objects do not think their connections 
-   * are correct after a failure followed by a restart. 
-   * After the restart the node is up again and the Ndb object 
-   * might not have noticed the failure.
-   */
-
-  DBUG_ENTER("TransporterFacade::ReportNodeFailureComplete");
-  DBUG_PRINT("enter",("nodeid= %d", tNodeId));
-  Uint32 sz = m_threads.m_statusNext.size();
-  for (Uint32 i = 0; i < sz ; i ++) {
-    if (m_threads.getInUse(i)){
-      void * obj = m_threads.m_objectExecute[i].m_object;
-      NodeStatusFunction RegPC = m_threads.m_statusFunction[i];
-      (*RegPC) (obj, tNodeId, false, true);
-    }
-  }
-  DBUG_VOID_RETURN;
-}
-
-void
-TransporterFacade::ReportNodeAlive(NodeId tNodeId)
-{
-  /**
-   * When a node fails we must report this to each Ndb object. 
-   * The function that is used for communicating node failures is called.
-   * This is to ensure that the Ndb objects do not think there connections 
-   * are correct after a failure
-   * followed by a restart. 
-   * After the restart the node is up again and the Ndb object 
-   * might not have noticed the failure.
-   */
-  Uint32 sz = m_threads.m_statusNext.size();
-  for (Uint32 i = 0; i < sz ; i ++) {
-    if (m_threads.getInUse(i)){
-      void * obj = m_threads.m_objectExecute[i].m_object;
-      NodeStatusFunction RegPC = m_threads.m_statusFunction[i];
-      (*RegPC) (obj, tNodeId, true, false);
-    }
-  }
-}
-
 int 
-TransporterFacade::close(BlockNumber blockNumber, Uint64 trans_id)
+TransporterFacade::close_clnt(trp_client* clnt)
 {
-  NdbMutex_Lock(theMutexPtr);
-  Uint32 low_bits = (Uint32)trans_id;
-  m_max_trans_id = m_max_trans_id > low_bits ? m_max_trans_id : low_bits;
-  close_local(blockNumber);
-  NdbMutex_Unlock(theMutexPtr);
-  return 0;
-}
-
-int 
-TransporterFacade::close_local(BlockNumber blockNumber){
-  m_threads.close(blockNumber);
-  return 0;
+  int ret = -1;
+  if (clnt)
+  {
+    NdbMutex_Lock(theMutexPtr);
+    if (m_threads.get(clnt->m_blockNo) == clnt)
+    {
+      m_threads.close(clnt->m_blockNo);
+      ret = 0;
+    }
+    else
+    {
+      assert(0);
+    }
+    NdbMutex_Unlock(theMutexPtr);
+  }
+  return ret;
 }
 
 int
-TransporterFacade::open(void* objRef, 
-                        ExecuteFunction fun, 
-                        NodeStatusFunction statusFun,
-                        int blockNo)
+TransporterFacade::open_clnt(trp_client * clnt, int blockNo)
 {
   DBUG_ENTER("TransporterFacade::open");
-  int r= m_threads.open(objRef, fun, statusFun);
+  Guard g(theMutexPtr);
+  int r= m_threads.open(clnt);
   if (r < 0)
+  {
     DBUG_RETURN(r);
+  }
 
-  if (unlikely(blockNo != -1)){
+  if (unlikely(blockNo != -1))
+  {
     // Using fixed block number, add fixed->dymamic mapping
-    Uint32 fixed_index= blockNo - MIN_API_FIXED_BLOCK_NO;
-
+    Uint32 fixed_index = blockNo - MIN_API_FIXED_BLOCK_NO;
+    
     assert(blockNo >= MIN_API_FIXED_BLOCK_NO &&
            fixed_index <= NO_API_FIXED_BLOCKS);
-
+    
     m_fixed2dynamic[fixed_index]= r;
   }
 
 #if 1
-  if (theOwnId > 0) {
-    (*statusFun)(objRef, numberToRef(r, theOwnId), true, true);
+  if (theOwnId > 0)
+  {
+    clnt->trp_node_status(numberToRef(r, theOwnId), NS_CONNECTED);
   }
 #endif
   DBUG_RETURN(r);
@@ -1049,9 +780,8 @@ TransporterFacade::~TransporterFacade()
 {  
   DBUG_ENTER("TransporterFacade::~TransporterFacade");
 
-  NdbMutex_Lock(theMutexPtr);
   delete theClusterMgr;  
-  delete theArbitMgr;
+  NdbMutex_Lock(theMutexPtr);
   delete theTransporterRegistry;
   NdbMutex_Unlock(theMutexPtr);
   NdbMutex_Destroy(theMutexPtr);
@@ -1773,12 +1503,10 @@ TransporterFacade::ThreadData::ThreadData(Uint32 size){
 
 void
 TransporterFacade::ThreadData::expand(Uint32 size){
-  Object_Execute oe = { 0 ,0 };
-  NodeStatusFunction fun = 0;
+  trp_client * oe = 0;
 
   const Uint32 sz = m_statusNext.size();
   m_objectExecute.fill(sz + size, oe);
-  m_statusFunction.fill(sz + size, fun);
   for(Uint32 i = 0; i<size; i++){
     m_statusNext.push_back(sz + i + 1);
   }
@@ -1789,9 +1517,7 @@ TransporterFacade::ThreadData::expand(Uint32 size){
 
 
 int
-TransporterFacade::ThreadData::open(void* objRef, 
-				    ExecuteFunction fun, 
-				    NodeStatusFunction fun2)
+TransporterFacade::ThreadData::open(trp_client * clnt)
 {
   Uint32 nextFree = m_firstFree;
 
@@ -1807,11 +1533,8 @@ TransporterFacade::ThreadData::open(void* objRef,
   m_use_cnt++;
   m_firstFree = m_statusNext[nextFree];
 
-  Object_Execute oe = { objRef , fun };
-
   m_statusNext[nextFree] = INACTIVE;
-  m_objectExecute[nextFree] = oe;
-  m_statusFunction[nextFree] = fun2;
+  m_objectExecute[nextFree] = clnt;
 
   return indexToNumber(nextFree);
 }
@@ -1819,14 +1542,12 @@ TransporterFacade::ThreadData::open(void* objRef,
 int
 TransporterFacade::ThreadData::close(int number){
   number= numberToIndex(number);
-  assert(getInUse(number));
+  assert(m_objectExecute[number] != 0);
   m_statusNext[number] = m_firstFree;
   assert(m_use_cnt);
   m_use_cnt--;
   m_firstFree = number;
-  Object_Execute oe = { 0, 0 };
-  m_objectExecute[number] = oe;
-  m_statusFunction[number] = 0;
+  m_objectExecute[number] = 0;
   return 0;
 }
 
@@ -1836,103 +1557,20 @@ TransporterFacade::get_active_ndb_objects() const
   return m_threads.m_use_cnt;
 }
 
-PollGuard::PollGuard(TransporterFacade *tp, NdbWaiter *aWaiter,
-                     Uint32 block_no)
+
+void
+TransporterFacade::start_poll(trp_client* clnt)
 {
-  m_tp= tp;
-  m_waiter= aWaiter;
-  m_locked= true;
-  m_block_no= block_no;
-  tp->lock_mutex();
+  lock_mutex();
+  clnt->m_poll.m_locked = true;
 }
 
-/*
-  This is a common routine for possibly forcing the send of buffered signals
-  and receiving response the thread is waiting for. It is designed to be
-  useful from:
-  1) PK, UK lookups using the asynchronous interface
-     This routine uses the wait_for_input routine instead since it has
-     special end conditions due to the asynchronous nature of its usage.
-  2) Scans
-  3) dictSignal
-  It uses a NdbWaiter object to wait on the events and this object is
-  linked into the conditional wait queue. Thus this object contains
-  a reference to its place in the queue.
-
-  It replaces the method receiveResponse previously used on the Ndb object
-*/
-int PollGuard::wait_n_unlock(int wait_time, NodeId nodeId, Uint32 state,
-                             bool forceSend)
+void
+TransporterFacade::do_poll(trp_client* clnt, Uint32 wait_time)
 {
-  int ret_val;
-  m_waiter->set_node(nodeId);
-  m_waiter->set_state(state);
-  ret_val= wait_for_input_in_loop(wait_time, forceSend);
-  unlock_and_signal();
-  return ret_val;
-}
-
-int PollGuard::wait_scan(int wait_time, NodeId nodeId, bool forceSend)
-{
-  m_waiter->set_node(nodeId);
-  m_waiter->set_state(WAIT_SCAN);
-  return wait_for_input_in_loop(wait_time, forceSend);
-}
-
-int PollGuard::wait_for_input_in_loop(int wait_time, bool forceSend)
-{
-  int ret_val;
-  if (forceSend)
-    m_tp->forceSend(m_block_no);
-  else
-    m_tp->checkForceSend(m_block_no);
-
-  NDB_TICKS curr_time = NdbTick_CurrentMillisecond();
-  NDB_TICKS max_time = curr_time + (NDB_TICKS)wait_time;
-  const int maxsleep = (wait_time == -1 || wait_time > 10) ? 10 : wait_time;
-  do
-  {
-    wait_for_input(maxsleep);
-    Uint32 state= m_waiter->get_state();
-    if (state == NO_WAIT)
-    {
-      return 0;
-    }
-    else if (state == WAIT_NODE_FAILURE)
-    {
-      ret_val= -2;
-      break;
-    }
-    if (wait_time == -1)
-    {
-#ifdef NOT_USED
-      ndbout << "Waited WAITFOR_RESPONSE_TIMEOUT, continuing wait" << endl;
-#endif
-      continue;
-    }
-    wait_time= int(max_time - NdbTick_CurrentMillisecond());
-    if (wait_time <= 0)
-    {
-#ifdef VM_TRACE
-      ndbout << "Time-out state is " << m_waiter->get_state() << endl;
-#endif
-      m_waiter->set_state(WST_WAIT_TIMEOUT);
-      ret_val= -1;
-      break;
-    }
-  } while (1);
-#ifdef VM_TRACE
-  ndbout << "ERR: receiveResponse - theImpl->theWaiter.m_state = ";
-  ndbout << m_waiter->get_state() << endl;
-#endif
-  m_waiter->set_state(NO_WAIT);
-  return ret_val;
-}
-
-void PollGuard::wait_for_input(int wait_time)
-{
-  NdbWaiter *t_poll_owner= m_tp->get_poll_owner();
-  if (t_poll_owner != NULL && t_poll_owner != m_waiter)
+  assert(clnt->m_poll.m_locked == true);
+  trp_client* owner = m_poll_owner;
+  if (owner != NULL && owner != clnt)
   {
     /*
       We didn't get hold of the poll "right". We will sleep on a
@@ -1943,11 +1581,11 @@ void PollGuard::wait_for_input(int wait_time)
       queue if it hasn't happened already. It is usually already out of the
       queue but at time-out it could be that the object is still there.
     */
-    (void) m_tp->put_in_cond_wait_queue(m_waiter);
-    m_waiter->wait(wait_time);
-    if (m_waiter->get_cond_wait_index() != TransporterFacade::MAX_NO_THREADS)
+    add_to_poll_queue(clnt);
+    clnt->cond_wait(wait_time, theMutexPtr); // release/reacquire mutex
+    if (clnt != m_poll_owner)
     {
-      m_tp->remove_from_cond_wait_queue(m_waiter);
+      remove_from_poll_queue(clnt);
     }
   }
   else
@@ -1957,25 +1595,22 @@ void PollGuard::wait_for_input(int wait_time)
       receiving data we will check if all data is received, if not we
       poll again.
     */
-#ifdef NDB_SHM_TRANSPORTER
-    /*
-      If shared memory transporters are used we need to set our sigmask
-      such that we wake up also on interrupts on the shared memory
-      interrupt signal.
-    */
-    NdbThread_set_shm_sigmask(FALSE);
-#endif
-    m_tp->set_poll_owner(m_waiter);
-    m_waiter->set_poll_owner(true);
-    m_tp->external_poll((Uint32)wait_time);
+    assert(owner == clnt || clnt->m_poll.m_poll_owner == false);
+    m_poll_owner = clnt;
+    clnt->m_poll.m_poll_owner = true;
+    external_poll(wait_time);
   }
 }
 
-void PollGuard::unlock_and_signal()
+void
+TransporterFacade::complete_poll(trp_client* clnt)
 {
-  NdbWaiter *t_signal_cond_waiter= 0;
-  if (!m_locked)
+  if (!clnt->m_poll.m_locked)
+  {
+    assert(clnt->m_poll.m_poll_owner == false);
     return;
+  }
+
   /*
    When completing the poll for this thread we must return the poll
    ownership if we own it. We will give it to the last thread that
@@ -1988,29 +1623,95 @@ void PollGuard::unlock_and_signal()
    API Volume 1 Third Edition on page 703-704 for a discussion on this
    subject.
   */
-  if (m_tp->get_poll_owner() == m_waiter)
+  trp_client* new_owner = 0;
+  if (m_poll_owner == clnt)
   {
-#ifdef NDB_SHM_TRANSPORTER
-    /*
-      If shared memory transporters are used we need to reset our sigmask
-      since we are no longer the thread to receive interrupts.
-    */
-    NdbThread_set_shm_sigmask(TRUE);
-#endif
-    m_waiter->set_poll_owner(false);
-    t_signal_cond_waiter= m_tp->rem_last_from_cond_wait_queue();
-    m_tp->set_poll_owner(t_signal_cond_waiter);
-    if (t_signal_cond_waiter)
-      t_signal_cond_waiter->set_poll_owner(true);
+    assert(clnt->m_poll.m_poll_owner == true);
+    m_poll_owner = new_owner = remove_last_from_poll_queue();
   }
-  if (t_signal_cond_waiter)
-    t_signal_cond_waiter->cond_signal();
-  m_tp->unlock_mutex();
-  m_locked=false;
+  if (new_owner)
+  {
+    assert(new_owner->m_poll.m_poll_owner == false);
+    new_owner->cond_signal();
+    new_owner->m_poll.m_poll_owner = true;
+  }
+  clnt->m_poll.m_locked = false;
+  clnt->m_poll.m_poll_owner = false;
+  unlock_mutex();
 }
 
-template class Vector<NodeStatusFunction>;
-template class Vector<TransporterFacade::ThreadData::Object_Execute>;
+void
+TransporterFacade::add_to_poll_queue(trp_client* clnt)
+{
+  assert(clnt != 0);
+  assert(clnt->m_poll.m_prev == 0);
+  assert(clnt->m_poll.m_next == 0);
+  assert(clnt->m_poll.m_locked == true);
+  assert(clnt->m_poll.m_poll_owner == false);
+
+  if (m_poll_queue_head == 0)
+  {
+    assert(m_poll_queue_tail == 0);
+    m_poll_queue_head = clnt;
+    m_poll_queue_tail = clnt;
+  }
+  else
+  {
+    assert(m_poll_queue_tail->m_poll.m_next == 0);
+    m_poll_queue_tail->m_poll.m_next = clnt;
+    clnt->m_poll.m_prev = m_poll_queue_tail;
+    m_poll_queue_tail = clnt;
+  }
+}
+
+void
+TransporterFacade::remove_from_poll_queue(trp_client* clnt)
+{
+  assert(clnt != 0);
+  assert(clnt->m_poll.m_locked == true);
+  assert(clnt->m_poll.m_poll_owner == false);
+
+  if (clnt->m_poll.m_prev != 0)
+  {
+    clnt->m_poll.m_prev->m_poll.m_next = clnt->m_poll.m_next;
+  }
+  else
+  {
+    assert(m_poll_queue_head == clnt);
+    m_poll_queue_head = clnt->m_poll.m_next;
+  }
+
+  if (clnt->m_poll.m_next != 0)
+  {
+    clnt->m_poll.m_next->m_poll.m_prev = clnt->m_poll.m_prev;
+  }
+  else
+  {
+    assert(m_poll_queue_tail == clnt);
+    m_poll_queue_tail = clnt->m_poll.m_prev;
+  }
+
+  if (m_poll_queue_head == 0)
+    assert(m_poll_queue_tail == 0);
+  else if (m_poll_queue_tail == 0)
+    assert(m_poll_queue_head == 0);
+
+  clnt->m_poll.m_prev = 0;
+  clnt->m_poll.m_next = 0;
+}
+
+trp_client*
+TransporterFacade::remove_last_from_poll_queue()
+{
+  trp_client * clnt = m_poll_queue_tail;
+  if (clnt == 0)
+    return 0;
+
+  remove_from_poll_queue(clnt);
+  return clnt;
+}
+
+template class Vector<trp_client*>;
 
 #include "SignalSender.hpp"
 
@@ -2050,7 +1751,7 @@ SignalSender::sendSignal(Uint16 nodeId, const SimpleSignal * s){
 
   if (ss == SEND_OK)
   {
-    assert(getNodeInfo(nodeId).m_api_reg_conf == true ||
+    assert(getNodeInfo(nodeId).is_confirmed() ||
            s->readSignalNumber() == GSN_API_REGREQ);
     theFacade->forceSend(m_blockNo);
   }
