@@ -17,13 +17,11 @@
 */
 
 #include <ndb_global.h>
-#include <my_pthread.h>
-#include <my_sys.h>
 
 #include "ndb_cluster_connection_impl.hpp"
 #include <mgmapi_configuration.hpp>
 #include <mgmapi_config_parameters.h>
-#include <TransporterFacade.hpp>
+#include "TransporterFacade.hpp"
 #include <NdbOut.hpp>
 #include <NdbSleep.h>
 #include <NdbThread.h>
@@ -33,6 +31,8 @@
 #include <mgmapi_debug.h>
 #include <mgmapi_internal.h>
 #include <md5_hash.hpp>
+#include "NdbImpl.hpp"
+#include "NdbDictionaryImpl.hpp"
 
 #include <NdbMutex.h>
 #ifdef VM_TRACE
@@ -47,16 +47,23 @@ static int g_ndb_connection_count = 0;
 /*
  * Ndb_cluster_connection
  */
-
 Ndb_cluster_connection::Ndb_cluster_connection(const char *connect_string)
-  : m_impl(* new Ndb_cluster_connection_impl(connect_string, 0))
+  : m_impl(* new Ndb_cluster_connection_impl(connect_string, 0, 0))
+{
+}
+
+Ndb_cluster_connection::Ndb_cluster_connection(const char *connect_string,
+                                               int force_api_nodeid)
+  : m_impl(* new Ndb_cluster_connection_impl(connect_string, 0,
+                                             force_api_nodeid))
 {
 }
 
 Ndb_cluster_connection::Ndb_cluster_connection(const char *connect_string,
                                                Ndb_cluster_connection *
                                                main_connection)
-  : m_impl(* new Ndb_cluster_connection_impl(connect_string, main_connection))
+  : m_impl(* new Ndb_cluster_connection_impl(connect_string,
+                                             main_connection, 0))
 {
 }
 
@@ -94,7 +101,9 @@ const char *Ndb_cluster_connection::get_connectstring(char *buf,
   return 0;
 }
 
-pthread_handler_t run_ndb_cluster_connection_connect_thread(void *me)
+extern "C"
+void *
+run_ndb_cluster_connection_connect_thread(void *me)
 {
   Ndb_cluster_connection_impl* connection= (Ndb_cluster_connection_impl*) me;
   connection->m_run_connect_thread= 1;
@@ -286,16 +295,17 @@ const char *Ndb_cluster_connection::get_latest_error_msg() const
 
 Ndb_cluster_connection_impl::
 Ndb_cluster_connection_impl(const char * connect_string,
-                            Ndb_cluster_connection *main_connection)
+                            Ndb_cluster_connection *main_connection,
+                            int force_api_nodeid)
   : Ndb_cluster_connection(*this),
     m_main_connection(main_connection),
     m_optimized_node_selection(1),
-    m_name(0),
     m_run_connect_thread(0),
     m_latest_trans_gci(0),
     m_first_ndb_object(0),
     m_latest_error_msg(),
-    m_latest_error(0)
+    m_latest_error(0),
+    m_max_trans_id(0)
 {
   DBUG_ENTER("Ndb_cluster_connection");
   DBUG_PRINT("enter",("Ndb_cluster_connection this=0x%lx", (long) this));
@@ -327,7 +337,8 @@ Ndb_cluster_connection_impl(const char * connect_string,
     ndb_print_state_mutex= NdbMutex_Create();
 #endif
   m_config_retriever=
-    new ConfigRetriever(connect_string, NDB_VERSION, NDB_MGM_NODE_TYPE_API);
+    new ConfigRetriever(connect_string, force_api_nodeid,
+                        NDB_VERSION, NDB_MGM_NODE_TYPE_API);
   if (m_config_retriever->hasError())
   {
     m_latest_error= 1;
@@ -335,13 +346,6 @@ Ndb_cluster_connection_impl(const char * connect_string,
       ("Could not initialize handle to management server: %s",
        m_config_retriever->getErrorString());
     printf("%s\n", get_latest_error_msg());
-    delete m_config_retriever;
-    m_config_retriever= 0;
-  }
-  if (m_name)
-  {
-    NdbMgmHandle h= m_config_retriever->get_mgmHandle();
-    ndb_mgm_set_name(h, m_name);
   }
   if (!m_main_connection)
   {
@@ -354,6 +358,11 @@ Ndb_cluster_connection_impl(const char * connect_string,
     m_globalDictCache = 0;
     m_transporter_facade=
       new TransporterFacade(m_main_connection->m_impl.m_globalDictCache);
+
+    // The secondary connection can't use same nodeid, clear the nodeid
+    // in ConfigRetriver to avoid asking for the same nodeid again
+    m_config_retriever->setNodeId(0);
+
   }
 
   DBUG_VOID_RETURN;
@@ -395,8 +404,6 @@ Ndb_cluster_connection_impl::~Ndb_cluster_connection_impl()
     ndb_print_state_mutex= NULL;
   }
 #endif
-  if (m_name)
-    free(m_name);
 
   NdbMutex_Lock(g_ndb_connection_mutex);
   if(--g_ndb_connection_count == 0)
@@ -449,6 +456,7 @@ Ndb_cluster_connection_impl::link_ndb_object(Ndb* p)
   p->theImpl->m_next_ndb_object = m_first_ndb_object;
   m_first_ndb_object = p;
   
+  p->theFirstTransId += m_max_trans_id;
   unlock_ndb_objects();
 }
 
@@ -477,20 +485,20 @@ Ndb_cluster_connection_impl::unlink_ndb_object(Ndb* p)
   p->theImpl->m_prev_ndb_object = 0;
   p->theImpl->m_next_ndb_object = 0;
 
+  Uint32 transId = (Uint32)p->theFirstTransId;
+  if (transId > m_max_trans_id)
+  {
+    m_max_trans_id = transId;
+  }
+
   unlock_ndb_objects();  
 }
 
 void
 Ndb_cluster_connection_impl::set_name(const char *name)
 {
-  if (m_name)
-    free(m_name);
-  m_name= strdup(name);
-  if (m_config_retriever && m_name)
-  {
-    NdbMgmHandle h= m_config_retriever->get_mgmHandle();
-    ndb_mgm_set_name(h, m_name);
-  }
+  NdbMgmHandle h= m_config_retriever->get_mgmHandle();
+  ndb_mgm_set_name(h, name);
 }
 
 int
@@ -592,6 +600,46 @@ Ndb_cluster_connection_impl::init_nodes_vector(Uint32 nodeid,
   DBUG_RETURN(0);
 }
 
+int
+Ndb_cluster_connection_impl::configure(Uint32 nodeId,
+                                       const ndb_mgm_configuration &config)
+{
+  DBUG_ENTER("Ndb_cluster_connection_impl::configure");
+  {
+    ndb_mgm_configuration_iterator iter(config, CFG_SECTION_NODE);
+    if(iter.find(CFG_NODE_ID, nodeId))
+      DBUG_RETURN(-1);
+
+    // Configure scan settings
+    Uint32 scan_batch_size= 0;
+    if (!iter.get(CFG_MAX_SCAN_BATCH_SIZE, &scan_batch_size)) {
+      m_config.m_scan_batch_size= scan_batch_size;
+    }
+    Uint32 batch_byte_size= 0;
+    if (!iter.get(CFG_BATCH_BYTE_SIZE, &batch_byte_size)) {
+      m_config.m_batch_byte_size= batch_byte_size;
+    }
+    Uint32 batch_size= 0;
+    if (!iter.get(CFG_BATCH_SIZE, &batch_size)) {
+      m_config.m_batch_size= batch_size;
+    }
+
+    // Configure timeouts
+    Uint32 timeout = 120000;
+    for (iter.first(); iter.valid(); iter.next())
+    {
+      Uint32 tmp1 = 0, tmp2 = 0;
+      iter.get(CFG_DB_TRANSACTION_CHECK_INTERVAL, &tmp1);
+      iter.get(CFG_DB_TRANSACTION_DEADLOCK_TIMEOUT, &tmp2);
+      tmp1 += tmp2;
+      if (tmp1 > timeout)
+        timeout = tmp1;
+    }
+    m_config.m_waitfor_timeout = timeout;
+  }
+  DBUG_RETURN(init_nodes_vector(nodeId, config));
+}
+
 void
 Ndb_cluster_connection_impl::do_test()
 {
@@ -686,19 +734,19 @@ int Ndb_cluster_connection_impl::connect(int no_retries,
     if(props == 0)
       break;
 
-    if (m_transporter_facade->start_instance(nodeId, props) < 0)
-    {
-      ndb_mgm_destroy_configuration(props);
-      DBUG_RETURN(-1);
-    }
-
-    if (init_nodes_vector(nodeId, *props))
+    if (configure(nodeId, *props))
     {
       ndb_mgm_destroy_configuration(props);
       DBUG_PRINT("exit", ("malloc failure, ret: -1"));
       DBUG_RETURN(-1);
     }
 
+    if (m_transporter_facade->start_instance(nodeId, props) < 0)
+    {
+      ndb_mgm_destroy_configuration(props);
+      DBUG_RETURN(-1);
+    }
+
     ndb_mgm_destroy_configuration(props);
     m_transporter_facade->connected();
     m_latest_error = 0;
