@@ -16,15 +16,20 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#include "NdbImpl.hpp"
+#ifndef NDB_INTERNAL_HPP
+#define NDB_INTERNAL_HPP
 
+/**
+ * This class exposes non-public funcionality to various test/utility programs
+ */
 class Ndb_internal
 {
-private:
-  friend class NdbEventBuffer;
-  friend class BackupRestore;
+public:
   Ndb_internal() {}
   virtual ~Ndb_internal() {}
-  static int send_event_report(bool has_lock, Ndb *ndb, Uint32 *data, Uint32 length)
-    { return ndb->theImpl->send_event_report(has_lock, data, length); }
+
+  static int send_event_report(bool has_lock, Ndb *ndb, Uint32*data,Uint32 len);
+  static void setForceShortRequests(Ndb*, bool val);
 };
+
+#endif
