@@ -168,7 +168,7 @@ ErrorBundle ErrorCodes[] = {
    * TemporaryResourceError
    */
   { 217,  DMEC, TR, "217" },
-  { 218,  DMEC, TR, "218" },
+  { 218,  DMEC, TR, "Out of LongMessageBuffer" },
   { 219,  DMEC, TR, "219" },
   { 233,  DMEC, TR,
     "Out of operation records in transaction coordinator (increase MaxNoOfConcurrentOperations)" },
@@ -179,6 +179,7 @@ ErrorBundle ErrorCodes[] = {
   { 419,  DMEC, TR, "419" },
   { 245,  DMEC, TR, "Too many active scans" },
   { 488,  DMEC, TR, "Too many active scans" },
+  { 489,  DMEC, TR, "Too many active scans" },
   { 490,  DMEC, TR, "Too many active scans" },
   { 805,  DMEC, TR, "Out of attrinfo records in tuple manager" },
   { 830,  DMEC, TR, "Out of add fragment operation records" },
@@ -205,6 +206,7 @@ ErrorBundle ErrorCodes[] = {
   { 623,  HA_ERR_RECORD_FILE_FULL, IS, "623" },
   { 624,  HA_ERR_RECORD_FILE_FULL, IS, "624" },
   { 625,  HA_ERR_INDEX_FILE_FULL, IS, "Out of memory in Ndb Kernel, hash index part (increase IndexMemory)" },
+  { 633,  HA_ERR_INDEX_FILE_FULL, IS, "Table fragment hash index has reached maximum possible size" },
   { 640,  DMEC, IS, "Too many hash indexes (should not happen)" },
   { 826,  HA_ERR_RECORD_FILE_FULL, IS, "Too many tables and attributes (increase MaxNoOfAttributes or MaxNoOfTables)" },
   { 827,  HA_ERR_RECORD_FILE_FULL, IS, "Out of memory in Ndb Kernel, table data (increase DataMemory)" },
@@ -301,6 +303,10 @@ ErrorBundle ErrorCodes[] = {
   { 4349, DMEC, IE, "Inconsistency detected at index usage" },
   { 4350, DMEC, IE, "Transaction already aborted" },
   { 4351, DMEC, TO, "Timeout/deadlock during index build" },
+  { 294,  DMEC, IE, "Unlocked operation has out of range index" },
+  { 295,  DMEC, IE, "Unlocked operation has invalid state" },
+  { 298,  DMEC, IE, "Invalid distribution key" },
+  { 416,  DMEC, IE, "Bad state handling unlock request" },
 
   /**
    * Application error
@@ -311,6 +317,7 @@ ErrorBundle ErrorCodes[] = {
   { 823,  DMEC, AE, "Too much attrinfo from application in tuple manager" },
   { 829,  DMEC, AE, "Corrupt data received for insert/update" },
   { 831,  DMEC, AE, "Too many nullable/bitfields in table definition" },
+  { 850,  DMEC, AE, "Too long or too short default value"},
   { 876,  DMEC, AE, "876" },
   { 877,  DMEC, AE, "877" },
   { 878,  DMEC, AE, "878" },
@@ -321,6 +328,7 @@ ErrorBundle ErrorCodes[] = {
   { 886,  DMEC, AE, "More than 65535 instructions executed in interpreter" },
   { 897,  DMEC, AE, "Update attempt of primary key via ndbcluster internal api (if this occurs via the MySQL server it is a bug, please report)" },
   { 892,  DMEC, AE, "Unsupported type in scan filter" },
+  { 1233, DMEC, AE, "Table read-only" },
   { 4256, DMEC, AE, "Must call Ndb::init() before this function" },
   { 4257, DMEC, AE, "Tried to read too much - too many getValue calls" },
   { 320,  DMEC, AE, "Invalid no of nodes specified for new nodegroup" },
@@ -328,6 +336,7 @@ ErrorBundle ErrorCodes[] = {
   { 322,  DMEC, AE, "Invalid node(s) specified for new nodegroup, node already in nodegroup" },
   { 323,  DMEC, AE, "Invalid nodegroup id, nodegroup already existing" },
   { 324,  DMEC, AE, "Invalid node(s) specified for new nodegroup, no node in nodegroup is started" },
+  { 417,  DMEC, AE, "Bad operation reference - double unlock" },
 
   /** 
    * Scan application errors
@@ -400,7 +409,10 @@ ErrorBundle ErrorCodes[] = {
   { 772,  HA_WRONG_CREATE_OPTION, IE, "Given fragmentType doesn't exist" },
   { 749,  HA_WRONG_CREATE_OPTION, IE, "Primary Table in wrong state" },
   { 779,  HA_WRONG_CREATE_OPTION, SE, "Invalid undo buffer size" },
+  { 791,  HA_WRONG_CREATE_OPTION, SE, "Too many total bits in bitfields" },
+  { 795,  DMEC, IE, "Out of LongMessageBuffer in DICT" },
   { 764,  HA_WRONG_CREATE_OPTION, SE, "Invalid extent size" },
+  { 789,  HA_WRONG_CREATE_OPTION, AE, "Logfile group not found" },
   { 765,  DMEC, SE, "Out of filegroup records" },
   { 750,  IE, SE, "Invalid file type" },
   { 751,  DMEC, SE, "Out of file records" },
@@ -422,6 +434,7 @@ ErrorBundle ErrorCodes[] = {
   { 770,  DMEC, SE, "Cant drop file, file is used" },
   { 774,  DMEC, SE, "Invalid schema object for drop" },
   { 790,  HA_WRONG_CREATE_OPTION, SE, "Invalid hashmap" },
+  { 793,  DMEC, AE, "Object definition too big" },
   { 241,  HA_ERR_TABLE_DEF_CHANGED, SE, "Invalid schema object version" },
   { 283,  HA_ERR_NO_SUCH_TABLE, SE, "Table is being dropped" },
   { 284,  HA_ERR_TABLE_DEF_CHANGED, SE, "Table not defined in transaction coordinator" },
@@ -453,7 +466,7 @@ ErrorBundle ErrorCodes[] = {
   { 1513, DMEC, IE, "Filegroup not online" },
   { 1514, DMEC, SE, "Currently there is a limit of one logfile group" },
   { 1515, DMEC, SE, "Currently there is a 4G limit of one undo/data-file in 32-bit host" },
-  { 1516, DMEC, SE, "File to small" },
+  { 1516, DMEC, SE, "File too small" },
 
   { 773,  DMEC, SE, "Out of string memory, please modify StringMemory config parameter" },
   { 775,  DMEC, SE, "Create file is not supported when Diskless=1" },
@@ -461,6 +474,8 @@ ErrorBundle ErrorCodes[] = {
   { 777,  DMEC, AE, "Cannot create a temporary index on a non-temporary table" },
   { 778,  DMEC, AE, "A temporary table or index must be specified as not logging" },
   { 786,  DMEC, NR, "Schema transaction aborted due to node-failure" },
+  { 792,  DMEC, SE, "Default value for primary key column not supported" },
+  { 794,  DMEC, AE, "Schema feature requires data node upgrade" },
   
   /**
    * FunctionNotImplemented
@@ -581,6 +596,7 @@ ErrorBundle ErrorCodes[] = {
   { 4401, DMEC, AE, "Only one schema operation per schema transaction" },
   { 4402, DMEC, AE, "No schema operation defined before calling execute" },
   { 4410, DMEC, AE, "Schema transaction is already started" },
+  { 4411, DMEC, AE, "Schema transaction not possible until upgrade complete" },
 
   { 4501, DMEC, AE, "Insert in hash table failed when getting table information from Ndb" },
   { 4502, DMEC, AE, "GetValue not allowed in Update operation" },
@@ -620,6 +636,13 @@ ErrorBundle ErrorCodes[] = {
   { 4546, DMEC, AE, "Explicit partitioning info not allowed for table and operation" },
   { 4547, DMEC, AE, "RecordSpecification has overlapping offsets" },
   { 4548, DMEC, AE, "RecordSpecification has too many elements" },
+  { 4549, DMEC, AE, "getLockHandle only supported for primary key read with a lock" },
+  { 4550, DMEC, AE, "Cannot releaseLockHandle until operation executed" },
+  { 4551, DMEC, AE, "NdbLockHandle already released" },
+  { 4552, DMEC, AE, "NdbLockHandle does not belong to transaction" },
+  { 4553, DMEC, AE, "NdbLockHandle original operation not executed successfully" },
+  { 4554, DMEC, AE, "NdbBlob can only be closed from Active state" },
+  { 4555, DMEC, AE, "NdbBlob cannot be closed with pending operations" },
 
   { 4200, DMEC, AE, "Status Error when defining an operation" },
   { 4201, DMEC, AE, "Variable Arrays not yet supported" },
