@@ -106,6 +106,7 @@ static int setAttrNames(void);
 static int setTableNames(void);
 static int readArguments(int, const char**);
 static int createTables(Ndb*);
+static int dropTables(Ndb*);
 static void sleepBeforeStartingTest(int seconds);
 static int checkThreadResults(ThreadNdb *threadArrayP, const char* phase);
 
@@ -318,6 +319,9 @@ NDB_COMMAND(flexHammer, "flexHammer", "flexHammer", "flexHammer", 65535)
     NdbThread_WaitFor(pThreads[i].threadLife, &tmp);
     NdbThread_Destroy(&pThreads[i].threadLife);
   }
+
+  dropTables(pMyNdb);
+
   delete flexHammerErrorData;
   delete [] pThreads;
   delete pMyNdb;
@@ -332,7 +336,7 @@ void*
 flexHammerThread(void* pArg)
 {
   ThreadNdb* pThreadData = (ThreadNdb*)pArg;
-  unsigned int threadNo = pThreadData->threadNo;
+  //unsigned int threadNo = pThreadData->threadNo;
   Ndb* pMyNdb = NULL ;
   NdbConnection *pMyTransaction = NULL ;
   //  NdbOperation*	pMyOperation[MAXTABLES] = {NULL};
@@ -346,11 +350,10 @@ flexHammerThread(void* pArg)
   int count_tables = 0;
   int count_attributes = 0;
   int i = 0;
-  int j = 0;
   int tThreadResult = 0;
   MyOpType tMyOpType = otLast;
   int pkValue = 0;
-  int readValue[MAXATTR][MAXATTRSIZE] = {0};
+  int readValue[MAXATTR][MAXATTRSIZE]; bzero(readValue, sizeof(readValue));
   int attrValue[MAXATTRSIZE];
   NdbRecAttr* tTmp = NULL;
   int tNoOfAttempts = 0;
@@ -830,6 +833,25 @@ createTables(Ndb* pMyNdb)
 
 } // createTables 
 
+static int
+dropTables(Ndb* pMyNdb)
+{
+  int i = 0;
+
+  if (theTableCreateFlag == 0)
+  {
+    for (i = 0; i < tNoOfTables; i++)
+    {
+      ndbout << "Dropping " << tableName[i] << "...";
+      pMyNdb->getDictionary()->dropTable(tableName[i]);
+      ndbout << "done" << endl;
+    }
+  }
+
+  return(0);
+
+} // createTables 
+
 
 static int setAttrNames()
 {
