@@ -24,6 +24,9 @@
 #include <NdbRestarts.hpp>
 #include <Vector.hpp>
 #include <signaldata/DumpStateOrd.hpp>
+#include <NodeBitmask.hpp>
+#include <NdbSqlUtil.hpp>
+#include <BlockNumbers.h>
 
 #define CHECK(b) if (!(b)) { \
   g_err << "ERR: "<< step->getName() \
@@ -31,6 +34,12 @@
   result = NDBT_FAILED; break;\
 } 
 
+#define CHECKRET(b) if (!(b)) { \
+  g_err << "ERR: "<< step->getName() \
+         << " failed on line " << __LINE__ << endl; \
+  return NDBT_FAILED;                             \
+} 
+
 
 struct Attrib {
   bool indexCreated;
@@ -142,6 +151,7 @@ int create_index(NDBT_Context* ctx, int indxNum,
 		 const NdbDictionary::Table* pTab, 
 		 Ndb* pNdb, Attrib* attr, bool logged){
   bool orderedIndex = ctx->getProperty("OrderedIndex", (unsigned)0);
+  bool notOnlyPkId = ctx->getProperty("NotOnlyPkId", (unsigned)0);
   int result  = NDBT_OK;
 
   HugoCalculator calc(*pTab);
@@ -166,12 +176,42 @@ int create_index(NDBT_Context* ctx, int indxNum,
     pIdx.setType(NdbDictionary::Index::OrderedIndex);
   else
     pIdx.setType(NdbDictionary::Index::UniqueHashIndex);
+  
+  bool includesOnlyPkIdCols = true;
   for (int c = 0; c< attr->numAttribs; c++){
     int attrNo = attr->attribs[c];
-    pIdx.addIndexColumn(pTab->getColumn(attrNo)->getName());
-    ndbout << pTab->getColumn(attrNo)->getName()<<" ";
+    const NdbDictionary::Column* col = pTab->getColumn(attrNo);
+    switch(col->getType())
+    {
+    case NDB_TYPE_BIT:
+    case NDB_TYPE_BLOB:
+    case NDB_TYPE_TEXT:
+      /* Not supported */
+      ndbout << col->getName() << " - bad type )" << endl;
+      return SKIP_INDEX;
+    default:
+      break;
+    }
+    if (col->getStorageType() == NDB_STORAGETYPE_DISK)
+    {
+      ndbout << col->getName() << " - disk based )" << endl;
+      return SKIP_INDEX;
+    }
+
+    pIdx.addIndexColumn(col->getName());
+    ndbout << col->getName()<<" ";
+
+    if (! (col->getPrimaryKey() ||
+           calc.isIdCol(attrNo)))
+      includesOnlyPkIdCols = false;
   }
-  
+
+  if (notOnlyPkId && includesOnlyPkIdCols)
+  {
+    ndbout << " Only PK/id cols included - skipping" << endl;
+    return SKIP_INDEX;
+  }
+
   if (!orderedIndex)
   {
     /**
@@ -598,7 +638,7 @@ int runRestarts(NDBT_Context* ctx, NDBT_Step* step){
   int sync_threads = ctx->getProperty("Threads", (unsigned)0);
 
   while(i<loops && result != NDBT_FAILED && !ctx->isTestStopped()){
-    if(restarts.executeRestart("RestartRandomNodeAbort", timeout) != 0){
+    if(restarts.executeRestart(ctx, "RestartRandomNodeAbort", timeout) != 0){
       g_err << "Failed to executeRestart(" <<pCase->getName() <<")" << endl;
       result = NDBT_FAILED;
       break;
@@ -767,6 +807,298 @@ int runInsertDelete(NDBT_Context* ctx, NDBT_Step* step){
   
   return result;
 }
+
+int tryAddUniqueIndex(Ndb* pNdb,
+                      const NdbDictionary::Table* pTab,
+                      const char* idxName,
+                      HugoCalculator& calc,
+                      int& chosenCol)
+{
+  for(int c = 0; c < pTab->getNoOfColumns(); c++)
+  {
+    const NdbDictionary::Column* col = pTab->getColumn(c);
+    
+    if (!col->getPrimaryKey() &&
+        !calc.isUpdateCol(c) &&
+        !col->getNullable() &&
+        col->getStorageType() != NDB_STORAGETYPE_DISK)
+    {
+      chosenCol = c;
+      break;
+    }
+  }
+  
+  if (chosenCol == -1)
+  {
+    return 1;
+  }
+  
+
+  /* Create unique index on chosen column */
+
+  const char* colName = pTab->getColumn(chosenCol)->getName();
+  ndbout << "Creating unique index :" << idxName << " on ("
+         << colName << ")" << endl;
+
+  NdbDictionary::Index idxDef(idxName);
+  idxDef.setTable(pTab->getName());
+  idxDef.setType(NdbDictionary::Index::UniqueHashIndex);
+  
+  idxDef.addIndexColumn(colName);
+  idxDef.setStoredIndex(false);
+
+  if (pNdb->getDictionary()->createIndex(idxDef) != 0)
+  {
+    ndbout << "FAILED!" << endl;
+    const NdbError err = pNdb->getDictionary()->getNdbError();
+    ERR(err);
+    return -1;
+  }
+
+  return 0;
+}
+
+int tryInsertUniqueRecord(NDBT_Step* step,
+                          HugoOperations& hugoOps,
+                          int& recordNum)
+{
+  Ndb* pNdb = GETNDB(step);
+  do
+  {
+    CHECKRET(hugoOps.startTransaction(pNdb) == 0);
+    CHECKRET(hugoOps.pkInsertRecord(pNdb,
+                                    recordNum,
+                                    1,  // NumRecords 
+                                    0)  // UpdatesValue
+             == 0);
+    if (hugoOps.execute_Commit(pNdb) != 0)
+    {
+      NdbError err = hugoOps.getTransaction()->getNdbError();
+      hugoOps.closeTransaction(pNdb);
+      if (err.code == 839)
+      {
+        /* Unique constraint violation, try again with
+         * different record
+         */
+        recordNum++;
+        continue;
+      }
+      else
+      {
+        ERR(err);
+        return NDBT_FAILED;
+      }
+    }
+    
+    hugoOps.closeTransaction(pNdb);
+    break;
+  } while (true);
+
+  return NDBT_OK;
+}
+                     
+
+int runConstraintDetails(NDBT_Context* ctx, NDBT_Step* step)
+{
+  const NdbDictionary::Table* pTab = ctx->getTab();
+  Ndb* pNdb = GETNDB(step);
+
+  /* Steps in testcase
+   * 1) Choose a column to index - not pk or updates column
+   * 2) Insert a couple of unique rows
+   * 3) For a number of different batch sizes :
+   *    i)  Insert a row with a conflicting values
+   *    ii) Update an existing row with a conflicting value
+   *    Verify :
+   *    - The correct error is received
+   *    - The failing constraint is detected
+   *    - The error details string is as expected.
+   */
+  HugoCalculator calc(*pTab);
+
+  /* Choose column to add unique index to */
+
+  int chosenCol = -1;
+  const char* idxName = "constraintCheck";
+  
+  int rc = tryAddUniqueIndex(pNdb, pTab, idxName, calc, chosenCol);
+
+  if (rc)
+  {
+    if (rc == 1)
+    {
+      ndbout << "No suitable column in this table, skipping" << endl;
+      return NDBT_OK;
+    }
+    return NDBT_FAILED;
+  }
+
+  const NdbDictionary::Index* pIdx = 
+    pNdb->getDictionary()->getIndex(idxName, pTab->getName());
+  CHECKRET(pIdx != 0);
+
+
+  /* Now insert a couple of rows */
+
+  HugoOperations hugoOps(*pTab);
+  int firstRecordNum = 0;
+  CHECKRET(tryInsertUniqueRecord(step, hugoOps, firstRecordNum) == NDBT_OK);
+  int secondRecordNum = firstRecordNum + 1;
+  CHECKRET(tryInsertUniqueRecord(step, hugoOps, secondRecordNum) == NDBT_OK);
+
+
+  /* Now we'll attempt to insert/update records 
+   * in various sized batches and check the errors which 
+   * are returned
+   */
+
+  int maxBatchSize = 10;
+  int recordOffset = secondRecordNum + 1;
+  char buff[NDB_MAX_TUPLE_SIZE];
+  Uint32 real_len;
+  CHECKRET(calc.calcValue(firstRecordNum, chosenCol, 0, &buff[0], 
+                          pTab->getColumn(chosenCol)->getSizeInBytes(), 
+                          &real_len) != 0);
+  
+  for (int optype = 0; optype < 2; optype ++)
+  {
+    bool useInsert = (optype == 0);
+    ndbout << "Verifying constraint violation for " 
+           << (useInsert?"Insert":"Update")
+           << " operations" << endl;
+      
+    for (int batchSize = 1; batchSize <= maxBatchSize; batchSize++)
+    {
+      NdbTransaction* trans = pNdb->startTransaction();
+      CHECKRET(trans != 0);
+      
+      for (int rows = 0; rows < batchSize; rows ++)
+      {
+        int rowId = recordOffset + rows;
+        NdbOperation* op = trans->getNdbOperation(pTab);
+        CHECKRET(op != 0);
+        if (useInsert)
+        {
+          CHECKRET(op->insertTuple() == 0);
+          
+          CHECKRET(hugoOps.setValues(op, rowId, 0) == 0);
+          
+          /* Now override setValue for the indexed column to cause
+           * constraint violation
+           */
+          CHECKRET(op->setValue(chosenCol, &buff[0], real_len) == 0);
+        }
+        else
+        {
+          /* Update value of 'second' row to conflict with
+           * first
+           */
+          CHECKRET(op->updateTuple() == 0);
+          CHECKRET(hugoOps.equalForRow(op, secondRecordNum) == 0);
+          
+          CHECKRET(op->setValue(chosenCol, &buff[0], real_len) == 0);
+        }
+      }
+      
+      CHECKRET(trans->execute(Commit) == -1);
+      
+      NdbError err = trans->getNdbError();
+      
+      ERR(err);
+      
+      CHECKRET(err.code == 893);
+      
+      /* Ugliness - current NdbApi puts index schema object id
+       * as abs. value of char* in NdbError struct
+       */
+
+      int idxObjId = (int) ((UintPtr) err.details - UintPtr(0));
+      char detailsBuff[100];
+      const char* errIdxName = NULL;
+      
+      ndbout_c("Got details column val of %p and string of %s\n",
+               err.details, pNdb->getNdbErrorDetail(err, 
+                                                    &detailsBuff[0],
+                                                    100));
+      if (idxObjId == pIdx->getObjectId())
+      {
+        /* Insert / update failed on the constraint we added */
+        errIdxName = pIdx->getName();
+      }
+      else
+      {
+        /* We failed on a different constraint.
+         * Some NDBT tables already have constraints (e.g. I3)
+         * Check that the failing constraint contains our column
+         */
+        NdbDictionary::Dictionary::List tableIndices;
+        
+        CHECKRET(pNdb->getDictionary()->listIndexes(tableIndices,
+                                                    pTab->getName()) == 0);
+        
+        bool ok = false;
+        for (unsigned ind = 0; ind < tableIndices.count; ind ++)
+        {
+          if (tableIndices.elements[ind].id == (unsigned) idxObjId)
+          {
+            const char* otherIdxName = tableIndices.elements[ind].name;
+            ndbout << "Found other violated constraint : " << otherIdxName << endl;
+            const NdbDictionary::Index* otherIndex = 
+              pNdb->getDictionary()->getIndex(otherIdxName,
+                                              pTab->getName());
+            CHECKRET(otherIndex != NULL);
+            
+            for (unsigned col = 0; col < otherIndex->getNoOfColumns(); col++)
+            {
+              if (strcmp(otherIndex->getColumn(col)->getName(),
+                         pTab->getColumn(chosenCol)->getName()) == 0)
+              {
+                /* Found our column in the index */
+                ok = true;
+                errIdxName = otherIndex->getName();
+                break;
+              }
+            }
+            
+            if (ok)
+            {
+              ndbout << "  Constraint contains unique column " << endl;
+              break;
+            }
+            ndbout << "  Constraint does not contain unique col - fail" << endl;
+            CHECKRET(false);
+          }
+        }
+        
+        if (!ok)
+        {
+          ndbout << "Did not find violated constraint" << endl;
+          CHECKRET(false);
+        }
+      }
+
+      /* Finally verify the name returned is :
+       * <db>/<schema>/<table>/<index> 
+       */
+      BaseString expected;
+
+      expected.assfmt("%s/%s/%s/%s",
+                      pNdb->getDatabaseName(),
+                      pNdb->getSchemaName(),
+                      pTab->getName(),
+                      errIdxName);
+
+      CHECKRET(strcmp(expected.c_str(), &detailsBuff[0]) == 0);
+
+      ndbout << " OK " << endl;
+
+      trans->close();
+    }
+  }
+  
+  return NDBT_OK;
+}
+
 int runLoadTable(NDBT_Context* ctx, NDBT_Step* step){
   int records = ctx->getNumRecords();
   
@@ -1750,6 +2082,19 @@ runBug21384(NDBT_Context* ctx, NDBT_Step* step)
   return NDBT_OK;
 }
 
+int
+runReadIndexUntilStopped(NDBT_Context* ctx, NDBT_Step* step)
+{
+  Ndb* pNdb = GETNDB(step);
+  HugoTransactions hugoTrans(*ctx->getTab());
+  int rows = ctx->getNumRecords();
+  while (!ctx->isTestStopped())
+  {
+    hugoTrans.indexReadRecords(pNdb, pkIdxName, rows, 1);
+  }
+  return NDBT_OK;
+}
+
 int 
 runBug25059(NDBT_Context* ctx, NDBT_Step* step)
 {
@@ -2003,6 +2348,379 @@ runBug46069_scandel(NDBT_Context* ctx, NDBT_Step* step)
   return NDBT_OK;
 }
 
+int
+runBug50118(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NdbSleep_MilliSleep(500);
+  int loops = ctx->getNumLoops();
+  while (loops--)
+  {
+    createPkIndex_Drop(ctx, step);
+    createPkIndex(ctx, step);
+  }
+  ctx->stopTest();
+  return NDBT_OK;
+}
+
+int
+runTrigOverload(NDBT_Context* ctx, NDBT_Step* step)
+{
+  /* Test inserts, deletes and updates via
+   * PK with error inserts
+   */
+  Ndb* pNdb = GETNDB(step);
+  HugoOperations hugoOps(*ctx->getTab());
+  NdbRestarter restarter;
+
+  unsigned numScenarios = 3;
+  unsigned errorInserts[3] = {8085, 8086, 0};
+  int results[3] = {218, 218, 0};
+
+  unsigned iterations = 50;
+
+  /* Insert some records */
+  if (hugoOps.startTransaction(pNdb) ||
+      hugoOps.pkInsertRecord(pNdb, 0, iterations) ||
+      hugoOps.execute_Commit(pNdb))
+  {
+    g_err << "Failed on initial insert "
+          << pNdb->getNdbError()
+          << endl;
+    return NDBT_FAILED;
+  }
+  
+  hugoOps.closeTransaction(pNdb);
+
+  for(unsigned i = 0 ; i < iterations; i++)
+  {
+    unsigned scenario = i % numScenarios;
+    unsigned errorVal = errorInserts[ scenario ];
+    g_err << "Iteration :" << i 
+          << " inserting error " << errorVal
+          << " expecting result : " << results[scenario]
+          << endl;
+    restarter.insertErrorInAllNodes(errorVal);
+    //    NdbSleep_MilliSleep(500); // Error insert latency?
+    
+    CHECKRET(hugoOps.startTransaction(pNdb) == 0);
+
+    CHECKRET(hugoOps.pkInsertRecord(pNdb, iterations + i, 1) == 0);
+
+    hugoOps.execute_Commit(pNdb);
+
+    int errorCode = hugoOps.getTransaction()->getNdbError().code;
+    
+    if (errorCode != results[scenario])
+    {
+      g_err << "For Insert in scenario " << scenario 
+            << " expected code " << results[scenario]
+            << " but got " << hugoOps.getTransaction()->getNdbError()
+            << endl;
+      return NDBT_FAILED;
+    }
+    
+    hugoOps.closeTransaction(pNdb);
+    
+    CHECKRET(hugoOps.startTransaction(pNdb) == 0);
+    
+    CHECKRET(hugoOps.pkUpdateRecord(pNdb, i, 1, iterations) == 0);
+    
+    hugoOps.execute_Commit(pNdb);
+
+    errorCode = hugoOps.getTransaction()->getNdbError().code;
+    
+    if (errorCode != results[scenario])
+    {
+      g_err << "For Update in scenario " << scenario 
+            << " expected code " << results[scenario]
+            << " but got " << hugoOps.getTransaction()->getNdbError()
+            << endl;
+      return NDBT_FAILED;
+    }
+    
+    hugoOps.closeTransaction(pNdb);
+
+    CHECKRET(hugoOps.startTransaction(pNdb) == 0);
+    
+    CHECKRET(hugoOps.pkDeleteRecord(pNdb, i, 1) == 0);
+    
+    hugoOps.execute_Commit(pNdb);
+
+    errorCode = hugoOps.getTransaction()->getNdbError().code;
+    
+    if (errorCode != results[scenario])
+    {
+      g_err << "For Delete in scenario " << scenario 
+            << " expected code " << results[scenario]
+            << " but got " << hugoOps.getTransaction()->getNdbError()
+            << endl;
+      return NDBT_FAILED;
+    }
+    
+    hugoOps.closeTransaction(pNdb);
+
+  }
+
+  restarter.insertErrorInAllNodes(0);
+  
+  return NDBT_OK;
+}
+
+int
+runClearError(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NdbRestarter restarter;
+
+  restarter.insertErrorInAllNodes(0);
+
+  return NDBT_OK;
+}
+
+// bug#56829
+
+#undef CHECK2 // previous no good
+#define CHECK2(b, e) \
+  if (!(b)) { \
+    g_err << "ERR: " << #b << " failed at line " << __LINE__ \
+          << ": " << e << endl; \
+    result = NDBT_FAILED; \
+    break; \
+  }
+
+static int
+get_data_memory_pages(NdbMgmHandle h, NdbNodeBitmask dbmask, int* pages_out)
+{
+  int result = NDBT_OK;
+  int pages = 0;
+
+  while (1)
+  {
+    // sends dump 1000 and retrieves all replies
+    ndb_mgm_events* e = 0;
+    CHECK2((e = ndb_mgm_dump_events(h, NDB_LE_MemoryUsage, 0, 0)) != 0, ndb_mgm_get_latest_error_msg(h));
+
+    // sum up pages (also verify sanity)
+    for (int i = 0; i < e->no_of_events; i++)
+    {
+      ndb_logevent* le = &e->events[i];
+      CHECK2(le->type == NDB_LE_MemoryUsage, "bad event type " << le->type);
+      const ndb_logevent_MemoryUsage* lem = &le->MemoryUsage;
+      if (lem->block != DBTUP)
+        continue;
+      int nodeId = le->source_nodeid;
+      CHECK2(dbmask.get(nodeId), "duplicate event from node " << nodeId);
+      dbmask.clear(nodeId);
+      pages += lem->pages_used;
+      g_info << "i:" << i << " node:" << le->source_nodeid << " pages:" << lem->pages_used << endl;
+    }
+    free(e);
+    CHECK2(result == NDBT_OK, "failed");
+
+    char buf[NdbNodeBitmask::TextLength + 1];
+    CHECK2(dbmask.isclear(), "no response from nodes " << dbmask.getText(buf));
+    break;
+  }
+
+  *pages_out = pages;
+  return result;
+}
+
+int
+runBug56829(NDBT_Context* ctx, NDBT_Step* step)
+{
+  Ndb* pNdb = GETNDB(step);
+  NdbDictionary::Dictionary* pDic = pNdb->getDictionary();
+  const int loops = ctx->getNumLoops();
+  int result = NDBT_OK;
+  const NdbDictionary::Table tab(*ctx->getTab());
+  const int rows = ctx->getNumRecords();
+  const char* mgm = 0;//XXX ctx->getRemoteMgm();
+
+  char tabname[100];
+  strcpy(tabname, tab.getName());
+  char indname[100];
+  strcpy(indname, tabname);
+  strcat(indname, "X1");
+
+  (void)pDic->dropTable(tabname);
+
+  NdbMgmHandle h = 0;
+  NdbNodeBitmask dbmask;
+  // entry n marks if row with PK n exists
+  char* rowmask = new char [rows];
+  memset(rowmask, 0, rows);
+  int loop = 0;
+  while (loop < loops)
+  {
+    CHECK2(rows > 0, "rows must be != 0");
+    g_info << "loop " << loop << "<" << loops << endl;
+
+    // at first loop connect to mgm
+    if (loop == 0)
+    {
+      CHECK2((h = ndb_mgm_create_handle()) != 0, "mgm: failed to create handle");
+      CHECK2(ndb_mgm_set_connectstring(h, mgm) == 0, ndb_mgm_get_latest_error_msg(h));
+      CHECK2(ndb_mgm_connect(h, 0, 0, 0) == 0, ndb_mgm_get_latest_error_msg(h));
+      g_info << "mgm: connected to " << (mgm ? mgm : "default") << endl;
+
+      // make bitmask of DB nodes
+      dbmask.clear();
+      ndb_mgm_cluster_state* cs = 0;
+      CHECK2((cs = ndb_mgm_get_status(h)) != 0, ndb_mgm_get_latest_error_msg(h));
+      for (int j = 0; j < cs->no_of_nodes; j++)
+      {
+        ndb_mgm_node_state* ns = &cs->node_states[j];
+        if (ns->node_type == NDB_MGM_NODE_TYPE_NDB)
+        {
+          CHECK2(ns->node_status == NDB_MGM_NODE_STATUS_STARTED, "node " << ns->node_id << " not started status " << ns->node_status);
+          CHECK2(!dbmask.get(ns->node_id), "duplicate node id " << ns->node_id);
+          dbmask.set(ns->node_id);
+          g_info << "added DB node " << ns->node_id << endl;
+        }
+      }
+      free(cs);
+      CHECK2(result == NDBT_OK, "some DB nodes are not started");
+      CHECK2(!dbmask.isclear(), "found no DB nodes");
+    }
+
+    // data memory pages after following events
+    // 0-initial 1,2-create table,index 3-load 4-delete 5,6-drop index,table
+    int pages[7];
+
+    // initial
+    CHECK2(get_data_memory_pages(h, dbmask, &pages[0]) == NDBT_OK, "failed");
+    g_info << "initial pages " << pages[0] << endl;
+
+    // create table
+    g_info << "create table " << tabname << endl;
+    const NdbDictionary::Table* pTab = 0;
+    CHECK2(pDic->createTable(tab) == 0, pDic->getNdbError());
+    CHECK2((pTab = pDic->getTable(tabname)) != 0, pDic->getNdbError());
+    CHECK2(get_data_memory_pages(h, dbmask, &pages[1]) == NDBT_OK, "failed");
+    g_info << "create table pages " << pages[1] << endl;
+
+    // choice of index attributes is not relevant to this bug
+    // choose one non-PK updateable column
+    NdbDictionary::Index ind;
+    ind.setName(indname);
+    ind.setTable(tabname);
+    ind.setType(NdbDictionary::Index::OrderedIndex);
+    ind.setLogging(false);
+    {
+      HugoCalculator calc(*pTab);
+      for (int j = 0; j < pTab->getNoOfColumns(); j++)
+      {
+        const NdbDictionary::Column* col = pTab->getColumn(j);
+        if (col->getPrimaryKey() || calc.isUpdateCol(j))
+          continue;
+        CHARSET_INFO* cs = col->getCharset();
+        if (NdbSqlUtil::check_column_for_ordered_index(col->getType(), col->getCharset()) == 0)
+        {
+          ind.addColumn(*col);
+          break;
+        }
+      }
+    }
+    CHECK2(ind.getNoOfColumns() == 1, "cannot use table " << tabname);
+
+    // create index
+    g_info << "create index " << indname << " on " << ind.getColumn(0)->getName() << endl;
+    const NdbDictionary::Index* pInd = 0;
+    CHECK2(pDic->createIndex(ind, *pTab) == 0, pDic->getNdbError());
+    CHECK2((pInd = pDic->getIndex(indname, tabname)) != 0, pDic->getNdbError());
+    CHECK2(get_data_memory_pages(h, dbmask, &pages[2]) == NDBT_OK, "failed");
+    g_info << "create index pages " << pages[2] << endl;
+
+    HugoTransactions trans(*pTab);
+
+    // load all records
+    g_info << "load records" << endl;
+    CHECK2(trans.loadTable(pNdb, rows) == 0, trans.getNdbError());
+    memset(rowmask, 1, rows);
+    CHECK2(get_data_memory_pages(h, dbmask, &pages[3]) == NDBT_OK, "failed");
+    g_info << "load records pages " << pages[3] << endl;
+
+    // test index with random ops
+    g_info << "test index ops" << endl;
+    {
+      HugoOperations ops(*pTab);
+      for (int i = 0; i < rows; i++)
+      {
+        CHECK2(ops.startTransaction(pNdb) == 0, ops.getNdbError());
+        for (int j = 0; j < 32; j++)
+        {
+          int n = rand() % rows;
+          if (!rowmask[n])
+          {
+            CHECK2(ops.pkInsertRecord(pNdb, n) == 0, ops.getNdbError());
+            rowmask[n] = 1;
+          }
+          else if (rand() % 2 == 0)
+          {
+            CHECK2(ops.pkDeleteRecord(pNdb, n) == 0, ops.getNdbError());
+            rowmask[n] = 0;
+          }
+          else
+          {
+            CHECK2(ops.pkUpdateRecord(pNdb, n) == 0, ops.getNdbError());
+          }
+        }
+        CHECK2(result == NDBT_OK, "index ops batch failed");
+        CHECK2(ops.execute_Commit(pNdb) == 0, ops.getNdbError());
+        ops.closeTransaction(pNdb);
+      }
+      CHECK2(result == NDBT_OK, "index ops failed");
+    }
+
+    // delete all records
+    g_info << "delete records" << endl;
+    CHECK2(trans.clearTable(pNdb) == 0, trans.getNdbError());
+    memset(rowmask, 0, rows);
+    CHECK2(get_data_memory_pages(h, dbmask, &pages[4]) == NDBT_OK, "failed");
+    g_info << "delete records pages " << pages[4] << endl;
+
+    // drop index
+    g_info << "drop index" <<  endl;
+    CHECK2(pDic->dropIndex(indname, tabname) == 0, pDic->getNdbError());
+    CHECK2(get_data_memory_pages(h, dbmask, &pages[5]) == NDBT_OK, "failed");
+    g_info << "drop index pages " << pages[5] << endl;
+
+    // drop table
+    g_info << "drop table" << endl;
+    CHECK2(pDic->dropTable(tabname) == 0, pDic->getNdbError());
+    CHECK2(get_data_memory_pages(h, dbmask, &pages[6]) == NDBT_OK, "failed");
+    g_info << "drop table pages " << pages[6] << endl;
+
+    // verify
+    CHECK2(pages[1] == pages[0], "pages after create table " << pages[1]
+                                  << " not == initial pages " << pages[0]);
+    CHECK2(pages[2] == pages[0], "pages after create index " << pages[2]
+                                  << " not == initial pages " << pages[0]);
+    CHECK2(pages[3] >  pages[0], "pages after load " << pages[3]
+                                  << " not >  initial pages " << pages[0]);
+    CHECK2(pages[4] == pages[0], "pages after delete " << pages[4]
+                                  << " not == initial pages " << pages[0]);
+    CHECK2(pages[5] == pages[0], "pages after drop index " << pages[5]
+                                  << " not == initial pages " << pages[0]);
+    CHECK2(pages[6] == pages[0], "pages after drop table " << pages[6]
+                                  << " not == initial pages " << pages[0]);
+
+    loop++;
+
+    // at last loop disconnect from mgm
+    if (loop == loops)
+    {
+      CHECK2(ndb_mgm_disconnect(h) == 0, ndb_mgm_get_latest_error_msg(h));
+      ndb_mgm_destroy_handle(&h);
+      g_info << "mgm: disconnected" << endl;
+    }
+  }
+  delete [] rowmask;
+
+  return result;
+}
+
+
 NDBT_TESTSUITE(testIndex);
 TESTCASE("CreateAll", 
 	 "Test that we can create all various indexes on each table\n"
@@ -2376,6 +3094,39 @@ TESTCASE("Bug46069", ""){
   STEPS(runBug46069_scandel, 2);
   FINALIZER(createPkIndex_Drop);
 }
+TESTCASE("ConstraintDetails",
+         "Test that the details part of the returned NdbError is as "
+         "expected"){
+  INITIALIZER(runConstraintDetails);
+}
+TESTCASE("Bug50118", ""){
+  TC_PROPERTY("LoggedIndexes", (unsigned)0);
+  INITIALIZER(runClearTable);
+  INITIALIZER(runLoadTable);
+  INITIALIZER(createPkIndex);
+  STEP(runReadIndexUntilStopped);
+  STEP(runReadIndexUntilStopped);
+  STEP(runReadIndexUntilStopped);
+  STEP(runBug50118);
+  FINALIZER(createPkIndex_Drop);
+  FINALIZER(runClearTable);
+}
+TESTCASE("FireTrigOverload", ""){
+  TC_PROPERTY("LoggedIndexes", (unsigned)0);
+  TC_PROPERTY("NotOnlyPkId", (unsigned)1); // Index must be non PK to fire triggers
+  TC_PROPERTY(NDBT_TestCase::getStepThreadStackSizePropName(), 128*1024);
+  INITIALIZER(createRandomIndex);
+  INITIALIZER(runClearTable);
+  STEP(runTrigOverload);
+  FINALIZER(runClearError);
+  FINALIZER(createRandomIndex_Drop);
+}
+TESTCASE("Bug56829",
+         "Return empty ordered index nodes to index fragment "
+         "so that empty fragment pages can be freed"){
+  STEP(runBug56829);
+}
+  
 NDBT_TESTSUITE_END(testIndex);
 
 int main(int argc, const char** argv){
