@@ -41,7 +41,11 @@ int runTestApiSession(NDBT_Context* ctx, NDBT_Step* step)
   h= ndb_mgm_create_handle();
   ndb_mgm_set_connectstring(h, mgmd.getConnectString());
   ndb_mgm_connect(h,0,0,0);
+#ifdef NDB_WIN
+  SOCKET s = ndb_mgm_get_fd(h);
+#else
   int s= ndb_mgm_get_fd(h);
+#endif
   session_id= ndb_mgm_get_session_id(h);
   ndbout << "MGM Session id: " << session_id << endl;
   send(s,"get",3,0);
@@ -450,7 +454,7 @@ int runTestMgmApiEventTimeout(NDBT_Context* ctx, NDBT_Step* step)
                      1, NDB_MGM_EVENT_CATEGORY_STARTUP,
                      0 };
 
-    my_socket my_fd;
+    NDB_SOCKET_TYPE my_fd;
 #ifdef NDB_WIN
     SOCKET fd= ndb_mgm_listen_event(h, filter);
     my_fd.s= fd;
@@ -465,8 +469,11 @@ int runTestMgmApiEventTimeout(NDBT_Context* ctx, NDBT_Step* step)
       result= NDBT_FAILED;
     }
 
-    Uint32 theData[25];
-    EventReport *fake_event = (EventReport*)theData;
+    union {
+      Uint32 theData[25];
+      EventReport repData;
+    };
+    EventReport *fake_event = &repData;
     fake_event->setEventType(NDB_LE_NDBStopForced);
     fake_event->setNodeId(42);
     theData[2]= 0;
@@ -575,8 +582,11 @@ int runTestMgmApiStructEventTimeout(NDBT_Context* ctx, NDBT_Step* step)
     {
       if(error_ins==0 || (error_ins!=0 && i<5))
       {
-        Uint32 theData[25];
-        EventReport *fake_event = (EventReport*)theData;
+        union {
+	  Uint32 theData[25];
+	  EventReport repData;
+	};
+        EventReport *fake_event = &repData;
         fake_event->setEventType(NDB_LE_NDBStopForced);
         fake_event->setNodeId(42);
         theData[2]= 0;
@@ -704,154 +714,6 @@ int runGetConfigUntilStopped(NDBT_Context* ctx, NDBT_Step* step)
 }
 
 
-
-int getMgmLogInfo(NdbMgmHandle h, off_t *current_size, off_t *max_size)
-{
-  int r, ncol;
-  char rowbuf[1024];
-  char **cols;
-  int current_size_colnum= 0;
-  int max_size_colnum= 0;
-
-  int rows;
-  r= ndb_mgm_ndbinfo(h,"LOGDESTINATION", &rows);
-
-  ncol= ndb_mgm_ndbinfo_colcount(h);
-
-  cols= (char**)malloc(ncol*sizeof(char*));
-  for(int i=0;i<ncol;i++)
-    cols[i]= (char*) malloc(100*sizeof(char));
-
-  ndb_mgm_ndbinfo_getcolums(h,ncol,100,cols);
-
-  for(int i=0;i<ncol;i++)
-  {
-    if(strcmp(cols[i],"CURRENT_SIZE")==0)
-      current_size_colnum= i;
-    if(strcmp(cols[i],"MAX_SIZE")==0)
-      max_size_colnum= i;
-    free(cols[i]);
-  }
-  ndbout << endl;
-
-  while(r--)
-  {
-    ndb_mgm_ndbinfo_getrow(h, rowbuf, sizeof(rowbuf));
-    char *col= rowbuf;
-    for(int i=0;i<ncol;i++)
-    {
-      int len;
-      col= ndb_mgm_ndbinfo_nextcolumn(col, &len);
-      if(!col)
-        break;
-      if(col[len]=='\'')
-        col[len++]='\0';
-      col[len]='\0';
-      if(i==current_size_colnum)
-        *current_size= (off_t)strtoll(col,NULL,10);
-      if(i==max_size_colnum)
-        *max_size= (off_t)strtoll(col,NULL,10);
-      col= &col[len+1];
-    }
-
-  }
-
-  ndbout_c("CURRENT SIZE = %lu",*current_size);
-  ndbout_c("MAX SIZE = %lu",*max_size);
-
-  free(cols);
-
-  return 0;
-}
-
-int runTestMgmLogRotation(NDBT_Context* ctx, NDBT_Step* step)
-{
-  NdbMgmd mgmd;
-  const char *mgm= mgmd.getConnectString();
-  int result= NDBT_FAILED;
-  int mgmid= 0;
-  off_t current_size= 0, max_size= 0;
-  int i,j;
-
-  NdbMgmHandle h= NULL,h1,h2,h3,h4;
-  h= ndb_mgm_create_handle();
-  ndb_mgm_set_connectstring(h, mgm);
-
-  ndb_mgm_connect(h,0,0,0);
-
-  h1= ndb_mgm_create_handle();
-  ndb_mgm_set_connectstring(h1, mgm);
-  ndb_mgm_connect(h1,0,0,0);
-
-  h2= ndb_mgm_create_handle();
-  ndb_mgm_set_connectstring(h2, mgm);
-  ndb_mgm_connect(h2,0,0,0);
-
-  h3= ndb_mgm_create_handle();
-  ndb_mgm_set_connectstring(h3, mgm);
-  ndb_mgm_connect(h3,0,0,0);
-
-  h4= ndb_mgm_create_handle();
-  ndb_mgm_set_connectstring(h4, mgm);
-  ndb_mgm_connect(h4,0,0,0);
-
-
-  if(ndb_mgm_check_connection(h) < 0)
-  {
-    result= NDBT_FAILED;
-    goto done;
-  }
-
-  mgmid= ndb_mgm_get_mgmd_nodeid(h);
-
-  ndbout_c("Connected to MGM server at NodeID: %d",mgmid);
-
-  if(getMgmLogInfo(h, &current_size, &max_size))
-  {
-    result= NDBT_FAILED;
-    goto done;
-  }
-
-  for(i=0;i<max_size/4;i++)
-  {
-        Uint32 theData[25];
-        memset(theData,0,sizeof(theData));
-        EventReport *fake_event = (EventReport*)theData;
-
-        for(j=0;j<100;j++)
-        {
-          fake_event->setEventType((Ndb_logevent_type)j);
-          fake_event->setNodeId(j+100);
-          ndb_mgm_report_event(h, theData, 6);
-          fake_event->setNodeId(j+200);
-          ndb_mgm_report_event(h1, theData, 6);
-          fake_event->setNodeId(j+300);
-          ndb_mgm_report_event(h2, theData, 6);
-          fake_event->setNodeId(j+400);
-          ndb_mgm_report_event(h3, theData, 6);
-          fake_event->setNodeId(j+500);
-          ndb_mgm_report_event(h4, theData, 6);
-
-        }
-        off_t c,m;
-        getMgmLogInfo(h, &c, &m);
-        if(c < current_size)
-        {
-          result= NDBT_OK;
-          break;
-        }
-  }
-
-  return result;
-
-done:
-  if(h)
-    ndb_mgm_destroy_handle(&h);
-
-  return result;
-}
-
-
 int runTestStatus(NDBT_Context* ctx, NDBT_Step* step)
 {
   ndb_mgm_node_type types[2] = {
@@ -1067,8 +929,6 @@ check_get_nodeid_invalid_nodetype1(NdbMgmd& mgmd)
                                     "unknown nodetype 37");
 }
 
-
-#if 0
 static bool
 check_get_nodeid_invalid_nodeid(NdbMgmd& mgmd)
 {
@@ -1078,13 +938,12 @@ check_get_nodeid_invalid_nodeid(NdbMgmd& mgmd)
     Properties args;
     args.put("nodeid", nodeId);
     BaseString expected;
-    expected.assfmt("No node defined with id=%d", nodeId);
+    expected.assfmt("illegal nodeid %d", nodeId);
     if (!get_nodeid_result_contains(mgmd, args, expected.c_str()))
       return false;
   }
   return true;
 }
-#endif
 
 static bool
 check_get_nodeid_dynamic_nodeid(NdbMgmd& mgmd)
@@ -1211,8 +1070,8 @@ check_get_nodeid_wrong_nodetype(NdbMgmd& mgmd)
   assert(nodeId && nodeType != (Uint32)NDB_MGM_NODE_TYPE_UNKNOWN);
 
   nodeType = (nodeType + 1) / NDB_MGM_NODE_TYPE_MAX;
-  assert(nodeType >= NDB_MGM_NODE_TYPE_MIN &&
-         nodeType <= NDB_MGM_NODE_TYPE_MAX);
+  assert((int)nodeType >= (int)NDB_MGM_NODE_TYPE_MIN &&
+         (int)nodeType <= (int)NDB_MGM_NODE_TYPE_MAX);
 
   Properties args, reply;
   args.put("nodeid",nodeId);
@@ -1242,7 +1101,7 @@ int runTestGetNodeId(NDBT_Context* ctx, NDBT_Step* step)
       check_get_nodeid_invalid_endian1(mgmd) &&
       check_get_nodeid_invalid_endian2(mgmd) &&
       check_get_nodeid_invalid_nodetype1(mgmd) &&
-//      check_get_nodeid_invalid_nodeid(mgmd) &&
+      check_get_nodeid_invalid_nodeid(mgmd) &&
       check_get_nodeid_dynamic_nodeid(mgmd) &&
       check_get_nodeid_nonode(mgmd) &&
 //      check_get_nodeid_nodeid1(mgmd) &&
@@ -1278,47 +1137,6 @@ int runSleepAndStop(NDBT_Context* ctx, NDBT_Step* step)
 }
 
 
-static bool
-get_version(NdbMgmd& mgmd,
-            Properties& reply)
-{
-  Properties args;
-  if (!mgmd.call("get version", args,
-                 "version", reply))
-  {
-    g_err << "get_version: mgmd.call failed" << endl;
-    return false;
-  }
-
-  //reply.print();
-  return true;
-}
-
-int runTestGetVersion(NDBT_Context* ctx, NDBT_Step* step)
-{
-  NdbMgmd mgmd;
-
-  if (!mgmd.connect())
-    return NDBT_FAILED;
-
-  Properties reply;
-  if (!get_version(mgmd, reply))
-   return NDBT_FAILED;
-
-  return NDBT_OK;
-}
-
-int runTestGetVersionUntilStopped(NDBT_Context* ctx, NDBT_Step* step)
-{
-  int result= NDBT_OK;
-  while(!ctx->isTestStopped() &&
-        (result= runTestGetVersion(ctx, step)) == NDBT_OK)
-    ;
-  return result;
-}
-
-
-
 static bool
 check_connection(NdbMgmd& mgmd)
 {
@@ -1337,8 +1155,7 @@ check_transporter_connect(NdbMgmd& mgmd, const char * hello)
   SocketOutputStream out(mgmd.socket());
 
   // Call 'transporter connect'
-  if (out.println("transporter connect") ||
-      out.println(""))
+  if (out.println("transporter connect\n"))
   {
     g_err << "Send failed" << endl;
     return false;
@@ -2470,6 +2287,202 @@ int runTestBug45497(NDBT_Context* ctx, NDBT_Step* step)
 }
 
 
+static int
+runTestGetVersion(NDBT_Context* ctx, NDBT_Step* step)
+{
+
+  NdbMgmd mgmd;
+
+  if (!mgmd.connect())
+    return NDBT_FAILED;
+
+  char verStr[64];
+  int major, minor, build;
+  if (ndb_mgm_get_version(mgmd.handle(),
+                          &major, &minor, &build,
+                          sizeof(verStr), verStr) != 1)
+  {
+    g_err << "ndb_mgm_get_version failed,"
+          << "error: " << ndb_mgm_get_latest_error_msg(mgmd.handle())
+          << "desc: " << ndb_mgm_get_latest_error_desc(mgmd.handle()) << endl;
+    return NDBT_FAILED;
+  }
+
+  g_info << "Using major: " << major
+         << " minor: " << minor
+         << " build: " << build
+         << " string: " << verStr << endl;
+
+  int l = 0;
+  int loops = ctx->getNumLoops();
+  while(l < loops)
+  {
+    char verStr2[64];
+    int major2, minor2, build2;
+    if (ndb_mgm_get_version(mgmd.handle(),
+                            &major2, &minor2, &build2,
+                            sizeof(verStr2), verStr2) != 1)
+    {
+      g_err << "ndb_mgm_get_version failed,"
+            << "error: " << ndb_mgm_get_latest_error_msg(mgmd.handle())
+            << "desc: " << ndb_mgm_get_latest_error_desc(mgmd.handle()) << endl;
+      return NDBT_FAILED;
+    }
+
+    if (major != major2)
+    {
+      g_err << "Got different major: " << major2
+            << " excpected: " << major << endl;
+      return NDBT_FAILED;
+    }
+
+    if (minor != minor2)
+    {
+      g_err << "Got different minor: " << minor2
+            << " excpected: " << minor << endl;
+      return NDBT_FAILED;
+    }
+
+    if (build != build2)
+    {
+      g_err << "Got different build: " << build2
+            << " excpected: " << build << endl;
+      return NDBT_FAILED;
+    }
+
+    if (strcmp(verStr, verStr2) != 0)
+    {
+      g_err << "Got different verStr: " << verStr2
+            << " excpected: " << verStr << endl;
+      return NDBT_FAILED;
+    }
+
+    l++;
+  }
+
+  return NDBT_OK;
+}
+
+
+static int
+runTestGetVersionUntilStopped(NDBT_Context* ctx, NDBT_Step* step)
+{
+  int result= NDBT_OK;
+  while(!ctx->isTestStopped() &&
+        (result= runTestGetVersion(ctx, step)) == NDBT_OK)
+    ;
+  return result;
+}
+
+
+int runTestDumpEvents(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NdbMgmd mgmd;
+
+  if (!mgmd.connect())
+    return NDBT_FAILED;
+
+  // Test with unsupported logevent_type
+  {
+    const Ndb_logevent_type unsupported = NDB_LE_NDBStopForced;
+    g_info << "ndb_mgm_dump_events(" << unsupported << ")" << endl;
+
+    const struct ndb_mgm_events* events =
+      ndb_mgm_dump_events(mgmd.handle(), unsupported, 0, 0);
+    if (events != NULL)
+    {
+      g_err << "ndb_mgm_dump_events returned events "
+            << "for unsupported Ndb_logevent_type" << endl;
+      return NDBT_FAILED;
+    }
+
+    if (ndb_mgm_get_latest_error(mgmd.handle()) != NDB_MGM_USAGE_ERROR ||
+        strcmp("ndb_logevent_type 59 not supported",
+               ndb_mgm_get_latest_error_desc(mgmd.handle())))
+    {
+      g_err << "Unexpected error for unsupported logevent type, "
+            << ndb_mgm_get_latest_error(mgmd.handle())
+            << ", desc: " << ndb_mgm_get_latest_error_desc(mgmd.handle())
+            << endl;
+      return NDBT_FAILED;
+    }
+  }
+
+  // Test with nodes >= MAX_NDB_NODES
+  for (int i = MAX_NDB_NODES; i < MAX_NDB_NODES + 3; i++)
+  {
+    g_info << "ndb_mgm_dump_events(NDB_LE_MemoryUsage, 1, "
+           << i << ")" << endl;
+
+    const struct ndb_mgm_events* events =
+      ndb_mgm_dump_events(mgmd.handle(), NDB_LE_MemoryUsage, 1, &i);
+    if (events != NULL)
+    {
+      g_err << "ndb_mgm_dump_events returned events "
+            << "for too large nodeid" << endl;
+      return NDBT_FAILED;
+    }
+
+    int invalid_nodeid;
+    if (ndb_mgm_get_latest_error(mgmd.handle()) != NDB_MGM_USAGE_ERROR ||
+        sscanf(ndb_mgm_get_latest_error_desc(mgmd.handle()),
+               "invalid nodes: '%d'", &invalid_nodeid) != 1 ||
+        invalid_nodeid != i)
+    {
+      g_err << "Unexpected error for too large nodeid, "
+            << ndb_mgm_get_latest_error(mgmd.handle())
+            << ", desc: " << ndb_mgm_get_latest_error_desc(mgmd.handle())
+            << endl;
+      return NDBT_FAILED;
+    }
+
+  }
+
+  int l = 0;
+  int loops = ctx->getNumLoops();
+  while (l<loops)
+  {
+    const Ndb_logevent_type supported[] =
+      {
+        NDB_LE_MemoryUsage,
+        NDB_LE_BackupStatus,
+        (Ndb_logevent_type)0
+      };
+
+    // Test with supported logevent_type
+    for (int i = 0; supported[i]; i++)
+    {
+      g_info << "ndb_mgm_dump_events(" << supported[i] << ")" << endl;
+
+      struct ndb_mgm_events* events =
+        ndb_mgm_dump_events(mgmd.handle(), supported[i], 0, 0);
+      if (events == NULL)
+      {
+        g_err << "ndb_mgm_dump_events failed, type: " << supported[i]
+              << ", error: " << ndb_mgm_get_latest_error(mgmd.handle())
+              << ", msg: " << ndb_mgm_get_latest_error_msg(mgmd.handle())
+              << endl;
+        return NDBT_FAILED;
+      }
+
+      if (events->no_of_events < 0)
+      {
+        g_err << "ndb_mgm_dump_events returned a negative number of events: "
+              << events->no_of_events << endl;
+        free(events);
+        return NDBT_FAILED;
+      }
+
+      g_info << "Got " << events->no_of_events << " events" << endl;
+      free(events);
+    }
+
+    l++;
+  }
+
+  return NDBT_OK;
+}
+
 NDBT_TESTSUITE(testMgm);
 DRIVER(DummyDriver); /* turn off use of NdbApi */
 TESTCASE("ApiSessionFailure",
@@ -2530,13 +2543,6 @@ TESTCASE("TestSetConfigParallel",
 TESTCASE("GetConfig", "Run ndb_mgm_get_configuration in parallel"){
   STEPS(runGetConfig, 100);
 }
-#if 0
-TESTCASE("MgmLogRotation",
-	 "Test log rotation"){
-  INITIALIZER(runTestMgmLogRotation);
-
-}
-#endif
 TESTCASE("TestStatus",
 	 "Test status and status2"){
   INITIALIZER(runTestStatus);
@@ -2551,10 +2557,9 @@ TESTCASE("TestGetNodeId",
 	 "Test 'get nodeid'"){
   INITIALIZER(runTestGetNodeId);
 }
-
 TESTCASE("TestGetVersion",
-	 "Test 'get version'"){
-  INITIALIZER(runTestGetVersion);
+ 	 "Test 'get version' and 'ndb_mgm_get_version'"){
+  STEPS(runTestGetVersion, 20);
 }
 TESTCASE("TestTransporterConnect",
 	 "Test 'transporter connect'"){
@@ -2585,7 +2590,7 @@ TESTCASE("Stress",
   STEP(runSetConfigUntilStopped);
   STEPS(runGetConfigUntilStopped, 10);
   STEPS(runTestStatusUntilStopped, 10);
-//  STEPS(runTestGetVersionUntilStopped, 5);
+  STEPS(runTestGetVersionUntilStopped, 5);
   STEP(runSleepAndStop);
 }
 TESTCASE("Stress2",
@@ -2594,13 +2599,21 @@ TESTCASE("Stress2",
   STEPS(runTestSetConfigParallelUntilStopped, 5);
   STEPS(runGetConfigUntilStopped, 10);
   STEPS(runTestStatusUntilStopped, 10);
-//  STEPS(runTestGetVersionUntilStopped, 5);
+  STEPS(runTestGetVersionUntilStopped, 5);
   STEP(runSleepAndStop);
 }
 TESTCASE("Bug45497",
          "Connect to ndb_mgmd until it can't handle more connections"){
   STEP(runTestBug45497);
 }
+TESTCASE("TestGetVersion",
+ 	 "Test 'get version' and 'ndb_mgm_get_version'"){
+  STEPS(runTestGetVersion, 20);
+}
+TESTCASE("TestDumpEvents",
+ 	 "Test 'dump events'"){
+  STEPS(runTestDumpEvents, 1);
+}
 NDBT_TESTSUITE_END(testMgm);
 
 int main(int argc, const char** argv){
