@@ -26,17 +26,30 @@
 #include <NDBT_Find.hpp>
 #include <NDBT_Workingdir.hpp>
 
-static bool file_exists(const char* path)
+static bool file_exists(const char* path, Uint32 timeout = 1)
 {
   g_info << "File '" << path << "' ";
-  if (access(path, F_OK) == 0)
+  /**
+   * ndb_mgmd does currently not fsync the directory
+   *   after committing config-bin,
+   *   which means that it can be on disk, wo/ being visible
+   *   remedy this by retrying some
+   */
+  for (Uint32 i = 0; i < 10 * timeout; i++)
   {
-    g_info << "exists" << endl;
-    return true;
+    if (access(path, F_OK) == 0)
+    {
+      g_info << "exists" << endl;
+      return true;
+    }
+    if (i == 0)
+    {
+      g_info << "does not exist, retrying...";
+    }
+    NdbSleep_MilliSleep(100);
   }
   g_info << "does not exist" << endl;
   return false;
-
 }
 
 // Util function that concatenate strings to form a path
@@ -171,8 +184,7 @@ public:
     // Diconnect and close our "builtin" client
     m_mgmd_client.close();
 
-    assert(m_proc);
-    if (!m_proc->stop())
+    if (m_proc == 0 || !m_proc->stop())
     {
       fprintf(stderr, "Failed to stop process %s\n", name());
       return false; // Can't kill with -9 -> fatal error
@@ -183,7 +195,13 @@ public:
       fprintf(stderr, "Failed to wait for process %s\n", name());
       return false; // Can't wait after kill with -9 -> fatal error
     }
-    assert(ret == 9);
+
+    if (ret != 9)
+    {
+      fprintf(stderr, "stop ret: %u\n", ret);
+      return false; // Can't wait after kill with -9 -> fatal error
+    }
+
     delete m_proc;
     m_proc = 0;
 
@@ -191,12 +209,11 @@ public:
 
   }
 
-  bool wait(int& ret, int timeout = 30)
+  bool wait(int& ret, int timeout = 300)
   {
     g_info << "Waiting for " << name() << endl;
 
-    assert(m_proc);
-    if (!m_proc->wait(ret, timeout))
+    if (m_proc == 0 || !m_proc->wait(ret, timeout))
     {
       fprintf(stderr, "Failed to wait for process %s\n", name());
       return false;
@@ -336,8 +353,8 @@ int runTestBasic2Mgm(NDBT_Context* ctx, NDBT_Step* step)
   for (int i = 1; i <= 2; i++)
   {
     Mgmd* mgmd = new Mgmd(i);
-    CHECK(mgmd->start_from_config_ini(wd.path()));
     mgmds.push_back(mgmd);
+    CHECK(mgmd->start_from_config_ini(wd.path()));
   }
 
   // Connect the ndb_mgmd(s)
@@ -402,8 +419,8 @@ int runTestBug45495(NDBT_Context* ctx, NDBT_Step* step)
   for (int i = 1; i <= 2; i++)
   {
     Mgmd* mgmd = new Mgmd(i);
-    CHECK(mgmd->start_from_config_ini(wd.path()));
     mgmds.push_back(mgmd);
+    CHECK(mgmd->start_from_config_ini(wd.path()));
   }
 
   // Connect the ndb_mgmd(s)
@@ -522,22 +539,23 @@ int runTestBug45495(NDBT_Context* ctx, NDBT_Step* step)
                                           "--reload", NULL));
     CHECK(mgmds[i]->connect(config));
     CHECK(mgmds[i]->wait_confirmed_config());
-
-    CHECK(file_exists(path(wd.path(),
-                           "ndb_1_config.bin.1",
-                           NULL).c_str()));
-    CHECK(file_exists(path(wd.path(),
-                           "ndb_2_config.bin.1",
-                           NULL).c_str()));
-
-    CHECK(file_exists(path(wd.path(),
-                            "ndb_1_config.bin.2",
-                            NULL).c_str()));
-    CHECK(file_exists(path(wd.path(),
-                            "ndb_2_config.bin.2",
-                            NULL).c_str()));
   }
 
+  CHECK(file_exists(path(wd.path(),
+                         "ndb_1_config.bin.1",
+                         NULL).c_str()));
+  CHECK(file_exists(path(wd.path(),
+                         "ndb_2_config.bin.1",
+                         NULL).c_str()));
+
+  Uint32 timeout = 30;
+  CHECK(file_exists(path(wd.path(),
+                         "ndb_1_config.bin.2",
+                         NULL).c_str(), timeout));
+  CHECK(file_exists(path(wd.path(),
+                         "ndb_2_config.bin.2",
+                         NULL).c_str(), timeout));
+
   g_err << "** Reload mgmd initial(from generation=2)" << endl;
   for (unsigned i = 0; i < mgmds.size(); i++)
   {
@@ -545,6 +563,7 @@ int runTestBug45495(NDBT_Context* ctx, NDBT_Step* step)
     CHECK(mgmds[i]->start_from_config_ini(wd.path(),
                                           "-f config2.ini",
                                           "--reload", "--initial", NULL));
+
     CHECK(mgmds[i]->connect(config));
     CHECK(mgmds[i]->wait_confirmed_config());
 
@@ -560,14 +579,16 @@ int runTestBug45495(NDBT_Context* ctx, NDBT_Step* step)
       tmp.assfmt("ndb_%d_config.bin.2", j+1);
       CHECK(file_exists(path(wd.path(),
                              tmp.c_str(),
-                             NULL).c_str()));
+                             NULL).c_str(),
+                        timeout));
     }
   }
 
   return NDBT_OK;
-
 }
 
+
+
 int runTestBug42015(NDBT_Context* ctx, NDBT_Step* step)
 {
   NDBT_Workingdir wd("test_mgmd"); // temporary working directory
@@ -582,16 +603,16 @@ int runTestBug42015(NDBT_Context* ctx, NDBT_Step* step)
   MgmdProcessList mgmds;
   // Start ndb_mgmd 1 from config.ini
   Mgmd* mgmd = new Mgmd(1);
-  CHECK(mgmd->start_from_config_ini(wd.path()));
   mgmds.push_back(mgmd);
+  CHECK(mgmd->start_from_config_ini(wd.path()));
 
   // Start ndb_mgmd 2 by fetching from first
   Mgmd* mgmd2 = new Mgmd(2);
+  mgmds.push_back(mgmd2);
   CHECK(mgmd2->start(wd.path(),
                      "--ndb-connectstring",
                      mgmd->connectstring(config).c_str(),
                      NULL));
-  mgmds.push_back(mgmd2);
 
   // Connect the ndb_mgmd(s)
   for (unsigned i = 0; i < mgmds.size(); i++)
@@ -613,6 +634,307 @@ int runTestBug42015(NDBT_Context* ctx, NDBT_Step* step)
 
 }
 
+/* Test for bug 53008:  --skip-config-cache */
+int runTestNoConfigCache(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NDBT_Workingdir wd("test_mgmd"); // temporary working directory
+  
+  g_err << "** Create config.ini" << endl;
+  Properties config = ConfigFactory::create();
+  CHECK(ConfigFactory::write_config_ini(config,
+                                        path(wd.path(),
+                                             "config.ini",
+                                             NULL).c_str()));
+  
+  // Start ndb_mgmd  from config.ini
+  Mgmd* mgmd = new Mgmd(1);
+  CHECK(mgmd->start_from_config_ini(wd.path(), "--skip-config-cache", NULL));
+     
+  // Connect the ndb_mgmd(s)
+  CHECK(mgmd->connect(config));
+  
+  // wait for confirmed config
+  CHECK(mgmd->wait_confirmed_config());
+  
+  // Check binary config files *not* created
+  bool bin_conf_file = file_exists(path(wd.path(),
+                                        "ndb_1_config.bin.1", 
+                                        NULL).c_str());
+  CHECK(bin_conf_file == false);
+  
+  mgmd->stop();  
+  return NDBT_OK;
+}  
+
+
+int runTestNowaitNodes(NDBT_Context* ctx, NDBT_Step* step)
+{
+  MgmdProcessList mgmds;
+  NDBT_Workingdir wd("test_mgmd"); // temporary working directory
+
+  // Create config.ini
+  Properties config = ConfigFactory::create(2);
+  CHECK(ConfigFactory::write_config_ini(config,
+                                        path(wd.path(),
+                                             "config.ini",
+                                             NULL).c_str()));
+  // Start first ndb_mgmd
+  Mgmd* mgmd1 = new Mgmd(1);
+  {
+    mgmds.push_back(mgmd1);
+    CHECK(mgmd1->start_from_config_ini(wd.path(),
+                                    "--initial",
+                                       "--nowait-nodes=2",
+                                       NULL));
+
+    // Connect the ndb_mgmd
+    CHECK(mgmd1->connect(config));
+
+    // wait for confirmed config
+    CHECK(mgmd1->wait_confirmed_config());
+
+    // Check binary config file created
+    CHECK(file_exists(path(wd.path(),
+                           "ndb_1_config.bin.1",
+                           NULL).c_str()));
+  }
+
+  // Start second ndb_mgmd
+  {
+    Mgmd* mgmd2 = new Mgmd(2);
+    mgmds.push_back(mgmd2);
+    CHECK(mgmd2->start_from_config_ini(wd.path(),
+                                       "--initial",
+                                       NULL));
+
+    // Connect the ndb_mgmd
+    CHECK(mgmd2->connect(config));
+
+    // wait for confirmed config
+    CHECK(mgmd2->wait_confirmed_config());
+
+    // Check binary config file created
+    CHECK(file_exists(path(wd.path(),
+                           "ndb_2_config.bin.1",
+                           NULL).c_str()));
+
+  }
+
+  // Create new config.ini
+  g_err << "** Create config2.ini" << endl;
+  CHECK(ConfigFactory::put(config, "ndb_mgmd", 1, "ArbitrationDelay", 100));
+  CHECK(ConfigFactory::write_config_ini(config,
+                                        path(wd.path(),
+                                             "config2.ini",
+                                             NULL).c_str()));
+
+  g_err << "** Reload second mgmd from config2.ini" << endl;
+  {
+    Mgmd* mgmd2 = mgmds[1];
+    CHECK(mgmd2->stop());
+    // Start from config2.ini
+    CHECK(mgmd2->start_from_config_ini(wd.path(),
+                                       "-f config2.ini",
+                                       "--reload", NULL));
+    CHECK(mgmd2->connect(config));
+    CHECK(mgmd1->wait_confirmed_config());
+    CHECK(mgmd2->wait_confirmed_config());
+
+    CHECK(file_exists(path(wd.path(),
+                           "ndb_1_config.bin.1",
+                           NULL).c_str()));
+    CHECK(file_exists(path(wd.path(),
+                           "ndb_2_config.bin.1",
+                           NULL).c_str()));
+
+    // Both ndb_mgmd(s) should have reloaded and new binary config exist
+    CHECK(file_exists(path(wd.path(),
+                           "ndb_1_config.bin.2",
+                           NULL).c_str()));
+    CHECK(file_exists(path(wd.path(),
+                           "ndb_2_config.bin.2",
+                           NULL).c_str()));
+  }
+
+  // Stop the ndb_mgmd(s)
+  for (unsigned i = 0; i < mgmds.size(); i++)
+    CHECK(mgmds[i]->stop());
+
+  return NDBT_OK;
+}
+
+
+int runTestNowaitNodes2(NDBT_Context* ctx, NDBT_Step* step)
+{
+  int ret;
+  NDBT_Workingdir wd("test_mgmd"); // temporary working directory
+
+  // Create config.ini
+  Properties config = ConfigFactory::create(2);
+  CHECK(ConfigFactory::write_config_ini(config,
+                                        path(wd.path(),
+                                             "config.ini",
+                                             NULL).c_str()));
+
+  g_err << "** Start mgmd1 from config.ini" << endl;
+  MgmdProcessList mgmds;
+  Mgmd* mgmd1 = new Mgmd(1);
+  mgmds.push_back(mgmd1);
+  CHECK(mgmd1->start_from_config_ini(wd.path(),
+                                     "--initial",
+                                     "--nowait-nodes=1-255",
+                                     NULL));
+  CHECK(mgmd1->connect(config));
+  CHECK(mgmd1->wait_confirmed_config());
+
+  // check config files exist
+  CHECK(file_exists(path(wd.path(),
+                         "ndb_1_config.bin.1",
+                         NULL).c_str()));
+
+  g_err << "** Create config2.ini" << endl;
+  CHECK(ConfigFactory::put(config, "ndb_mgmd", 1, "ArbitrationDelay", 100));
+  CHECK(ConfigFactory::write_config_ini(config,
+                                        path(wd.path(),
+                                             "config2.ini",
+                                             NULL).c_str()));
+
+  g_err << "** Start mgmd2 from config2.ini" << endl;
+  Mgmd* mgmd2 = new Mgmd(2);
+  mgmds.push_back(mgmd2);
+  CHECK(mgmd2->start_from_config_ini(wd.path(),
+                                     "-f config2.ini",
+                                     "--initial",
+                                     "--nowait-nodes=1-255",
+                                     NULL));
+  CHECK(mgmd2->wait(ret));
+  CHECK(ret == 1);
+
+  CHECK(mgmd1->stop());
+
+  g_err << "** Start mgmd2 again from config2.ini" << endl;
+  CHECK(mgmd2->start_from_config_ini(wd.path(),
+                                     "-f config2.ini",
+                                     "--initial",
+                                     "--nowait-nodes=1-255",
+                                     NULL));
+
+
+  CHECK(mgmd2->connect(config));
+  CHECK(mgmd2->wait_confirmed_config());
+
+  // check config files exist
+  CHECK(file_exists(path(wd.path(),
+                         "ndb_2_config.bin.1",
+                         NULL).c_str()));
+
+  g_err << "** Start mgmd1 from config.ini, mgmd2 should shutdown" << endl;
+  CHECK(mgmd1->start_from_config_ini(wd.path(),
+                                     "--initial",
+                                     "--nowait-nodes=1-255",
+                                     NULL));
+  CHECK(mgmd2->wait(ret));
+  CHECK(ret == 1);
+
+  CHECK(mgmd1->stop());
+
+  return NDBT_OK;
+}
+
+int
+runBug56844(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NDBT_Workingdir wd("test_mgmd"); // temporary working directory
+
+  g_err << "** Create config.ini" << endl;
+  Properties config = ConfigFactory::create(2);
+  CHECK(ConfigFactory::write_config_ini(config,
+                                        path(wd.path(),
+                                             "config.ini",
+                                             NULL).c_str()));
+  // Start ndb_mgmd(s)
+  MgmdProcessList mgmds;
+  for (int i = 1; i <= 2; i++)
+  {
+    Mgmd* mgmd = new Mgmd(i);
+    mgmds.push_back(mgmd);
+    CHECK(mgmd->start_from_config_ini(wd.path()));
+  }
+
+  // Connect the ndb_mgmd(s)
+  for (unsigned i = 0; i < mgmds.size(); i++)
+  {
+    CHECK(mgmds[i]->connect(config));
+  }
+
+  // wait for confirmed config
+  for (unsigned i = 0; i < mgmds.size(); i++)
+  {
+    CHECK(mgmds[i]->wait_confirmed_config());
+  }
+
+  // stop them
+  for (unsigned i = 0; i < mgmds.size(); i++)
+  {
+    CHECK(mgmds[i]->stop());
+  }
+
+  // Check binary config files created
+  CHECK(file_exists(path(wd.path(),
+                         "ndb_1_config.bin.1",
+                         NULL).c_str()));
+  CHECK(file_exists(path(wd.path(),
+                         "ndb_2_config.bin.1",
+                         NULL).c_str()));
+
+  CHECK(ConfigFactory::put(config, "ndb_mgmd", 1, "ArbitrationDelay", 100));
+  CHECK(ConfigFactory::write_config_ini(config,
+                                        path(wd.path(),
+                                             "config2.ini",
+                                             NULL).c_str()));
+  Uint32 no = 2;
+  int loops = ctx->getNumLoops();
+  for (int l = 0; l < loops; l++, no++)
+  {
+    g_err << l << ": *** Reload from config.ini" << endl;
+    for (unsigned i = 0; i < mgmds.size(); i++)
+    {
+      // Start from config2.ini
+      CHECK(mgmds[i]->start_from_config_ini(wd.path(),
+                                            (l & 1) == 1 ?
+                                            "-f config.ini" :
+                                            "-f config2.ini",
+                                            "--reload", NULL));
+    }
+    for (unsigned i = 0; i < mgmds.size(); i++)
+    {
+      CHECK(mgmds[i]->connect(config));
+      CHECK(mgmds[i]->wait_confirmed_config());
+    }
+
+    /**
+     * Since it will first be confirmed...
+     *   and then once connected to other ndb_nmgmd start a config
+     *   change, it can take a bit until new config exists...
+     *   allow 30s
+     */
+    Uint32 timeout = 30;
+    for (unsigned i = 0; i < mgmds.size(); i++)
+    {
+      BaseString p = path(wd.path(), "", NULL);
+      p.appfmt("ndb_%u_config.bin.%u", i+1, no);
+      g_err << "CHECK(" << p.c_str() << ")" << endl;
+      CHECK(file_exists(p.c_str(), timeout));
+    }
+
+    for (unsigned i = 0; i < mgmds.size(); i++)
+    {
+      CHECK(mgmds[i]->stop());
+    }
+  }
+  return NDBT_OK;
+}
+
 NDBT_TESTSUITE(testMgmd);
 DRIVER(DummyDriver); /* turn off use of NdbApi */
 
@@ -622,17 +944,44 @@ TESTCASE("Basic2Mgm",
   INITIALIZER(runTestBasic2Mgm);
 }
 
-TESTCASE("Bug45495",
-         "Test that mgmd can be restarted in any order")
-{
-  INITIALIZER(runTestBug45495);
-}
-
 TESTCASE("Bug42015",
          "Test that mgmd can fetch configuration from another mgmd")
 {
   INITIALIZER(runTestBug42015);
 }
+TESTCASE("NowaitNodes",
+         "Test that one mgmd(of 2) can start alone with usage "
+         "of --nowait-nodes, then start the second mgmd and it should join")
+{
+  INITIALIZER(runTestNowaitNodes);
+}
+TESTCASE("NowaitNodes2",
+         "Test that one mgmd(of 2) can start alone with usage "
+         "of --nowait-nodes, then start the second mgmd from different "
+         "configuration and the one with lowest nodeid should shutdown")
+{
+  INITIALIZER(runTestNowaitNodes2);
+}
+
+TESTCASE("NoCfgCache",
+         "Test that when an mgmd is started with --skip-config-cache, "
+         "no ndb_xx_config.xx.bin file is created, but you can "
+         "connect to the mgm node and retrieve the config.")
+{
+  INITIALIZER(runTestNoConfigCache);
+}
+
+TESTCASE("Bug56844",
+         "Test that mgmd can be restarted in any order")
+{
+  INITIALIZER(runBug56844);
+}
+
+TESTCASE("Bug45495",
+         "Test that mgmd can be restarted in any order")
+{
+  INITIALIZER(runTestBug45495);
+}
 
 NDBT_TESTSUITE_END(testMgmd);
 
