@@ -24,6 +24,7 @@
 #include <Vector.hpp>
 #include "ScanFunctions.hpp"
 #include <random.h>
+#include <signaldata/DumpStateOrd.hpp>
 
 const NdbDictionary::Table *
 getTable(Ndb* pNdb, int i){
@@ -315,12 +316,19 @@ int runRandScanRead(NDBT_Context* ctx, NDBT_Step* step){
   int parallelism = ctx->getProperty("Parallelism", 240);
   int abort = ctx->getProperty("AbortProb", 5);
   int tupscan = ctx->getProperty("TupScan", (Uint32)0);
+  int lmarg = ctx->getProperty("LockMode", ~Uint32(0));
+  int nocount = ctx->getProperty("NoCount", Uint32(0));
+
+  if (nocount)
+    records = 0;
 
   int i = 0;
   HugoTransactions hugoTrans(*ctx->getTab());
   while (i<loops && !ctx->isTestStopped()) {
     g_info << i << ": ";
     NdbOperation::LockMode lm = (NdbOperation::LockMode)(rand() % 3);
+    if (lmarg != ~0)
+      lm = (NdbOperation::LockMode)lmarg;
     int scan_flags = 0;
   
     if (tupscan == 1)
@@ -1319,6 +1327,81 @@ finalizeBug42559(NDBT_Context* ctx, NDBT_Step* step){
 }
 
 
+int
+runBug54945(NDBT_Context* ctx, NDBT_Step* step)
+{
+
+  int loops = ctx->getNumLoops();
+  const NdbDictionary::Table*  pTab = ctx->getTab();
+
+  Ndb* pNdb = GETNDB(step);
+  NdbRestarter res;
+
+  if (res.getNumDbNodes() < 2)
+  {
+    ctx->stopTest();
+    return NDBT_OK;
+  }
+
+  while (loops--)
+  {
+    int node = res.getNode(NdbRestarter::NS_RANDOM);
+    int err = 0;
+    printf("node: %u ", node);
+    switch(loops % 2){
+    case 0:
+      if (res.getNumDbNodes() >= 2)
+      {
+        err = 8088;
+        int val[] = { DumpStateOrd::CmvmiSetRestartOnErrorInsert, 1 };
+        res.dumpStateOneNode(node, val, 2);
+        res.insertErrorInNode(node, 8088);
+        ndbout_c("error 8088");
+        break;
+      }
+      // fall through
+    case 1:
+      err = 5057;
+      res.insertErrorInNode(node, 5057);
+      ndbout_c("error 5057");
+      break;
+    }
+
+    for (int i = 0; i< 25; i++)
+    {
+      NdbTransaction* pCon = pNdb->startTransaction();
+      NdbScanOperation* pOp = pCon->getNdbScanOperation(pTab->getName());
+      if (pOp == NULL) {
+        ERR(pCon->getNdbError());
+        return NDBT_FAILED;
+      }
+      
+      if( pOp->readTuples(NdbOperation::LM_Read) != 0) 
+      {
+        ERR(pCon->getNdbError());
+        return NDBT_FAILED;
+      }
+      
+      if( pOp->getValue(NdbDictionary::Column::ROW_COUNT) == 0)
+      {
+        ERR(pCon->getNdbError());
+        return NDBT_FAILED;
+      }
+      
+      pCon->execute(NoCommit);
+      pCon->close();
+    } 
+    if (err == 8088)
+    {
+      res.waitNodesNoStart(&node, 1);
+      res.startAll();
+      res.waitClusterStarted();
+    }
+  }
+
+  return NDBT_OK;
+}
+
 NDBT_TESTSUITE(testScan);
 TESTCASE("ScanRead", 
 	 "Verify scan requirement: It should be possible "\
@@ -1778,6 +1861,17 @@ TESTCASE("InsertDelete",
   STEP(runInsertDelete);
   FINALIZER(runClearTable);
 }
+TESTCASE("Bug48700", 
+	 "Load and delete all while scan updating and scan reading\n"\
+	 "Alexander Lukas special"){
+  TC_PROPERTY("AbortProb", Uint32(0));
+  TC_PROPERTY("NoCount", 1);
+  TC_PROPERTY("LockMode", NdbOperation::LM_CommittedRead);
+  INITIALIZER(runClearTable);
+  STEPS(runRandScanRead, 10);
+  STEP(runInsertDelete);
+  FINALIZER(runClearTable);
+}
 TESTCASE("CheckAfterTerror", 
 	 "Check that we can still scan read after this terror of NdbApi"){
   INITIALIZER(runLoadTable);
@@ -1834,6 +1928,10 @@ TESTCASE("Bug42559", "")
   FINALIZER(finalizeBug42559);
   FINALIZER(runClearTable);
 }
+TESTCASE("Bug54945", "")
+{
+  INITIALIZER(runBug54945);
+}
 NDBT_TESTSUITE_END(testScan);
 
 int main(int argc, const char** argv){
