@@ -1835,12 +1835,12 @@ runTO(NDBT_Context* ctx, NDBT_Step* step)
   }
 
   struct ndb_logevent event;
+  int val[] = { DumpStateOrd::DihMinTimeBetweenLCP, 0 };
 
   Uint32 i = 0;
   while(i<=loops && result != NDBT_FAILED)
   {
-    int val = DumpStateOrd::DihMinTimeBetweenLCP;
-    CHECK(res.dumpStateAllNodes(&val, 1) == 0);
+    CHECK(res.dumpStateAllNodes(val, 1) == 0);
 
     int filter[] = { 15, NDB_MGM_EVENT_CATEGORY_CHECKPOINT, 0 };
     NdbLogEventHandle handle = 
@@ -1860,7 +1860,8 @@ runTO(NDBT_Context* ctx, NDBT_Step* step)
       CHECK(res.waitNodesNoStart(&node, 1) == 0);
       for (Uint32 j = 0; j<25; j++)
       {
-        CHECK(hugoTrans.scanUpdateRecords(pNdb, 0) == 0);
+        if (! (hugoTrans.scanUpdateRecords(pNdb, 0) == 0))
+          break;
       }
       while(ndb_logevent_get_next(handle, &event, 0) >= 0 &&
             event.type != NDB_LE_LocalCheckpointCompleted);
@@ -1872,13 +1873,23 @@ runTO(NDBT_Context* ctx, NDBT_Step* step)
     Uint32 LCP = event.LocalCheckpointCompleted.lci;
     ndbout_c("LCP: %u", LCP);
     
-    do {
+    do 
+    {
+      bzero(&event, sizeof(event));
       while(ndb_logevent_get_next(handle, &event, 0) >= 0 &&
             event.type != NDB_LE_LocalCheckpointCompleted)
+        bzero(&event, sizeof(event));
+      
+      if (event.type == NDB_LE_LocalCheckpointCompleted &&
+          event.LocalCheckpointCompleted.lci < LCP + 3)
       {
-        CHECK(hugoTrans.scanUpdateRecords(pNdb, 0) == 0);
+        hugoTrans.scanUpdateRecords(pNdb, 0);
       }
-    } while (event.LocalCheckpointCompleted.lci < LCP + 3);
+      else
+      {
+        break;
+      }
+    } while (true);
     
     ndbout_c("LCP: %u", event.LocalCheckpointCompleted.lci);
     
@@ -1905,7 +1916,7 @@ runTO(NDBT_Context* ctx, NDBT_Step* step)
     hugoTrans.clearTable(pNdb);
     hugoTrans.loadTable(pNdb, rows);
     
-    CHECK(res.dumpStateAllNodes(&val, 1) == 0);
+    CHECK(res.dumpStateAllNodes(val, 1) == 0);
     
     while(ndb_logevent_get_next(handle, &event, 0) >= 0 &&
           event.type != NDB_LE_LocalCheckpointCompleted);
@@ -1914,7 +1925,9 @@ runTO(NDBT_Context* ctx, NDBT_Step* step)
     
     i++;
   }
-  
+
+  res.dumpStateAllNodes(val, 2); // Reset LCP time
+
   ctx->stopTest();  
   return result;
 }
@@ -2193,6 +2206,192 @@ loop:
   return NDBT_OK;
 }
 
+int
+runScanUpdateUntilStopped(NDBT_Context* ctx, NDBT_Step* step)
+{
+  Ndb* pNdb = GETNDB(step);
+  HugoTransactions hugoTrans(*ctx->getTab());
+
+  NullOutputStream null;
+  OutputStream * save[1];
+  save[0] = g_err.m_out;
+  g_err.m_out = &null;
+  while (!ctx->isTestStopped())
+  {
+    hugoTrans.scanUpdateRecords(pNdb, 0);
+  }
+  g_err.m_out = save[0];
+  return NDBT_OK;
+}
+
+int
+runBug48436(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NdbRestarter res;
+  Uint32 loops = ctx->getNumLoops();
+  const Uint32 nodeCount = res.getNumDbNodes();
+  if(nodeCount < 2)
+  {
+    return NDBT_OK;
+  }
+
+  for (Uint32 l = 0; l<loops; l++)
+  {
+    int nodes[2];
+    nodes[0] = res.getNode(NdbRestarter::NS_RANDOM);
+    nodes[1] = res.getRandomNodeSameNodeGroup(nodes[0], rand());
+    int val = 7099;
+    int val2[] = { DumpStateOrd::CmvmiSetRestartOnErrorInsert, 1 };
+
+    ndbout_c("nodes %u %u", nodes[0], nodes[1]);
+
+    for (Uint32 j = 0; j<5; j++)
+    {
+      int c = (rand()) % 11;
+      ndbout_c("case: %u", c);
+      switch(c){
+      case 0:
+      case 1:
+        res.dumpStateAllNodes(&val, 1);
+      case 2:
+      case 3:
+      case 4:
+      case 5:
+        res.restartOneDbNode(nodes[0], false, true, true);
+        res.waitNodesNoStart(nodes+0,1);
+        res.dumpStateOneNode(nodes[0], val2, 2);
+        res.insertErrorInNode(nodes[0], 5054); // crash during restart
+        res.startAll();
+        sleep(3);
+        res.waitNodesNoStart(nodes+0,1);
+        res.startAll();
+        break;
+      case 6:
+        res.restartOneDbNode(nodes[0], false, true, true);
+        res.waitNodesNoStart(nodes+0, 1);
+        res.startAll();
+        break;
+      case 7:
+        res.dumpStateAllNodes(&val, 1);
+      case 8:
+        res.restartOneDbNode(nodes[1], false, true, true);
+        res.waitNodesNoStart(nodes+1,1);
+        res.dumpStateOneNode(nodes[1], val2, 2);
+        res.insertErrorInNode(nodes[1], 5054); // crash during restart
+        res.startAll();
+        sleep(3);
+        res.waitNodesNoStart(nodes+1,1);
+        res.startAll();
+        break;
+      case 9:
+        res.restartAll(false, true, true);
+        res.waitClusterNoStart();
+        res.startAll();
+        break;
+      case 10:
+      {
+        res.dumpStateAllNodes(val2, 2);
+        int node = res.getMasterNodeId();
+        res.insertErrorInNode(node, 7222);
+        res.waitClusterNoStart();
+        res.startAll();
+        break;
+      }
+      }
+      res.waitClusterStarted();
+    }
+    res.restartAll(false, true, true);
+    res.waitClusterNoStart();
+    res.startAll();
+    res.waitClusterStarted();
+  }
+  ctx->stopTest();
+
+  return NDBT_OK;
+}
+
+int
+runBug54611(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NdbRestarter res;
+  Uint32 loops = ctx->getNumLoops();
+  Ndb* pNdb = GETNDB(step);
+  int rows = ctx->getNumRecords();
+
+  HugoTransactions hugoTrans(*ctx->getTab());
+
+  for (Uint32 l = 0; l<loops; l++)
+  {
+    int val = DumpStateOrd::DihMinTimeBetweenLCP;
+    res.dumpStateAllNodes(&val, 1);
+
+    for (Uint32 i = 0; i < 5; i++)
+    {
+      hugoTrans.scanUpdateRecords(pNdb, rows);
+    }
+
+    int val2[] = { DumpStateOrd::CmvmiSetRestartOnErrorInsert, 1 };
+    res.dumpStateAllNodes(val2, 2);
+
+    int node = res.getMasterNodeId();
+    res.insertErrorInNode(node, 7222);
+
+    while (hugoTrans.scanUpdateRecords(pNdb, rows) == 0);
+    res.waitClusterNoStart();
+
+    res.insertErrorInAllNodes(5055);
+    res.startAll();
+    res.waitClusterStarted();
+  }
+
+  return NDBT_OK;
+}
+
+int
+runBug56961(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NdbRestarter res;
+  Uint32 loops = ctx->getNumLoops();
+  Ndb* pNdb = GETNDB(step);
+  int rows = ctx->getNumRecords();
+
+  int node = res.getNode(NdbRestarter::NS_RANDOM);
+  int val2[] = { DumpStateOrd::CmvmiSetRestartOnErrorInsert, 1 };
+  HugoTransactions hugoTrans(*ctx->getTab());
+
+  for (Uint32 l = 0; l<loops; l++)
+  {
+    ndbout_c("Waiting for %d to restart (5058)", node);
+    res.dumpStateOneNode(node, val2, 2);
+    res.insertErrorInNode(node, 5058);
+
+    hugoTrans.clearTable(pNdb);
+    hugoTrans.loadTable(pNdb, rows);
+    while (hugoTrans.scanUpdateRecords(pNdb, rows) == NDBT_OK &&
+           res.getNodeStatus(node) != NDB_MGM_NODE_STATUS_NOT_STARTED &&
+           res.getNodeStatus(node) != NDB_MGM_NODE_STATUS_NO_CONTACT);
+    res.waitNodesNoStart(&node, 1);
+    res.startNodes(&node, 1);
+    ndbout_c("Waiting for %d to start", node);
+    res.waitClusterStarted();
+
+    ndbout_c("Waiting for %d to restart (5059)", node);
+    res.dumpStateOneNode(node, val2, 2);
+    res.insertErrorInNode(node, 5059);
+
+    hugoTrans.clearTable(pNdb);
+    hugoTrans.loadTable(pNdb, rows);
+    while (hugoTrans.scanUpdateRecords(pNdb, rows) == NDBT_OK &&
+           res.getNodeStatus(node) != NDB_MGM_NODE_STATUS_NOT_STARTED &&
+           res.getNodeStatus(node) != NDB_MGM_NODE_STATUS_NO_CONTACT);
+    res.waitNodesNoStart(&node, 1);
+    res.startNodes(&node, 1);
+    ndbout_c("Waiting for %d to start", node);
+    res.waitClusterStarted();
+  }
+
+  return NDBT_OK;
+}
 
 NDBT_TESTSUITE(testSystemRestart);
 TESTCASE("SR1", 
@@ -2531,6 +2730,22 @@ TESTCASE("Bug46412", "")
 {
   INITIALIZER(runBug46412);
 }
+TESTCASE("Bug48436", "")
+{
+  INITIALIZER(runLoadTable);
+  STEP(runBug48436);
+  STEP(runScanUpdateUntilStopped);
+}
+TESTCASE("Bug54611", "")
+{
+  INITIALIZER(runLoadTable);
+  INITIALIZER(runBug54611);
+}
+TESTCASE("Bug56961", "")
+{
+  INITIALIZER(runLoadTable);
+  INITIALIZER(runBug56961);
+}
 NDBT_TESTSUITE_END(testSystemRestart);
 
 int main(int argc, const char** argv){
