@@ -319,8 +319,11 @@ eventOperation(Ndb* pNdb, const NdbDictionary::Table &tab, void* pstats, int rec
 	}
 	g_info << endl;
       }
-    } else
+    }
+    else
+    {
       ;//printf("timed out\n");
+    }
   }
 
   g_info << "dropping event operation" << endl;
@@ -1005,6 +1008,7 @@ int runRestarter(NDBT_Context* ctx, NDBT_Step* step){
   NdbRestarter restarter;
   int i = 0;
   int lastId = 0;
+  bool abort = ctx->getProperty("Graceful", Uint32(0)) == 0;
 
   if (restarter.getNumDbNodes() < 2){
     ctx->stopTest();
@@ -1021,7 +1025,12 @@ int runRestarter(NDBT_Context* ctx, NDBT_Step* step){
     int id = lastId % restarter.getNumDbNodes();
     int nodeId = restarter.getDbNodeId(id);
     ndbout << "Restart node " << nodeId << endl; 
-    if(restarter.restartOneDbNode(nodeId, false, false, true) != 0){
+    if (abort == false && ((i % 3) == 0))
+    {
+      restarter.insertErrorInNode(nodeId, 13043);
+    }
+
+    if(restarter.restartOneDbNode(nodeId, false, false, abort) != 0){
       g_err << "Failed to restartNextDbNode" << endl;
       result = NDBT_FAILED;
       break;
@@ -1660,7 +1669,7 @@ static int runMulti_NR(NDBT_Context* ctx, NDBT_Step* step)
     {
       // restart a node
       int timeout = 240;
-      if (restarts.executeRestart("RestartRandomNodeAbort", timeout))
+      if (restarts.executeRestart(ctx, "RestartRandomNodeAbort", timeout))
       {
 	DBUG_RETURN(NDBT_FAILED);
       }
@@ -2913,7 +2922,8 @@ executeOps(Ndb* pNdb,
 
   for (Uint32 i=keyOffset; i < (keyOffset + rowCount); i++)
   {
-    *keyPtr= *attrPtr= i;
+    memcpy(keyPtr, &i, sizeof(i));
+    memcpy(attrPtr, &i, sizeof(i));
     opts.optionsPresent |= NdbOperation::OperationOptions::OO_ANYVALUE;
     opts.anyValue= anyValueOffset + i;
     bool allowInterpreted= 
@@ -3271,7 +3281,7 @@ runBug44915(NDBT_Context* ctx, NDBT_Step* step)
   
   NdbRestarter res;
   int error[] = { 13031, 13044, 13045, 0 };
-  for (int i = 0; error[i]; i++)
+  for (int i = 0; error[i] && result == NDBT_OK; i++)
   {
     ndbout_c("error: %d", error[i]);
     res.insertErrorInNode(res.getDbNodeId(rand() % res.getNumDbNodes()),
@@ -3279,11 +3289,94 @@ runBug44915(NDBT_Context* ctx, NDBT_Step* step)
     
     result = runCreateEvent(ctx, step); // should fail due to error insert
     result = runCreateEvent(ctx, step); // should pass
+    result = runDropEvent(ctx, step);
+  }
+  return result;
+}
+
+int
+runBug56579(NDBT_Context* ctx, NDBT_Step* step)
+{
+  int result = NDBT_OK;
+
+  NdbRestarter res;
+  Ndb* pNdb = GETNDB(step);
+
+  int error_all[] = { 13046, 0 };
+  for (int i = 0; error_all[i] && result == NDBT_OK; i++)
+  {
+    ndbout_c("error: %d", error_all[i]);
+    res.insertErrorInAllNodes(error_all[i]);
+
+    if (createEventOperation(pNdb, *ctx->getTab()) != 0)
+    {
+      return NDBT_FAILED;
+    }
   }
 
   return result;
 }
 
+int
+runBug57886_create_drop(NDBT_Context* ctx, NDBT_Step* step)
+{
+  int loops = ctx->getNumLoops();
+  Ndb* pNdb = GETNDB(step);
+
+  NdbDictionary::Dictionary *pDict = pNdb->getDictionary();
+  NdbDictionary::Table tab = * ctx->getTab();
+
+  sleep(5);
+
+  while (loops --)
+  {
+    if (pDict->dropTable(tab.getName()) != 0)
+    {
+      return NDBT_FAILED;
+    }
+
+    if (pDict->createTable(tab) != 0)
+    {
+      return NDBT_FAILED;
+    }
+
+    sleep(1);
+  }
+
+  ctx->stopTest();
+  return NDBT_OK;
+}
+
+int
+runBug57886_subscribe_unsunscribe(NDBT_Context* ctx, NDBT_Step* step)
+{
+  Ndb* pNdb;
+  Ndb_cluster_connection *pCC;
+  if (cc(&pCC, &pNdb))
+  {
+    // too few api slots...
+    return NDBT_OK;
+  }
+
+  NdbDictionary::Table tab = * ctx->getTab();
+
+  while (!ctx->isTestStopped())
+  {
+    createEvent(pNdb, tab, false, false);
+
+    NdbEventOperation* op = createEventOperation(pNdb, tab, 0);
+    if (op)
+    {
+      pNdb->dropEventOperation(op);
+    }
+    dropEvent(pNdb, tab);
+  }
+
+  delete pNdb;
+  delete pCC;
+  return NDBT_OK;
+}
+
 NDBT_TESTSUITE(test_event);
 TESTCASE("BasicEventOperation", 
 	 "Verify that we can listen to Events"
@@ -3342,6 +3435,20 @@ TESTCASE("EventOperationApplier_NR",
   FINALIZER(runVerify);
   FINALIZER(runDropShadowTable);
 }
+TESTCASE("EventOperationApplier_NS",
+	 "Verify that if we apply the data we get from event "
+	 "operation is the same as the original table"
+	 "NOTE! No errors are allowed!" ){
+  TC_PROPERTY("Graceful", 1);
+  INITIALIZER(runCreateEvent);
+  INITIALIZER(runCreateShadowTable);
+  STEP(runEventApplier);
+  STEP(runEventMixedLoad);
+  STEP(runRestarter);
+  FINALIZER(runDropEvent);
+  FINALIZER(runVerify);
+  FINALIZER(runDropShadowTable);
+}
 TESTCASE("MergeEventOperationApplier", 
 	 "Verify that if we apply the data we get from merged event "
 	 "operation is the same as the original table"
@@ -3492,6 +3599,17 @@ TESTCASE("Bug44915", "")
 {
   INITIALIZER(runBug44915);
 }
+TESTCASE("Bug56579", "")
+{
+  INITIALIZER(runCreateEvent);
+  STEP(runBug56579);
+  FINALIZER(runDropEvent);
+}
+TESTCASE("Bug57886", "")
+{
+  STEP(runBug57886_create_drop);
+  STEPS(runBug57886_subscribe_unsunscribe, 5);
+}
 NDBT_TESTSUITE_END(test_event);
 
 int main(int argc, const char** argv){
