@@ -20,7 +20,6 @@
 #include <ndb_opts.h>
 #include <NdbApi.hpp>
 #include <NdbTest.hpp>
-#include <my_sys.h>
 #include <ndb_version.h>
 
 #include <ndb_rand.h>
@@ -372,7 +371,7 @@ createtable(Tab& t)
     char pk2[1 + g_charlen + 1];
     pk1 = g_maxpk;
     sprintf(pk2 + 1, "%-u", pk1);
-    *(uchar*)pk2 = strlen(pk2 + 1);
+    *(uchar*)pk2 = (uchar)(strlen(pk2 + 1));
     chkdb(g_op->equal("pk1", (char*)&pk1) == 0);
     chkdb(g_op->equal("pk2", (char*)&pk2[0]) == 0);
     chkdb(g_con->execute(Commit) == 0);
@@ -1073,7 +1072,7 @@ copydata(const Data& d1, Data& d3, bool pk, bool nonpk)
   uint i;
   for (i = 0; i < ncol(); i++) {
     const Col& c = g_col[i];
-    if (c.pk && pk || ! c.pk && nonpk)
+    if ((c.pk && pk) || (! c.pk && nonpk))
       copycol(c, d1, d3);
   }
 }
@@ -1084,7 +1083,7 @@ compdata(const Data& d1, const Data& d2, Data& d3, bool pk, bool nonpk)
   uint i;
   for (i = 0; i < ncol(); i++) {
     const Col& c = g_col[i];
-    if (c.pk && pk || ! c.pk && nonpk) {
+    if ((c.pk && pk) || (! c.pk && nonpk)) {
       const Data* d = 0;
       if (d1.ind[i] == -1 && d2.ind[i] == -1)
         d3.ind[i] = -1;
@@ -1244,7 +1243,7 @@ waitgci(uint ngci)
       char pk2[1 + g_charlen + 1];
       pk1 = g_maxpk;
       sprintf(pk2 + 1, "%-u", pk1);
-      *(uchar*)pk2 = strlen(pk2 + 1);
+      *(uchar*)pk2 = (uchar)(strlen(pk2 + 1));
       chkdb((g_op = g_con->getNdbOperation(t.tabname)) != 0);
       chkdb(g_op->readTuple() == 0);
       chkdb(g_op->equal("pk1", (char*)&pk1) == 0);
@@ -1545,9 +1544,9 @@ makeops(Run& r)
           break;
         do {
           optype = (Op::Type)urandom(g_optypes);
-        } while (tot_op->type == Op::NUL &&
-                 (optype == Op::DEL || optype == Op::UPD) ||
-                 tot_op->type == Op::INS && optype == Op::INS);
+        } while ((tot_op->type == Op::NUL &&
+                  (optype == Op::DEL || optype == Op::UPD)) ||
+                 (tot_op->type == Op::INS && optype == Op::INS));
       } else {
         const char* str = g_opstringpart[g_loop % g_opstringparts];
         uint m = strlen(str);
@@ -1807,8 +1806,8 @@ cmppostpre(Run& r)
         for (i = 0; i < ncol(); i++) {
           const Col& c = getcol(i);
           bool eq =
-            d[0].ind[i] == 1 && d[1].ind[i] == 1 ||
-            d[0].ind[i] == 0 && d[1].ind[i] == 0 && cmpcol(c, d[0], d[1]) == 0;
+            (d[0].ind[i] == 1 && d[1].ind[i] == 1) ||
+            (d[0].ind[i] == 0 && d[1].ind[i] == 0 && cmpcol(c, d[0], d[1])== 0);
           if (eq) {
             d[0].ppeq |= (1 << i);
             d[1].ppeq |= (1 << i);
@@ -2240,60 +2239,60 @@ static struct my_option
 my_long_options[] =
 {
   NDB_STD_OPTS("test_event_merge"),
-  { "abort-on-error", 1001, "Do abort() on any error",
+  { "abort-on-error", NDB_OPT_NOSHORT, "Do abort() on any error",
     (uchar **)&g_opts.abort_on_error, (uchar **)&g_opts.abort_on_error, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "loglevel", 1002, "Logging level in this program 0-3 (default 0)",
+  { "loglevel", NDB_OPT_NOSHORT, "Logging level in this program 0-3 (default 0)",
     (uchar **)&g_opts.loglevel, (uchar **)&g_opts.loglevel, 0,
     GET_INT, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "loop", 1003, "Number of test loops (default 5, 0=forever)",
+  { "loop", NDB_OPT_NOSHORT, "Number of test loops (default 5, 0=forever)",
     (uchar **)&g_opts.loop, (uchar **)&g_opts.loop, 0,
     GET_INT, REQUIRED_ARG, 5, 0, 0, 0, 0, 0 },
-  { "maxops", 1004, "Approx number of PK operations per table (default 1000)",
+  { "maxops", NDB_OPT_NOSHORT, "Approx number of PK operations per table (default 1000)",
     (uchar **)&g_opts.maxops, (uchar **)&g_opts.maxops, 0,
     GET_UINT, REQUIRED_ARG, 1000, 0, 0, 0, 0, 0 },
-  { "maxpk", 1005, "Number of different PK values (default 10, max 1000)",
+  { "maxpk", NDB_OPT_NOSHORT, "Number of different PK values (default 10, max 1000)",
     (uchar **)&g_opts.maxpk, (uchar **)&g_opts.maxpk, 0,
     GET_UINT, REQUIRED_ARG, 10, 0, 0, 0, 0, 0 },
-  { "maxtab", 1006, "Number of tables (default 10, max 100)",
+  { "maxtab", NDB_OPT_NOSHORT, "Number of tables (default 10, max 100)",
     (uchar **)&g_opts.maxtab, (uchar **)&g_opts.maxtab, 0,
     GET_INT, REQUIRED_ARG, 10, 0, 0, 0, 0, 0 },
-  { "no-blobs", 1007, "Omit blob attributes (5.0: true)",
+  { "no-blobs", NDB_OPT_NOSHORT, "Omit blob attributes (5.0: true)",
     (uchar **)&g_opts.no_blobs, (uchar **)&g_opts.no_blobs, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "no-implicit-nulls", 1008, "Insert must include all attrs"
+  { "no-implicit-nulls", NDB_OPT_NOSHORT, "Insert must include all attrs"
                                " i.e. no implicit NULLs",
     (uchar **)&g_opts.no_implicit_nulls, (uchar **)&g_opts.no_implicit_nulls, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "no-missing-update", 1009, "Update must include all non-PK attrs",
+  { "no-missing-update", NDB_OPT_NOSHORT, "Update must include all non-PK attrs",
     (uchar **)&g_opts.no_missing_update, (uchar **)&g_opts.no_missing_update, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "no-multiops", 1010, "Allow only 1 operation per commit",
+  { "no-multiops", NDB_OPT_NOSHORT, "Allow only 1 operation per commit",
     (uchar **)&g_opts.no_multiops, (uchar **)&g_opts.no_multiops, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "no-nulls", 1011, "Create no NULL values",
+  { "no-nulls", NDB_OPT_NOSHORT, "Create no NULL values",
     (uchar **)&g_opts.no_nulls, (uchar **)&g_opts.no_nulls, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "one-blob", 1012, "Only one blob attribute (default 2)",
+  { "one-blob", NDB_OPT_NOSHORT, "Only one blob attribute (default 2)",
     (uchar **)&g_opts.one_blob, (uchar **)&g_opts.one_blob, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "opstring", 1013, "Operations to run e.g. idiucdc (c is commit) or"
+  { "opstring", NDB_OPT_NOSHORT, "Operations to run e.g. idiucdc (c is commit) or"
                       " iuuc:uudc (the : separates loops)",
     (uchar **)&g_opts.opstring, (uchar **)&g_opts.opstring, 0,
     GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "seed", 1014, "Random seed (0=loop number, default -1=random)",
+  { "seed", NDB_OPT_NOSHORT, "Random seed (0=loop number, default -1=random)",
     (uchar **)&g_opts.seed, (uchar **)&g_opts.seed, 0,
     GET_INT, REQUIRED_ARG, -1, 0, 0, 0, 0, 0 },
-  { "separate-events", 1015, "Do not combine events per GCI (5.0: true)",
+  { "separate-events", NDB_OPT_NOSHORT, "Do not combine events per GCI (5.0: true)",
     (uchar **)&g_opts.separate_events, (uchar **)&g_opts.separate_events, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "tweak", 1016, "Whatever the source says",
+  { "tweak", NDB_OPT_NOSHORT, "Whatever the source says",
     (uchar **)&g_opts.tweak, (uchar **)&g_opts.tweak, 0,
     GET_UINT, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "use-table", 1017, "Use existing tables",
+  { "use-table", NDB_OPT_NOSHORT, "Use existing tables",
     (uchar **)&g_opts.use_table, (uchar **)&g_opts.use_table, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "blob-version", 1018, "Blob version 1 or 2 (default 2)",
+  { "blob-version", NDB_OPT_NOSHORT, "Blob version 1 or 2 (default 2)",
     (uchar**)&g_opts.blob_version, (uchar**)&g_opts.blob_version, 0,
     GET_INT, REQUIRED_ARG, 2, 0, 0, 0, 0, 0 },
   { 0, 0, 0,
@@ -2301,13 +2300,6 @@ my_long_options[] =
     GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0 }
 };
 
-#if 0
-static void
-usage()
-{
-  my_print_help(my_long_options);
-}
-#endif
 
 static int
 checkopts()
