@@ -82,7 +82,7 @@ char * g_env_path = 0;
 const char* g_mysqld_host = 0;
 
 /** Dummy, extern declared in ndb_opts.h */
-int g_print_full_config = 0, opt_ndb_shm;
+int g_print_full_config = 0;
 my_bool opt_core;
 
 static struct my_option g_options[] =
@@ -177,32 +177,50 @@ main(int argc, char ** argv)
   g_logger.createConsoleHandler();
   
   if(!parse_args(argc, argv))
+  {
+    g_logger.critical("Failed to parse arguments");
     goto end;
+  }
   
   g_logger.info("Starting...");
   g_config.m_generated = false;
   g_config.m_replication = g_replicate;
   if (!setup_config(g_config, g_mysqld_host))
+  {
+    g_logger.critical("Failed to setup configuration");
     goto end;
+  }
 
   if (!configure(g_config, g_do_setup))
+  {
+    g_logger.critical("Failed to configure");
     goto end;
+  }
   
   g_logger.info("Setting up directories...");
   if (!setup_directories(g_config, g_do_setup))
+  {
+    g_logger.critical("Failed to set up directories");
     goto end;
+  }
 
   if (g_do_setup)
   {
     g_logger.info("Setting up files...");
     if (!setup_files(g_config, g_do_setup, g_do_sshx))
+    {
+      g_logger.critical("Failed to set up files");
       goto end;
+    }
   }
   
   if (g_do_deploy)
   {
     if (!deploy(g_config))
+    {
+      g_logger.critical("Failed to deploy");
       goto end;
+    }
   }
 
   if (g_do_quit)
@@ -212,19 +230,28 @@ main(int argc, char ** argv)
   }
 
   if(!setup_hosts(g_config))
+  {
+    g_logger.critical("Failed to setup hosts");
     goto end;
+  }
 
   if (g_do_sshx)
   {
     g_logger.info("Starting xterm-ssh");
     if (!sshx(g_config, g_do_sshx))
+    {
+      g_logger.critical("Failed to start xterm-ssh");
       goto end;
+    }
 
     g_logger.info("Done...sleeping");
     while(true)
     {
       if (!do_command(g_config))
+      {
+        g_logger.critical("Failed to do ssh command");
         goto end;
+      }
 
       NdbSleep_SecSleep(1);
     }
@@ -234,16 +261,26 @@ main(int argc, char ** argv)
  
   g_logger.info("Connecting to hosts...");
   if(!connect_hosts(g_config))
+  {
+    g_logger.critical("Failed to connect to CPCD on hosts");
     goto end;
+  }
 
+#ifndef _WIN32
   if (g_do_start && !g_test_case_filename)
   {
     g_logger.info("Starting server processes: %x", g_do_start);    
     if (!start(g_config, g_do_start))
+    {
+      g_logger.critical("Failed to start server processes");
       goto end;
+    }
     
     if (!setup_db(g_config))
+    {
+      g_logger.critical("Failed to setup database");
       goto end;
+    }
 
     g_logger.info("Done...sleeping");
     while(true)
@@ -259,12 +296,14 @@ main(int argc, char ** argv)
     return_code = 0;
     goto end;
   }
+#endif
 
   return_code = 0;
   
   /**
    * Main loop
    */
+  g_logger.debug("Entering main loop");
   while(!feof(g_test_case_file))
   {
     /**
@@ -276,25 +315,41 @@ main(int argc, char ** argv)
       g_logger.info("(Re)starting server processes...");
 
       if(!stop_processes(g_config, ~0))
-	goto end;
+      {
+        g_logger.critical("Failed to stop all processes");
+        goto end;
+      }
       
       if (!setup_directories(g_config, 2))
-	goto end;
+      {
+        g_logger.critical("Failed to setup directories");
+        goto end;
+      }
       
       if (!setup_files(g_config, 2, 1))
-	goto end;
+      {
+        g_logger.critical("Failed to setup files");
+        goto end;
+      }
       
       if(!setup_hosts(g_config))
+      {
+        g_logger.critical("Failed to setup hosts");
         goto end;
+      }
       
+      g_logger.debug("Setup complete, starting servers");
       if (!start(g_config, p_ndb | p_servers))
       {
-        g_logger.info("Failed to start server processes");
+        g_logger.critical("Failed to start server processes");
         g_logger.info("Gathering logs and saving them as test %u", test_no);
         
         int tmp;
         if(!gather_result(g_config, &tmp))
+        {
+          g_logger.critical("Failed to gather results");
           goto end;
+        }
         
         if(g_report_file != 0)
         {
@@ -317,7 +372,10 @@ main(int argc, char ** argv)
       }
 
       if (!setup_db(g_config))
-	goto end;
+      {
+        g_logger.critical("Failed to setup database");
+        goto end;
+      }
       
       g_logger.info("All servers start completed");
     }
@@ -333,10 +391,16 @@ main(int argc, char ** argv)
     
     // Assign processes to programs
     if(!setup_test_case(g_config, test_case))
+    {
+      g_logger.critical("Failed to setup test case");
       goto end;
+    }
     
     if(!start_processes(g_config, p_clients))
+    {
+      g_logger.critical("Failed to start client processes");
       goto end;
+    }
 
     int result = 0;
     
@@ -345,7 +409,10 @@ main(int argc, char ** argv)
     do 
     {
       if(!update_status(g_config, atrt_process::AP_ALL))
-	goto end;
+      {
+        g_logger.critical("Failed to get updated status for all processes");
+        goto end;
+      }
       
       if(is_running(g_config, p_ndb) != 2)
       {
@@ -373,7 +440,9 @@ main(int argc, char ** argv)
       now = time(0);
       if(now  > (start + test_case.m_max_time))
       {
+        g_logger.debug("Timed out");
 	result = ERR_MAX_TIME_ELAPSED;
+        g_logger.info("Timeout '%s' after %ld seconds", test_case.m_name.c_str(), test_case.m_max_time);
 	break;
       }
       NdbSleep_SecSleep(1);
@@ -382,11 +451,17 @@ main(int argc, char ** argv)
     const time_t elapsed = time(0) - start;
    
     if(!stop_processes(g_config, p_clients))
+    {
+      g_logger.critical("Failed to stop client processes");
       goto end;
+    }
     
     int tmp, *rp = result ? &tmp : &result;
     if(!gather_result(g_config, rp))
+    {
+      g_logger.critical("Failed to gather result after test run");
       goto end;
+    }
     
     g_logger.info("#%d %s(%d)", 
 		  test_no, 
@@ -452,7 +527,7 @@ main(int argc, char ** argv)
   return return_code;
 }
 
-static 
+extern "C"
 my_bool 
 get_one_option(int arg, const struct my_option * opt, char * value)
 {
@@ -710,8 +785,14 @@ parse_args(int argc, char** argv)
     return false;
   }
   
+  /* Read username from environment, default to sakila */
   g_user = strdup(getenv("LOGNAME"));
-  
+  if (g_user == 0)
+  {
+    g_user = "sakila";
+    g_logger.info("No default user specified, will use 'sakila'.");
+    g_logger.info("Please set LOGNAME environment variable for other username");
+  }
   return true;
 }
 
@@ -947,7 +1028,8 @@ bool
 start_processes(atrt_config& config, int types){
   for(size_t i = 0; i<config.m_processes.size(); i++){
     atrt_process & proc = *config.m_processes[i];
-    if((types & proc.m_type) != 0 && proc.m_proc.m_path != ""){
+    if(IF_WIN(!(proc.m_type & atrt_process::AP_MYSQLD), 1)
+       && (types & proc.m_type) != 0 && proc.m_proc.m_path != ""){
       if(!start_process(proc)){
 	return false;
       }
@@ -1051,6 +1133,11 @@ is_running(atrt_config& config, int types){
       found++;
       if(proc.m_proc.m_status == "running")
 	running++;
+      else {
+        if(IF_WIN(proc.m_type & atrt_process::AP_MYSQLD, 0))  {
+          running++;
+        }
+      }
     }
   }
   
@@ -1181,7 +1268,6 @@ setup_test_case(atrt_config& config, const atrt_testcase& tc){
         cmd.appfmt("%s/bin/", g_prefix);
       }
       cmd.append(tc.m_command.c_str());
-      to_native(cmd);
 
       if (0) // valgrind
       {
@@ -1397,13 +1483,6 @@ started:
   return true;
 }
 
-void
-require(bool x)
-{
-  if (!x)
-    abort();
-}
-
 bool
 reset_config(atrt_config & config)
 {
