@@ -18,6 +18,13 @@
 
 #include <HugoOperations.hpp>
 
+#undef ERR
+#define ERR(error) \
+{ \
+  const NdbError &_error= (error); \
+  if (!m_quiet) ERR_OUT(g_err, _error); \
+}
+
 int HugoOperations::startTransaction(Ndb* pNdb,
                                      const NdbDictionary::Table *table,
                                      const char  *keyData, Uint32 keyLen){
@@ -30,6 +37,7 @@ int HugoOperations::startTransaction(Ndb* pNdb,
   if (pTrans == NULL) {
     const NdbError err = pNdb->getNdbError();
     ERR(err);
+    setNdbError(err);
     return NDBT_FAILED;
   }
   return NDBT_OK;
@@ -90,6 +98,7 @@ int HugoOperations::pkReadRecord(Ndb* pNdb,
     }
     if (pOp == NULL) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
@@ -123,6 +132,7 @@ rand_lock_mode:
     
     if( check == -1 ) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
@@ -148,6 +158,7 @@ rand_lock_mode:
 	if((rows[r]->attributeStore(a) = 
 	    pOp->getValue(tab.getColumn(a)->getName())) == 0) {
 	  ERR(pTrans->getNdbError());
+          setNdbError(pTrans->getNdbError());
 	  return NDBT_FAILED;
 	}
       } 
@@ -181,6 +192,7 @@ int HugoOperations::pkReadRandRecord(Ndb* pNdb,
     }
     if (pOp == NULL) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
@@ -210,6 +222,7 @@ rand_lock_mode:
     
     if( check == -1 ) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
@@ -237,6 +250,7 @@ rand_lock_mode:
 	if((rows[r]->attributeStore(a) = 
 	    pOp->getValue(tab.getColumn(a)->getName())) == 0) {
 	  ERR(pTrans->getNdbError());
+          setNdbError(pTrans->getNdbError());
 	  return NDBT_FAILED;
 	}
       } 
@@ -249,6 +263,115 @@ rand_lock_mode:
   return NDBT_OK;
 }
 
+int HugoOperations::pkReadRecordLockHandle(Ndb* pNdb,
+                                           Vector<const NdbLockHandle*>& lockHandles,
+                                           int recordNo,
+                                           int numRecords,
+                                           NdbOperation::LockMode lm,
+                                           NdbOperation::LockMode *lmused){
+  if (idx)
+  {
+    g_err << "ERROR : Cannot call pkReadRecordLockHandle on an index"
+          << endl;
+    return NDBT_FAILED;
+  }
+
+  /* If something other than LM_Read or LM_Exclusive is 
+   * passed in then we'll choose, and PkReadRecord
+   * will update lm_used
+   */
+  while (lm != NdbOperation::LM_Read &&
+         lm != NdbOperation::LM_Exclusive)
+  {
+    lm = (NdbOperation::LockMode)((rand() >> 16) & 1);
+  }
+
+  const NdbOperation* prevOp = pTrans->getLastDefinedOperation();
+
+  int readRc = pkReadRecord(pNdb,
+                            recordNo,
+                            numRecords,
+                            lm,
+                            lmused);
+
+  if (readRc == NDBT_OK)
+  {
+    /* Now traverse operations added, requesting
+     * LockHandles
+     */
+    const NdbOperation* definedOp = (prevOp)? prevOp->next() : 
+      pTrans->getFirstDefinedOperation();
+    
+    while (definedOp)
+    {
+      /* Look away now */
+      const NdbLockHandle* lh = 
+        (const_cast<NdbOperation*>(definedOp))->getLockHandle();
+      
+      if (lh == NULL)
+      {
+        ERR(definedOp->getNdbError());
+        setNdbError(definedOp->getNdbError());
+        return NDBT_FAILED;
+      }
+
+      lockHandles.push_back(lh);
+      definedOp = definedOp->next();
+    }
+  }
+
+  return readRc;
+}
+
+int HugoOperations::pkUnlockRecord(Ndb* pNdb,
+                                   Vector<const NdbLockHandle*>& lockHandles,
+                                   int offset,
+                                   int numRecords,
+                                   NdbOperation::AbortOption ao)
+{
+  if (numRecords == ~(0))
+  {
+    numRecords = lockHandles.size() - offset;
+  }
+
+  if (lockHandles.size() < (unsigned)(offset + numRecords))
+  {
+    g_err << "ERROR : LockHandles size is "
+          << lockHandles.size()
+          << " offset (" 
+          << offset
+          << ") and/or numRecords ("
+          << numRecords
+          << ") too large." << endl;
+    return NDBT_FAILED;
+  }
+  
+  for (int i = 0; i < numRecords; i++)
+  {
+    const NdbLockHandle* lh = lockHandles[offset + i];
+    if (lh != NULL)
+    {
+      const NdbOperation* unlockOp = pTrans->unlock(lh, ao);
+      
+      if (unlockOp == NULL)
+      {
+        ERR(pTrans->getNdbError());
+        setNdbError(pTrans->getNdbError());
+        return NDBT_FAILED;
+      }
+    }
+    else
+    {
+      g_err << "ERROR : LockHandle number "
+            << i+offset << " is NULL. "
+            << " offset is " << offset << endl;
+      return NDBT_FAILED;
+    }
+  }
+
+  return NDBT_OK;
+}
+
 int HugoOperations::pkUpdateRecord(Ndb* pNdb,
 				   int recordNo,
 				   int numRecords,
@@ -259,12 +382,14 @@ int HugoOperations::pkUpdateRecord(Ndb* pNdb,
     NdbOperation* pOp = getOperation(pTrans, NdbOperation::UpdateRequest);
     if (pOp == NULL) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     check = pOp->updateTuple();
     if( check == -1 ) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
@@ -293,6 +418,7 @@ HugoOperations::setValues(NdbOperation* pOp, int rowId, int updateId)
     if (tab.getColumn(a)->getPrimaryKey() == false){
       if(setValueForAttr(pOp, a, rowId, updateId ) != 0){ 
 	ERR(pTrans->getNdbError());
+        setNdbError(pTrans->getNdbError());
 	return NDBT_FAILED;
       }
     }
@@ -311,17 +437,20 @@ int HugoOperations::pkInsertRecord(Ndb* pNdb,
     NdbOperation* pOp = getOperation(pTrans, NdbOperation::InsertRequest);
     if (pOp == NULL) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     check = pOp->insertTuple();
     if( check == -1 ) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     if(setValues(pOp, r+recordNo, updatesValue) != NDBT_OK)
     {
+      m_error.code = pTrans->getNdbError().code;
       return NDBT_FAILED;
     }
 
@@ -342,12 +471,14 @@ int HugoOperations::pkWriteRecord(Ndb* pNdb,
     NdbOperation* pOp = pTrans->getNdbOperation(tab.getName());	
     if (pOp == NULL) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     check = pOp->writeTuple();
     if( check == -1 ) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
@@ -365,6 +496,7 @@ int HugoOperations::pkWriteRecord(Ndb* pNdb,
       if (tab.getColumn(a)->getPrimaryKey() == false){
 	if(setValueForAttr(pOp, a, recordNo+r, updatesValue ) != 0){ 
 	  ERR(pTrans->getNdbError());
+          setNdbError(pTrans->getNdbError());
 	  return NDBT_FAILED;
 	}
       }
@@ -382,12 +514,14 @@ int HugoOperations::pkWritePartialRecord(Ndb* pNdb,
     NdbOperation* pOp = pTrans->getNdbOperation(tab.getName());	
     if (pOp == NULL) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     check = pOp->writeTuple();
     if( check == -1 ) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
@@ -412,12 +546,14 @@ int HugoOperations::pkDeleteRecord(Ndb* pNdb,
     NdbOperation* pOp = getOperation(pTrans, NdbOperation::DeleteRequest);
     if (pOp == NULL) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     check = pOp->deleteTuple();
     if( check == -1 ) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
@@ -441,10 +577,12 @@ int HugoOperations::execute_Commit(Ndb* pNdb,
   const NdbError err = pTrans->getNdbError();
   if( check == -1 || err.code) {
     ERR(err);
+    setNdbError(err);
     NdbOperation* pOp = pTrans->getNdbErrorOperation();
     if (pOp != NULL){
       const NdbError err2 = pOp->getNdbError();
       ERR(err2);
+      setNdbError(err2);
     }
     if (err.code == 0)
       return NDBT_FAILED;
@@ -463,6 +601,7 @@ int HugoOperations::execute_Commit(Ndb* pNdb,
     case -1:
       const NdbError err = pTrans->getNdbError();
       ERR(err);
+      setNdbError(err);
       return (err.code > 0 ? err.code : NDBT_FAILED);
     }
 
@@ -490,12 +629,16 @@ int HugoOperations::execute_NoCommit(Ndb* pNdb, AbortOption eao){
   const NdbError err = pTrans->getNdbError();
   if( check == -1 || err.code) {
     ERR(err);
+    setNdbError(err);
     const NdbOperation* pOp = pTrans->getNdbErrorOperation();
     while (pOp != NULL)
     {
       const NdbError err2 = pOp->getNdbError();
       if (err2.code)
+      {
 	ERR(err2);
+        setNdbError(err2);
+      }
       pOp = pTrans->getNextCompletedOperation(pOp);
     }
     if (err.code == 0)
@@ -515,6 +658,7 @@ int HugoOperations::execute_NoCommit(Ndb* pNdb, AbortOption eao){
     case -1:
       const NdbError err = pTrans->getNdbError();
       ERR(err);
+      setNdbError(err);
       return (err.code > 0 ? err.code : NDBT_FAILED);
     }
 
@@ -540,6 +684,7 @@ int HugoOperations::execute_Rollback(Ndb* pNdb){
   if( check == -1 ) {
     const NdbError err = pTrans->getNdbError();
     ERR(err);
+    setNdbError(err);
     return NDBT_FAILED;
   }
   return NDBT_OK;
@@ -617,7 +762,8 @@ HugoOperations::HugoOperations(const NdbDictionary::Table& _tab,
 			       const NdbDictionary::Index* idx):
   UtilTransactions(_tab, idx),
   pIndexScanOp(NULL),
-  calc(_tab)
+  calc(_tab),
+  m_quiet(false)
 {
 }
 
@@ -640,6 +786,7 @@ HugoOperations::equalForRow(NdbOperation* pOp, int row)
       if(equalForAttr(pOp, a, row) != 0)
       {
         ERR(pOp->getNdbError());
+        setNdbError(pOp->getNdbError());
         return NDBT_FAILED;
       }
     }
@@ -700,13 +847,32 @@ int HugoOperations::setValueForAttr(NdbOperation* pOp,
 				      int updateId){
   const NdbDictionary::Column* attr = tab.getColumn(attrId);     
   
-  int len = attr->getSizeInBytes();
-  char buf[NDB_MAX_TUPLE_SIZE];
-  memset(buf, 0, sizeof(buf));
-  Uint32 real_len;
-  const char * value = calc.calcValue(rowId, attrId, 
-				      updateId, buf, len, &real_len);
-  return pOp->setValue( attr->getName(), value, real_len);
+  if (! (attr->getType() == NdbDictionary::Column::Blob))
+  {
+    int len = attr->getSizeInBytes();
+    char buf[NDB_MAX_TUPLE_SIZE];
+    memset(buf, 0, sizeof(buf));
+    Uint32 real_len;
+    const char * value = calc.calcValue(rowId, attrId,
+                                        updateId, buf, len, &real_len);
+    return pOp->setValue( attr->getName(), value, real_len);
+  }
+  else
+  {
+    char buf[32000];
+    int len = (int)sizeof(buf);
+    Uint32 real_len;
+    const char * value = calc.calcValue(rowId, attrId,
+                                        updateId, buf, len, &real_len);
+    NdbBlob * b = pOp->getBlobHandle(attrId);
+    if (b == 0)
+      return -1;
+
+    if (real_len == 0)
+      return b->setNull();
+    else
+      return b->setValue(value, real_len);
+  }
 }
 
 int
@@ -819,6 +985,7 @@ int HugoOperations::indexReadRecords(Ndb*, const char * idxName, int recordNo,
     NdbOperation* pOp = pTrans->getNdbIndexOperation(idxName, tab.getName());
     if (pOp == NULL) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
@@ -828,6 +995,7 @@ int HugoOperations::indexReadRecords(Ndb*, const char * idxName, int recordNo,
       check = pOp->readTuple();
     if( check == -1 ) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
@@ -840,6 +1008,7 @@ int HugoOperations::indexReadRecords(Ndb*, const char * idxName, int recordNo,
       if((rows[r]->attributeStore(a) = 
 	  pOp->getValue(tab.getColumn(a)->getName())) == 0) {
 	ERR(pTrans->getNdbError());
+        setNdbError(pTrans->getNdbError());
 	return NDBT_FAILED;
       }
     } 
@@ -860,12 +1029,14 @@ HugoOperations::indexUpdateRecord(Ndb*,
     NdbOperation* pOp = pTrans->getNdbIndexOperation(idxName, tab.getName());
     if (pOp == NULL) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     check = pOp->updateTuple();
     if( check == -1 ) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
@@ -878,6 +1049,7 @@ HugoOperations::indexUpdateRecord(Ndb*,
       if (tab.getColumn(a)->getPrimaryKey() == false){
 	if(setValueForAttr(pOp, a, recordNo+r, updatesValue ) != 0){ 
 	  ERR(pTrans->getNdbError());
+          setNdbError(pTrans->getNdbError());
 	  return NDBT_FAILED;
 	}
       }
@@ -904,6 +1076,7 @@ HugoOperations::scanReadRecords(Ndb* pNdb, NdbScanOperation::LockMode lm,
     if((rows[0]->attributeStore(a) = 
 	pOp->getValue(tab.getColumn(a)->getName())) == 0) {
       ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
   } 
@@ -914,4 +1087,78 @@ HugoOperations::scanReadRecords(Ndb* pNdb, NdbScanOperation::LockMode lm,
   return 0;
 }
 
+int
+HugoOperations::releaseLockHandles(Ndb* pNdb,
+                                   Vector<const NdbLockHandle*>& lockHandles,
+                                   int offset,
+                                   int numRecords)
+{
+  int totalSize = lockHandles.size();
+  if (numRecords == ~(0))
+  {
+    numRecords = totalSize - offset;
+  }
+
+  if (totalSize < (offset + numRecords))
+  {
+    g_err << "ERROR : LockHandles size is "
+          << lockHandles.size()
+          << " offset (" 
+          << offset
+          << ") and/or numRecords ("
+          << numRecords
+          << ") too large." << endl;
+    return NDBT_FAILED;
+  }
+
+  for (int i = 0; i < numRecords; i++)
+  {
+    const NdbLockHandle* lh = lockHandles[offset + i];
+    if (lh != NULL)
+    {
+      if (pTrans->releaseLockHandle(lh) != 0)
+      {
+        ERR(pTrans->getNdbError());
+        setNdbError(pTrans->getNdbError());
+        return NDBT_FAILED;
+      }
+      const NdbLockHandle* nullPtr = NULL;
+      //lockHandles.set(nullPtr, offset + i, nullPtr);
+    }
+    else
+    {
+      g_err << "ERROR : LockHandle number "
+            << i+offset << " is NULL. "
+            << " offset is " << offset << endl;
+      return NDBT_FAILED;
+    }
+  }
+
+  return NDBT_OK;
+
+}
+
+static void
+update(const NdbError & _err)
+{
+  NdbError & error = (NdbError &) _err;
+  ndberror_struct ndberror = (ndberror_struct)error;
+  ndberror_update(&ndberror);
+  error = NdbError(ndberror);
+}
+
+const NdbError &
+HugoOperations::getNdbError() const
+{
+  update(m_error);
+  return m_error;
+}
+
+void
+HugoOperations::setNdbError(const NdbError& error)
+{
+  m_error.code = error.code ? error.code : 1;
+}
+
 template class Vector<HugoOperations::RsPair>;
+template class Vector<const NdbLockHandle*>;
