@@ -17,19 +17,11 @@
 */
 
 #include <ndb_global.h>
-//#define NDB_OPTS_SKIP_USAGE
 #include <ndb_opts.h>
-#include <my_pthread.h>
-#include <ndb_rand.h>
 
 #include "NDBT.hpp"
 #include "NDBT_Test.hpp"
 
-#include <PortDefs.h>
-
-#include <time.h>
-
-
 NDBT_Context::NDBT_Context(Ndb_cluster_connection& con)
   : m_cluster_connection(con)
 {
@@ -168,7 +160,7 @@ NDBT_Context::incProperty(const char * name){
 
 void  NDBT_Context::setProperty(const char* _name, const char* _val){ 
   NdbMutex_Lock(propertyMutexPtr);
-  const bool b = props.put(_name, _val);
+  const bool b = props.put(_name, _val, true);
   assert(b == true);
   NdbCondition_Broadcast(propertyCondPtr);
   NdbMutex_Unlock(propertyMutexPtr);
@@ -268,6 +260,7 @@ NDBT_Step::NDBT_Step(NDBT_TestCase* ptest, const char* pname,
 {
 }
 
+#include "../../src/ndbapi/ndb_internal.hpp"
 
 int
 NDBT_Step::setUp(Ndb_cluster_connection& con){
@@ -282,6 +275,9 @@ NDBT_Step::setUp(Ndb_cluster_connection& con){
     m_ndb = new Ndb(&con, "TEST_DB" );
     m_ndb->init(1024);
 
+    Ndb_internal::setForceShortRequests(m_ndb, 
+                                        m_ctx->suite->getForceShort());
+
     int result = m_ndb->waitUntilReady(300); // 5 minutes
     if (result != 0){
       g_err << "Ndb was not ready" << endl;
@@ -543,9 +539,12 @@ void NDBT_TestCaseImpl1::startStepInThread(int stepNo, NDBT_Context* ctx){
   pStep->setContext(ctx);
   char buf[16];
   BaseString::snprintf(buf, sizeof(buf), "step_%d", stepNo);
+  Uint32 stackSize = ctx->getProperty(NDBT_TestCase::getStepThreadStackSizePropName(), 
+                                      Uint32(0));
+
   NdbThread* pThread = NdbThread_Create(runStep_C,
 					(void**)pStep,
-                                        0, // default stack size
+                                        stackSize,
 					buf, 
 					NDB_THREAD_PRIO_LOW);
   threads.push_back(pThread);
@@ -792,6 +791,7 @@ NDBT_TestSuite::NDBT_TestSuite(const char* pname) :
    temporaryTables = false;
    runonce = false;
    m_noddl = false;
+   m_forceShort = false;
 }
 
 
@@ -833,6 +833,10 @@ bool NDBT_TestSuite::getLogging() const {
   return m_logging;
 }
 
+bool NDBT_TestSuite::getForceShort() const {
+  return m_forceShort;
+}
+
 bool NDBT_TestSuite::timerIsOn(){
   return (timer != 0);
 }
@@ -927,6 +931,8 @@ NDBT_TestSuite::executeOneCtx(Ndb_cluster_connection& con,
     Ndb ndb(&con, "TEST_DB");
     ndb.init(1024);
 
+    Ndb_internal::setForceShortRequests(&ndb, m_forceShort);
+
     int result = ndb.waitUntilReady(300); // 5 minutes
     if (result != 0){
       g_err << name <<": Ndb was not ready" << endl;
@@ -1256,14 +1262,6 @@ int NDBT_TestSuite::reportAllTables(const char* _testname){
   return result;
 }
 
-enum test_options {
-  OPT_INTERACTIVE = NDB_STD_OPTIONS_LAST,
-  OPT_PRINT,
-  OPT_PRINT_HTML,
-  OPT_PRINT_CASES
-  
-};
-
 static int opt_print = false;
 static int opt_print_html = false;
 static int opt_print_cases = false;
@@ -1272,21 +1270,22 @@ static int opt_loops;
 static int opt_timer;
 static char * opt_testname = NULL;
 static int opt_verbose;
-static int opt_seed = 0;
+unsigned opt_seed = 0;
 static int opt_nologging = 0;
 static int opt_temporary = 0;
 static int opt_noddl = 0;
+static int opt_forceShort = 0;
 
 static struct my_option my_long_options[] =
 {
   NDB_STD_OPTS(""),
-  { "print", OPT_PRINT, "Print execution tree",
+  { "print", NDB_OPT_NOSHORT, "Print execution tree",
     (uchar **) &opt_print, (uchar **) &opt_print, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "print_html", OPT_PRINT_HTML, "Print execution tree in html table format",
+  { "print_html", NDB_OPT_NOSHORT, "Print execution tree in html table format",
     (uchar **) &opt_print_html, (uchar **) &opt_print_html, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "print_cases", OPT_PRINT_CASES, "Print list of test cases",
+  { "print_cases", NDB_OPT_NOSHORT, "Print list of test cases",
     (uchar **) &opt_print_cases, (uchar **) &opt_print_cases, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { "records", 'r', "Number of records", 
@@ -1295,9 +1294,9 @@ static struct my_option my_long_options[] =
   { "loops", 'l', "Number of loops",
     (uchar **) &opt_loops, (uchar **) &opt_loops, 0,
     GET_INT, REQUIRED_ARG, 5, 0, 0, 0, 0, 0 },
-  { "seed", 1024, "Random seed",
+  { "seed", NDB_OPT_NOSHORT, "Random seed",
     (uchar **) &opt_seed, (uchar **) &opt_seed, 0,
-    GET_INT, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
+    GET_UINT, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
   { "testname", 'n', "Name of test to run",
     (uchar **) &opt_testname, (uchar **) &opt_testname, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
@@ -1310,12 +1309,16 @@ static struct my_option my_long_options[] =
   { "temporary-tables", 'T', "Create temporary table(s)",
     (uchar **) &opt_temporary, (uchar **) &opt_temporary, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "nologging", 0, "Create table(s) wo/ logging",
+  { "nologging", NDB_OPT_NOSHORT, "Create table(s) wo/ logging",
     (uchar **) &opt_nologging, (uchar **) &opt_nologging, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "noddl", 0, "Don't create/drop tables as part of running tests",
+  { "noddl", NDB_OPT_NOSHORT,
+    "Don't create/drop tables as part of running tests",
     (uchar**) &opt_noddl, (uchar**) &opt_noddl, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+  { "forceshortreqs", NDB_OPT_NOSHORT, "Use short signals for NdbApi requests",
+    (uchar**) &opt_forceShort, (uchar**) &opt_forceShort, 0,
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { 0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
@@ -1325,7 +1328,7 @@ const char *load_default_groups[]= { "mysql_cluster",0 };
 
 static void short_usage_sub(void)
 {
-  ndb_short_usage_sub(my_progname, "[tabname1 tabname2 ... tabnameN]");
+  ndb_short_usage_sub("[tabname1 tabname2 ... tabnameN]");
 }
 static void usage()
 {
@@ -1359,7 +1362,7 @@ int NDBT_TestSuite::execute(int argc, const char** argv){
   if (!my_progname)
     my_progname= _argv[0];
 
-  ndb_opt_set_usage_funcs(NULL, short_usage_sub, usage);
+  ndb_opt_set_usage_funcs(short_usage_sub, usage);
 
   load_defaults("my",load_default_groups,&argc,&_argv);
 
@@ -1386,10 +1389,11 @@ int NDBT_TestSuite::execute(int argc, const char** argv){
     setLogging(false);
   temporaryTables = opt_temporary;
   m_noddl = opt_noddl;
+  m_forceShort = opt_forceShort;
 
   if (opt_seed == 0)
   {
-    opt_seed = (int)NdbTick_CurrentMillisecond();
+    opt_seed = (unsigned)NdbTick_CurrentMillisecond();
   }
   ndbout_c("random seed: %u", opt_seed);
   srand(opt_seed);
@@ -1465,7 +1469,7 @@ int NDBT_TestSuite::execute(int argc, const char** argv){
     return 0;
   }
 
-  Ndb_cluster_connection con;
+  Ndb_cluster_connection con(opt_ndb_connectstring, opt_ndb_nodeid);
   if(m_connect_cluster && con.connect(12, 5, 1))
   {
     return NDBT_ProgramExit(NDBT_FAILED);
