@@ -73,11 +73,14 @@ int
 NdbRestarter::restartOneDbNode(int _nodeId,
 			       bool inital,
 			       bool nostart,
-			       bool abort){
+			       bool abort,
+                               bool force)
+{
   return restartNodes(&_nodeId, 1,
                       (inital ? NRRF_INITIAL : 0) |
                       (nostart ? NRRF_NOSTART : 0) |
-                      (abort ? NRRF_ABORT : 0));
+                      (abort ? NRRF_ABORT : 0) |
+                      (force ? NRRF_FORCE : 0));
 }
 
 int
@@ -88,14 +91,16 @@ NdbRestarter::restartNodes(int * nodes, int cnt,
     return -1;
 
   int ret = 0;
-  
-  if ((ret = ndb_mgm_restart2(handle, cnt, nodes,
+  int unused;
+  if ((ret = ndb_mgm_restart4(handle, cnt, nodes,
                               (flags & NRRF_INITIAL),
                               (flags & NRRF_NOSTART),
-                              (flags & NRRF_ABORT))) <= 0)
+                              (flags & NRRF_ABORT),
+                              (flags & NRRF_FORCE),
+                              &unused)) <= 0)
   {
     /**
-     * ndb_mgm_restart2 returned error, one reason could
+     * ndb_mgm_restart4 returned error, one reason could
      * be that the node have not stopped fast enough!
      * Check status of the node to see if it's on the 
      * way down. If that's the case ignore the error
@@ -104,7 +109,7 @@ NdbRestarter::restartNodes(int * nodes, int cnt,
     if (getStatus() != 0)
       return -1;
 
-    g_info << "ndb_mgm_restart2 returned with error, checking node state"
+    g_info << "ndb_mgm_restart4 returned with error, checking node state"
            << endl;
 
     for (int j = 0; j<cnt; j++)
@@ -623,12 +628,15 @@ int NdbRestarter::getNumDbNodes(){
 
 int NdbRestarter::restartAll(bool initial,
 			     bool nostart,
-			     bool abort){
-  
+			     bool abort,
+                             bool force)
+{
   if (!isConnected())
     return -1;
 
-  if (ndb_mgm_restart2(handle, 0, NULL, initial, 1, abort) == -1) {
+  int unused;
+  if (ndb_mgm_restart4(handle, 0, NULL, initial, 1, abort,
+                       force, &unused) == -1) {
     MGMERR(handle);
     g_err  << "Could not restart(stop) all nodes " << endl;
     // return -1; Continue anyway - Magnus
@@ -898,4 +906,91 @@ loop:
   return 0;
 }
 
+int
+NdbRestarter::getMasterNodeVersion(int& version)
+{
+  int masterNodeId = getMasterNodeId();
+  if (masterNodeId != -1)
+  {
+    for(size_t i = 0; i < ndbNodes.size(); i++)
+    {
+      if (ndbNodes[i].node_id == masterNodeId)
+      {
+        version =  ndbNodes[i].version;
+        return 0;
+      }
+    }
+  }
+
+  g_err << "Could not find node info for master node id "
+        << masterNodeId << endl;
+  return -1;
+}
+
+int
+NdbRestarter::getNodeTypeVersionRange(ndb_mgm_node_type type,
+                                      int& minVer,
+                                      int& maxVer)
+{
+  if (!isConnected())
+    return -1;
+  
+  if (getStatus() != 0)
+    return -1;
+  
+  Vector<ndb_mgm_node_state>* nodeVec = NULL;
+
+  switch (type)
+  {
+  case NDB_MGM_NODE_TYPE_API:
+    nodeVec = &apiNodes;
+    break;
+  case NDB_MGM_NODE_TYPE_NDB:
+    nodeVec = &ndbNodes;
+    break;
+  case NDB_MGM_NODE_TYPE_MGM:
+    nodeVec = &mgmNodes;
+    break;
+  default:
+    g_err << "Bad node type : " << type << endl;
+    return -1;
+  }
+
+  if (nodeVec->size() == 0)
+  {
+    g_err << "No nodes of type " << type << " online" << endl;
+    return -1;
+  }
+
+  minVer = 0;
+  maxVer = 0;
+  
+  for(size_t i = 0; i < nodeVec->size(); i++)
+  {
+    int nodeVer = (*nodeVec)[i].version;
+    if ((minVer == 0) ||
+        (nodeVer < minVer))
+      minVer = nodeVer;
+    
+    if (nodeVer > maxVer)
+      maxVer = nodeVer;
+  }
+
+  return 0;
+}
+
+int
+NdbRestarter::getNodeStatus(int nodeid)
+{
+  if (getStatus() != 0)
+    return -1;
+
+  for (size_t n = 0; n < ndbNodes.size(); n++)
+  {
+    if (ndbNodes[n].node_id == nodeid)
+      return ndbNodes[n].node_status;
+  }
+  return -1;
+}
+
 template class Vector<ndb_mgm_node_state>;
