@@ -29,8 +29,11 @@
 #include <ndb_limits.h>
 #include <NdbAutoPtr.hpp>
 
+#include <sys/stat.h>
+
 #include "../../../../sql/ha_ndbcluster_tables.h"
 extern NdbRecordPrintFormat g_ndbrecord_print_format;
+extern bool ga_skip_unknown_objects;
 
 Uint16 Twiddle16(Uint16 in); // Byte shift 16-bit data
 Uint32 Twiddle32(Uint32 in); // Byte shift 32-bit data
@@ -429,13 +432,39 @@ RestoreMetaData::readMetaTableDesc() {
       delete dst;
     obj.m_objPtr = dst;
 
+    if (!m_hostByteOrder)
+    {
+      /**
+       * Bloddy byte-array, need to twiddle
+       */
+      Vector<Uint32> values;
+      Uint32 len = dst->getMapLen();
+      Uint32 zero = 0;
+      values.fill(len - 1, zero);
+      dst->getMapValues(values.getBase(), values.size());
+      for (Uint32 i = 0; i<len; i++)
+      {
+        values[i] = Twiddle16(values[i]);
+      }
+      dst->setMap(values.getBase(), values.size());
+    }
+
     m_objects.push(obj, 0); // Put first
     return true;
     break;
   }
   default:
-    err << "Unsupported table type!! " << sectionInfo[2] << endl;
-    return false;
+    if (ga_skip_unknown_objects)
+    {
+      info << "Skipping schema object with unknown table type "
+           << sectionInfo[2] << endl;
+      return true;
+    }
+    else
+    {
+      err << "Unsupported table type!! " << sectionInfo[2] << endl;
+      return false;
+    }
   }
   if (errcode)
   {
@@ -501,8 +530,12 @@ RestoreMetaData::markSysTables()
         strcmp(tableName, OLD_NDB_REP_DB "/def/" OLD_NDB_SCHEMA_TABLE) == 0 ||
         strcmp(tableName, NDB_REP_DB "/def/" NDB_APPLY_TABLE) == 0 ||
         strcmp(tableName, NDB_REP_DB "/def/" NDB_SCHEMA_TABLE)== 0 )
-
-      table->isSysTable = true;
+    {
+      table->m_isSysTable = true;
+      if (strcmp(tableName, "SYSTAB_0") == 0 ||
+          strcmp(tableName, "sys/def/SYSTAB_0") == 0)
+        table->m_isSYSTAB_0 = true;
+    }
   }
   for (i = 0; i < getNoOfTables(); i++) {
     TableS* blobTable = allTables[i];
@@ -517,8 +550,8 @@ RestoreMetaData::markSysTables()
       for (j = 0; j < getNoOfTables(); j++) {
         TableS* table = allTables[j];
         if (table->getTableId() == (Uint32) id1) {
-          if (table->isSysTable)
-            blobTable->isSysTable = true;
+          if (table->m_isSysTable)
+            blobTable->m_isSysTable = true;
           blobTable->m_main_table = table;
           blobTable->m_main_column_id = id2;
           break;
@@ -565,7 +598,16 @@ RestoreMetaData::fixBlobs()
           break;
         }
       }
-      assert(blobTable != NULL);
+      if (blobTable == NULL)
+      {
+        /* Corrupt backup, has main table, but no blob table */
+        err << "Table " << table->m_dictTable->getName()
+            << " has blob column " << j << " (" 
+            << c->m_name.c_str()
+            << ") with missing parts table in backup."
+            << endl;
+        return false;
+      }
       assert(blobTable->m_dictTable != NULL);
       NdbTableImpl& bt = NdbTableImpl::getImpl(*blobTable->m_dictTable);
       const char* colName = c->m_blobVersion == 1 ? "DATA" : "NDB$DATA";
@@ -581,29 +623,26 @@ RestoreMetaData::fixBlobs()
 bool
 RestoreMetaData::readGCPEntry() {
 
-  Uint32 data[4];
+  BackupFormat::CtlFile::GCPEntry dst;
   
-  BackupFormat::CtlFile::GCPEntry * dst = 
-    (BackupFormat::CtlFile::GCPEntry *)&data[0];
-  
-  if(buffer_read(dst, 4, 4) != 4){
+  if(buffer_read(&dst, 1, sizeof(dst)) != sizeof(dst)){
     err << "readGCPEntry read error" << endl;
     return false;
   }
   
-  dst->SectionType = ntohl(dst->SectionType);
-  dst->SectionLength = ntohl(dst->SectionLength);
+  dst.SectionType = ntohl(dst.SectionType);
+  dst.SectionLength = ntohl(dst.SectionLength);
   
-  if(dst->SectionType != BackupFormat::GCP_ENTRY){
+  if(dst.SectionType != BackupFormat::GCP_ENTRY){
     err << "readGCPEntry invalid format" << endl;
     return false;
   }
   
-  dst->StartGCP = ntohl(dst->StartGCP);
-  dst->StopGCP = ntohl(dst->StopGCP);
+  dst.StartGCP = ntohl(dst.StartGCP);
+  dst.StopGCP = ntohl(dst.StopGCP);
   
-  m_startGCP = dst->StartGCP;
-  m_stopGCP = dst->StopGCP;
+  m_startGCP = dst.StartGCP;
+  m_stopGCP = dst.StopGCP;
   return true;
 }
 
@@ -662,7 +701,8 @@ TableS::TableS(Uint32 version, NdbTableImpl* tableImpl)
   m_max_auto_val= 0;
   m_noOfRecords= 0;
   backupVersion = version;
-  isSysTable = false;
+  m_isSysTable = false;
+  m_isSYSTAB_0 = false;
   m_main_table = NULL;
   m_main_column_id = ~(Uint32)0;
   
@@ -768,8 +808,9 @@ RestoreDataIterator::free_bitfield_storage()
 Uint32
 RestoreDataIterator::get_free_bitfield_storage() const
 {
-  return (m_bitfield_storage_ptr + m_bitfield_storage_len) - 
-    m_bitfield_storage_curr_ptr;
+  
+  return Uint32((m_bitfield_storage_ptr + m_bitfield_storage_len) - 
+    m_bitfield_storage_curr_ptr);
 }
 
 Uint32*
@@ -808,7 +849,7 @@ TableS * TupleS::getTable() const {
   return m_currentTable;
 }
 
-const AttributeDesc * TupleS::getDesc(int i) const {
+AttributeDesc * TupleS::getDesc(int i) const {
   return m_currentTable->allAttributesDesc[i];
 }
 
@@ -927,6 +968,11 @@ RestoreDataIterator::getNextTuple(int  & res)
   return &m_tuple;
 } // RestoreDataIterator::getNextTuple
 
+TableS *
+RestoreDataIterator::getCurrentTable()
+{
+  return m_currentTable;
+}
 
 int
 RestoreDataIterator::readTupleData_packed(Uint32 *buf_ptr, 
@@ -1376,7 +1422,7 @@ Uint32 BackupFile::buffer_get_ptr_ahead(void **p_buf_ptr, Uint32 size, Uint32 nm
         }
       }
       m_file_pos -= r;
-      m_buffer_data_left += r;
+      m_buffer_data_left += (Uint32)r;
       //move to the end of buffer
       m_buffer_ptr = buffer_end;
     }
@@ -1384,7 +1430,7 @@ Uint32 BackupFile::buffer_get_ptr_ahead(void **p_buf_ptr, Uint32 size, Uint32 nm
     {
       memmove(m_buffer, m_buffer_ptr, m_buffer_data_left);
       int error;
-      size_t r = azread(&m_file,
+      Uint32 r = azread(&m_file,
                         ((char *)m_buffer) + m_buffer_data_left,
                         m_buffer_sz - m_buffer_data_left, &error);
       m_file_pos += r;
@@ -1697,7 +1743,7 @@ RestoreDataIterator::validateFragmentFooter() {
 } // RestoreDataIterator::getFragmentFooter
 
 AttributeDesc::AttributeDesc(NdbDictionary::Column *c)
-  : m_column(c)
+  : m_column(c), truncation_detected(false)
 {
   size = 8*NdbColumnImpl::getImpl(* c).m_attrSize;
   arraySize = NdbColumnImpl::getImpl(* c).m_arraySize;
@@ -1742,6 +1788,35 @@ void TableS::createAttr(NdbDictionary::Column *column)
   m_variableAttribs.push_back(d);
 } // TableS::createAttr
 
+bool
+TableS::get_auto_data(const TupleS & tuple, Uint32 * syskey, Uint64 * nextid) const
+{
+  /*
+    Read current (highest) auto_increment value for
+    a table. Currently there can only be one per table.
+    The values are stored in sustable SYSTAB_0 as
+    {SYSKEY,NEXTID} values where SYSKEY (32-bit) is
+    the table_id and NEXTID (64-bit) is the next auto_increment
+    value in the sequence (note though that sequences of
+    values can have been fetched and that are cached in NdbAPI).
+    SYSTAB_0 can contain other data so we need to check that
+    the found SYSKEY value is a valid table_id (< 0x10000000).
+   */
+  AttributeData * attr_data = tuple.getData(0);
+  AttributeDesc * attr_desc = tuple.getDesc(0);
+  const AttributeS attr1 = {attr_desc, *attr_data};
+  memcpy(syskey ,attr1.Data.u_int32_value, sizeof(Uint32));
+  attr_data = tuple.getData(1);
+  attr_desc = tuple.getDesc(1);
+  const AttributeS attr2 = {attr_desc, *attr_data};
+  memcpy(nextid, attr2.Data.u_int64_value, sizeof(Uint64));
+  if (*syskey < 0x10000000)
+  {
+    return true;
+  }
+  return false;
+};
+
 Uint16 Twiddle16(Uint16 in)
 {
   Uint16 retVal = 0;
@@ -1921,7 +1996,7 @@ RestoreLogIterator::getNextLogEntry(int & res) {
     if(unlikely(!m_hostByteOrder))
       *(Uint32*)ah = Twiddle32(*(Uint32*)ah);
 
-    attr->Desc = (* tab)[ah->getAttributeId()];
+    attr->Desc = tab->getAttributeDesc(ah->getAttributeId());
     assert(attr->Desc != 0);
 
     const Uint32 sz = ah->getDataSize();
@@ -1973,7 +2048,7 @@ operator<<(NdbOut& ndbout, const TupleS& tuple)
     if (i > 0)
       ndbout << g_ndbrecord_print_format.fields_terminated_by;
     AttributeData * attr_data = tuple.getData(i);
-    const AttributeDesc * attr_desc = tuple.getDesc(i);
+    AttributeDesc * attr_desc = tuple.getDesc(i);
     const AttributeS attr = {attr_desc, *attr_data};
     debug << i << " " << attr_desc->m_column->getName();
     ndbout << attr;
