@@ -37,9 +37,10 @@ static Uint32 g_tableCompabilityMask = 0;
 static int ga_nodeId = 0;
 static int ga_nParallelism = 128;
 static int ga_backupId = 0;
-static bool ga_dont_ignore_systab_0 = false;
+bool ga_dont_ignore_systab_0 = false;
 static bool ga_no_upgrade = false;
 static bool ga_promote_attributes = false;
+static bool ga_demote_attributes = false;
 static Vector<class BackupConsumer *> g_consumers;
 static BackupPrinter* g_printer = NULL;
 
@@ -64,6 +65,17 @@ Vector<BaseString> g_include_databases, g_exclude_databases;
 NdbRecordPrintFormat g_ndbrecord_print_format;
 unsigned int opt_no_binlog;
 
+class RestoreOption
+{
+public:
+  virtual ~RestoreOption() { }
+  int optid;
+  BaseString argument;
+};
+
+Vector<class RestoreOption *> g_include_exclude;
+static void save_include_exclude(int optid, char * argument);
+
 /**
  * print and restore flags
  */
@@ -80,30 +92,19 @@ static int _restore_data = 0;
 static int _restore_meta = 0;
 static int _no_restore_disk = 0;
 static bool _preserve_trailing_spaces = false;
+static bool ga_disable_indexes = false;
+static bool ga_rebuild_indexes = false;
+bool ga_skip_unknown_objects = false;
 BaseString g_options("ndb_restore");
 
 const char *load_default_groups[]= { "mysql_cluster","ndb_restore",0 };
 
 enum ndb_restore_options {
-  OPT_PRINT= NDB_STD_OPTIONS_LAST,
-  OPT_PRINT_DATA,
-  OPT_PRINT_LOG,
-  OPT_PRINT_META,
-  OPT_BACKUP_PATH,
-  OPT_HEX_FORMAT,
-  OPT_FIELDS_ENCLOSED_BY,
-  OPT_FIELDS_TERMINATED_BY,
-  OPT_FIELDS_OPTIONALLY_ENCLOSED_BY,
-  OPT_LINES_TERMINATED_BY,
-  OPT_APPEND,
-  OPT_PROGRESS_FREQUENCY,
-  OPT_NO_BINLOG,
-  OPT_VERBOSE,
+  OPT_VERBOSE = NDB_STD_OPTIONS_LAST,
   OPT_INCLUDE_TABLES,
   OPT_EXCLUDE_TABLES,
   OPT_INCLUDE_DATABASES,
-  OPT_EXCLUDE_DATABASES,
-  OPT_EXCLUDE_MISSING_COLUMNS
+  OPT_EXCLUDE_DATABASES
 };
 static const char *opt_fields_enclosed_by= NULL;
 static const char *opt_fields_terminated_by= NULL;
@@ -121,7 +122,7 @@ static struct my_option my_long_options[] =
 {
   NDB_STD_OPTS("ndb_restore"),
   { "connect", 'c', "same as --connect-string",
-    (uchar**) &opt_connect_str, (uchar**) &opt_connect_str, 0,
+    (uchar**) &opt_ndb_connectstring, (uchar**) &opt_ndb_connectstring, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
   { "nodeid", 'n', "Backup files from node with id",
     (uchar**) &ga_nodeId, (uchar**) &ga_nodeId, 0,
@@ -145,6 +146,11 @@ static struct my_option my_long_options[] =
     "Allow attributes to be promoted when restoring data from backup",
     (uchar**) &ga_promote_attributes, (uchar**) &ga_promote_attributes, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+  { "lossy-conversions", 'L',
+    "Allow lossy conversions for attributes (type demotions or integral"
+    " signed/unsigned type changes) when restoring data from backup",
+    (uchar**) &ga_demote_attributes, (uchar**) &ga_demote_attributes, 0,
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { "preserve-trailing-spaces", 'P',
     "Allow to preserve the tailing spaces (including paddings) When char->varchar or binary->varbinary is promoted",
     (uchar**) &_preserve_trailing_spaces, (uchar**)_preserve_trailing_spaces , 0,
@@ -167,23 +173,23 @@ static struct my_option my_long_options[] =
     "(parallelism can be 1 to 1024)", 
     (uchar**) &ga_nParallelism, (uchar**) &ga_nParallelism, 0,
     GET_INT, REQUIRED_ARG, 128, 1, 1024, 0, 1, 0 },
-  { "print", OPT_PRINT, "Print metadata, data and log to stdout",
+  { "print", NDB_OPT_NOSHORT, "Print metadata, data and log to stdout",
     (uchar**) &_print, (uchar**) &_print, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "print_data", OPT_PRINT_DATA, "Print data to stdout", 
+  { "print_data", NDB_OPT_NOSHORT, "Print data to stdout",
     (uchar**) &_print_data, (uchar**) &_print_data, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "print_meta", OPT_PRINT_META, "Print meta data to stdout",
+  { "print_meta", NDB_OPT_NOSHORT, "Print meta data to stdout",
     (uchar**) &_print_meta, (uchar**) &_print_meta,  0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "print_log", OPT_PRINT_LOG, "Print log to stdout",
+  { "print_log", NDB_OPT_NOSHORT, "Print log to stdout",
     (uchar**) &_print_log, (uchar**) &_print_log,  0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "backup_path", OPT_BACKUP_PATH, "Path to backup files",
+  { "backup_path", NDB_OPT_NOSHORT, "Path to backup files",
     (uchar**) &ga_backupPath, (uchar**) &ga_backupPath, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
   { "dont_ignore_systab_0", 'f',
-    "Experimental. Do not ignore system table during restore.", 
+    "Do not ignore system table during --print-data.", 
     (uchar**) &ga_dont_ignore_systab_0, (uchar**) &ga_dont_ignore_systab_0, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { "ndb-nodegroup-map", OPT_NDB_NODEGROUP_MAP,
@@ -192,38 +198,38 @@ static struct my_option my_long_options[] =
     (uchar**) &opt_nodegroup_map_str,
     0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "fields-enclosed-by", OPT_FIELDS_ENCLOSED_BY,
+  { "fields-enclosed-by", NDB_OPT_NOSHORT,
     "Fields are enclosed by ...",
     (uchar**) &opt_fields_enclosed_by, (uchar**) &opt_fields_enclosed_by, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "fields-terminated-by", OPT_FIELDS_TERMINATED_BY,
+  { "fields-terminated-by", NDB_OPT_NOSHORT,
     "Fields are terminated by ...",
     (uchar**) &opt_fields_terminated_by,
     (uchar**) &opt_fields_terminated_by, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "fields-optionally-enclosed-by", OPT_FIELDS_OPTIONALLY_ENCLOSED_BY,
+  { "fields-optionally-enclosed-by", NDB_OPT_NOSHORT,
     "Fields are optionally enclosed by ...",
     (uchar**) &opt_fields_optionally_enclosed_by,
     (uchar**) &opt_fields_optionally_enclosed_by, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "hex", OPT_HEX_FORMAT, "print binary types in hex format", 
+  { "hex", NDB_OPT_NOSHORT, "print binary types in hex format", 
     (uchar**) &opt_hex_format, (uchar**) &opt_hex_format, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { "tab", 'T', "Creates tab separated textfile for each table to "
     "given path. (creates .txt files)",
    (uchar**) &tab_path, (uchar**) &tab_path, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-  { "append", OPT_APPEND, "for --tab append data to file", 
+  { "append", NDB_OPT_NOSHORT, "for --tab append data to file", 
     (uchar**) &opt_append, (uchar**) &opt_append, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "lines-terminated-by", OPT_LINES_TERMINATED_BY, "",
+  { "lines-terminated-by", NDB_OPT_NOSHORT, "",
     (uchar**) &opt_lines_terminated_by, (uchar**) &opt_lines_terminated_by, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "progress-frequency", OPT_PROGRESS_FREQUENCY,
+  { "progress-frequency", NDB_OPT_NOSHORT,
     "Print status uf restore periodically in given seconds", 
     (uchar**) &opt_progress_frequency, (uchar**) &opt_progress_frequency, 0,
     GET_INT, REQUIRED_ARG, 0, 0, 65535, 0, 0, 0 },
-  { "no-binlog", OPT_NO_BINLOG,
+  { "no-binlog", NDB_OPT_NOSHORT,
     "If a mysqld is connected and has binary log, do not log the restored data", 
     (uchar**) &opt_no_binlog, (uchar**) &opt_no_binlog, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
@@ -248,11 +254,24 @@ static struct my_option my_long_options[] =
     "Example: db1.t1,db3.t1",
     (uchar**) &opt_exclude_tables, (uchar**) &opt_exclude_tables, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "exclude-missing-columns", OPT_EXCLUDE_MISSING_COLUMNS,
+  { "exclude-missing-columns", NDB_OPT_NOSHORT,
     "Ignore columns present in backup but not in database",
     (uchar**) &ga_exclude_missing_columns,
     (uchar**) &ga_exclude_missing_columns, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+  { "disable-indexes", NDB_OPT_NOSHORT,
+    "Disable indexes",
+    (uchar**) &ga_disable_indexes,
+    (uchar**) &ga_disable_indexes, 0,
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+  { "rebuild-indexes", NDB_OPT_NOSHORT,
+    "Rebuild indexes",
+    (uchar**) &ga_rebuild_indexes,
+    (uchar**) &ga_rebuild_indexes, 0,
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+  { "skip-unknown-objects", 256, "Skip unknown object when parsing backup",
+    (uchar**) &ga_skip_unknown_objects, (uchar**) &ga_skip_unknown_objects, 0,
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { 0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
@@ -367,7 +386,7 @@ static bool analyse_nodegroup_map(const char *ng_map_str,
 
 static void short_usage_sub(void)
 {
-  ndb_short_usage_sub(my_progname,"[<path to backup files>]");
+  ndb_short_usage_sub("[<path to backup files>]");
 }
 static void usage()
 {
@@ -419,6 +438,12 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
       exit(NDBT_ProgramExit(NDBT_WRONGARGS));
     }
     break;
+  case OPT_INCLUDE_DATABASES:
+  case OPT_EXCLUDE_DATABASES:
+  case OPT_INCLUDE_TABLES:
+  case OPT_EXCLUDE_TABLES:
+    save_include_exclude(optid, argument);
+    break;
   }
   return 0;
 }
@@ -471,10 +496,9 @@ BaseString
 makeExternalTableName(const BaseString &internalName)
 {
    // Make dbname/def/table1 into dbname.table1
-  int idx;
   BaseString externalName;
   
-  idx = internalName.indexOf('/');
+  ssize_t idx = internalName.indexOf('/');
   externalName = internalName.substr(0,idx);
   externalName.append(".");
   externalName.append(internalName.substr(idx + SCHEMA_NAME_SIZE,
@@ -494,7 +518,7 @@ readArguments(int *pargc, char*** pargv)
   load_defaults("my",load_default_groups,pargc,pargv);
   debug << "handle_options" << endl;
 
-  ndb_opt_set_usage_funcs(NULL, short_usage_sub, usage);
+  ndb_opt_set_usage_funcs(short_usage_sub, usage);
 
   if (handle_options(pargc, pargv, my_long_options, get_one_option))
   {
@@ -535,7 +559,9 @@ o verify nodegroup mapping
   if (g_printer == NULL)
     return false;
 
-  BackupRestore* restore = new BackupRestore(opt_nodegroup_map,
+  BackupRestore* restore = new BackupRestore(opt_ndb_connectstring,
+                                             opt_ndb_nodeid,
+                                             opt_nodegroup_map,
                                              opt_nodegroup_map_len,
                                              ga_nParallelism);
   if (restore == NULL) 
@@ -599,6 +625,16 @@ o verify nodegroup mapping
     restore->m_restore_epoch = true;
   }
 
+  if (ga_disable_indexes)
+  {
+    restore->m_disable_indexes = true;
+  }
+
+  if (ga_rebuild_indexes)
+  {
+    restore->m_rebuild_indexes = true;
+  }
+
   {
     BackupConsumer * c = g_printer;
     g_consumers.push_back(c);
@@ -710,11 +746,22 @@ o verify nodegroup mapping
   g_ndbrecord_print_format.hex_prefix= "";
   g_ndbrecord_print_format.hex_format= opt_hex_format;
 
+  if (ga_skip_table_check)
+  {
+    g_tableCompabilityMask = ~(Uint32)0;
+    ga_skip_unknown_objects = true;
+  }
+
   if (ga_promote_attributes)
   {
     g_tableCompabilityMask |= TCM_ATTRIBUTE_PROMOTION;
   }
 
+  if (ga_demote_attributes)
+  {
+    g_tableCompabilityMask |= TCM_ATTRIBUTE_DEMOTION;
+  }
+
   if (ga_exclude_missing_columns)
   {
     g_tableCompabilityMask |= TCM_EXCLUDE_MISSING_COLUMNS;
@@ -733,7 +780,7 @@ clearConsumers()
 static inline bool
 checkSysTable(const TableS* table)
 {
-  return ga_dont_ignore_systab_0 || ! table->getSysTable();
+  return ! table->getSysTable();
 }
 
 static inline bool
@@ -757,8 +804,13 @@ isIndex(const TableS* table)
 }
 
 static inline bool
-isInList(BaseString &needle, Vector<BaseString> &lst)
+isSYSTAB_0(const TableS* table)
 {
+  return table->isSYSTAB_0();
+}
+
+static inline bool
+isInList(BaseString &needle, Vector<BaseString> &lst){
   unsigned int i= 0;
   for (i= 0; i < lst.size(); i++)
   {
@@ -783,43 +835,120 @@ getTableName(const TableS* table)
   return table_name;
 }
 
+static void save_include_exclude(int optid, char * argument)
+{
+  BaseString arg = argument;
+  Vector<BaseString> args;
+  arg.split(args, ",");
+  for (uint i = 0; i < args.size(); i++)
+  {
+    RestoreOption * option = new RestoreOption();
+    BaseString arg;
+    
+    option->optid = optid;
+    switch (optid) {
+    case OPT_INCLUDE_TABLES:
+    case OPT_EXCLUDE_TABLES:
+      if (makeInternalTableName(args[i], arg))
+      {
+        info << "`" << args[i] << "` is not a valid tablename!" << endl;
+        exit(NDBT_ProgramExit(NDBT_WRONGARGS));
+      }
+      break;
+    default:
+      arg = args[i];
+      break;
+    }
+    option->argument = arg;
+    g_include_exclude.push_back(option);
+  }
+}
+static bool check_include_exclude(BaseString database, BaseString table)
+{
+  const char * db = database.c_str();
+  const char * tbl = table.c_str();
+  bool do_include = true;
+
+  if (g_include_databases.size() != 0 ||
+      g_include_tables.size() != 0)
+  {
+    /*
+      User has explicitly specified what databases
+      and/or tables should be restored. If no match is
+      found then DON'T restore table.
+     */
+    do_include = false;
+  }
+  if (do_include &&
+      (g_exclude_databases.size() != 0 ||
+       g_exclude_tables.size() != 0))
+  {
+    /*
+      User has not explicitly specified what databases
+      and/or tables should be restored.
+      User has explicitly specified what databases
+      and/or tables should NOT be restored. If no match is
+      found then DO restore table.
+     */
+    do_include = true;
+  }
+
+  if (g_include_exclude.size() != 0)
+  {
+    /*
+      Scan include exclude arguments in reverse.
+      First matching include causes table to be restored.
+      first matching exclude causes table NOT to be restored.      
+     */
+    for(uint i = g_include_exclude.size(); i > 0; i--)
+    {
+      RestoreOption *option = g_include_exclude[i-1];
+      switch (option->optid) {
+      case OPT_INCLUDE_TABLES:
+        if (strcmp(tbl, option->argument.c_str()) == 0)
+          return true; // do include
+        break;
+      case OPT_EXCLUDE_TABLES:
+        if (strcmp(tbl, option->argument.c_str()) == 0)
+          return false; // don't include
+        break;
+      case OPT_INCLUDE_DATABASES:
+        if (strcmp(db, option->argument.c_str()) == 0)
+          return true; // do include
+        break;
+      case OPT_EXCLUDE_DATABASES:
+        if (strcmp(db, option->argument.c_str()) == 0)
+          return false; // don't include
+        break;
+      default:
+        continue;
+      }
+    }
+  }
+  
+  return do_include;
+}
+
 static inline bool
 checkDoRestore(const TableS* table)
 {
-  int idx;
   bool ret = true;
   BaseString db, tbl;
   
   tbl.assign(getTableName(table));
-  idx = tbl.indexOf('/');
+  ssize_t idx = tbl.indexOf('/');
   db = tbl.substr(0, idx);
   
-  /* Included tables overrides
-   * Excluded tables which overrides
-   * Included databases which overrides
-   * Excluded databases.
-   * If any databases are included, then only
-   * included databases are restored.
-   * If any tables are included, then only
-   * included tables are restored.
+  /*
+    Include/exclude flags are evaluated right
+    to left, and first match overrides any other
+    matches. Non-overlapping arguments are accumulative.
+    If no include flags are specified this means all databases/tables
+    except any excluded are restored.
+    If include flags are specified than only those databases
+    or tables specified are restored.
    */
-
-  if (g_exclude_databases.size() != 0) {
-    if (isInList(db, g_exclude_databases))
-      ret = false;
-  }
-  if (g_include_databases.size() != 0) {
-    ret= isInList(db, g_include_databases);
-  }
-  
-  if (g_exclude_tables.size() != 0) {
-    if (isInList(tbl, g_exclude_tables))
-      ret = false;
-  }
-  if (g_include_tables.size() != 0) {
-    ret= isInList(tbl, g_include_tables);
-  }
-  
+  ret = check_include_exclude(db, tbl);
   return ret;
 }
 
@@ -878,7 +1007,6 @@ free_data_callback()
     g_consumers[i]->tuple_free();
 }
 
-const char * g_connect_string = 0;
 static void exitHandler(int code)
 {
   NDBT_ProgramExit(code);
@@ -919,6 +1047,27 @@ static void report_progress(const char *prefix, const BackupFile &f)
     info << prefix << f.get_file_pos() << " bytes\n";
 }
 
+/**
+ * Reports, clears information on columns where data truncation was detected.
+ */
+static void
+check_data_truncations(const TableS * table)
+{
+  assert(table);
+  const char * tname = table->getTableName();
+  const int n = table->getNoOfAttributes();
+  for (int i = 0; i < n; i++) {
+    AttributeDesc * desc = table->getAttributeDesc(i);
+    if (desc->truncation_detected) {
+      const char * cname = desc->m_column->getName();
+      info.setLevel(254);
+      info << "Data truncation(s) detected for attribute: "
+           << tname << "." << cname << endl;
+      desc->truncation_detected = false;
+    }
+  }
+}
+
 int
 main(int argc, char** argv)
 {
@@ -938,6 +1087,8 @@ main(int argc, char** argv)
     g_options.appfmt(" -u");
   if (ga_promote_attributes)
     g_options.appfmt(" -A");
+  if (ga_demote_attributes)
+    g_options.appfmt(" -L");
   if (_preserve_trailing_spaces)
     g_options.appfmt(" -P");
   if (ga_skip_table_check)
@@ -950,10 +1101,12 @@ main(int argc, char** argv)
     g_options.appfmt(" -d");
   if (ga_exclude_missing_columns)
     g_options.append(" --exclude-missing-columns");
+  if (ga_disable_indexes)
+    g_options.append(" --disable-indexes");
+  if (ga_rebuild_indexes)
+    g_options.append(" --rebuild-indexes");
   g_options.appfmt(" -p %d", ga_nParallelism);
 
-  g_connect_string = opt_connect_str;
-
   init_progress();
 
   /**
@@ -985,8 +1138,6 @@ main(int argc, char** argv)
 
   info << endl;
 
-  info << "Stop GCP of Backup: " << metaData.getStopGCP() << endl;
-
   /**
    * check wheater we can restore the backup (right version).
    */
@@ -1011,6 +1162,8 @@ main(int argc, char** argv)
 
   debug << "Load content" << endl;
   int res  = metaData.loadContent();
+
+  info << "Stop GCP of Backup: " << metaData.getStopGCP() << endl;
   
   if (res == 0)
   {
@@ -1075,6 +1228,10 @@ main(int argc, char** argv)
     table_output.push_back(NULL);
     if (!checkDbAndTableName(table))
       continue;
+    if (isSYSTAB_0(table))
+    {
+      table_output[i]= ndbout.m_out;
+    }
     if (checkSysTable(table))
     {
       if (!tab_path || isBlobTable(table) || isIndex(table))
@@ -1145,40 +1302,20 @@ main(int argc, char** argv)
   {
     if(_restore_data || _print_data)
     {
-      if (!ga_skip_table_check && g_tableCompabilityMask == 0)
-      {
-        for(i=0; i < metaData.getNoOfTables(); i++)
+      // Check table compatibility
+      for (i=0; i < metaData.getNoOfTables(); i++){
+        if (checkSysTable(metaData, i) &&
+            checkDbAndTableName(metaData[i]))
         {
-          if (checkSysTable(metaData, i) && 
-             checkDbAndTableName(metaData[i]))
+          for(Uint32 j= 0; j < g_consumers.size(); j++)
           {
-            for(Uint32 j= 0; j < g_consumers.size(); j++)
-              if (!g_consumers[j]->table_equal(* metaData[i]))
-              {
-                err << "Restore: Failed to restore data, ";
-                err << metaData[i]->getTableName() << " table structure doesn't match backup ... Exiting " << endl;
-                exitHandler(NDBT_FAILED);
-              }
-          }
-        }
-      }
-      if (g_tableCompabilityMask != 0)
-      { 
-        //if want to promote attributes, compability check is done firstly
-        for (i=0; i < metaData.getNoOfTables(); i++){
-          if (checkSysTable(metaData, i) &&
-              checkDbAndTableName(metaData[i]))
-          {
-            for(Uint32 j= 0; j < g_consumers.size(); j++)
+            if (!g_consumers[j]->table_compatible_check(*metaData[i]))
             {
-              if (!g_consumers[j]->table_compatible_check(*metaData[i]))
-              {
-                err << "Restore: Failed to restore data, ";
-                err << metaData[i]->getTableName() << " table structure incompatible with backup's ... Exiting " << endl;
-                exitHandler(NDBT_FAILED);
-              } 
+              err << "Restore: Failed to restore data, ";
+              err << metaData[i]->getTableName() << " table structure incompatible with backup's ... Exiting " << endl;
+              exitHandler(NDBT_FAILED);
             } 
-          }
+          } 
         }
       }
       RestoreDataIterator dataIter(metaData, &free_data_callback);
@@ -1283,9 +1420,10 @@ main(int argc, char** argv)
     
     if(_restore_data)
     {
-      for(i = 0; i<metaData.getNoOfTables(); i++)
+      for(i = 0; i < metaData.getNoOfTables(); i++)
       {
         const TableS* table = metaData[i];
+        check_data_truncations(table);
         OutputStream *output = table_output[table->getLocalId()];
         if (!output)
           continue;
@@ -1319,7 +1457,25 @@ main(int argc, char** argv)
                "please look at configuration.");
     }               
   }
-  
+
+  if (ga_rebuild_indexes)
+  {
+    debug << "Rebuilding indexes" << endl;
+    for(i = 0; i<metaData.getNoOfTables(); i++)
+    {
+      const TableS *table= metaData[i];
+      if (! (checkSysTable(table) && checkDbAndTableName(table)))
+        continue;
+      if (isBlobTable(table) || isIndex(table))
+        continue;
+      for(Uint32 j= 0; j < g_consumers.size(); j++)
+      {
+        if (!g_consumers[j]->rebuild_indexes(* table))
+          return -1;
+      }
+    }
+  }
+
   /* report to clusterlog if applicable */
   for (i = 0; i < g_consumers.size(); i++)
     g_consumers[i]->report_completed(ga_backupId, ga_nodeId);
@@ -1345,3 +1501,4 @@ main(int argc, char** argv)
 
 template class Vector<BackupConsumer*>;
 template class Vector<OutputStream*>;
+template class Vector<RestoreOption *>;
