@@ -27,7 +27,7 @@
 
 int vio_errno(Vio *vio __attribute__((unused)))
 {
-  return my_socket_errno();
+  return socket_errno;         /* On Win32 this mapped to WSAGetLastError() */
 }
 
 
@@ -35,21 +35,17 @@ size_t vio_read(Vio * vio, uchar* buf, size_t size)
 {
   size_t r;
   DBUG_ENTER("vio_read");
-  DBUG_PRINT("enter", ("sd: " MY_SOCKET_FORMAT "  buf: 0x%lx  size: %u",
-                       MY_SOCKET_FORMAT_VALUE(vio->sd), (long) buf,
+  DBUG_PRINT("enter", ("sd: %d  buf: 0x%lx  size: %u", vio->sd, (long) buf,
                        (uint) size));
 
   /* Ensure nobody uses vio_read_buff and vio_read simultaneously */
   DBUG_ASSERT(vio->read_end == vio->read_pos);
-
-  /*
-    Callers of 'vio_read' checks "errno" even if no system function
-    failed, to avoid false positives "errno" must be reset
-  */
-  my_socket_set_errno(0);
-
-  r = my_recv(vio->sd, buf, size, 0);
-
+#ifdef __WIN__
+  r = recv(vio->sd, buf, size,0);
+#else
+  errno=0;                                     /* For linux */
+  r = read(vio->sd, buf, size);
+#endif /* __WIN__ */
 #ifndef DBUG_OFF
   if (r == (size_t) -1)
   {
@@ -71,8 +67,7 @@ size_t vio_read_buff(Vio *vio, uchar* buf, size_t size)
   size_t rc;
 #define VIO_UNBUFFERED_READ_MIN_SIZE 2048
   DBUG_ENTER("vio_read_buff");
-  DBUG_PRINT("enter", ("sd: " MY_SOCKET_FORMAT "  buf: 0x%lx  size: %u",
-                       MY_SOCKET_FORMAT_VALUE(vio->sd), (long) buf,
+  DBUG_PRINT("enter", ("sd: %d  buf: 0x%lx  size: %u", vio->sd, (long) buf,
                        (uint) size));
 
   if (vio->read_pos < vio->read_end)
@@ -111,11 +106,13 @@ size_t vio_write(Vio * vio, const uchar* buf, size_t size)
 {
   size_t r;
   DBUG_ENTER("vio_write");
-  DBUG_PRINT("enter", ("sd: " MY_SOCKET_FORMAT "  buf: 0x%lx  size: %u",
-                       MY_SOCKET_FORMAT_VALUE(vio->sd), (long)buf, (uint)size));
-
-  r = my_send(vio->sd, buf, size, 0);
-
+  DBUG_PRINT("enter", ("sd: %d  buf: 0x%lx  size: %u", vio->sd, (long) buf,
+                       (uint) size));
+#ifdef __WIN__
+  r = send(vio->sd, buf, size,0);
+#else
+  r = write(vio->sd, buf, size);
+#endif /* __WIN__ */
 #ifndef DBUG_OFF
   if (r == (size_t) -1)
   {
@@ -138,7 +135,7 @@ int vio_blocking(Vio * vio __attribute__((unused)), my_bool set_blocking_mode,
 
 #if !defined(__WIN__)
 #if !defined(NO_FCNTL_NONBLOCK)
-  if (my_socket_valid(vio->sd))
+  if (vio->sd >= 0)
   {
     int old_fcntl=vio->fcntl_mode;
     if (set_blocking_mode)
@@ -147,10 +144,10 @@ int vio_blocking(Vio * vio __attribute__((unused)), my_bool set_blocking_mode,
       vio->fcntl_mode |= O_NONBLOCK; /* set bit */
     if (old_fcntl != vio->fcntl_mode)
     {
-      r= my_socket_nonblock(vio->sd, (vio->fcntl_mode & O_NONBLOCK));
-      if (r!=0)
+      r= fcntl(vio->sd, F_SETFL, vio->fcntl_mode);
+      if (r == -1)
       {
-        DBUG_PRINT("info", ("fcntl failed, errno %d", r));
+        DBUG_PRINT("info", ("fcntl failed, errno %d", errno));
         vio->fcntl_mode= old_fcntl;
       }
     }
@@ -174,7 +171,7 @@ int vio_blocking(Vio * vio __attribute__((unused)), my_bool set_blocking_mode,
       vio->fcntl_mode |= O_NONBLOCK; /* set bit */
     }
     if (old_fcntl != vio->fcntl_mode)
-      r = ioctlsocket(vio->sd.s,FIONBIO,(void*) &arg);
+      r = ioctlsocket(vio->sd,FIONBIO,(void*) &arg);
   }
   else
     r=  test(!(vio->fcntl_mode & O_NONBLOCK)) != set_blocking_mode;
@@ -202,7 +199,7 @@ int vio_fastsend(Vio * vio __attribute__((unused)))
 #if defined(IPTOS_THROUGHPUT)
   {
     int tos = IPTOS_THROUGHPUT;
-    r= my_setsockopt(vio->sd, IPPROTO_IP, IP_TOS, (void *) &tos, sizeof(tos));
+    r= setsockopt(vio->sd, IPPROTO_IP, IP_TOS, (void *) &tos, sizeof(tos));
   }
 #endif                                    /* IPTOS_THROUGHPUT */
   if (!r)
@@ -213,8 +210,9 @@ int vio_fastsend(Vio * vio __attribute__((unused)))
     int nodelay = 1;
 #endif
 
-    r= my_setsockopt(vio->sd, IPPROTO_TCP, TCP_NODELAY,
-                  &nodelay, sizeof(nodelay));
+    r= setsockopt(vio->sd, IPPROTO_TCP, TCP_NODELAY,
+                  IF_WIN(const char*, void*) &nodelay,
+                  sizeof(nodelay));
 
   }
   if (r)
@@ -231,13 +229,13 @@ int vio_keepalive(Vio* vio, my_bool set_keep_alive)
   int r=0;
   uint opt = 0;
   DBUG_ENTER("vio_keepalive");
-  DBUG_PRINT("enter", ("sd: " MY_SOCKET_FORMAT "  set_keep_alive: %d",
-                       MY_SOCKET_FORMAT_VALUE(vio->sd), (int)set_keep_alive));
+  DBUG_PRINT("enter", ("sd: %d  set_keep_alive: %d", vio->sd, (int)
+                      set_keep_alive));
   if (vio->type != VIO_TYPE_NAMEDPIPE)
   {
     if (set_keep_alive)
       opt = 1;
-    r = my_setsockopt(vio->sd, SOL_SOCKET, SO_KEEPALIVE, (char *) &opt,
+    r = setsockopt(vio->sd, SOL_SOCKET, SO_KEEPALIVE, (char *) &opt,
 		   sizeof(opt));
   }
   DBUG_RETURN(r);
@@ -266,23 +264,17 @@ int vio_close(Vio * vio)
 {
   int r=0;
   DBUG_ENTER("vio_close");
-#ifdef __WIN__
-  if (vio->type == VIO_TYPE_NAMEDPIPE)
-  {
-#if defined(__NT__) && defined(MYSQL_SERVER)
-    CancelIo(vio->hPipe);
-    DisconnectNamedPipe(vio->hPipe);
-#endif
-    r=CloseHandle(vio->hPipe);
-  }
-  else
-#endif /* __WIN__ */
+
  if (vio->type != VIO_CLOSED)
   {
-    DBUG_ASSERT(my_socket_valid(vio->sd));
-    if (my_shutdown(vio->sd, SHUT_RDWR))
+    DBUG_ASSERT(vio->type ==  VIO_TYPE_TCPIP ||
+      vio->type == VIO_TYPE_SOCKET ||
+      vio->type == VIO_TYPE_SSL);
+
+    DBUG_ASSERT(vio->sd >= 0);
+    if (shutdown(vio->sd, SHUT_RDWR))
       r= -1;
-    if (my_socket_close(vio->sd))
+    if (closesocket(vio->sd))
       r= -1;
   }
   if (r)
@@ -291,7 +283,7 @@ int vio_close(Vio * vio)
     /* FIXME: error handling (not critical for MySQL) */
   }
   vio->type= VIO_CLOSED;
-  my_socket_invalidate(&(vio->sd));
+  vio->sd=   -1;
   DBUG_RETURN(r);
 }
 
@@ -403,8 +395,8 @@ my_bool vio_peer_addr(Vio * vio, char *ip_buffer, uint16 *port,
                       size_t ip_buffer_size)
 {
   DBUG_ENTER("vio_peer_addr");
-  DBUG_PRINT("enter", ("Client socket fd: " MY_SOCKET_FORMAT,
-                       MY_SOCKET_FORMAT_VALUE(vio->sd)));
+  DBUG_PRINT("enter", ("Client socket fd: %d", (int) vio->sd));
+
   if (vio->localhost)
   {
     /*
@@ -435,7 +427,7 @@ my_bool vio_peer_addr(Vio * vio, char *ip_buffer, uint16 *port,
 
     /* Get sockaddr by socked fd */
 
-    err_code= my_getpeername(vio->sd, addr, &addr_length);
+    err_code= getpeername(vio->sd, addr, &addr_length);
 
     if (err_code)
     {
@@ -482,7 +474,7 @@ my_bool vio_poll_read(Vio *vio,uint timeout)
   struct pollfd fds;
   int res;
   DBUG_ENTER("vio_poll");
-  fds.fd=vio->sd.fd; /* FIXME: Vista has WSAPoll(), can emulate elsewhere */
+  fds.fd=vio->sd;
   fds.events=POLLIN;
   fds.revents=0;
   if ((res=poll(&fds,1,(int) timeout*1000)) <= 0)
@@ -511,8 +503,9 @@ void vio_timeout(Vio *vio, uint which, uint timeout)
   wait_timeout.tv_usec= 0;
 #endif
 
-  r= my_setsockopt(vio->sd, SOL_SOCKET, which ? SO_SNDTIMEO : SO_RCVTIMEO,
-                   &wait_timeout, sizeof(wait_timeout));
+  r= setsockopt(vio->sd, SOL_SOCKET, which ? SO_SNDTIMEO : SO_RCVTIMEO,
+                IF_WIN(const char*, const void*)&wait_timeout,
+                sizeof(wait_timeout));
 
   }
 
@@ -532,59 +525,134 @@ void vio_timeout(Vio *vio, uint which, uint timeout)
 
 
 #ifdef __WIN__
-size_t vio_read_pipe(Vio * vio, uchar* buf, size_t size)
+
+/*
+  Finish pending IO on pipe. Honor wait timeout
+*/
+static size_t pipe_complete_io(Vio* vio, char* buf, size_t size, DWORD timeout_ms)
 {
   DWORD length;
+  DWORD ret;
+
+  DBUG_ENTER("pipe_complete_io");
+
+  ret= WaitForSingleObject(vio->pipe_overlapped.hEvent, timeout_ms);
+  /*
+    WaitForSingleObjects will normally return WAIT_OBJECT_O (success, IO completed)
+    or WAIT_TIMEOUT.
+  */
+  if(ret != WAIT_OBJECT_0)
+  {
+    CancelIo(vio->hPipe);
+    DBUG_PRINT("error",("WaitForSingleObject() returned  %d", ret));
+    DBUG_RETURN((size_t)-1);
+  }
+
+  if (!GetOverlappedResult(vio->hPipe,&(vio->pipe_overlapped),&length, FALSE))
+  {
+    DBUG_PRINT("error",("GetOverlappedResult() returned last error  %d", 
+      GetLastError()));
+    DBUG_RETURN((size_t)-1);
+  }
+
+  DBUG_RETURN(length);
+}
+
+
+size_t vio_read_pipe(Vio * vio, uchar *buf, size_t size)
+{
+  DWORD bytes_read;
+  size_t retval;
   DBUG_ENTER("vio_read_pipe");
   DBUG_PRINT("enter", ("sd: %d  buf: 0x%lx  size: %u", vio->sd, (long) buf,
                        (uint) size));
 
-  if (!ReadFile(vio->hPipe, buf, size, &length, NULL))
-    DBUG_RETURN(-1);
+  if (ReadFile(vio->hPipe, buf, (DWORD)size, &bytes_read,
+      &(vio->pipe_overlapped)))
+  {
+    retval= bytes_read;
+  }
+  else
+  {
+    if (GetLastError() != ERROR_IO_PENDING)
+    {
+      DBUG_PRINT("error",("ReadFile() returned last error %d",
+        GetLastError()));
+      DBUG_RETURN((size_t)-1);
+    }
+    retval= pipe_complete_io(vio, buf, size,vio->read_timeout_ms);
+  }
 
-  DBUG_PRINT("exit", ("%d", length));
-  DBUG_RETURN((size_t) length);
+  DBUG_PRINT("exit", ("%lld", (longlong)retval));
+  DBUG_RETURN(retval);
 }
 
 
 size_t vio_write_pipe(Vio * vio, const uchar* buf, size_t size)
 {
-  DWORD length;
+  DWORD bytes_written;
+  size_t retval;
   DBUG_ENTER("vio_write_pipe");
   DBUG_PRINT("enter", ("sd: %d  buf: 0x%lx  size: %u", vio->sd, (long) buf,
                        (uint) size));
 
-  if (!WriteFile(vio->hPipe, (char*) buf, size, &length, NULL))
-    DBUG_RETURN(-1);
+  if (WriteFile(vio->hPipe, buf, (DWORD)size, &bytes_written, 
+      &(vio->pipe_overlapped)))
+  {
+    retval= bytes_written;
+  }
+  else
+  {
+    if (GetLastError() != ERROR_IO_PENDING)
+    {
+      DBUG_PRINT("vio_error",("WriteFile() returned last error %d",
+        GetLastError()));
+      DBUG_RETURN((size_t)-1);
+    }
+    retval= pipe_complete_io(vio, (char *)buf, size, vio->write_timeout_ms);
+  }
 
-  DBUG_PRINT("exit", ("%d", length));
-  DBUG_RETURN((size_t) length);
+  DBUG_PRINT("exit", ("%lld", (longlong)retval));
+  DBUG_RETURN(retval);
 }
 
+
 int vio_close_pipe(Vio * vio)
 {
   int r;
   DBUG_ENTER("vio_close_pipe");
-#if defined(__NT__) && defined(MYSQL_SERVER)
-  CancelIo(vio->hPipe);
+
+  CloseHandle(vio->pipe_overlapped.hEvent);
   DisconnectNamedPipe(vio->hPipe);
-#endif
-  r=CloseHandle(vio->hPipe);
+  r= CloseHandle(vio->hPipe);
   if (r)
   {
     DBUG_PRINT("vio_error", ("close() failed, error: %d",GetLastError()));
     /* FIXME: error handling (not critical for MySQL) */
   }
   vio->type= VIO_CLOSED;
-  my_socket_invalidate(&(vio->sd));
+  vio->sd=   -1;
   DBUG_RETURN(r);
 }
 
 
-void vio_ignore_timeout(Vio *vio __attribute__((unused)),
-			uint which __attribute__((unused)),
-			uint timeout __attribute__((unused)))
+void vio_win32_timeout(Vio *vio, uint which , uint timeout_sec)
 {
+    DWORD timeout_ms;
+    /*
+      Windows is measuring timeouts in milliseconds. Check for possible int 
+      overflow.
+    */
+    if (timeout_sec > UINT_MAX/1000)
+      timeout_ms= INFINITE;
+    else
+      timeout_ms= timeout_sec * 1000;
+
+    /* which == 1 means "write", which == 0 means "read".*/
+    if(which)
+      vio->write_timeout_ms= timeout_ms;
+    else
+      vio->read_timeout_ms= timeout_ms;
 }
 
 
@@ -619,7 +687,7 @@ size_t vio_read_shared_memory(Vio * vio, uchar* buf, size_t size)
          WAIT_ABANDONED_0 and WAIT_TIMEOUT - fail.  We can't read anything
       */
       if (WaitForMultipleObjects(array_elements(events), events, FALSE,
-                                 vio->net->read_timeout*1000) != WAIT_OBJECT_0)
+                                 vio->read_timeout_ms) != WAIT_OBJECT_0)
       {
         DBUG_RETURN(-1);
       };
@@ -676,7 +744,7 @@ size_t vio_write_shared_memory(Vio * vio, const uchar* buf, size_t size)
   while (remain != 0)
   {
     if (WaitForMultipleObjects(array_elements(events), events, FALSE,
-                               vio->net->write_timeout*1000) != WAIT_OBJECT_0)
+                               vio->write_timeout_ms) != WAIT_OBJECT_0)
     {
       DBUG_RETURN((size_t) -1);
     }
@@ -755,7 +823,7 @@ int vio_close_shared_memory(Vio * vio)
     }
   }
   vio->type= VIO_CLOSED;
-  my_socket_invalidate(&(vio->sd));
+  vio->sd=   -1;
   DBUG_RETURN(error_count);
 }
 #endif /* HAVE_SMEM */
@@ -788,8 +856,11 @@ my_bool vio_get_normalized_ip_string(const struct sockaddr *addr,
                                      char *ip_string,
                                      size_t ip_string_size)
 {
-  struct sockaddr_storage norm_addr_storage;
-  struct sockaddr *norm_addr= (struct sockaddr *) &norm_addr_storage;
+  union {
+    struct sockaddr_storage norm_addr_storage;
+    struct sockaddr _avoid_strict_alias_warning;
+  } tmp_addr;
+  struct sockaddr *norm_addr= (struct sockaddr *) &tmp_addr.norm_addr_storage;
   int norm_addr_length;
   int err_code;
 
@@ -810,9 +881,11 @@ my_bool vio_get_normalized_ip_string(const struct sockaddr *addr,
 
 /**
   This is a wrapper for the system getnameinfo(), because different OS
-  differ in the getnameinfo() implementation. For instance, Solaris 10
-  requires that the 2nd argument (salen) must match the actual size of the
-  struct sockaddr_storage passed to it.
+  differ in the getnameinfo() implementation:
+    - Solaris 10 requires that the 2nd argument (salen) must match the
+      actual size of the struct sockaddr_storage passed to it;
+    - Mac OS X has sockaddr_in::sin_len and sockaddr_in6::sin6_len and
+      requires them to be filled.
 */
 
 int vio_getnameinfo(const struct sockaddr *sa,
@@ -825,11 +898,17 @@ int vio_getnameinfo(const struct sockaddr *sa,
   switch (sa->sa_family) {
   case AF_INET:
     sa_length= sizeof (struct sockaddr_in);
+#ifdef HAVE_SOCKADDR_IN_SIN_LEN
+    ((struct sockaddr_in *) sa)->sin_len= sa_length;
+#endif /* HAVE_SOCKADDR_IN_SIN_LEN */
     break;
 
 #ifdef HAVE_IPV6
   case AF_INET6:
     sa_length= sizeof (struct sockaddr_in6);
+# ifdef HAVE_SOCKADDR_IN6_SIN6_LEN
+    ((struct sockaddr_in6 *) sa)->sin6_len= sa_length;
+# endif /* HAVE_SOCKADDR_IN6_SIN6_LEN */
     break;
 #endif /* HAVE_IPV6 */
   }
